@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -30,17 +30,10 @@ public:
 
     int getBetCount() const;
 
-#if (!defined(_MSC_VER) || (_MSC_VER >= 1300)) && (!defined(__BCPLUSPLUS__))
-//
-// Some compilers don't let local classes access private members
-//
 private:
-#endif
 
     void decrementBetCount();
-
-private:
-
+    
     int _betCount;
     IceUtil::Mutex _mutex;
     std::vector<IceUtil::TimerPtr> _timers;
