@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -17,7 +17,11 @@ BookFactory::BookFactory(const LibraryIPtr& library) :
 }
 
 Ice::ObjectPtr
+#ifndef NDEBUG
 BookFactory::create(const string& type)
+#else
+BookFactory::create(const string&)
+#endif
 {
     assert(_library);
     assert(type == "::Demo::Book");
