@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,7 +14,7 @@ using namespace std;
 using namespace Demo;
 
 int
-runParser(int argc, char* argv[], const Ice::CommunicatorPtr& communicator)
+runParser(int, char* argv[], const Ice::CommunicatorPtr& communicator)
 {
     LibraryPrx library = LibraryPrx::checkedCast(communicator->propertyToProxy("Library.Proxy"));
     if(!library)
