@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -29,7 +29,7 @@ int main(int argc, char* argv[])
 }
 
 int
-ReadNew::run(int argc, char* argv[])
+ReadNew::run(int, char*[])
 {
     ConnectionPtr connection = createConnection(communicator(), "dbnew");
     bool createDb = true;
