@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -20,7 +20,7 @@ NestedI::NestedI(const NestedPrx& self) :
 }
 
 void
-NestedI::nestedCall(Int level, const NestedPrx& proxy, const Ice::Current& current)
+NestedI::nestedCall(Int level, const NestedPrx& proxy, const Ice::Current&)
 {
     cout << level << endl;
     if(--level > 0)
