@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,7 +16,7 @@ class PluginServer : public Ice::Application
 public:
 
     virtual int 
-    run(int argc, char* argv[])
+    run(int argc, char*[])
     {
         if(argc > 1)
         {
