@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -26,6 +26,11 @@ ObjectFactory::create(const string& type)
         return new DerivedPrinterI;
     }
 
+    if(type == Demo::ClientPrinter::ice_staticId())
+    {
+        return new ClientPrinterI;
+    }
+
     assert(false);
     return 0;
 }
