@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -9,6 +9,7 @@
 
 #include <Ice/Ice.h>
 #include <ValueI.h>
+#include <ObjectFactory.h>
 
 using namespace std;
 
@@ -27,7 +28,7 @@ main(int argc, char* argv[])
 }
 
 int
-ValueServer::run(int argc, char* argv[])
+ValueServer::run(int argc, char*[])
 {
     if(argc > 1)
     {
@@ -35,6 +36,9 @@ ValueServer::run(int argc, char* argv[])
         return EXIT_FAILURE;
     }
 
+    Ice::ObjectFactoryPtr factory = new ObjectFactory;
+    communicator()->addObjectFactory(factory, Demo::Printer::ice_staticId());
+
     Ice::ObjectAdapterPtr adapter = communicator()->createObjectAdapter("Value");
     Demo::InitialPtr initial = new InitialI(adapter);
     adapter->add(initial, communicator()->stringToIdentity("initial"));
