@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -21,6 +21,7 @@ public:
     virtual Demo::SimplePtr getSimple(const Ice::Current&);
     virtual void getPrinter(::Demo::PrinterPtr&, Demo::PrinterPrx&, const Ice::Current&);
     virtual Demo::PrinterPtr getDerivedPrinter(const Ice::Current&);
+    virtual Demo::PrinterPtr updatePrinterMessage(const Demo::PrinterPtr&, const Ice::Current&);
     virtual void throwDerivedPrinter(const Ice::Current&);
     virtual void shutdown(const Ice::Current&);
 
@@ -46,4 +47,8 @@ public:
     virtual void printUppercase(const Ice::Current&);
 };
 
+class ClientPrinterI : virtual public Demo::ClientPrinter, virtual public PrinterI
+{
+};
+
 #endif
