@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -36,7 +36,7 @@ main(int argc, char* argv[])
 }
 
 int
-Server::run(int argc, char* argv[])
+Server::run(int argc, char*[])
 {
     if(argc > 1)
     {
