@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -21,7 +21,7 @@ main(int argc, char * argv[])
     try
     {
         ic = Ice::initialize(argc, argv);
-        Ice::ObjectPrx base = ic->stringToProxy("SimplePrinter:default -p 10000");
+        Ice::ObjectPrx base = ic->stringToProxy("SimplePrinter:default -h localhost -p 10000");
         PrinterPrx printer = PrinterPrx::checkedCast(base);
         if(!printer)
         {
