@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -42,10 +42,6 @@ public:
     {
         return _ref;
     }
-
-protected:
-
-    friend class IceInternal::GC; // Allows IceInternal::GC to read value of _ref.
 };
 
 }
