@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -10,8 +10,10 @@
 #ifndef ICE_ICE_H
 #define ICE_ICE_H
 
+#include <IceUtil/Config.h>
 #include <Ice/Initialize.h>
 #include <Ice/LocalException.h>
+#include <Ice/PropertiesAdmin.h>
 #include <Ice/Properties.h>
 #include <Ice/Logger.h>
 #include <Ice/LoggerUtil.h>
@@ -23,10 +25,13 @@
 #include <Ice/ServantLocator.h>
 #include <Ice/Proxy.h>
 #include <Ice/Object.h>
+#include <Ice/SlicedData.h>
 #include <Ice/OutgoingAsync.h>
 #include <Ice/IncomingAsync.h>
 #include <Ice/Process.h>
-#include <Ice/Application.h>
+#ifndef ICE_OS_WINRT
+#   include <Ice/Application.h>
+#endif
 #include <Ice/Connection.h>
 #include <Ice/ConnectionAsync.h>
 #include <Ice/Functional.h>
@@ -36,9 +41,14 @@
 #include <Ice/Router.h>
 #include <Ice/DispatchInterceptor.h>
 #include <Ice/Plugin.h>
-
+#include <Ice/NativePropertiesAdmin.h>
+#include <Ice/Instrumentation.h>
+#include <Ice/Metrics.h>
+#ifndef ICE_OS_WINRT
+#   include <Ice/Service.h>
+#endif
 #ifndef _WIN32
-#include <Ice/IconvStringConverter.h>
+#   include <Ice/IconvStringConverter.h>
 #endif
 
 #endif
