@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -29,12 +29,14 @@ public:
 
     virtual bool operator==(const LocalObject&) const;
     virtual bool operator<(const LocalObject&) const;
-    virtual ::Ice::Int ice_getHash() const;
-    
-    ICE_DEPRECATED_API ::Ice::Int ice_hash() const
+    ICE_DEPRECATED_API ::Ice::Int ice_getHash() const
     {
-        return ice_getHash();
+        return internal_getHash();
     }
+
+protected:
+
+    virtual ::Ice::Int internal_getHash() const;
 };
 
 }
