@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,15 +13,6 @@
 #include <IceUtil/Handle.h>
 #include <Ice/Config.h>
 
-//
-// We include Handle.h here to make sure that the Ice::Handle template
-// is defined before any definition of upCast().
-//
-// See http://gcc.gnu.org/bugzilla/show_bug.cgi?id=25495 for information
-// on why this is necessary.
-//
-#include <Ice/Handle.h>
-
 #include <iosfwd>
 
 namespace IceInternal
@@ -123,17 +114,6 @@ template<typename T>
 class ProxyHandle : public ::IceUtil::HandleBase<T>
 {
 public:
-
-#if defined(__BCPLUSPLUS__) && (__BCPLUSPLUS__ >= 0x0600)
-    //
-    // C++Builder 2009 does not allow setting Prx to 0.
-    //
-    ProxyHandle(int p)
-    {
-	assert(p == 0);
-        this->_ptr = 0;
-    }
-#endif
     
     ProxyHandle(T* p = 0)
     {
