@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -21,12 +21,12 @@ class ICE_API UserExceptionFactory : public IceUtil::Shared
 {
 public:
 
-    virtual void createAndThrow() = 0;
+    virtual void createAndThrow(const ::std::string&) = 0;
     virtual ~UserExceptionFactory() {}
 };
 
 typedef ::IceUtil::Handle<UserExceptionFactory> UserExceptionFactoryPtr;
 
-};
+}
 
 #endif
