@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -10,7 +10,10 @@
 #ifndef ICE_SSL_ICE_SSL_H
 #define ICE_SSL_ICE_SSL_H
 
-#include <IceSSL/Plugin.h>
+#include <IceUtil/Config.h>
+#ifndef ICE_OS_WINRT
+#   include <IceSSL/Plugin.h>
+#endif
 #include <IceSSL/EndpointInfo.h>
 #include <IceSSL/ConnectionInfo.h>
 
