@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,7 +19,9 @@
 #include <IceUtil/AbstractMutex.h>
 #include <IceUtil/Cache.h>
 #include <IceUtil/Cond.h>
-#include <IceUtil/CtrlCHandler.h>
+#ifndef ICE_OS_WINRT
+#   include <IceUtil/CtrlCHandler.h>
+#endif
 #include <IceUtil/Exception.h>
 #include <IceUtil/Functional.h>
 #include <IceUtil/Handle.h>
@@ -28,15 +30,15 @@
 #include <IceUtil/MutexProtocol.h>
 #include <IceUtil/Mutex.h>
 #include <IceUtil/MutexPtrLock.h>
-#include <IceUtil/RWRecMutex.h>
 #include <IceUtil/RecMutex.h>
+#include <IceUtil/ScopedArray.h>
 #include <IceUtil/Shared.h>
-#include <IceUtil/StaticMutex.h>
 #include <IceUtil/Thread.h>
 #include <IceUtil/ThreadException.h>
 #include <IceUtil/Time.h>
 #include <IceUtil/Timer.h>
 #include <IceUtil/UUID.h>
 #include <IceUtil/Unicode.h>
+#include <IceUtil/UniquePtr.h>
 
 #endif
