@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,7 +16,7 @@ namespace IceUtil
 {
 
 template<typename T>
-class ScopedArray : private IceUtil::noncopyable
+class ScopedArray
 {
 public:
 
@@ -49,6 +49,18 @@ public:
         _ptr = ptr;
     }
 
+    ScopedArray& operator=(const ScopedArray& other)
+    {
+        if(_ptr != 0)
+        {
+            delete[] _ptr;
+        }
+        _ptr = other._ptr;
+        const_cast<ScopedArray&>(other)._ptr = 0;
+        return *this;
+    }
+
+
     T& operator[](size_t i) const
     {
         assert(_ptr != 0);
@@ -68,6 +80,13 @@ public:
         _ptr = tmp;
     }
 
+    T* release()
+    {
+        T* tmp = _ptr;
+        _ptr = 0;
+        return tmp;
+    }
+
 private:
 
     T* _ptr;
