@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -44,7 +44,7 @@ private:
 
     Index& _index;
     std::string _dbName;
-    std::auto_ptr<Db> _db;
+    IceUtil::UniquePtr<Db> _db;
     ObjectStoreBase* _store;
 };
 
