@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,6 +14,8 @@
 #include <Freeze/CatalogIndexList.h>
 #include <algorithm>
 
+#include <Ice/StringConverter.h>
+
 using namespace std;
 using namespace Ice;
 using namespace Freeze;
@@ -70,7 +72,8 @@ Freeze::MapDb::~MapDb()
         }
         catch(const ::DbException& dx)
         {
-            throw DatabaseException(__FILE__, __LINE__, dx.what());
+            Ice::Error error(_communicator->getLogger());
+            error << "Freeze.Map: closing Db " << _dbName << " raised DbException: " << dx.what();
         }
     }
 }
@@ -84,6 +87,7 @@ Freeze::MapDb::MapDb(const ConnectionIPtr& connection,
                      bool createDb) :
     Db(connection->dbEnv()->getEnv(), 0),
     _communicator(connection->communicator()),
+    _encoding(connection->encoding()),
     _dbName(dbName),
     _trace(connection->trace()),
     _keyCompare(keyCompare)
@@ -179,7 +183,8 @@ Freeze::MapDb::MapDb(const ConnectionIPtr& connection,
             {
                 flags |= DB_CREATE;
             }
-            open(txn, _dbName.c_str(), 0, DB_BTREE, flags, FREEZE_DB_MODE);
+
+            open(txn, Ice::nativeToUTF8(_communicator, _dbName).c_str(), 0, DB_BTREE, flags, FREEZE_DB_MODE);
             
             
             StringSeq oldIndices;
@@ -204,8 +209,9 @@ Freeze::MapDb::MapDb(const ConnectionIPtr& connection,
                 assert(indexBase->_impl == 0);
                 assert(indexBase->_communicator == 0);
                 indexBase->_communicator = connection->communicator();
+                indexBase->_encoding = connection->encoding();
                 
-                auto_ptr<MapIndexI> indexI;
+                IceUtil::UniquePtr<MapIndexI> indexI;
 
                 try
                 {
@@ -223,11 +229,13 @@ Freeze::MapDb::MapDb(const ConnectionIPtr& connection,
                     throw DatabaseException(__FILE__, __LINE__, message);
                 }
                 
-#ifndef NDEBUG
+#ifdef NDEBUG
+                _indices.insert(IndexMap::value_type(indexBase->name(), indexI.get()));
+#else
                 bool inserted = 
-#endif
                     _indices.insert(IndexMap::value_type(indexBase->name(), indexI.get())).second;
                 assert(inserted);
+#endif
                 
                 indexBase->_impl = indexI.release();
                 
@@ -366,9 +374,15 @@ Freeze::MapDb::MapDb(const ConnectionIPtr& connection,
     }
 }
 
-Freeze::MapDb::MapDb(const Ice::CommunicatorPtr& communicator, const string& dbName, const string& keyTypeId, const string& valueTypeId, DbEnv* env) :
+Freeze::MapDb::MapDb(const Ice::CommunicatorPtr& communicator, 
+                     const Ice::EncodingVersion& encoding,
+                     const string& dbName, 
+                     const string& keyTypeId, 
+                     const string& valueTypeId, 
+                     DbEnv* env) :
     Db(env, 0),
     _communicator(communicator),
+    _encoding(encoding),
     _dbName(dbName),
     _key(keyTypeId),
     _value(valueTypeId),
@@ -420,7 +434,8 @@ Freeze::MapDb::MapDb(const Ice::CommunicatorPtr& communicator, const string& dbN
         }
 
         u_int32_t flags = DB_THREAD | DB_CREATE | DB_AUTO_COMMIT;
-        open(0, _dbName.c_str(), 0, DB_BTREE, flags, FREEZE_DB_MODE);
+
+        open(0, Ice::nativeToUTF8(_communicator, _dbName).c_str(), 0, DB_BTREE, flags, FREEZE_DB_MODE);
     }
     catch(const ::DbException& dx)
     {
@@ -442,6 +457,7 @@ Freeze::MapDb::connectIndices(const vector<MapIndexBasePtr>& indices) const
         assert(q != _indices.end());
         indexBase->_impl = q->second;
         indexBase->_communicator = _communicator;
+        indexBase->_encoding = _encoding;
     }
 }
 
