@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -28,7 +28,9 @@ Freeze::ObjectStoreBase::ObjectStoreBase(const string& facet, const string& face
     _facet(facet),
     _evictor(evictor),
     _indices(indices),
-    _communicator(evictor->communicator())
+    _communicator(evictor->communicator()),
+    _encoding(evictor->encoding()),
+    _keepStats(false)
 {
     if(facet == "")
     {
@@ -59,10 +61,17 @@ Freeze::ObjectStoreBase::ObjectStoreBase(const string& facet, const string& face
     Catalog::iterator p = catalog.find(evictor->filename());
     if(p != catalog.end())
     {
-        if(p->second.evictor == false)
+        if(p->second.evictor)
+        {
+            //
+            // empty means the value is ::Freeze::ObjectRecord
+            //
+            _keepStats = p->second.value.empty();
+        }
+        else
         {
             DatabaseException ex(__FILE__, __LINE__);
-            ex.message = evictor->filename() + " is an evictor database";
+            ex.message = evictor->filename() + " is not an evictor database";
             throw ex;
         }
     }
@@ -140,6 +149,8 @@ Freeze::ObjectStoreBase::ObjectStoreBase(const string& facet, const string& face
         {
             CatalogData catalogData;
             catalogData.evictor = true;
+            catalogData.key = "Ice::Identity";
+            catalogData.value = "Object";
             catalog.put(Catalog::value_type(evictor->filename(), catalogData));
         }
 
@@ -197,15 +208,22 @@ Freeze::ObjectStoreBase::~ObjectStoreBase()
         
         for(size_t i = 0; i < _indices.size(); ++i)
         {
-            _indices[i]->_impl->close();
+            try
+            {
+                _indices[i]->_impl->close();
+            }
+            catch(const DatabaseException& ex)
+            {
+                Ice::Error error(_communicator->getLogger());
+                error << "Freeze: closing ObjectStore " << _dbName << " raised DatabaseException: " << ex.what();
+            }
         }
         _indices.clear();
     }
     catch(const DbException& dx)
     {
-        DatabaseException ex(__FILE__, __LINE__);
-        ex.message = dx.what();
-        throw ex;
+        Ice::Error error(_communicator->getLogger());
+        error << "Freeze: closing ObjectStore " << _dbName << " raised DbException: " << dx.what();
     }
 }
     
@@ -223,7 +241,7 @@ Freeze::ObjectStoreBase::dbHasObject(const Identity& ident, const TransactionIPt
     }
 
     Key key;    
-    marshal(ident, key, _communicator);
+    marshal(ident, key, _communicator, _encoding);
     Dbt dbKey;
     initializeInDbt(key, dbKey);
     
@@ -318,48 +336,73 @@ Freeze::ObjectStoreBase::save(Key& key, Value& value, Byte status, DbTxn* tx)
 }
 
 void 
-Freeze::ObjectStoreBase::marshal(const Identity& ident, Key& bytes, const CommunicatorPtr& communicator)
+Freeze::ObjectStoreBase::marshal(const Identity& ident, 
+                                 Key& bytes, 
+                                 const CommunicatorPtr& communicator, 
+                                 const EncodingVersion& encoding)
 {
     IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
-    IceInternal::BasicStream stream(instance.get());
-    ident.__write(&stream);
+    IceInternal::BasicStream stream(instance.get(), encoding, true);
+    stream.write(ident);
     vector<Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
 }
     
 void 
-Freeze::ObjectStoreBase::unmarshal(Identity& ident, const Key& bytes, const CommunicatorPtr& communicator)
+Freeze::ObjectStoreBase::unmarshal(Identity& ident, 
+                                   const Key& bytes, 
+                                   const CommunicatorPtr& communicator, 
+                                   const EncodingVersion& encoding)
 {
     IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
-    IceInternal::BasicStream stream(instance.get());
-    stream.b.resize(bytes.size());
-    memcpy(&stream.b[0], &bytes[0], bytes.size());
-    stream.i = stream.b.begin();
-    ident.__read(&stream);
+    IceInternal::BasicStream stream(instance.get(), encoding, &bytes[0], &bytes[0] + bytes.size());
+    stream.read(ident);
 }
 
 void
-Freeze::ObjectStoreBase::marshal(const ObjectRecord& v, Value& bytes, const CommunicatorPtr& communicator)
+Freeze::ObjectStoreBase::marshal(const ObjectRecord& v, 
+                                 Value& bytes, 
+                                 const CommunicatorPtr& communicator,
+                                 const EncodingVersion& encoding,
+                                 bool keepStats)
 {
     IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
-    IceInternal::BasicStream stream(instance.get());
+    IceInternal::BasicStream stream(instance.get(), encoding, true);
     stream.startWriteEncaps();
-    v.__write(&stream);
+    if(keepStats)
+    {
+        stream.write(v);
+    }
+    else
+    {
+        stream.write(v.servant);
+    }
+
     stream.writePendingObjects();
     stream.endWriteEncaps();
     vector<Byte>(stream.b.begin(), stream.b.end()).swap(bytes);
 }
 
 void
-Freeze::ObjectStoreBase::unmarshal(ObjectRecord& v, const Value& bytes, const CommunicatorPtr& communicator)
+Freeze::ObjectStoreBase::unmarshal(ObjectRecord& v,
+                                   const Value& bytes,
+                                   const CommunicatorPtr& communicator,
+                                   const EncodingVersion& encoding,
+                                   bool keepStats)
 {
     IceInternal::InstancePtr instance = IceInternal::getInstance(communicator);
-    IceInternal::BasicStream stream(instance.get());
+    IceInternal::BasicStream stream(instance.get(), encoding, &bytes[0], &bytes[0] + bytes.size());
     stream.sliceObjects(false);
-    stream.b.resize(bytes.size());
-    memcpy(&stream.b[0], &bytes[0], bytes.size());
-    stream.i = stream.b.begin();
     stream.startReadEncaps();
-    v.__read(&stream);
+    
+    if(keepStats)
+    {
+        stream.read(v);
+    }
+    else
+    {
+        stream.read(v.servant);
+    }
+    
     stream.readPendingObjects();
     stream.endReadEncaps();
 }
@@ -380,7 +423,7 @@ Freeze::ObjectStoreBase::load(const Identity& ident, const TransactionIPtr& tran
     }
 
     Key key;
-    marshal(ident, key, _communicator);
+    marshal(ident, key, _communicator, _encoding);
 
     Dbt dbKey;
     initializeInDbt(key, dbKey);
@@ -423,7 +466,7 @@ Freeze::ObjectStoreBase::load(const Identity& ident, const TransactionIPtr& tran
         }
     }
     
-    unmarshal(rec, value, _communicator);
+    unmarshal(rec, value, _communicator, _encoding, _keepStats);
     _evictor->initialize(ident, _facet, rec.servant);
     return true;
 }
@@ -444,10 +487,10 @@ Freeze::ObjectStoreBase::update(const Identity& ident, const ObjectRecord& rec,
     }
 
     Key key;
-    marshal(ident, key, _communicator);
+    marshal(ident, key, _communicator, _encoding);
 
     Value value;
-    marshal(rec, value, _communicator);
+    marshal(rec, value, _communicator, _encoding, _keepStats);
 
     Dbt dbKey;
     Dbt dbValue;
@@ -489,10 +532,10 @@ Freeze::ObjectStoreBase::insert(const Identity& ident, const ObjectRecord& rec,
     }
 
     Key key;
-    marshal(ident, key, _communicator);
+    marshal(ident, key, _communicator, _encoding);
 
     Value value;
-    marshal(rec, value, _communicator);
+    marshal(rec, value, _communicator, _encoding, _keepStats);
 
     Dbt dbKey;
     Dbt dbValue;
@@ -547,7 +590,7 @@ Freeze::ObjectStoreBase::remove(const Identity& ident, const TransactionIPtr& tr
     }
 
     Key key;
-    marshal(ident, key, _communicator);
+    marshal(ident, key, _communicator, _encoding);
    
     Dbt dbKey;
     initializeInDbt(key, dbKey);
@@ -595,7 +638,7 @@ bool
 Freeze::ObjectStoreBase::loadImpl(const Identity& ident, ObjectRecord& rec)
 {
     Key key;
-    marshal(ident, key, _communicator);
+    marshal(ident, key, _communicator, _encoding);
 
     Dbt dbKey;
     initializeInDbt(key, dbKey);
@@ -640,7 +683,7 @@ Freeze::ObjectStoreBase::loadImpl(const Identity& ident, ObjectRecord& rec)
         }
     }
     
-    unmarshal(rec, value, _communicator);
+    unmarshal(rec, value, _communicator, _encoding, _keepStats);
     _evictor->initialize(ident, _facet, rec.servant);
     return true;
 }
