@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -23,7 +23,7 @@ public:
     virtual ~FailureException() throw();
     virtual std::string ice_name() const;
     virtual void ice_print(std::ostream&) const;
-    virtual IceUtil::Exception* ice_clone() const;
+    virtual FailureException* ice_clone() const;
     virtual void ice_throw() const;
 
     std::string reason() const;
