@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -529,7 +529,7 @@ FreezeScript::TransformVisitor::transformObject(const ObjectDataPtr& dest, const
         objectDataMap.erase(p);
 
 #if (defined(_MSC_VER) && (_MSC_VER >= 1600))
-	objectDataMap.insert(ObjectDataMap::value_type(src.get(), nullptr));
+        objectDataMap.insert(ObjectDataMap::value_type(src.get(), static_cast<ObjectDataPtr>(nullptr)));
 #else
         objectDataMap.insert(ObjectDataMap::value_type(src.get(), 0));
 #endif
