@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -56,7 +56,7 @@ std::string typeToString(const Slice::TypePtr&);
 bool ignoreType(const std::string&);
 void createEvictorSliceTypes(const Slice::UnitPtr&);
 bool parseSlice(const std::string&, const Slice::UnitPtr&, const std::vector<std::string>&, 
-                const std::vector<std::string>&, bool);
+                const std::vector<std::string>&, bool, const std::string&);
 
 typedef std::map<std::string, Freeze::CatalogData> CatalogDataMap;
 
