@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,24 +13,22 @@
 #include <IceUtil/Shared.h>
 #include <Ice/AcceptorF.h>
 #include <Ice/TransceiverF.h>
-
-#ifdef _WIN32
-#include <winsock2.h>
-typedef int ssize_t;
-#else
-#   define SOCKET int
-#endif
+#include <Ice/Network.h>
 
 namespace IceInternal
 {
 
-class ICE_API Acceptor : public ::IceUtil::Shared
+class ICE_API Acceptor : virtual public ::IceUtil::Shared
 {
 public:
 
-    virtual SOCKET fd() = 0;
+    virtual NativeInfoPtr getNativeInfo() = 0;
     virtual void close() = 0;
     virtual void listen() = 0;
+#if defined(ICE_USE_IOCP) || defined(ICE_OS_WINRT)
+    virtual void startAccept() = 0;
+    virtual void finishAccept() = 0;
+#endif
     virtual TransceiverPtr accept() = 0;
     virtual std::string toString() const = 0;
 };
