@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,7 +16,7 @@ using namespace Ice;
 using namespace IceInternal;
 
 DispatchStatus
-Ice::DispatchInterceptor::__dispatch(IceInternal::Incoming& in, const Current& current)
+Ice::DispatchInterceptor::__dispatch(IceInternal::Incoming& in, const Current& /*current*/)
 {
     try
     {
