@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,13 +16,13 @@ namespace IceInternal
 inline void
 hashAdd(Ice::Int& hashCode, Ice::Int value)
 {
-    hashCode = hashCode * 5 + value;
+    hashCode = ((hashCode << 5) + hashCode) ^ (2654435761u * value);
 }
 
 inline void
 hashAdd(Ice::Int& hashCode, bool value)
 {
-    hashCode = hashCode * 5 + static_cast<Ice::Int>(value);
+    hashCode = ((hashCode << 5) + hashCode) ^ (value ? 1 : 0);
 }
 
 inline void
@@ -30,7 +30,7 @@ hashAdd(Ice::Int& hashCode, const std::string& value)
 {
     for(std::string::const_iterator p = value.begin(); p != value.end(); ++p)
     {
-        hashCode = 5 * hashCode + *p;
+        hashCode = ((hashCode << 5) + hashCode) ^ *p;
     }    
 }
 
@@ -53,7 +53,6 @@ hashAdd(Ice::Int& hashCode, const std::map<K, V>& map)
     }
 }
 
-
 }
 
 #endif
