@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,9 +22,11 @@
 #include <Ice/Protocol.h>
 #include <Ice/ReplyStatus.h>
 #include <IceUtil/StringUtil.h>
+#include <typeinfo>
 
 using namespace std;
 using namespace Ice;
+using namespace Ice::Instrumentation;
 using namespace IceInternal;
 
 namespace IceUtilInternal
@@ -39,7 +41,7 @@ IceInternal::IncomingBase::IncomingBase(Instance* instance, ConnectionI* connect
                                         bool response, Byte compress, Int requestId) :
     _response(response),
     _compress(compress),
-    _os(instance),
+    _os(instance, Ice::currentProtocolEncoding),
     _connection(connection)
 {
     _current.adapter = adapter;
@@ -49,15 +51,17 @@ IceInternal::IncomingBase::IncomingBase(Instance* instance, ConnectionI* connect
 
 IceInternal::IncomingBase::IncomingBase(IncomingBase& in) :
     _current(in._current), // copy
-    _os(in._os.instance()),
+    _os(in._os.instance(), Ice::currentProtocolEncoding),
     _interceptorAsyncCallbackQueue(in._interceptorAsyncCallbackQueue) // copy
 {
-    adopt(in); // adopt everything else
+    __adopt(in); // adopt everything else
 }
 
 void
-IceInternal::IncomingBase::adopt(IncomingBase& other)
+IceInternal::IncomingBase::__adopt(IncomingBase& other)
 {
+    _observer.adopt(other._observer);
+
     _servant = other._servant;
     other._servant = 0;
 
@@ -79,6 +83,86 @@ IceInternal::IncomingBase::adopt(IncomingBase& other)
     other._connection = 0;
 }
 
+BasicStream* 
+IncomingBase::__startWriteParams(FormatType format)
+{
+    if(_response)
+    {
+        assert(_os.b.size() == headerSize + 4); // Reply status position.
+        assert(_current.encoding >= Ice::Encoding_1_0); // Encoding for reply is known.
+        _os.write(static_cast<Ice::Byte>(0));
+        _os.startWriteEncaps(_current.encoding, format);
+    }
+    
+    //
+    // We still return the stream even if no response is expected. The
+    // servant code might still write some out parameters if for
+    // example a method with out parameters somehow and erroneously
+    // invoked as oneway (or if the invocation is invoked on a 
+    // blobject and the blobject erroneously writes a response).
+    //
+    return &_os;
+}
+
+void 
+IncomingBase::__endWriteParams(bool ok)
+{
+    if(!ok)
+    {
+        _observer.userException();
+    }
+
+    if(_response)
+    {
+        *(_os.b.begin() + headerSize + 4) = ok ? replyOK : replyUserException; // Reply status position.
+        _os.endWriteEncaps();
+    }
+}
+
+void 
+IncomingBase::__writeEmptyParams()
+{
+    if(_response)
+    {
+        assert(_os.b.size() == headerSize + 4); // Reply status position.
+        assert(_current.encoding >= Ice::Encoding_1_0); // Encoding for reply is known.
+        _os.write(replyOK);
+        _os.writeEmptyEncaps(_current.encoding);
+    }
+}
+
+void 
+IncomingBase::__writeParamEncaps(const Byte* v, Ice::Int sz, bool ok)
+{
+    if(!ok)
+    {
+        _observer.userException();
+    }
+
+    if(_response)
+    {
+        assert(_os.b.size() == headerSize + 4); // Reply status position.
+        assert(_current.encoding >= Ice::Encoding_1_0); // Encoding for reply is known.
+        _os.write(ok ? replyOK : replyUserException);
+        if(sz == 0)
+        {
+            _os.writeEmptyEncaps(_current.encoding);
+        }
+        else
+        {
+            _os.writeEncaps(v, sz);
+        }
+    }
+}
+
+void 
+IncomingBase::__writeUserException(const Ice::UserException& ex, Ice::FormatType format)
+{
+    ::IceInternal::BasicStream* __os = __startWriteParams(format);
+    __os->write(ex);
+    __endWriteParams(false);
+}
+
 void
 IceInternal::IncomingBase::__warning(const Exception& ex) const
 {
@@ -95,7 +179,7 @@ IceInternal::IncomingBase::__warning(const Exception& ex) const
         Ice::IPConnectionInfoPtr ipConnInfo = Ice::IPConnectionInfoPtr::dynamicCast(connInfo);
         if(ipConnInfo)
         {
-            out << "\nremote host: " << ipConnInfo->remoteAddress + " remote port: " << ipConnInfo->remotePort;
+            out << "\nremote host: " << ipConnInfo->remoteAddress << " remote port: " << ipConnInfo->remotePort;
         }
     }
 }
@@ -116,7 +200,7 @@ IceInternal::IncomingBase::__warning(const string& msg) const
         Ice::IPConnectionInfoPtr ipConnInfo = Ice::IPConnectionInfoPtr::dynamicCast(connInfo);
         if(ipConnInfo)
         {
-            out << "\nremote host: " << ipConnInfo->remoteAddress + " remote port: " << ipConnInfo->remotePort;
+            out << "\nremote host: " << ipConnInfo->remoteAddress << " remote port: " << ipConnInfo->remotePort;
         }
     }
 }
@@ -134,17 +218,19 @@ IceInternal::IncomingBase::__servantLocatorFinished()
     {
         assert(_connection);
 
+        _observer.userException();
+
         //
         // The operation may have already marshaled a reply; we must overwrite that reply.
         //
         if(_response)
         {
-            _os.endWriteEncaps();
             _os.b.resize(headerSize + 4); // Reply status position.
             _os.write(replyUserException);
-            _os.startWriteEncaps();
+            _os.startWriteEncaps(_current.encoding, DefaultFormat);
             _os.write(ex);
             _os.endWriteEncaps();
+            _observer.reply(static_cast<Int>(_os.b.size() - headerSize - 4));
             _connection->sendResponse(&_os, _compress);
         }
         else
@@ -152,6 +238,7 @@ IceInternal::IncomingBase::__servantLocatorFinished()
             _connection->sendNoResponse();
         }
 
+        _observer.detach();
         _connection = 0;
     }
     catch(const std::exception& ex)
@@ -195,9 +282,13 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
             __warning(*rfe);
         }
 
+        if(_observer)
+        {
+            _observer.failed(rfe->ice_name());
+        }
+
         if(_response)
         {
-            _os.endWriteEncaps();
             _os.b.resize(headerSize + 4); // Reply status position.
             if(dynamic_cast<ObjectNotExistException*>(rfe))
             {
@@ -216,7 +307,7 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
                 assert(false);
             }
 
-            rfe->id.__write(&_os);
+            _os.write(rfe->id);
 
             //
             // For compatibility with the old FacetPath.
@@ -232,6 +323,7 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
 
             _os.write(rfe->operation, false);
 
+            _observer.reply(static_cast<Int>(_os.b.size() - headerSize - 4));
             _connection->sendResponse(&_os, _compress);
         }
         else
@@ -241,15 +333,18 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
     }
     else if(const Exception* ex = dynamic_cast<const Exception*>(&exc))
     {
-
         if(_os.instance()->initializationData().properties->getPropertyAsIntWithDefault("Ice.Warn.Dispatch", 1) > 0)
         {
             __warning(*ex);
         }
 
+        if(_observer)
+        {
+            _observer.failed(ex->ice_name());
+        }
+
         if(_response)
         {
-            _os.endWriteEncaps();
             _os.b.resize(headerSize + 4); // Reply status position.
             if(const UnknownLocalException* ule = dynamic_cast<const UnknownLocalException*>(&exc))
             {
@@ -271,12 +366,10 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
                 _os.write(replyUnknownLocalException);
                 ostringstream str;
                 str << *le;
-#ifdef __GNUC__
                 if(IceUtilInternal::printStackTraces)
                 {
                     str <<  '\n' << ex->ice_stackTrace();
                 }
-#endif
                 _os.write(str.str(), false);
             }
             else if(const UserException* ue = dynamic_cast<const UserException*>(&exc))
@@ -284,12 +377,10 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
                 _os.write(replyUnknownUserException);
                 ostringstream str;
                 str << *ue;
-#ifdef __GNUC__
                 if(IceUtilInternal::printStackTraces)
                 {
                     str <<  '\n' << ex->ice_stackTrace();
                 }
-#endif
                 _os.write(str.str(), false);
             }
             else
@@ -297,8 +388,14 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
                 _os.write(replyUnknownException);
                 ostringstream str;
                 str << *ex;
+                if(IceUtilInternal::printStackTraces)
+                {
+                    str <<  '\n' << ex->ice_stackTrace();
+                }
                 _os.write(str.str(), false);
             }
+
+            _observer.reply(static_cast<Int>(_os.b.size() - headerSize - 4));
             _connection->sendResponse(&_os, _compress);
         }
         else
@@ -313,14 +410,20 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
             __warning(string("std::exception: ") + exc.what());
         }
 
+        if(_observer)
+        {
+            _observer.failed(typeid(exc).name());
+        }
+
         if(_response)
         {
-            _os.endWriteEncaps();
             _os.b.resize(headerSize + 4); // Reply status position.
             _os.write(replyUnknownException);
             ostringstream str;
             str << "std::exception: " << exc.what();
             _os.write(str.str(), false);
+
+            _observer.reply(static_cast<Int>(_os.b.size() - headerSize - 4));
             _connection->sendResponse(&_os, _compress);
         }
         else
@@ -329,6 +432,7 @@ IceInternal::IncomingBase::__handleException(const std::exception& exc)
         }
     }
 
+    _observer.detach();
     _connection = 0;
 }
 
@@ -342,13 +446,18 @@ IceInternal::IncomingBase::__handleException()
 
     assert(_connection);
 
+    if(_observer)
+    {
+        _observer.failed("unknown");
+    }
+
     if(_response)
     {
-        _os.endWriteEncaps();
         _os.b.resize(headerSize + 4); // Reply status position.
         _os.write(replyUnknownException);
         string reason = "unknown c++ exception";
         _os.write(reason, false);
+        _observer.reply(static_cast<Int>(_os.b.size() - headerSize - 4));
         _connection->sendResponse(&_os, _compress);
     }
     else
@@ -356,17 +465,28 @@ IceInternal::IncomingBase::__handleException()
         _connection->sendNoResponse();
     }
 
+    _observer.detach();
     _connection = 0;
 }
 
 
-IceInternal::Incoming::Incoming(Instance* instance, ConnectionI* connection,
-                                const ObjectAdapterPtr& adapter,
+IceInternal::Incoming::Incoming(Instance* instance, ConnectionI* connection, const ObjectAdapterPtr& adapter,
                                 bool response, Byte compress, Int requestId) :
     IncomingBase(instance, connection, adapter, response, compress, requestId),
-    _is(instance),
     _inParamPos(0)
 {
+    //
+    // Prepare the response if necessary.
+    //
+    if(response)
+    {
+        _os.writeBlob(replyHdr, sizeof(replyHdr));
+        
+        //
+        // Add the request ID.
+        //
+        _os.write(requestId);
+    }
 }
 
 
@@ -390,7 +510,7 @@ IceInternal::Incoming::startOver()
         //
         // That's the first startOver, so almost nothing to do
         //
-        _inParamPos = _is.i;
+        _inParamPos = _is->i;
     }
     else
     {
@@ -399,15 +519,8 @@ IceInternal::Incoming::startOver()
         //
         // Let's rewind _is and clean-up _os
         //
-        _is.i = _inParamPos;
-
-        if(_response)
-        {
-            _os.endWriteEncaps();
-            _os.b.resize(headerSize + 4);
-            _os.write(static_cast<Byte>(0));
-            _os.startWriteEncaps();
-        }
+        _is->i = _inParamPos;
+        _os.b.resize(headerSize + 4); // Reply status position.
     }
 }
 
@@ -435,12 +548,16 @@ IceInternal::Incoming::setActive(IncomingAsync& cb)
 }
 
 void
-IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager)
+IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager, BasicStream* stream)
 {
+    _is = stream;
+
+    BasicStream::Container::iterator start = _is->i;
+
     //
     // Read the current.
     //
-    _current.id.__read(&_is);
+    _is->read(_current.id);
 
     //
     // For compatibility with the old FacetPath.
@@ -448,7 +565,7 @@ IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager)
     string facet;
     {
         vector<string> facetPath;
-        _is.read(facetPath);
+        _is->read(facetPath);
         if(!facetPath.empty())
         {
             if(facetPath.size() > 1)
@@ -460,34 +577,32 @@ IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager)
     }
     _current.facet.swap(facet);
 
-    _is.read(_current.operation, false);
+    _is->read(_current.operation, false);
 
     Byte b;
-    _is.read(b);
+    _is->read(b);
     _current.mode = static_cast<OperationMode>(b);
 
-    Int sz;
-    _is.readSize(sz);
+    Int sz = _is->readSize();
     while(sz--)
     {
         pair<const string, string> pr;
-        _is.read(const_cast<string&>(pr.first));
-        _is.read(pr.second);
+        _is->read(const_cast<string&>(pr.first));
+        _is->read(pr.second);
         _current.ctx.insert(_current.ctx.end(), pr);
     }
 
-    if(_response)
-    {
-        assert(_os.b.size() == headerSize + 4); // Reply status position.
-        _os.write(static_cast<Byte>(0));
-        _os.startWriteEncaps();
+    const CommunicatorObserverPtr& obsv = _is->instance()->getObserver();
+    if(obsv)
+    { 
+        // Read the parameter encapsulation size.
+        Ice::Int sz;
+        _is->read(sz); 
+        _is->i -= 4;
+
+        _observer.attach(obsv->getDispatchObserver(_current, static_cast<Int>(_is->i - start + sz)));
     }
 
-    // Initialize status to some value, to keep the compiler happy.
-    Ice::Byte replyStatus = replyOK;
-
-    DispatchStatus dispatchStatus = DispatchOK;
-
     //
     // Don't put the code above into the try block below. Exceptions
     // in the code above are considered fatal, and must propagate to
@@ -513,16 +628,37 @@ IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager)
                 }
                 catch(const UserException& ex)
                 {
-                    _os.write(ex);
-                    replyStatus = replyUserException;
+                    Ice::EncodingVersion encoding = _is->skipEncaps(); // Required for batch requests.
+
+                    _observer.userException();
+
+                    if(_response)
+                    {
+                        _os.write(replyUserException);
+                        _os.startWriteEncaps(encoding, DefaultFormat);
+                        _os.write(ex);
+                        _os.endWriteEncaps();
+                        _observer.reply(static_cast<Int>(_os.b.size() - headerSize - 4));
+                        _connection->sendResponse(&_os, _compress);
+                    }
+                    else
+                    {
+                        _connection->sendNoResponse();
+                    }
+
+                    _observer.detach();
+                    _connection = 0;
+                    return;
                 }
                 catch(const std::exception& ex)
                 {
+                    _is->skipEncaps(); // Required for batch requests.
                     __handleException(ex);
                     return;
                 }
                 catch(...)
                 {
+                    _is->skipEncaps(); // Required for batch requests.
                     __handleException();
                     return;
                 }
@@ -530,55 +666,60 @@ IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager)
         }
     }
 
-    if(_servant)
+    try
     {
-        try
+        if(_servant)
         {
-            assert(replyStatus == replyOK);
-            dispatchStatus = _servant->__dispatch(*this, _current);
-            if(dispatchStatus == DispatchUserException)
-            {
-                replyStatus = replyUserException;
+            //
+            // DispatchAsync is a "pseudo dispatch status", used internally only
+            // to indicate async dispatch.
+            //
+            if(_servant->__dispatch(*this, _current) == DispatchAsync)
+            {            
+                return;
             }
 
-            if(dispatchStatus != DispatchAsync)
-            {
-                if(_locator && !__servantLocatorFinished())
-                {
-                    return;
-                }
-            }
-        }
-        catch(const std::exception& ex)
-        {
             if(_locator && !__servantLocatorFinished())
             {
                 return;
             }
-            __handleException(ex);
-            return;
-        }
-        catch(...)
-        {
-            if(_locator && !__servantLocatorFinished())
-            {
-                return;
-            }
-            __handleException();
-            return;
-        }
-    }
-    else if(replyStatus == replyOK)
-    {
-        if(servantManager && servantManager->hasServant(_current.id))
-        {
-            replyStatus = replyFacetNotExist;
         }
         else
         {
-            replyStatus = replyObjectNotExist;
+            //
+            // Skip the input parameters, this is required for reading
+            // the next batch request if dispatching batch requests.
+            //
+            _is->skipEncaps(); 
+
+            if(servantManager && servantManager->hasServant(_current.id))
+            {
+                throw FacetNotExistException(__FILE__, __LINE__, _current.id, _current.facet, _current.operation);
+            }
+            else
+            {
+                throw ObjectNotExistException(__FILE__, __LINE__, _current.id, _current.facet, _current.operation);
+            }
         }
     }
+    catch(const std::exception& ex)
+    {
+        if(_servant && _locator && !__servantLocatorFinished())
+        {
+            return;
+        }
+        __handleException(ex);
+        return;
+    }
+    catch(...)
+    {
+        if(_servant && _locator && !__servantLocatorFinished())
+        {
+            return;
+        }
+        __handleException();
+        return;
+    }
 
     //
     // Don't put the code below into the try block above. Exceptions
@@ -586,53 +727,9 @@ IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager)
     // the caller of this operation.
     //
 
-    //
-    // DispatchAsync is "pseudo dispatch status", used internally only
-    // to indicate async dispatch.
-    //
-    if(dispatchStatus == DispatchAsync)
-    {
-        //
-        // If this was an asynchronous dispatch, we're done here.
-        //
-        return;
-    }
-
-    assert(_connection);
-
     if(_response)
     {
-        _os.endWriteEncaps();
-
-        if(replyStatus != replyOK && replyStatus != replyUserException)
-        {
-            assert(replyStatus == replyObjectNotExist ||
-                   replyStatus == replyFacetNotExist);
-
-            _os.b.resize(headerSize + 4); // Reply status position.
-            _os.write(replyStatus);
-
-            _current.id.__write(&_os);
-
-            //
-            // For compatibility with the old FacetPath.
-            //
-            if(_current.facet.empty())
-            {
-                _os.write(static_cast<string*>(0), static_cast<string*>(0));
-            }
-            else
-            {
-                _os.write(&_current.facet, &_current.facet + 1);
-            }
-
-            _os.write(_current.operation, false);
-        }
-        else
-        {
-            *(_os.b.begin() + headerSize + 4) = replyStatus; // Reply status position.
-        }
-
+        _observer.reply(static_cast<Int>(_os.b.size() - headerSize - 4));
         _connection->sendResponse(&_os, _compress);
     }
     else
@@ -640,10 +737,10 @@ IceInternal::Incoming::invoke(const ServantManagerPtr& servantManager)
         _connection->sendNoResponse();
     }
 
+    _observer.detach();
     _connection = 0;
 }
 
-
 bool
 IceInternal::IncomingRequest::isCollocated()
 {
