@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -25,20 +25,23 @@
 #include <Ice/ObjectAdapterFactory.h>
 #include <Ice/Exception.h>
 #include <Ice/PropertiesI.h>
+#include <Ice/PropertiesAdminI.h>
 #include <Ice/LoggerI.h>
 #include <Ice/Network.h>
 #include <Ice/EndpointFactoryManager.h>
 #include <Ice/RetryQueue.h>
-#include <Ice/TcpEndpointI.h>
-#include <Ice/UdpEndpointI.h>
 #include <Ice/DynamicLibrary.h>
 #include <Ice/PluginManagerI.h>
 #include <Ice/Initialize.h>
 #include <Ice/LoggerUtil.h>
 #include <IceUtil/StringUtil.h>
 #include <Ice/PropertiesI.h>
-#include <IceUtil/UUID.h>
 #include <Ice/Communicator.h>
+#include <Ice/GC.h>
+#include <Ice/MetricsAdminI.h>
+#include <Ice/InstrumentationI.h>
+ 
+#include <IceUtil/UUID.h>
 #include <IceUtil/Mutex.h>
 #include <IceUtil/MutexPtrLock.h>
 
@@ -53,6 +56,19 @@
 #   include <sys/types.h>
 #endif
 
+#if defined(__linux) || defined(__sun)
+#   include <grp.h> // for initgroups
+#endif
+
+#include <Ice/UdpEndpointI.h>
+
+#ifndef ICE_OS_WINRT
+#   include <Ice/TcpEndpointI.h>
+#else
+#   include <IceSSL/EndpointInfo.h> // For IceSSL::EndpointType
+#   include <Ice/winrt/StreamEndpointI.h>
+#endif
+
 using namespace std;
 using namespace Ice;
 using namespace IceInternal;
@@ -65,6 +81,13 @@ extern bool ICE_DECLSPEC_IMPORT printStackTraces;
 
 };
 
+namespace IceInternal
+{
+
+extern IceUtil::Handle<IceInternal::GC> theCollector;
+
+}
+
 namespace
 {
 
@@ -97,6 +120,34 @@ Init init;
 
 }
 
+namespace IceInternal
+{
+
+class ObserverUpdaterI : public Ice::Instrumentation::ObserverUpdater
+{
+public:
+
+    ObserverUpdaterI(InstancePtr instance) : _instance(instance)
+    {
+    }
+
+    void updateConnectionObservers()
+    {
+        _instance->updateConnectionObservers();
+    }
+
+    void updateThreadObservers()
+    {
+        _instance->updateThreadObservers();
+    }
+
+private:
+
+    InstancePtr _instance;
+};
+
+}
+
 IceUtil::Shared* IceInternal::upCast(Instance* p) { return p; }
 
 bool
@@ -237,16 +288,21 @@ IceInternal::Instance::objectAdapterFactory() const
 ProtocolSupport
 IceInternal::Instance::protocolSupport() const
 {
-    IceUtil::RecMutex::Lock sync(*this);
-
-    if(_state == StateDestroyed)
-    {
-        throw CommunicatorDestroyedException(__FILE__, __LINE__);
-    }
-
     return _protocolSupport;
 }
 
+bool
+IceInternal::Instance::preferIPv6() const
+{
+    return _preferIPv6;
+}
+
+NetworkProxyPtr
+IceInternal::Instance::networkProxy() const
+{
+    return _networkProxy;
+}
+
 ThreadPoolPtr
 IceInternal::Instance::clientThreadPool()
 {
@@ -262,7 +318,7 @@ IceInternal::Instance::clientThreadPool()
 }
 
 ThreadPoolPtr
-IceInternal::Instance::serverThreadPool()
+IceInternal::Instance::serverThreadPool(bool create)
 {
     IceUtil::RecMutex::Lock sync(*this);
 
@@ -271,7 +327,7 @@ IceInternal::Instance::serverThreadPool()
         throw CommunicatorDestroyedException(__FILE__, __LINE__);
     }
     
-    if(!_serverThreadPool) // Lazy initialization.
+    if(!_serverThreadPool && create) // Lazy initialization.
     {
         int timeout = _initData.properties->getPropertyAsInt("Ice.ServerIdleTime");
         _serverThreadPool = new ThreadPool(this, "Ice.ThreadPool.Server", timeout);
@@ -508,7 +564,6 @@ IceInternal::Instance::getAdmin()
         string instanceName = _initData.properties->getProperty("Ice.Admin.InstanceName");
 
         Ice::LocatorPrx defaultLocator = _referenceFactory->getDefaultLocator();
-
         if((defaultLocator != 0 && serverId != "") || instanceName != "")
         {
             if(_adminIdentity.name == "")
@@ -673,6 +728,35 @@ IceInternal::Instance::removeAdminFacet(const string& facet)
     {
         result = _adminAdapter->removeFacet(_adminIdentity, facet);  
     }
+
+    return result;
+}
+
+Ice::ObjectPtr
+IceInternal::Instance::findAdminFacet(const string& facet)
+{
+    IceUtil::RecMutex::Lock sync(*this);
+
+    if(_state == StateDestroyed)
+    {
+        throw CommunicatorDestroyedException(__FILE__, __LINE__);
+    }
+
+    ObjectPtr result;
+
+    if(_adminAdapter == 0 || (!_adminFacetFilter.empty() && _adminFacetFilter.find(facet) == _adminFacetFilter.end()))
+    {
+        FacetMap::iterator p = _adminFacets.find(facet);
+        if(p != _adminFacets.end())
+        {
+            result = p->second;
+        }
+    }
+    else
+    {
+        result = _adminAdapter->findFacet(_adminIdentity, facet);
+    }
+
     return result;
 } 
 
@@ -828,15 +912,31 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
                 string newUser = _initData.properties->getProperty("Ice.ChangeUser");
                 if(!newUser.empty())
                 {
+                    errno = 0;
                     struct passwd* pw = getpwnam(newUser.c_str());
                     if(!pw)
+                    {
+                        if(errno)
+                        {
+                            SyscallException ex(__FILE__, __LINE__);
+                            ex.error = getSystemErrno();
+                            throw ex;
+                        }
+                        else
+                        {
+                            InitializationException ex(__FILE__, __LINE__, "Unknown user account `" + newUser + "'");
+                            throw ex;
+                        }
+                    }
+                    
+                    if(setgid(pw->pw_gid) == -1)
                     {
                         SyscallException ex(__FILE__, __LINE__);
                         ex.error = getSystemErrno();
                         throw ex;
                     }
-                    
-                    if(setgid(pw->pw_gid) == -1)
+
+                    if(initgroups(pw->pw_name, pw->pw_gid) == -1)
                     {
                         SyscallException ex(__FILE__, __LINE__);
                         ex.error = getSystemErrno();
@@ -857,7 +957,7 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
             if(instanceCount == 1)
             {                   
                 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(ICE_OS_WINRT)
                 WORD version = MAKEWORD(1, 1);
                 WSADATA data;
                 if(WSAStartup(version, &data) != 0)
@@ -943,9 +1043,10 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
         //
         const_cast<Int&>(_clientACM) = _initData.properties->getPropertyAsIntWithDefault("Ice.ACM.Client", 60);
         const_cast<Int&>(_serverACM) = _initData.properties->getPropertyAsInt("Ice.ACM.Server");
+#ifndef ICE_OS_WINRT
         const_cast<ImplicitContextIPtr&>(_implicitContext) = 
             ImplicitContextI::create(_initData.properties->getProperty("Ice.ImplicitContext"));
-
+#endif
         _routerManager = new RouterManager;
 
         _locatorManager = new LocatorManager(_initData.properties);
@@ -954,8 +1055,21 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
 
         _proxyFactory = new ProxyFactory(this);
 
+        string proxyHost = _initData.properties->getProperty("Ice.SOCKSProxyHost");
+        int defaultIPv6 = 1; // IPv6 enabled by default.
+        if(!proxyHost.empty())
+        {
+#ifdef ICE_OS_WINRT
+            throw InitializationException(__FILE__, __LINE__, "SOCKS proxy not supported in WinRT");
+#else
+            int proxyPort = _initData.properties->getPropertyAsIntWithDefault("Ice.SOCKSProxyPort", 1080);
+            _networkProxy = new SOCKSNetworkProxy(proxyHost, proxyPort);
+            defaultIPv6 = 0; // IPv6 is not supported with SOCKS
+#endif
+        }
+
         bool ipv4 = _initData.properties->getPropertyAsIntWithDefault("Ice.IPv4", 1) > 0;
-        bool ipv6 = _initData.properties->getPropertyAsIntWithDefault("Ice.IPv6", 0) > 0;
+        bool ipv6 = _initData.properties->getPropertyAsIntWithDefault("Ice.IPv6", defaultIPv6) > 0;
         if(!ipv4 && !ipv6)
         {
             throw InitializationException(__FILE__, __LINE__, "Both IPV4 and IPv6 support cannot be disabled.");
@@ -972,9 +1086,25 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
         {
             _protocolSupport = EnableIPv6;
         }
+        _preferIPv6 = _initData.properties->getPropertyAsInt("Ice.PreferIPv6Address") > 0;
+
+#ifndef ICE_OS_WINRT
+        if(ipv6 && SOCKSNetworkProxyPtr::dynamicCast(_networkProxy))
+        {
+            throw InitializationException(__FILE__, __LINE__, "IPv6 is not supported with SOCKS4 proxies");
+        }
+#endif
+
         _endpointFactoryManager = new EndpointFactoryManager(this);
+#ifndef ICE_OS_WINRT
         EndpointFactoryPtr tcpEndpointFactory = new TcpEndpointFactory(this);
         _endpointFactoryManager->add(tcpEndpointFactory);
+#else
+        EndpointFactoryPtr tcpStreamEndpointFactory = new StreamEndpointFactory(this, TCPEndpointType);
+        _endpointFactoryManager->add(tcpStreamEndpointFactory);
+        EndpointFactoryPtr sslStreamEndpointFactory = new StreamEndpointFactory(this, IceSSL::EndpointType);
+        _endpointFactoryManager->add(sslStreamEndpointFactory);
+#endif
         EndpointFactoryPtr udpEndpointFactory = new UdpEndpointFactory(this);
         _endpointFactoryManager->add(udpEndpointFactory);
 
@@ -982,7 +1112,7 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
 
         _pluginManager = new PluginManagerI(communicator, _dynamicLibraryList);
 
-        _outgoingConnectionFactory = new OutgoingConnectionFactory(this);
+        _outgoingConnectionFactory = new OutgoingConnectionFactory(communicator, this);
 
         _servantFactoryManager = new ObjectFactoryManager();
 
@@ -990,7 +1120,6 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
         
         _retryQueue = new RetryQueue(this);
 
-
         if(_initData.wstringConverter == 0)
         {
             _initData.wstringConverter = new UnicodeWstringConverter();
@@ -1007,9 +1136,33 @@ IceInternal::Instance::Instance(const CommunicatorPtr& communicator, const Initi
             _adminFacetFilter.insert(facetSeq.begin(), facetSeq.end());
         }
 
-        _adminFacets.insert(FacetMap::value_type("Properties", new PropertiesAdminI(_initData.properties)));
         _adminFacets.insert(FacetMap::value_type("Process", new ProcessI(communicator)));
 
+        PropertiesAdminIPtr props = new PropertiesAdminI("Properties", _initData.properties, _initData.logger);
+        _adminFacets.insert(FacetMap::value_type("Properties",props));
+
+        _metricsAdmin = new MetricsAdminI(_initData.properties, _initData.logger);
+        _adminFacets.insert(FacetMap::value_type("Metrics", _metricsAdmin));
+
+        //
+        // Setup the communicator observer only if the user didn't already set an
+        // Ice observer resolver and if the admininistrative endpoints are set.
+        //
+        if((_adminFacetFilter.empty() || _adminFacetFilter.find("Metrics") != _adminFacetFilter.end()) &&
+           _initData.properties->getProperty("Ice.Admin.Endpoints") != "")
+        {
+            _observer = new CommunicatorObserverI(_metricsAdmin, _initData.observer);
+
+            //
+            // Make sure the admin plugin receives property updates.
+            //
+            props->addUpdateCallback(_metricsAdmin);
+        }
+        else
+        {
+            _observer = _initData.observer;
+        }
+
         __setNoDelete(false);
     }
     catch(...)
@@ -1048,7 +1201,7 @@ IceInternal::Instance::~Instance()
     IceUtilInternal::MutexPtrLock<IceUtil::Mutex> sync(staticMutex);
     if(--instanceCount == 0)
     {
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(ICE_OS_WINRT)
         WSACleanup();
 #endif
         
@@ -1075,6 +1228,14 @@ IceInternal::Instance::finishSetup(int& argc, char* argv[])
     assert(pluginManagerImpl);
     pluginManagerImpl->loadPlugins(argc, argv);
 
+    //
+    // Set observer updater
+    //
+    if(_observer)
+    {
+        theCollector->updateObserver(_observer);
+        _observer->setObserverUpdater(new ObserverUpdaterI(this));
+    }
 
     //
     // Create threads.
@@ -1098,7 +1259,7 @@ IceInternal::Instance::finishSetup(int& argc, char* argv[])
         out << "cannot create thread for timer:\n" << ex;
         throw;
     }
-    
+
     try
     {
         _endpointHostResolver = new EndpointHostResolver(this);
@@ -1109,7 +1270,7 @@ IceInternal::Instance::finishSetup(int& argc, char* argv[])
         out << "cannot create thread for endpoint host resolver:\n" << ex;
         throw;
     }
-
+    
     _clientThreadPool = new ThreadPool(this, "Ice.ThreadPool.Client", 0);
 
     //
@@ -1150,7 +1311,7 @@ IceInternal::Instance::finishSetup(int& argc, char* argv[])
     if(printProcessId)
     {
 #ifdef _MSC_VER
-        cout << _getpid() << endl;
+        cout << GetCurrentProcessId() << endl;
 #else
         cout << getpid() << endl;
 #endif
@@ -1239,10 +1400,26 @@ IceInternal::Instance::destroy()
         _retryQueue->destroy();
     }
 
+    if(_observer && theCollector)
+    {
+        theCollector->clearObserver(_observer);
+    }
+
+    if(_metricsAdmin)
+    {
+        _metricsAdmin->destroy();
+        _metricsAdmin = 0;
+
+        // Break cyclic reference counts. Don't clear _observer, it's immutable.
+        if(_observer)
+        {
+            CommunicatorObserverIPtr::dynamicCast(_observer)->destroy(); 
+        }
+    }
+
     ThreadPoolPtr serverThreadPool;
     ThreadPoolPtr clientThreadPool;
     EndpointHostResolverPtr endpointHostResolver;
-
     {
         IceUtil::RecMutex::Lock sync(*this);
 
@@ -1267,7 +1444,6 @@ IceInternal::Instance::destroy()
             _clientThreadPool->destroy();
             std::swap(_clientThreadPool, clientThreadPool);
         }
-
         if(_endpointHostResolver)
         {
             _endpointHostResolver->destroy();
@@ -1319,7 +1495,7 @@ IceInternal::Instance::destroy()
         // No destroy function defined.
         // _dynamicLibraryList->destroy();
         _dynamicLibraryList = 0;
-        
+
         _adminAdapter = 0;
         _adminFacets.clear();
 
@@ -1337,11 +1513,13 @@ IceInternal::Instance::destroy()
     {
         serverThreadPool->joinWithAllThreads();
     }
+#ifndef ICE_OS_WINRT
     if(endpointHostResolver)
     {
         endpointHostResolver->getThreadControl().join();
     }
-
+#endif
+    
     if(_initData.properties->getPropertyAsInt("Ice.Warn.UnusedProperties") > 0)
     {
         set<string> unusedProperties = static_cast<PropertiesI*>(_initData.properties.get())->getUnusedProperties();
@@ -1358,49 +1536,48 @@ IceInternal::Instance::destroy()
     return true;
 }
 
-IceInternal::UTF8BufferI::UTF8BufferI() :
-    _buffer(0),
-    _offset(0)
+void
+IceInternal::Instance::updateConnectionObservers()
 {
-}
-
-IceInternal::UTF8BufferI::~UTF8BufferI()
-{
-    free(_buffer);
-}
-
-Byte* 
-IceInternal::UTF8BufferI::getMoreBytes(size_t howMany, Byte* firstUnused)
-{
-    if(_buffer == 0)
+    try
     {
-        _buffer = (Byte*)malloc(howMany);
+        assert(_outgoingConnectionFactory);
+        _outgoingConnectionFactory->updateConnectionObservers();
+        assert(_objectAdapterFactory);
+        _objectAdapterFactory->updateObservers(&ObjectAdapterI::updateConnectionObservers);
     }
-    else
+    catch(const Ice::CommunicatorDestroyedException&)
     {
-        assert(firstUnused != 0);
-        _offset = firstUnused - _buffer;
-        _buffer = (Byte*)realloc(_buffer, _offset + howMany);
     }
-
-    return _buffer + _offset;
-}
-
-Byte* 
-IceInternal::UTF8BufferI::getBuffer()
-{
-    return _buffer;
 }
 
 void
-IceInternal::UTF8BufferI::reset()
+IceInternal::Instance::updateThreadObservers()
 {
-    free(_buffer);
-    _buffer = 0;
-    _offset = 0;
+    try
+    {
+        if(_clientThreadPool)
+        {
+            _clientThreadPool->updateObservers();
+        }
+        if(_serverThreadPool)
+        {
+            _serverThreadPool->updateObservers();
+        }
+        assert(_objectAdapterFactory);
+        _objectAdapterFactory->updateObservers(&ObjectAdapterI::updateThreadObservers);
+        if(_endpointHostResolver)
+        {
+            _endpointHostResolver->updateObserver();
+        }
+        assert(theCollector);
+        theCollector->updateObserver(_observer);
+    }
+    catch(const Ice::CommunicatorDestroyedException&)
+    {
+    }
 }
 
-
 IceInternal::ProcessI::ProcessI(const CommunicatorPtr& communicator) :
     _communicator(communicator)
 {
