@@ -1,12 +1,13 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+#include <IceUtil/DisableWarnings.h>
 #include <Ice/OutgoingAsync.h>
 #include <Ice/Object.h>
 #include <Ice/ConnectionI.h>
@@ -30,7 +31,7 @@ using namespace std;
 using namespace Ice;
 using namespace IceInternal;
 
-IceUtil::Shared* IceInternal::upCast(AsyncResult* p) { return p; }
+IceUtil::Shared* Ice::upCast(AsyncResult* p) { return p; }
 
 IceUtil::Shared* IceInternal::upCast(OutgoingAsyncMessageCallback* p) { return p; }
 IceUtil::Shared* IceInternal::upCast(OutgoingAsync* p) { return p; }
@@ -50,59 +51,62 @@ namespace
 class AsynchronousException : public DispatchWorkItem
 {
 public:
-    
+
     AsynchronousException(const IceInternal::InstancePtr& instance,
                           const Ice::AsyncResultPtr& result,
                           const Ice::Exception& ex) :
-        DispatchWorkItem(instance), _result(result), _exception(dynamic_cast<Ice::LocalException*>(ex.ice_clone()))
+        DispatchWorkItem(instance), _result(result), _exception(ex.ice_clone())
     {
     }
-    
+
     virtual void
     run()
     {
         _result->__exception(*_exception.get());
     }
-    
+
 private:
-    
+
     const Ice::AsyncResultPtr _result;
-    const auto_ptr<Ice::Exception> _exception;
+    const IceUtil::UniquePtr<Ice::Exception> _exception;
 };
 
 class AsynchronousSent : public DispatchWorkItem
 {
 public:
-    
-    AsynchronousSent(const IceInternal::InstancePtr& instance, const Ice::AsyncResultPtr& result) : 
+
+    AsynchronousSent(const IceInternal::InstancePtr& instance, const Ice::AsyncResultPtr& result) :
         DispatchWorkItem(instance), _result(result)
     {
     }
-    
+
     virtual void
     run()
     {
         _result->__sent();
     }
-    
+
 private:
-    
+
     const Ice::AsyncResultPtr _result;
 };
 
 };
 
-Ice::AsyncResult::AsyncResult(const IceInternal::InstancePtr& instance,
+Ice::AsyncResult::AsyncResult(const CommunicatorPtr& communicator,
+                              const IceInternal::InstancePtr& instance,
                               const string& op,
                               const CallbackBasePtr& del,
                               const LocalObjectPtr& cookie) :
+    _communicator(communicator),
     _instance(instance),
-    _operation(op), 
+    _operation(op),
     _callback(del),
     _cookie(cookie),
-    _is(instance.get()),
-    _os(instance.get()),
+    _is(instance.get(), Ice::currentProtocolEncoding),
+    _os(instance.get(), Ice::currentProtocolEncoding),
     _state(0),
+    _sentSynchronously(false),
     _exception(0)
 {
     if(!_callback)
@@ -141,7 +145,7 @@ Ice::AsyncResult::isCompleted() const
     return _state & Done;
 }
 
-void 
+void
 Ice::AsyncResult::waitForCompleted()
 {
     IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
@@ -151,7 +155,7 @@ Ice::AsyncResult::waitForCompleted()
     }
 }
 
-bool 
+bool
 Ice::AsyncResult::isSent() const
 {
     IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
@@ -223,19 +227,28 @@ Ice::AsyncResult::__sent()
 
     if(_callback)
     {
-	try
-	{
-	    AsyncResultPtr self(this);
-	    _callback->__sent(self);
-	}
-	catch(const std::exception& ex)
-	{
-	    __warning(ex);
-	}
-	catch(...)
-	{
-	    __warning();
-	}
+        try
+        {
+            AsyncResultPtr self(this);
+            _callback->__sent(self);
+        }
+        catch(const std::exception& ex)
+        {
+            __warning(ex);
+        }
+        catch(...)
+        {
+            __warning();
+        }
+    }
+
+    if(_observer)
+    {
+        Ice::ObjectPrx proxy = getProxy();
+        if(!proxy || !proxy->ice_isTwoway())
+        {
+            _observer.detach();
+        }
     }
 }
 
@@ -260,28 +273,31 @@ void
 Ice::AsyncResult::__exception(const Ice::Exception& ex)
 {
     {
-	IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
-	_state |= Done;
-	_exception.reset(ex.ice_clone());
-	_monitor.notifyAll();
+        IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
+        _state |= Done;
+        _os.resize(0); // Clear buffer now, instead of waiting for AsyncResult deallocation
+        _exception.reset(ex.ice_clone());
+        _monitor.notifyAll();
     }
 
     if(_callback)
     {
-	try
-	{
+        try
+        {
             AsyncResultPtr self(this);
             _callback->__completed(self);
-	}
-	catch(const std::exception& ex)
-	{
-	    __warning(ex);
-	}
-	catch(...)
-	{
-	    __warning();
-	}
+        }
+        catch(const std::exception& ex)
+        {
+            __warning(ex);
+        }
+        catch(...)
+        {
+            __warning();
+        }
     }
+
+    _observer.detach();
 }
 
 void
@@ -308,20 +324,22 @@ Ice::AsyncResult::__response()
 
     if(_callback)
     {
-	try
-	{
-	    AsyncResultPtr self(this);
+        try
+        {
+            AsyncResultPtr self(this);
             _callback->__completed(self);
-	}
-	catch(const std::exception& ex)
-	{
-	    __warning(ex);
-	}
-	catch(...)
-	{
-	    __warning();
-	}
+        }
+        catch(const std::exception& ex)
+        {
+            __warning(ex);
+        }
+        catch(...)
+        {
+            __warning();
+        }
     }
+
+    _observer.detach();
 }
 
 void
@@ -369,7 +387,7 @@ Ice::AsyncResult::__check(const AsyncResultPtr& r, const string& operation)
     }
     else if(&r->_operation != &operation)
     {
-        throw IceUtil::IllegalArgumentException(__FILE__, __LINE__, "Incorrect operation for end_" + operation + 
+        throw IceUtil::IllegalArgumentException(__FILE__, __LINE__, "Incorrect operation for end_" + operation +
                                                 " method: " + r->_operation);
     }
 }
@@ -403,12 +421,13 @@ Ice::AsyncResult::__warning() const
     }
 }
 
-IceInternal::OutgoingAsync::OutgoingAsync(const ObjectPrx& prx, 
+IceInternal::OutgoingAsync::OutgoingAsync(const ObjectPrx& prx,
                                           const std::string& operation,
                                           const CallbackBasePtr& delegate,
                                           const Ice::LocalObjectPtr& cookie) :
-    AsyncResult(prx->__reference()->getInstance(), operation, delegate, cookie),
-    _proxy(prx)
+    AsyncResult(prx->ice_getCommunicator(), prx->__reference()->getInstance(), operation, delegate, cookie),
+    _proxy(prx),
+    _encoding(getCompatibleEncoding(prx->__reference()->getEncoding()))
 {
 }
 
@@ -420,6 +439,10 @@ IceInternal::OutgoingAsync::__prepare(const std::string& operation, OperationMod
     _mode = mode;
     _sentSynchronously = false;
 
+    checkSupportedProtocol(getCompatibleProtocol(_proxy->__reference()->getProtocol()));
+
+    _observer.attach(_proxy.get(), operation, context);
+
     //
     // Can't call async via a batch proxy.
     //
@@ -432,7 +455,7 @@ IceInternal::OutgoingAsync::__prepare(const std::string& operation, OperationMod
 
     Reference* ref = _proxy->__reference().get();
 
-    ref->getIdentity().__write(&_os);
+    _os.write(ref->getIdentity());
 
     //
     // For compatibility with the old FacetPath.
@@ -456,7 +479,7 @@ IceInternal::OutgoingAsync::__prepare(const std::string& operation, OperationMod
         //
         // Explicit context
         //
-        __writeContext(&_os, *context);
+        _os.write(*context);
     }
     else
     {
@@ -467,15 +490,13 @@ IceInternal::OutgoingAsync::__prepare(const std::string& operation, OperationMod
         const Context& prxContext = ref->getContext()->getValue();
         if(implicitContext == 0)
         {
-            __writeContext(&_os, prxContext);
+            _os.write(prxContext);
         }
         else
         {
             implicitContext->write(prxContext, &_os);
         }
     }
-        
-    _os.startWriteEncaps();
 }
 
 bool
@@ -486,24 +507,23 @@ IceInternal::OutgoingAsync::__sent(Ice::ConnectionI* connection)
     bool alreadySent = _state & Sent; // Expected in case of a retry.
     _state |= Sent;
 
-    //
-    // It's possible for the request to be done already when using IOCP. This 
-    // is the case for example if the send callback is dispatched after the 
-    // read callback for the response/exception.
-    //
-    if(!(_state & Done))
+    assert(!(_state & Done));
+    if(!_proxy->ice_isTwoway())
     {
-        if(!_proxy->ice_isTwoway())
+        _remoteObserver.detach();
+        if(!_callback || !_callback->__hasSentCallback())
         {
-            _state |= Done | OK;
+            _observer.detach();
         }
-        else if(connection->timeout() > 0)
-        {
-            assert(!_timerTaskConnection);
-            _timerTaskConnection = connection;
+        _state |= Done | OK;
+        _os.resize(0); // Clear buffer now, instead of waiting for AsyncResult deallocation
+    }
+    else if(connection->timeout() > 0)
+    {
+        assert(!_timerTaskConnection);
+        _timerTaskConnection = connection;
             IceUtil::Time timeout = IceUtil::Time::milliSeconds(connection->timeout());
             _instance->timer()->schedule(this, timeout);
-        }
     }
     _monitor.notifyAll();
     return !alreadySent && _callback && _callback->__hasSentCallback();
@@ -512,11 +532,7 @@ IceInternal::OutgoingAsync::__sent(Ice::ConnectionI* connection)
 void
 IceInternal::OutgoingAsync::__sent()
 {
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    AsyncResult::__sent();
-#else
     ::Ice::AsyncResult::__sent();
-#endif
 }
 
 void
@@ -525,13 +541,15 @@ IceInternal::OutgoingAsync::__finished(const Ice::LocalException& exc, bool sent
     {
         IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
         assert(!(_state & Done));
+        _remoteObserver.failed(exc.ice_name());
+        _remoteObserver.detach();
         if(_timerTaskConnection)
         {
             _instance->timer()->cancel(this);
             _timerTaskConnection = 0;
         }
     }
-     
+
     //
     // NOTE: at this point, synchronization isn't needed, no other threads should be
     // calling on the callback.
@@ -563,6 +581,8 @@ IceInternal::OutgoingAsync::__finished(const LocalExceptionWrapper& exc)
     // calling on the callback. The LocalExceptionWrapper exception is only called
     // before the invocation is sent.
     //
+    _remoteObserver.failed(exc.get()->ice_name());
+    _remoteObserver.detach();
 
     try
     {
@@ -583,7 +603,7 @@ IceInternal::OutgoingAsync::__finished(const LocalExceptionWrapper& exc)
 }
 
 void
-IceInternal::OutgoingAsync::__finished(BasicStream& is)
+IceInternal::OutgoingAsync::__finished()
 {
     assert(_proxy->ice_isTwoway()); // Can only be called for twoways.
 
@@ -592,6 +612,13 @@ IceInternal::OutgoingAsync::__finished(BasicStream& is)
     {
         IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
         assert(!_exception.get() && !(_state & Done));
+        assert(!_is.b.empty());
+
+        if(_remoteObserver)
+        {
+            _remoteObserver->reply(static_cast<Int>(_is.b.size() - headerSize - 4));
+        }
+        _remoteObserver.detach();
 
         if(_timerTaskConnection)
         {
@@ -599,24 +626,27 @@ IceInternal::OutgoingAsync::__finished(BasicStream& is)
             _timerTaskConnection = 0;
         }
 
-        _is.swap(is);
         _is.read(replyStatus);
-        
+
         switch(replyStatus)
         {
             case replyOK:
-            case replyUserException:
             {
                 break;
             }
-            
+            case replyUserException:
+            {
+                _observer.userException();
+                break;
+            }
+
             case replyObjectNotExist:
             case replyFacetNotExist:
             case replyOperationNotExist:
             {
                 Identity ident;
-                ident.__read(&_is);
-                
+                _is.read(ident);
+
                 //
                 // For compatibility with the old FacetPath.
                 //
@@ -631,11 +661,11 @@ IceInternal::OutgoingAsync::__finished(BasicStream& is)
                     }
                     facet.swap(facetPath[0]);
                 }
-                
+
                 string operation;
                 _is.read(operation, false);
-                
-                auto_ptr<RequestFailedException> ex;
+
+                IceUtil::UniquePtr<RequestFailedException> ex;
                 switch(replyStatus)
                 {
                     case replyObjectNotExist:
@@ -643,19 +673,19 @@ IceInternal::OutgoingAsync::__finished(BasicStream& is)
                         ex.reset(new ObjectNotExistException(__FILE__, __LINE__));
                         break;
                     }
-                    
+
                     case replyFacetNotExist:
                     {
                         ex.reset(new FacetNotExistException(__FILE__, __LINE__));
                         break;
                     }
-                    
+
                     case replyOperationNotExist:
                     {
                         ex.reset(new OperationNotExistException(__FILE__, __LINE__));
                         break;
                     }
-                    
+
                     default:
                     {
                         assert(false);
@@ -668,15 +698,15 @@ IceInternal::OutgoingAsync::__finished(BasicStream& is)
                 ex->operation = operation;
                 ex->ice_throw();
             }
-            
+
             case replyUnknownException:
             case replyUnknownLocalException:
             case replyUnknownUserException:
             {
                 string unknown;
                 _is.read(unknown, false);
-                
-                auto_ptr<UnknownException> ex;
+
+                IceUtil::UniquePtr<UnknownException> ex;
                 switch(replyStatus)
                 {
                     case replyUnknownException:
@@ -684,42 +714,43 @@ IceInternal::OutgoingAsync::__finished(BasicStream& is)
                         ex.reset(new UnknownException(__FILE__, __LINE__));
                         break;
                     }
-                    
+
                     case replyUnknownLocalException:
                     {
                         ex.reset(new UnknownLocalException(__FILE__, __LINE__));
                         break;
                     }
-                    
+
                     case replyUnknownUserException:
                     {
                         ex.reset(new UnknownUserException(__FILE__, __LINE__));
                         break;
                     }
-                    
+
                     default:
                     {
                         assert(false);
                         break;
                     }
                 }
-                
+
                 ex->unknown = unknown;
                 ex->ice_throw();
             }
-            
+
             default:
             {
                 throw UnknownReplyStatusException(__FILE__, __LINE__);
             }
         }
 
-	_state |= Done;
-	if(replyStatus == replyOK)
-	{
-	    _state |= OK;
-	}
-	_monitor.notifyAll();
+        _state |= Done;
+        _os.resize(0); // Clear buffer now, instead of waiting for AsyncResult deallocation
+        if(replyStatus == replyOK)
+        {
+            _state |= OK;
+        }
+        _monitor.notifyAll();
     }
     catch(const LocalException& ex)
     {
@@ -784,11 +815,11 @@ IceInternal::OutgoingAsync::handleException(const LocalExceptionWrapper& ex)
 {
     if(_mode == Nonmutating || _mode == Idempotent)
     {
-        return _proxy->__handleExceptionWrapperRelaxed(_delegate, ex, false, _cnt);
+        return _proxy->__handleExceptionWrapperRelaxed(_delegate, ex, false, _cnt, _observer);
     }
     else
     {
-        return _proxy->__handleExceptionWrapper(_delegate, ex);
+        return _proxy->__handleExceptionWrapper(_delegate, ex, _observer);
     }
 }
 
@@ -803,20 +834,20 @@ IceInternal::OutgoingAsync::handleException(const Ice::LocalException& exc, bool
         // close connection message, the server guarantees that all outstanding requests
         // can safely be repeated.
         //
-        // An ObjectNotExistException can always be retried as well without violating 
+        // An ObjectNotExistException can always be retried as well without violating
         // "at-most-once" (see the implementation of the checkRetryAfterException method of
         // the ProxyFactory class for the reasons why it can be useful).
-        // 
-        if(!sent || 
-           dynamic_cast<const CloseConnectionException*>(&exc) || 
+        //
+        if(!sent ||
+           dynamic_cast<const CloseConnectionException*>(&exc) ||
            dynamic_cast<const ObjectNotExistException*>(&exc))
         {
             exc.ice_throw();
         }
 
         //
-        // Throw the exception wrapped in a LocalExceptionWrapper, to indicate that the 
-        // request cannot be resent without potentially violating the "at-most-once" 
+        // Throw the exception wrapped in a LocalExceptionWrapper, to indicate that the
+        // request cannot be resent without potentially violating the "at-most-once"
         // principle.
         //
         throw LocalExceptionWrapper(exc, false);
@@ -825,16 +856,16 @@ IceInternal::OutgoingAsync::handleException(const Ice::LocalException& exc, bool
     {
         if(_mode == Nonmutating || _mode == Idempotent)
         {
-            return _proxy->__handleExceptionWrapperRelaxed(_delegate, ex, false, _cnt);
+            return _proxy->__handleExceptionWrapperRelaxed(_delegate, ex, false, _cnt, _observer);
         }
         else
         {
-            return _proxy->__handleExceptionWrapper(_delegate, ex);
+            return _proxy->__handleExceptionWrapper(_delegate, ex, _observer);
         }
     }
     catch(const Ice::LocalException& ex)
     {
-        return _proxy->__handleException(_delegate, ex, false, _cnt);
+        return _proxy->__handleException(_delegate, ex, false, _cnt, _observer);
     }
     return 0; // Keep the compiler happy.
 }
@@ -855,52 +886,61 @@ IceInternal::OutgoingAsync::runTimerTask() // Implementation of TimerTask::runTi
     }
 }
 
-IceInternal::BatchOutgoingAsync::BatchOutgoingAsync(const InstancePtr& instance, 
+IceInternal::BatchOutgoingAsync::BatchOutgoingAsync(const CommunicatorPtr& communicator,
+                                                    const InstancePtr& instance,
                                                     const std::string& operation,
                                                     const CallbackBasePtr& delegate,
                                                     const Ice::LocalObjectPtr& cookie) :
-    AsyncResult(instance, operation, delegate, cookie)
+    AsyncResult(communicator, instance, operation, delegate, cookie)
 {
 }
 
 bool
-IceInternal::BatchOutgoingAsync::__sent(Ice::ConnectionI* connection)
+IceInternal::BatchOutgoingAsync::__sent(Ice::ConnectionI* /*connection*/)
 {
     IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
     assert(!_exception.get());
     _state |= Done | OK | Sent;
+    _os.resize(0); // Clear buffer now, instead of waiting for AsyncResult deallocation
+    _remoteObserver.detach();
     _monitor.notifyAll();
-    return _callback && _callback->__hasSentCallback();
+    if(!_callback || !_callback->__hasSentCallback())
+    {
+        _observer.detach();
+        return false;
+    }
+    return true;
 }
 
 void
 IceInternal::BatchOutgoingAsync::__sent()
 {
-#if defined(_MSC_VER) && (_MSC_VER < 1300) // VC++ 6 compiler bug
-    AsyncResult::__sent();
-#else
     ::Ice::AsyncResult::__sent();
-#endif
 }
 
 void
 IceInternal::BatchOutgoingAsync::__finished(const Ice::LocalException& exc, bool)
 {
+    _remoteObserver.failed(exc.ice_name());
+    _remoteObserver.detach();
     __exception(exc);
 }
 
-IceInternal::ProxyBatchOutgoingAsync::ProxyBatchOutgoingAsync(const Ice::ObjectPrx& proxy, 
+IceInternal::ProxyBatchOutgoingAsync::ProxyBatchOutgoingAsync(const Ice::ObjectPrx& proxy,
                                                               const std::string& operation,
                                                               const CallbackBasePtr& delegate,
                                                               const Ice::LocalObjectPtr& cookie) :
-    BatchOutgoingAsync(proxy->__reference()->getInstance(), operation, delegate, cookie),
+    BatchOutgoingAsync(proxy->ice_getCommunicator(), proxy->__reference()->getInstance(), operation, delegate, cookie),
     _proxy(proxy)
 {
+    _observer.attach(proxy.get(), operation, 0);
 }
 
 void
 IceInternal::ProxyBatchOutgoingAsync::__send()
 {
+    checkSupportedProtocol(_proxy->__reference()->getProtocol());
+
     //
     // We don't automatically retry if ice_flushBatchRequests fails. Otherwise, if some batch
     // requests were queued with the connection, they would be lost without being noticed.
@@ -922,18 +962,20 @@ IceInternal::ProxyBatchOutgoingAsync::__send()
     }
     catch(const ::Ice::LocalException& ex)
     {
-        _proxy->__handleException(delegate, ex, 0, cnt);
+        _proxy->__handleException(delegate, ex, 0, cnt, _observer);
     }
 }
 
 IceInternal::ConnectionBatchOutgoingAsync::ConnectionBatchOutgoingAsync(const ConnectionIPtr& con,
+                                                                        const CommunicatorPtr& communicator,
                                                                         const InstancePtr& instance,
                                                                         const string& operation,
                                                                         const CallbackBasePtr& delegate,
                                                                         const Ice::LocalObjectPtr& cookie) :
-    BatchOutgoingAsync(instance, operation, delegate, cookie),
+    BatchOutgoingAsync(communicator, instance, operation, delegate, cookie),
     _connection(con)
 {
+    _observer.attach(instance.get(), operation);
 }
 
 void
@@ -961,8 +1003,7 @@ IceInternal::CommunicatorBatchOutgoingAsync::CommunicatorBatchOutgoingAsync(cons
                                                                             const string& operation,
                                                                             const CallbackBasePtr& delegate,
                                                                             const Ice::LocalObjectPtr& cookie) :
-    BatchOutgoingAsync(instance, operation, delegate, cookie),
-    _communicator(communicator)
+    AsyncResult(communicator, instance, operation, delegate, cookie)
 {
     //
     // _useCount is initialized to 1 to prevent premature callbacks.
@@ -975,93 +1016,120 @@ IceInternal::CommunicatorBatchOutgoingAsync::CommunicatorBatchOutgoingAsync(cons
     // Assume all connections are flushed synchronously.
     //
     _sentSynchronously = true;
+    
+    //
+    // Attach observer
+    //
+    _observer.attach(instance.get(), operation);
 }
 
 void
-IceInternal::CommunicatorBatchOutgoingAsync::flushConnection(const ConnectionPtr& con)
+IceInternal::CommunicatorBatchOutgoingAsync::flushConnection(const ConnectionIPtr& con)
 {
+    class BatchOutgoingAsyncI : public BatchOutgoingAsync
+    {
+    public:
+
+        BatchOutgoingAsyncI(const CommunicatorBatchOutgoingAsyncPtr& outAsync,
+                            const InstancePtr& instance, 
+                            InvocationObserver& observer) :
+            BatchOutgoingAsync(outAsync->getCommunicator(), instance, outAsync->getOperation(), __dummyCallback, 0),
+            _outAsync(outAsync), _observer(observer)
+        {
+        }
+
+        virtual bool __sent(Ice::ConnectionI*)
+        {
+            _remoteObserver.detach();
+            _outAsync->check(false);
+            return false;
+        }
+
+#ifdef __SUNPRO_CC
+        using BatchOutgoingAsync::__sent;
+#endif
+
+        virtual void __finished(const Ice::LocalException& ex, bool)
+        {
+            _remoteObserver.failed(ex.ice_name());
+            _remoteObserver.detach();
+            _outAsync->check(false);
+        }
+
+        virtual void __attachRemoteObserver(const Ice::ConnectionInfoPtr& connection, const Ice::EndpointPtr& endpt,
+                                            Ice::Int requestId, Ice::Int sz)
+        {
+            _remoteObserver.attach(_observer.getRemoteObserver(connection, endpt, requestId, sz));
+        }
+
+    private:
+        
+        const CommunicatorBatchOutgoingAsyncPtr _outAsync;
+        InvocationObserver& _observer;
+    };
+
     {
         IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
         ++_useCount;
     }
-    CallbackPtr cb = newCallback(this, &CommunicatorBatchOutgoingAsync::completed,
-                                 &CommunicatorBatchOutgoingAsync::sent);
-    con->begin_flushBatchRequests(cb);
+
+    try
+    {
+        AsyncStatus status = con->flushAsyncBatchRequests(new BatchOutgoingAsyncI(this, _instance, _observer));
+        if(!(status & AsyncStatusSent))
+        {
+            _sentSynchronously = false;
+        }
+    }
+    catch(const Ice::LocalException&)
+    {
+        check(false);
+        throw;
+    }
 }
 
 void
 IceInternal::CommunicatorBatchOutgoingAsync::ready()
 {
-    check(0, 0, true);
+    check(true);
 }
 
 void
-IceInternal::CommunicatorBatchOutgoingAsync::completed(const AsyncResultPtr& r)
+IceInternal::CommunicatorBatchOutgoingAsync::check(bool userThread)
 {
-    ConnectionPtr con = r->getConnection();
-    assert(con);
-
-    try
-    {
-        con->end_flushBatchRequests(r);
-        assert(false); // completed() should only be called when an exception occurs.
-    }
-    catch(const Ice::LocalException& ex)
-    {
-        check(r, &ex, false);
-    }
-}
-
-void
-IceInternal::CommunicatorBatchOutgoingAsync::sent(const AsyncResultPtr& r)
-{
-    check(r, 0, r->sentSynchronously());
-}
-
-void
-IceInternal::CommunicatorBatchOutgoingAsync::check(const AsyncResultPtr& r, const LocalException* ex, bool userThread)
-{
-    bool done = false;
-
     {
         IceUtil::Monitor<IceUtil::Mutex>::Lock sync(_monitor);
         assert(_useCount > 0);
-        --_useCount;
-
-        //
-        // We report that the communicator flush request was sent synchronously
-        // if all of the connection flush requests are sent synchronously.
-        //
-        if((r && !r->sentSynchronously()) || ex)
+        if(--_useCount > 0)
         {
-            _sentSynchronously = false;
-        }
-
-        if(_useCount == 0)
-        {
-            done = true;
-            _state |= Done | OK | Sent;
-            _monitor.notifyAll();
-        }
+            return;
+        }         
+        _state |= Done | OK | Sent;
+        _os.resize(0); // Clear buffer now, instead of waiting for AsyncResult deallocation
+        _monitor.notifyAll();
     }
 
-    if(done)
+    if(!_callback || !_callback->__hasSentCallback())
+    {
+        _observer.detach();
+    }
+    else
     {
         //
         // _sentSynchronously is immutable here.
         //
-        if(!_sentSynchronously && userThread)
+        if(!_sentSynchronously || !userThread)
         {
             __sentAsync();
         }
         else
         {
-            assert(_sentSynchronously == userThread); // sentSynchronously && !userThread is impossible.
-            BatchOutgoingAsync::__sent();
+            AsyncResult::__sent();
         }
     }
 }
 
+
 namespace
 {
 
@@ -1090,8 +1158,8 @@ public:
     {
         //
         // Called by the AsyncResult constructor to verify the delegate. The dummy
-        // delegate is passed when the user used a begin_ method without delegate. 
-        // By returning 0 here, we tell the AsyncResult that no delegates was 
+        // delegate is passed when the user used a begin_ method without delegate.
+        // By returning 0 here, we tell the AsyncResult that no delegates was
         // provided.
         //
         return 0;
@@ -1133,3 +1201,4 @@ Ice::AMICallbackBase::__sent(bool sentSynchronously)
         dynamic_cast<AMISentCallback*>(this)->ice_sent();
     }
 }
+
