@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -73,21 +73,6 @@ private:
     const StringConverterPtr _converter;
 };
 
-
-class PropertiesAdminI : public PropertiesAdmin
-{
-public:
-    
-    PropertiesAdminI(const PropertiesPtr&);
-    
-    virtual std::string getProperty(const std::string&, const Current&);
-    virtual PropertyDict getPropertiesForPrefix(const std::string&, const Current&);
-    
-private:
-
-    const PropertiesPtr _properties; 
-};
-
 }
 
 #endif
