@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,15 +12,11 @@
 
 #include <Ice/TransceiverF.h>
 #include <Ice/InstanceF.h>
+#include <Ice/Protocol.h>
 #include <Ice/TraceLevelsF.h>
 #include <Ice/LoggerF.h>
 #include <Ice/Connector.h>
-
-#ifdef _WIN32
-#   include <winsock2.h>
-#else
-#   include <sys/socket.h>
-#endif
+#include <Ice/Network.h>
 
 namespace IceInternal
 {
@@ -40,14 +36,15 @@ public:
 
 private:
     
-    TcpConnector(const InstancePtr&, const struct sockaddr_storage&, Ice::Int, const std::string&);
+    TcpConnector(const InstancePtr&, const Address&, const NetworkProxyPtr&, Ice::Int, const std::string&);
     virtual ~TcpConnector();
     friend class TcpEndpointI;
 
     const InstancePtr _instance;
     const TraceLevelsPtr _traceLevels;
     const ::Ice::LoggerPtr _logger;
-    const struct sockaddr_storage _addr;
+    const Address _addr;
+    const NetworkProxyPtr _proxy;
     const Ice::Int _timeout;
     const std::string _connectionId;
 };
