@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,12 +14,7 @@
 #include <Ice/InstanceF.h>
 #include <Ice/Connector.h>
 #include <Ice/Protocol.h>
-
-#ifdef _WIN32
-#   include <winsock2.h>
-#else
-#   include <sys/socket.h> // For struct sockaddr_storage
-#endif
+#include <Ice/Network.h>
 
 namespace IceInternal
 {
@@ -39,22 +34,17 @@ public:
 
 private:
     
-    UdpConnector(const InstancePtr&, const struct sockaddr_storage&, const std::string&, int, Ice::Byte, Ice::Byte, 
-                 Ice::Byte, Ice::Byte, const std::string&);
+    UdpConnector(const InstancePtr&, const Address&, const std::string&, int, const std::string&);
+
     virtual ~UdpConnector();
     friend class UdpEndpointI;
 
     const InstancePtr _instance;
-    struct sockaddr_storage _addr;
+    const Address _addr;
     const std::string _mcastInterface;
     const int _mcastTtl;
-    const Ice::Byte _protocolMajor;
-    const Ice::Byte _protocolMinor;
-    const Ice::Byte _encodingMajor;
-    const Ice::Byte _encodingMinor;
     const std::string _connectionId;
 };
 
 }
-
 #endif
