@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -10,8 +10,10 @@
 #ifndef ICE_UDP_ENDPOINT_I_H
 #define ICE_UDP_ENDPOINT_I_H
 
+#include <IceUtil/Config.h>
 #include <Ice/EndpointI.h>
 #include <Ice/EndpointFactory.h>
+#include <Ice/Network.h>
 
 namespace IceInternal
 {
@@ -20,8 +22,8 @@ class UdpEndpointI : public EndpointI
 {
 public:
 
-    UdpEndpointI(const InstancePtr&, const std::string&, Ice::Int, const std::string&, Ice::Int, Ice::Byte, Ice::Byte,
-                 Ice::Byte, Ice::Byte, bool, const std::string&, bool);
+    UdpEndpointI(const InstancePtr&, const std::string&, Ice::Int, const std::string&, Ice::Int, bool, 
+                 const std::string&, bool);
     UdpEndpointI(const InstancePtr&, const std::string&, bool);
     UdpEndpointI(BasicStream*);
 
@@ -29,6 +31,7 @@ public:
     virtual std::string toString() const;
     virtual Ice::EndpointInfoPtr getInfo() const;
     virtual Ice::Short type() const;
+    virtual std::string protocol() const;
     virtual Ice::Int timeout() const;
     virtual EndpointIPtr timeout(Ice::Int) const;
     virtual EndpointIPtr connectionId(const ::std::string&) const;
@@ -37,19 +40,23 @@ public:
     virtual bool datagram() const;
     virtual bool secure() const;
     virtual TransceiverPtr transceiver(EndpointIPtr&) const;
-    virtual std::vector<ConnectorPtr> connectors() const;
-    virtual void connectors_async(const EndpointI_connectorsPtr&) const;
+    virtual std::vector<ConnectorPtr> connectors(Ice::EndpointSelectionType) const;
+    virtual void connectors_async(Ice::EndpointSelectionType, const EndpointI_connectorsPtr&) const;
     virtual AcceptorPtr acceptor(EndpointIPtr&, const std::string&) const;
     virtual std::vector<EndpointIPtr> expand() const;
     virtual bool equivalent(const EndpointIPtr&) const;
+    virtual std::vector<ConnectorPtr> connectors(const std::vector<Address>&, const NetworkProxyPtr&) const;
 
     virtual bool operator==(const Ice::LocalObject&) const;
     virtual bool operator<(const Ice::LocalObject&) const;
 
+#ifdef __SUNPRO_CC
+    using EndpointI::connectionId;
+#endif
+    
 private:
 
     virtual ::Ice::Int hashInit() const;
-    virtual std::vector<ConnectorPtr> connectors(const std::vector<struct sockaddr_storage>&) const;
 
     //
     // All members are const, because endpoints are immutable.
@@ -59,12 +66,7 @@ private:
     const Ice::Int _port;
     const std::string _mcastInterface;
     const Ice::Int _mcastTtl;
-    const Ice::Byte _protocolMajor;
-    const Ice::Byte _protocolMinor;
-    const Ice::Byte _encodingMajor;
-    const Ice::Byte _encodingMinor;
     const bool _connect;
-    const std::string _connectionId;
     const bool _compress;
 };
 
