@@ -1,35 +1,39 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-//
-// The following is required for the Vista PSDK to bring in
-// the definitions of the IN6_IS_ADDR_* macros.
-//
-#if defined(_WIN32) && !defined(_WIN32_WINNT)
-#       define _WIN32_WINNT 0x0501
-#endif
-
 #include <Ice/UdpTransceiver.h>
 #include <Ice/Connection.h>
 #include <Ice/Instance.h>
 #include <Ice/TraceLevels.h>
 #include <Ice/LoggerUtil.h>
-#include <Ice/Stats.h>
 #include <Ice/Buffer.h>
-#include <Ice/Network.h>
 #include <Ice/LocalException.h>
 #include <Ice/Properties.h>
 #include <IceUtil/StringUtil.h>
 
+#include <IceUtil/DisableWarnings.h>
+#include <Ice/Stats.h>
+
+#ifdef ICE_OS_WINRT
+#   include <ppltasks.h> // For Concurrency::task
+#endif
+
 using namespace std;
 using namespace Ice;
 using namespace IceInternal;
+#ifdef ICE_OS_WINRT
+using namespace Platform;
+using namespace Windows::Foundation;
+using namespace Windows::Storage::Streams;
+using namespace Windows::Networking;
+using namespace Windows::Networking::Sockets;
+#endif
 
 NativeInfoPtr
 IceInternal::UdpTransceiver::getNativeInfo()
@@ -53,10 +57,30 @@ IceInternal::UdpTransceiver::getAsyncInfo(SocketOperation status)
         return 0;
     }
 }
+#elif defined(ICE_OS_WINRT)
+void 
+IceInternal::UdpTransceiver::setCompletedHandler(SocketOperationCompletedHandler^ handler)
+{
+    _completedHandler = handler;
+    _writeOperationCompletedHandler = ref new AsyncOperationCompletedHandler<unsigned int>(
+        [=] (IAsyncOperation<unsigned int>^ operation, Windows::Foundation::AsyncStatus status)
+        {
+            if(status != Windows::Foundation::AsyncStatus::Completed)
+            {
+                _write.count = SOCKET_ERROR;
+                _write.error = operation->ErrorCode.Value;
+            }
+            else
+            {
+                _write.count = static_cast<int>(operation->GetResults());
+            }
+            _completedHandler(SocketOperationWrite);
+        });
+}
 #endif
 
 SocketOperation
-IceInternal::UdpTransceiver::initialize()
+IceInternal::UdpTransceiver::initialize(Buffer& /*readBuffer*/, Buffer& /*writeBuffer*/)
 {
     if(_state == StateNeedConnect)
     {
@@ -67,7 +91,16 @@ IceInternal::UdpTransceiver::initialize()
     {
         try
         {
+#if defined(ICE_USE_IOCP)
+            doFinishConnectAsync(_fd, _write);
+#elif defined(ICE_OS_WINRT)
+            if(_write.count == SOCKET_ERROR)
+            {
+                checkConnectErrorCode(__FILE__, __LINE__, _write.error, _addr.host);
+            }
+#else
             doFinishConnect(_fd);
+#endif
             _state = StateConnected;
         }
         catch(const Ice::LocalException& ex)
@@ -102,11 +135,30 @@ IceInternal::UdpTransceiver::close()
         out << "closing udp connection\n" << toString();
     }
 
+#ifdef ICE_OS_WINRT
+    IceUtil::Mutex::Lock lock(_mutex);
+    if(_readPending)
+    {
+        assert(_received.empty());
+        _completedHandler(SocketOperationRead);
+        _readPending = false;
+    }
+    _received.clear();
+#endif
+
     assert(_fd != INVALID_SOCKET);
     closeSocket(_fd);
     _fd = INVALID_SOCKET;
 }
 
+
+#ifdef ICE_OS_WINRT
+bool
+IceInternal::UdpTransceiver::write(Buffer&)
+{
+    return false;
+}
+#else
 bool
 IceInternal::UdpTransceiver::write(Buffer& buf)
 {
@@ -121,22 +173,22 @@ repeat:
     ssize_t ret;
     if(_state == StateConnected)
     {
-#ifdef _WIN32
+#   ifdef _WIN32
         ret = ::send(_fd, reinterpret_cast<const char*>(&buf.b[0]), static_cast<int>(buf.b.size()), 0);
-#else
+#   else
         ret = ::send(_fd, reinterpret_cast<const char*>(&buf.b[0]), buf.b.size(), 0);
-#endif
+#   endif
     }
     else
     {
-        socklen_t len = static_cast<socklen_t>(sizeof(_peerAddr));
-        if(_peerAddr.ss_family == AF_INET)
+        socklen_t len = static_cast<socklen_t>(sizeof(sockaddr_storage));
+        if(_peerAddr.saStorage.ss_family == AF_INET)
         {
-            len = sizeof(sockaddr_in);
+            len = static_cast<socklen_t>(sizeof(sockaddr_in));
         }
-        else if(_peerAddr.ss_family == AF_INET6)
+        else if(_peerAddr.saStorage.ss_family == AF_INET6)
         {
-            len = sizeof(sockaddr_in6);
+            len = static_cast<socklen_t>(sizeof(sockaddr_in6));
         }
         else
         {
@@ -146,13 +198,13 @@ repeat:
             throw ex;
         }
 
-#ifdef _WIN32
+#   ifdef _WIN32
         ret = ::sendto(_fd, reinterpret_cast<const char*>(&buf.b[0]), static_cast<int>(buf.b.size()), 0, 
-                       reinterpret_cast<struct sockaddr*>(&_peerAddr), len);
-#else
+                       &_peerAddr.sa, len);
+#   else
         ret = ::sendto(_fd, reinterpret_cast<const char*>(&buf.b[0]), buf.b.size(), 0,
-                       reinterpret_cast<struct sockaddr*>(&_peerAddr), len);
-#endif
+                       &_peerAddr.sa, len);
+#   endif
     }
 
     if(ret == SOCKET_ERROR)
@@ -187,7 +239,15 @@ repeat:
     buf.i = buf.b.end();
     return true;
 }
+#endif
 
+#ifdef ICE_OS_WINRT
+bool
+IceInternal::UdpTransceiver::read(Buffer&)
+{
+    return false;
+}
+#else
 bool
 IceInternal::UdpTransceiver::read(Buffer& buf)
 {
@@ -209,12 +269,12 @@ repeat:
     {
         assert(_incoming);
 
-        sockaddr_storage peerAddr;
-        memset(&peerAddr, 0, sizeof(struct sockaddr_storage));
-        socklen_t len = static_cast<socklen_t>(sizeof(peerAddr));
+        Address peerAddr;
+        memset(&peerAddr.saStorage, 0, sizeof(sockaddr_storage));
+        socklen_t len = static_cast<socklen_t>(sizeof(sockaddr_storage));
 
         ret = recvfrom(_fd, reinterpret_cast<char*>(&buf.b[0]), packetSize, 0, 
-                       reinterpret_cast<struct sockaddr*>(&peerAddr), &len);
+                       &peerAddr.sa, &len);
 
         if(ret != SOCKET_ERROR)
         {
@@ -265,12 +325,12 @@ repeat:
         //
         assert(_incoming); // Client connections should always be connected at this point.
 
-#ifndef NDEBUG
+#   ifndef NDEBUG
         bool connected = doConnect(_fd, _peerAddr);
         assert(connected);
-#else
+#   else
         doConnect(_fd, _peerAddr);
-#endif
+#   endif
         _state = StateConnected;
 
         if(_traceLevels->network >= 1)
@@ -295,8 +355,9 @@ repeat:
     buf.i = buf.b.end();
     return true;
 }
+#endif
 
-#ifdef ICE_USE_IOCP
+#if defined(ICE_USE_IOCP) || defined(ICE_OS_WINRT)
 bool
 IceInternal::UdpTransceiver::startWrite(Buffer& buf)
 {
@@ -304,9 +365,162 @@ IceInternal::UdpTransceiver::startWrite(Buffer& buf)
 
     // The caller is supposed to check the send size before by calling checkSendSize
     assert(min(_maxPacketSize, _sndSize - _udpOverhead) >= static_cast<int>(buf.b.size()));
-
     assert(_fd != INVALID_SOCKET);
 
+#ifdef ICE_OS_WINRT
+    if(_state < StateConnected)
+    {
+        try
+        {
+            bool multicast = isMulticast(_addr) || isAddressValid(_mcastAddr);
+            if(!multicast)
+            {
+                auto operation = safe_cast<DatagramSocket^>(_fd)->ConnectAsync(_addr.host, _addr.port);
+                if(!checkIfErrorOrCompleted(SocketOperationConnect, operation))
+                {
+                    operation->Completed = ref new AsyncActionCompletedHandler(
+                        [=] (IAsyncAction^ info, Windows::Foundation::AsyncStatus status)
+                        {
+                            if(status != Windows::Foundation::AsyncStatus::Completed)
+                            {
+                                _write.count = SOCKET_ERROR;
+                                _write.error = info->ErrorCode.Value;
+                            }
+                            else
+                            {
+                                _write.count = 0;
+                                _writer = ref new DataWriter(safe_cast<DatagramSocket^>(_fd)->OutputStream);
+                            }
+                            _completedHandler(SocketOperationConnect);
+                        });
+                }
+                else
+                {
+                    _write.count = 0;
+                    _writer = ref new DataWriter(safe_cast<DatagramSocket^>(_fd)->OutputStream);
+                }
+            }
+            else
+            {
+                auto operation = safe_cast<DatagramSocket^>(_fd)->GetOutputStreamAsync(_addr.host, _addr.port);
+                if(!checkIfErrorOrCompleted(SocketOperationConnect, operation))
+                {
+                    operation->Completed = ref new AsyncOperationCompletedHandler<IOutputStream^>(
+                        [=] (IAsyncOperation<IOutputStream^>^ info, Windows::Foundation::AsyncStatus status)
+                        {
+                            if(status != Windows::Foundation::AsyncStatus::Completed)
+                            {
+                                _write.count = SOCKET_ERROR;
+                                _write.error = info->ErrorCode.Value;
+                            }
+                            else
+                            {
+                                _write.count = 0;
+                                _writer = ref new DataWriter(info->GetResults());
+                            }
+                            if(_mcastAddr.host != nullptr)
+                            {
+                                setMcastGroup(_fd, _mcastAddr, "");
+                            }
+                            _completedHandler(SocketOperationConnect);
+                        });
+                }
+                else
+                {
+                    _write.count = 0;
+                    _writer = ref new DataWriter(operation->GetResults());
+                    if(_mcastAddr.host != nullptr)
+                    {
+                        setMcastGroup(_fd, _mcastAddr, "");
+                    }
+                }
+            }
+        }
+        catch(Platform::Exception^ ex)
+        {
+            checkConnectErrorCode(__FILE__, __LINE__, ex->HResult, _addr.host);
+        }
+        return false;
+    }
+    else if(_state == StateNotConnected)
+    {
+        try
+        {
+            DatagramSocket^ fd = safe_cast<DatagramSocket^>(_fd);
+            concurrency::create_task(fd->GetOutputStreamAsync(_peerAddr.host, _peerAddr.port)).then(
+                [=,&buf](concurrency::task<IOutputStream^> task)
+                {
+                    try
+                    {
+                        DataWriter^ writer = ref new DataWriter(task.get());
+                        writer->WriteBytes(ref new Array<unsigned char>(&*buf.i, static_cast<int>(buf.b.size())));
+                        DataWriterStoreOperation^ operation = writer->StoreAsync();
+
+                        Windows::Foundation::AsyncStatus status = operation->Status;
+                        if(status == Windows::Foundation::AsyncStatus::Completed)
+                        {
+                            //
+                            // NOTE: unlike other methods, it's important to modify _write.count
+                            // _before_ calling checkIfErrorOrCompleted since this isn't called
+                            // with the connection mutex but from a Windows thread pool thread.
+                            // So we can't modify the _write structure after calling the 
+                            // completed callback.
+                            //
+                            _write.count = operation->GetResults();
+                            _completedHandler(SocketOperationWrite);
+                        }
+                        else if(status == Windows::Foundation::AsyncStatus::Started)
+                        {
+                            operation->Completed = _writeOperationCompletedHandler;
+                        }
+                        else
+                        {
+                            if(_state < StateConnected)
+                            {
+                                checkConnectErrorCode(__FILE__, __LINE__, operation->ErrorCode.Value, _addr.host);
+                            }
+                            else 
+                            {
+                                checkErrorCode(__FILE__, __LINE__, operation->ErrorCode.Value);
+                            }
+                        }
+                    }
+                    catch(Platform::Exception^ pex)
+                    {
+                        _write.count = SOCKET_ERROR;
+                        _write.error = pex->HResult;
+                        _completedHandler(SocketOperationWrite);
+                    }
+                });
+        }
+        catch(Platform::Exception^ ex)
+        {
+            checkErrorCode(__FILE__, __LINE__, ex->HResult);
+        }
+        return true;
+    }
+    else
+    {
+        try
+        {
+            _writer->WriteBytes(ref new Array<unsigned char>(&*buf.i, static_cast<int>(buf.b.size())));
+            DataWriterStoreOperation^ operation = _writer->StoreAsync();
+            if(checkIfErrorOrCompleted(SocketOperationWrite, operation))
+            {
+                _write.count = operation->GetResults();
+            }
+            else
+            {
+                operation->Completed = _writeOperationCompletedHandler;
+            }
+        }
+        catch(Platform::Exception^ ex)
+        {
+            checkErrorCode(__FILE__, __LINE__, ex->HResult);
+        }
+        return true;
+    }
+#else
     _write.buf.len = static_cast<int>(buf.b.size());
     _write.buf.buf = reinterpret_cast<char*>(&*buf.i);
     int err;
@@ -316,12 +530,12 @@ IceInternal::UdpTransceiver::startWrite(Buffer& buf)
     }
     else
     {
-        socklen_t len = static_cast<socklen_t>(sizeof(_peerAddr));
-        if(_peerAddr.ss_family == AF_INET)
+        socklen_t len = static_cast<socklen_t>(sizeof(sockaddr_storage));
+        if(_peerAddr.saStorage.ss_family == AF_INET)
         {
             len = sizeof(sockaddr_in);
         }
-        else if(_peerAddr.ss_family == AF_INET6)
+        else if(_peerAddr.saStorage.ss_family == AF_INET6)
         {
             len = sizeof(sockaddr_in6);
         }
@@ -332,7 +546,7 @@ IceInternal::UdpTransceiver::startWrite(Buffer& buf)
             ex.error = 0;
             throw ex;
         }
-        err = WSASendTo(_fd, &_write.buf, 1, &_write.count, 0, reinterpret_cast<struct sockaddr*>(&_peerAddr), 
+        err = WSASendTo(_fd, &_write.buf, 1, &_write.count, 0, &_peerAddr.sa, 
                         len, &_write, NULL);
     }
 
@@ -354,14 +568,21 @@ IceInternal::UdpTransceiver::startWrite(Buffer& buf)
             }
         }
     }
+#endif
     return true;
 }
 
 void
 IceInternal::UdpTransceiver::finishWrite(Buffer& buf)
 {
-    if(_write.count == SOCKET_ERROR)
+    if(_state < StateConnected)
     {
+        return;
+    }
+
+    if(static_cast<int>(_write.count) == SOCKET_ERROR)
+    {
+#ifndef ICE_OS_WINRT
         WSASetLastError(_write.error);
         if(connectionLost())
         {
@@ -375,6 +596,9 @@ IceInternal::UdpTransceiver::finishWrite(Buffer& buf)
             ex.error = getSocketErrno();
             throw ex;
         }
+#else
+        checkErrorCode(__FILE__, __LINE__, _write.error);
+#endif
     }
 
     if(_traceLevels->network >= 3)
@@ -388,7 +612,7 @@ IceInternal::UdpTransceiver::finishWrite(Buffer& buf)
         _stats->bytesSent(type(), static_cast<Int>(_write.count));
     }
 
-    assert(_write.count == static_cast<ssize_t>(buf.b.size()));
+    assert(_write.count == buf.b.size());
     buf.i = buf.b.end();
 }
 
@@ -398,9 +622,8 @@ IceInternal::UdpTransceiver::startRead(Buffer& buf)
     const int packetSize = min(_maxPacketSize, _rcvSize - _udpOverhead);
     buf.b.resize(packetSize);
     buf.i = buf.b.begin();
-
     assert(!buf.b.empty() && buf.i != buf.b.end());
-
+#ifndef ICE_OS_WINRT
     _read.buf.len = packetSize;
     _read.buf.buf = reinterpret_cast<char*>(&*buf.i);
     int err;
@@ -410,11 +633,11 @@ IceInternal::UdpTransceiver::startRead(Buffer& buf)
     }
     else
     {
-        memset(&_readAddr, 0, sizeof(struct sockaddr_storage));
-        _readAddrLen = static_cast<socklen_t>(sizeof(_readAddr));
+        memset(&_readAddr.saStorage, 0, sizeof(struct sockaddr_storage));
+        _readAddrLen = static_cast<socklen_t>(sizeof(sockaddr_storage));
         
         err = WSARecvFrom(_fd, &_read.buf, 1, &_read.count, &_read.flags, 
-                          reinterpret_cast<struct sockaddr*>(&_readAddr), &_readAddrLen, &_read, NULL);
+                          &_readAddr.sa, &_readAddrLen, &_read, NULL);
     }
 
     if(err == SOCKET_ERROR)
@@ -439,12 +662,55 @@ IceInternal::UdpTransceiver::startRead(Buffer& buf)
             }
         }
     }
+#else
+    IceUtil::Mutex::Lock lock(_mutex);
+    assert(!_readPending);
+    if(!_received.empty())
+    {
+        _completedHandler(SocketOperationRead);
+    }
+    else
+    {
+        _readPending = true;
+    }
+#endif
 }
 
 void
 IceInternal::UdpTransceiver::finishRead(Buffer& buf)
 {
-    if(_read.count == SOCKET_ERROR)
+#ifdef ICE_OS_WINRT
+    IceUtil::Mutex::Lock lock(_mutex);
+    assert(!_readPending && (!_received.empty() || _fd == INVALID_SOCKET));
+    if(_fd == INVALID_SOCKET)
+    {
+        return;
+    }
+
+    DatagramSocketMessageReceivedEventArgs^ args = _received.front();
+    _received.pop_front();
+
+    int ret;
+    try
+    {
+        DataReader^ reader = args->GetDataReader();
+        ret = min(static_cast<int>(reader->UnconsumedBufferLength), static_cast<int>(buf.b.size())); // Truncate received data if too large.
+        
+        Array<unsigned char>^ data = ref new Array<unsigned char>(ret);
+        reader->ReadBytes(data);
+        memcpy(&*buf.i, data->Data, ret);
+        if(_state == StateNotConnected)
+        {
+            _peerAddr.host = args->RemoteAddress;
+            _peerAddr.port = args->RemotePort;
+        }
+    }
+    catch(Platform::Exception^ ex)
+    {
+        checkErrorCode(__FILE__, __LINE__, ex->HResult);
+    }
+#else
+    if(static_cast<int>(_read.count) == SOCKET_ERROR)
     {
         WSASetLastError(_read.error);
 
@@ -477,18 +743,21 @@ IceInternal::UdpTransceiver::finishRead(Buffer& buf)
         _peerAddr = _readAddr;
     }
 
+    int ret = _read.count;
+#endif
+
     if(_traceLevels->network >= 3)
     {
         Trace out(_logger, _traceLevels->networkCat);
-        out << "received " << _read.count << " bytes via udp\n" << toString();
+        out << "received " << ret << " bytes via udp\n" << toString();
     }
 
     if(_stats)
     {
-        _stats->bytesReceived(type(), static_cast<Int>(_read.count));
+        _stats->bytesReceived(type(), static_cast<Int>(ret));
     }
 
-    buf.b.resize(_read.count);
+    buf.b.resize(ret);
     buf.i = buf.b.end();
 }
 #endif
@@ -510,20 +779,27 @@ IceInternal::UdpTransceiver::toString() const
     ostringstream s;
     if(_state == StateNotConnected)
     {
-        struct sockaddr_storage localAddr;
+        Address localAddr;
         fdToLocalAddress(_fd, localAddr);
         s << "local address = " << addrToString(localAddr);
-        if(_peerAddr.ss_family != AF_UNSPEC)
+        if(isAddressValid(_peerAddr))
         {
             s << "\nremote address = " << addrToString(_peerAddr);
         }
     }
     else
     {
+#ifndef ICE_OS_WINRT
         s << fdToString(_fd);
+#else
+        Address localAddr;
+        fdToLocalAddress(_fd, localAddr);
+        s << "local address = " << addrToString(localAddr);
+        s << "\nremote address = " << addrToString(_addr);
+#endif
     }
 
-    if(_mcastAddr.ss_family != AF_UNSPEC)
+    if(isAddressValid(_mcastAddr))
     {
         s << "\nmulticast address = " + addrToString(_mcastAddr);
     }
@@ -533,14 +809,27 @@ IceInternal::UdpTransceiver::toString() const
 Ice::ConnectionInfoPtr
 IceInternal::UdpTransceiver::getInfo() const
 {
-    assert(_fd != INVALID_SOCKET);
     Ice::UDPConnectionInfoPtr info = new Ice::UDPConnectionInfo();
+#if defined(ICE_OS_WINRT)
+    if(isMulticast(_addr) || isAddressValid(_mcastAddr))
+    {
+        info->remotePort = 0;
+        info->localPort = 0;
+        info->mcastPort = 0;
+        return info;
+    }
+#endif
+    if(_fd == INVALID_SOCKET)
+    {
+        return info;
+    }
+
     if(_state == StateNotConnected)
     {
-        struct sockaddr_storage localAddr;
+        Address localAddr;
         fdToLocalAddress(_fd, localAddr);
         addrToAddressAndPort(localAddr, info->localAddress, info->localPort);
-        if(_peerAddr.ss_family != AF_UNSPEC)
+        if(isAddressValid(_peerAddr))
         {
             addrToAddressAndPort(_peerAddr, info->remoteAddress, info->remotePort);
         }
@@ -554,7 +843,7 @@ IceInternal::UdpTransceiver::getInfo() const
         fdToAddressAndPort(_fd, info->localAddress, info->localPort, info->remoteAddress, info->remotePort);   
     }
 
-    if(_mcastAddr.ss_family != AF_UNSPEC)
+    if(isAddressValid(_mcastAddr))
     {
         addrToAddressAndPort(_mcastAddr, info->mcastAddress, info->mcastPort);
     }
@@ -590,26 +879,54 @@ IceInternal::UdpTransceiver::effectivePort() const
     return getPort(_addr);
 }
 
-IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, const struct sockaddr_storage& addr,
-                                            const string& mcastInterface, int mcastTtl) :
+
+IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, 
+                                            const Address& addr,
+#ifdef ICE_OS_WINRT
+                                            const string&,
+                                            int
+#else
+                                            const string& mcastInterface,
+                                            int mcastTtl
+#endif
+                                            ) :
     _traceLevels(instance->traceLevels()),
     _logger(instance->initializationData().logger),
     _stats(instance->initializationData().stats),
     _incoming(false),
     _addr(addr),
     _state(StateNeedConnect)
-#ifdef ICE_USE_IOCP
+#if defined(ICE_USE_IOCP)
     , _read(SocketOperationRead), 
     _write(SocketOperationWrite)
+#elif defined(ICE_OS_WINRT)
+    , _readPending(false)
 #endif
 {
-    _mcastAddr.ss_family = AF_UNSPEC; // AF_UNSPEC means not multicast.
-    _peerAddr.ss_family = AF_UNSPEC; // Not initialized yet.
-
-    _fd = createSocket(true, _addr.ss_family);
+    _fd = createSocket(true, _addr);
     setBufSize(instance);
     setBlock(_fd, false);
-    
+
+#ifndef ICE_OS_WINRT
+    _mcastAddr.saStorage.ss_family = AF_UNSPEC;
+    _peerAddr.saStorage.ss_family = AF_UNSPEC; // Not initialized yet.
+
+    //
+    // NOTE: setting the multicast interface before performing the
+    // connect is important for some OS such as OS X.
+    //
+    if(isMulticast(_addr))
+    {
+        if(mcastInterface.length() > 0)
+        {
+            setMcastInterface(_fd, mcastInterface, _addr);
+        }
+        if(mcastTtl != -1)
+        {
+            setMcastTtl(_fd, mcastTtl, _addr);
+        }
+    }
+
     //
     // In general, connecting a datagram socket should be non-blocking as this just setups 
     // the default destination address for the socket. However, on some OS, connect sometime
@@ -621,6 +938,15 @@ IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, const s
     {
         _state = StateConnected;
     }
+#else
+    DatagramSocket^ socket = safe_cast<DatagramSocket^>(_fd);
+    socket->MessageReceived += ref new TypedEventHandler<DatagramSocket^, DatagramSocketMessageReceivedEventArgs^>(
+        [=](DatagramSocket^ fd, DatagramSocketMessageReceivedEventArgs^ args)
+        {
+            this->appendMessage(args);
+        });
+#endif
+
 
 #ifdef ICE_USE_IOCP
     //
@@ -631,18 +957,6 @@ IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, const s
     //
     assert(_state == StateConnected);
 #endif
-    
-    if(isMulticast(_addr))
-    {
-        if(mcastInterface.length() > 0)
-        {
-            setMcastInterface(_fd, mcastInterface, _addr.ss_family == AF_INET);
-        }
-        if(mcastTtl != -1)
-        {
-            setMcastTtl(_fd, mcastTtl, _addr.ss_family == AF_INET);
-        }
-    }
 }
 
 IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, const string& host, int port,
@@ -651,27 +965,39 @@ IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, const s
     _logger(instance->initializationData().logger),
     _stats(instance->initializationData().stats),
     _incoming(true),
-    _addr(getAddressForServer(host, port, instance->protocolSupport())),
+    _addr(getAddressForServer(host, port, instance->protocolSupport(), instance->preferIPv6())),
     _state(connect ? StateNeedConnect : StateNotConnected)
-#ifdef ICE_USE_IOCP
+#ifdef ICE_OS_WINRT
+    , _readPending(false)
+#elif defined(ICE_USE_IOCP)
     , _read(SocketOperationRead), 
     _write(SocketOperationWrite)
 #endif
 {
-    memset(&_mcastAddr, 0, sizeof(_mcastAddr));
-    memset(&_peerAddr, 0, sizeof(_peerAddr));
-
-    _fd = createSocket(true, _addr.ss_family);
+    _fd = createServerSocket(true, _addr, instance->protocolSupport());
     setBufSize(instance);
     setBlock(_fd, false);
+
+#ifndef ICE_OS_WINRT
+    memset(&_mcastAddr.saStorage, 0, sizeof(sockaddr_storage));
+    memset(&_peerAddr.saStorage, 0, sizeof(sockaddr_storage));
+    _peerAddr.saStorage.ss_family = AF_UNSPEC;
+    _mcastAddr.saStorage.ss_family = AF_UNSPEC;
+#else
+    DatagramSocket^ socket = safe_cast<DatagramSocket^>(_fd);
+    socket->MessageReceived += ref new TypedEventHandler<DatagramSocket^, DatagramSocketMessageReceivedEventArgs^>(
+        [=](DatagramSocket^ fd, DatagramSocketMessageReceivedEventArgs^ args)
+        {
+            this->appendMessage(args);
+        });
+#endif
+
     if(_traceLevels->network >= 2)
     {
         Trace out(_logger, _traceLevels->networkCat);
         out << "attempting to bind to udp socket " << addrToString(_addr);
     }
 
-    _peerAddr.ss_family = AF_UNSPEC; // Not assigned yet.
-
     if(isMulticast(_addr))
     {
         setReuseAddress(_fd, true);
@@ -685,11 +1011,10 @@ IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, const s
         // address won't be the multicast address and the client will
         // therefore reject the datagram.
         //
-        const_cast<struct sockaddr_storage&>(_addr) = 
-            getAddressForServer("", port, _mcastAddr.ss_family == AF_INET ? EnableIPv4 : EnableIPv6);
+        const_cast<Address&>(_addr) = getAddressForServer("", port, getProtocolSupport(_addr), false);
 #endif
 
-        const_cast<struct sockaddr_storage&>(_addr) = doBind(_fd, _addr);
+        const_cast<Address&>(_addr) = doBind(_fd, _addr);
         if(getPort(_mcastAddr) == 0)
         {
             setPort(_mcastAddr, getPort(_addr));
@@ -713,16 +1038,14 @@ IceInternal::UdpTransceiver::UdpTransceiver(const InstancePtr& instance, const s
         //
         setReuseAddress(_fd, true);
 #endif
-        const_cast<struct sockaddr_storage&>(_addr) = doBind(_fd, _addr);
-
-        _mcastAddr.ss_family = AF_UNSPEC; // AF_UNSPEC means not multicast.
+        const_cast<Address&>(_addr) = doBind(_fd, _addr);
     }
 
     if(_traceLevels->network >= 1)
     {
         Trace out(_logger, _traceLevels->networkCat);
         out << "starting to receive udp packets\n" << toString();
-        
+
         vector<string> interfaces = 
             getHostsForEndpointExpand(inetAddrToString(_addr), instance->protocolSupport(), true);
         if(!interfaces.empty())
@@ -741,7 +1064,6 @@ IceInternal::UdpTransceiver::~UdpTransceiver()
 //
 // Set UDP receive and send buffer sizes.
 //
-
 void
 IceInternal::UdpTransceiver::setBufSize(const InstancePtr& instance)
 {
@@ -759,7 +1081,6 @@ IceInternal::UdpTransceiver::setBufSize(const InstancePtr& instance)
             prop = "Ice.UDP.RcvSize";
             addr = &_rcvSize;
             dfltSize = getRecvBufferSize(_fd);
-            _rcvSize = dfltSize;
         }
         else
         {
@@ -767,9 +1088,14 @@ IceInternal::UdpTransceiver::setBufSize(const InstancePtr& instance)
             prop = "Ice.UDP.SndSize";
             addr = &_sndSize;
             dfltSize = getSendBufferSize(_fd);
-            _sndSize = dfltSize;
         }
 
+        if(dfltSize <= 0)
+        {
+            dfltSize = _maxPacketSize;
+        }
+        *addr = dfltSize;
+
         //
         // Get property for buffer size and check for sanity.
         //
@@ -802,7 +1128,11 @@ IceInternal::UdpTransceiver::setBufSize(const InstancePtr& instance)
             //
             // Warn if the size that was set is less than the requested size.
             //
-            if(*addr < sizeRequested)
+            if(*addr == 0) // set buffer size not supported.
+            {
+                *addr = sizeRequested;
+            }
+            else if(*addr < sizeRequested)
             {
                 Warning out(_logger);
                 out << "UDP " << direction << " buffer size: requested size of "
@@ -812,6 +1142,68 @@ IceInternal::UdpTransceiver::setBufSize(const InstancePtr& instance)
     }
 }
 
+#ifdef ICE_OS_WINRT
+bool
+IceInternal::UdpTransceiver::checkIfErrorOrCompleted(SocketOperation op, IAsyncInfo^ info)
+{
+    //
+    // NOTE: It's important to only check for info->Status once as it
+    // might change during the checks below (the Status can be changed
+    // by the Windows thread pool concurrently).
+    //
+    // We consider that a canceled async status is the same as an
+    // error. A canceled async status can occur if there's a timeout
+    // and the socket is closed.
+    //
+    Windows::Foundation::AsyncStatus status = info->Status;
+    if(status == Windows::Foundation::AsyncStatus::Completed)
+    {
+        _completedHandler(op);
+        return true;
+    }
+    else if(status == Windows::Foundation::AsyncStatus::Started)
+    {
+        return false;
+    }
+    else
+    {
+        if(_state < StateConnected)
+        {
+            checkConnectErrorCode(__FILE__, __LINE__, info->ErrorCode.Value, _addr.host);
+        }
+        else 
+        {
+            checkErrorCode(__FILE__, __LINE__, info->ErrorCode.Value);
+        }
+        return true; // Prevent compiler warning.
+    }
+}
+
+void
+IceInternal::UdpTransceiver::appendMessage(DatagramSocketMessageReceivedEventArgs^ args)
+{
+    IceUtil::Mutex::Lock lock(_mutex);
+    if(_fd == INVALID_SOCKET) // Transceiver was closed.
+    {
+        return;
+    }
+    _received.push_back(args);
+
+    //
+    // If the transceiver is waiting to read a message, notify the
+    // selector that the transceiver is ready for read. This will in
+    // turn cause finishRead() to be called. If the transceiver is
+    // ready for read, we just queue the message. It will be picked
+    // up by startRead() when it is invoked.
+    //
+    if(_readPending)
+    {
+        _completedHandler(SocketOperationRead);
+        _readPending = false;
+    }
+}
+#endif
+
 //
 // The maximum IP datagram size is 65535. Subtract 20 bytes for the IP header and 8 bytes for the UDP header
 // to get the maximum payload.
