@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,6 +12,11 @@
 #include <Ice/Ice.h>
 #include <Ice/DynamicLibrary.h>
 #include <Ice/SliceChecksums.h>
+#include <Ice/Initialize.h>
+#include <Ice/Instance.h>
+#include <Ice/PropertiesAdminI.h>
+#include <Ice/MetricsAdminI.h>
+#include <Ice/InstrumentationI.h>
 #include <IceBox/ServiceManagerI.h>
 
 using namespace Ice;
@@ -20,62 +25,10 @@ using namespace std;
 
 typedef IceBox::Service* (*SERVICE_FACTORY)(CommunicatorPtr);
 
-namespace 
+
+namespace
 {
 
-template<class T>
-class AMICallbackT : public T
-{
-public:
-    
-    AMICallbackT(const ServiceManagerIPtr& serviceManager, const ServiceObserverPrx& observer) :
-        _serviceManager(serviceManager),
-        _observer(observer)
-    {
-    }
-    
-    virtual void ice_response()
-    {
-        // ok, success
-    }
-    
-    virtual void ice_exception(const Ice::Exception& ex)
-    {
-        //
-        // Drop this observer
-        //
-        _serviceManager->removeObserver(_observer, ex);
-    }
-    
-private:
-    ServiceManagerIPtr _serviceManager;
-    ServiceObserverPrx _observer;
-};
-
-class PropertiesAdminI : public PropertiesAdmin
-{
-public:
-    
-    PropertiesAdminI(const PropertiesPtr& properties) :
-        _properties(properties)
-    {
-    }
-    
-    virtual string getProperty(const string& name, const Current&)
-    {
-        return _properties->getProperty(name);
-    }
-
-    virtual PropertyDict getPropertiesForPrefix(const string& prefix, const Current&)
-    {
-        return _properties->getPropertiesForPrefix(prefix);
-    }
-    
-private:
-
-    const PropertiesPtr _properties; 
-};
-
 struct StartServiceInfo
 {
     StartServiceInfo(const std::string& service, const std::string& value, const Ice::StringSeq& serverArgs)
@@ -83,28 +36,28 @@ struct StartServiceInfo
         name = service;
 
         //
-        // Separate the entry point from the arguments.
+        // Split the entire property value into arguments. An entry point containing spaces
+        // must be enclosed in quotes.
         //
-        string::size_type pos = value.find_first_of(" \t\n");
-        if(pos == string::npos)
+        try
         {
-            entryPoint = value;
+            args = IceUtilInternal::Options::split(value);
         }
-        else
+        catch(const IceUtilInternal::BadOptException& ex)
         {
-            entryPoint = value.substr(0, pos);
-            try
-            {
-                args = IceUtilInternal::Options::split(value.substr(pos + 1));
-            }
-            catch(const IceUtilInternal::BadOptException& ex)
-            {
-                FailureException e(__FILE__, __LINE__);
-                e.reason = "ServiceManager: invalid arguments for service `" + name + "':\n" + ex.reason;
-                throw e;        
-            }
+            FailureException e(__FILE__, __LINE__);
+            e.reason = "ServiceManager: invalid arguments for service `" + name + "':\n" + ex.reason;
+            throw e;
         }
 
+        assert(!args.empty());
+
+        //
+        // Shift the arguments.
+        //
+        entryPoint = args[0];
+        args.erase(args.begin());
+
         for(Ice::StringSeq::const_iterator p = serverArgs.begin(); p != serverArgs.end(); ++p)
         {
             if(p->find("--" + name + ".") == 0)
@@ -119,13 +72,13 @@ struct StartServiceInfo
     Ice::StringSeq args;
 };
 
-
 }
 
-IceBox::ServiceManagerI::ServiceManagerI(CommunicatorPtr communicator, int& argc, char* argv[]) : 
+IceBox::ServiceManagerI::ServiceManagerI(CommunicatorPtr communicator, int& argc, char* argv[]) :
     _communicator(communicator),
     _pendingStatusChanges(false),
-    _traceServiceObserver(0)
+    _traceServiceObserver(0),
+    _observerCompletedCB(newCallback(this, &ServiceManagerI::observerCompleted))
 { 
     _logger = _communicator->getLogger();
     _traceServiceObserver = _communicator->getProperties()->getPropertyAsInt("IceBox.Trace.ServiceObserver");
@@ -184,7 +137,7 @@ IceBox::ServiceManagerI::startService(const string& name, const Current&)
         info.service->start(name, info.communicator == 0 ? _sharedCommunicator : info.communicator, info.args);
         started = true;
     }
-    catch(const Ice::Exception& ex)
+    catch(const Exception& ex)
     {
         Warning out(_logger);
         out << "ServiceManager: exception in start for service " << info.name << ":\n";
@@ -195,12 +148,10 @@ IceBox::ServiceManagerI::startService(const string& name, const Current&)
         Warning out(_logger);
         out << "ServiceManager: unknown exception in start for service " << info.name;
     }
-    
+
     {
         IceUtil::Monitor<IceUtil::Mutex>::Lock lock(*this);
-
-        vector<ServiceInfo>::iterator p;
-        for(p = _services.begin(); p != _services.end(); ++p)
+        for(vector<ServiceInfo>::iterator p = _services.begin(); p != _services.end(); ++p)
         {
             if(p->name == name)
             {
@@ -262,7 +213,7 @@ IceBox::ServiceManagerI::stopService(const string& name, const Current&)
         info.service->stop();
         stopped = true;
     }
-    catch(const Ice::Exception& ex)
+    catch(const Exception& ex)
     {
         Warning out(_logger);
         out << "ServiceManager: exception while stopping service " << info.name << ":\n";
@@ -276,9 +227,7 @@ IceBox::ServiceManagerI::stopService(const string& name, const Current&)
 
     {
         IceUtil::Monitor<IceUtil::Mutex>::Lock lock(*this);
-
-        vector<ServiceInfo>::iterator p;
-        for(p = _services.begin(); p != _services.end(); ++p)
+        for(vector<ServiceInfo>::iterator p = _services.begin(); p != _services.end(); ++p)
         {
             if(p->name == name)
             {
@@ -303,7 +252,7 @@ IceBox::ServiceManagerI::stopService(const string& name, const Current&)
 }
 
 void
-IceBox::ServiceManagerI::addObserver(const ServiceObserverPrx& observer, const Ice::Current&)
+IceBox::ServiceManagerI::addObserver(const ServiceObserverPrx& observer, const Current&)
 {
     //
     // Null observers and duplicate registrations are ignored
@@ -316,7 +265,7 @@ IceBox::ServiceManagerI::addObserver(const ServiceObserverPrx& observer, const I
         {
             Trace out(_logger, "IceBox.ServiceObserver");
             out << "Added service observer " << _communicator->proxyToString(observer);
-        } 
+        }
 
         vector<string> activeServices;
         for(vector<ServiceInfo>::iterator p = _services.begin(); p != _services.end(); ++p)
@@ -327,34 +276,14 @@ IceBox::ServiceManagerI::addObserver(const ServiceObserverPrx& observer, const I
                 activeServices.push_back(info.name);
             }
         }
-       
+
         if(activeServices.size() > 0)
         {
-            observer->servicesStarted_async(new AMICallbackT<AMI_ServiceObserver_servicesStarted>(this, observer),
-                                            activeServices);
+            observer->begin_servicesStarted(activeServices, _observerCompletedCB);
         }
     }
 }
 
-void
-IceBox::ServiceManagerI::removeObserver(const ServiceObserverPrx& observer, const Ice::Exception& ex)
-{
-    IceUtil::Monitor<IceUtil::Mutex>::Lock lock(*this);
-
-    //
-    // It's possible to remove several times the same observer, e.g. multiple concurrent
-    // requests that fail
-    //
-    
-    set<ServiceObserverPrx>::iterator p = _observers.find(observer);
-    if(p != _observers.end())
-    {
-        ServiceObserverPrx observer = *p;
-        _observers.erase(p);
-        observerRemoved(observer, ex);
-    }
-} 
-
 void
 IceBox::ServiceManagerI::shutdown(const Current&)
 {
@@ -391,7 +320,7 @@ IceBox::ServiceManagerI::start()
         //
         // IceBox.Service.Foo=entry_point [args]
         //
-        // We parse the service properties specified in IceBox.LoadOrder 
+        // We parse the service properties specified in IceBox.LoadOrder
         // first, then the ones from remaining services.
         //
         const string prefix = "IceBox.Service.";
@@ -415,7 +344,7 @@ IceBox::ServiceManagerI::start()
         {
             servicesInfo.push_back(StartServiceInfo(p->first.substr(prefix.size()), p->second, _argv));
         }
-        
+
         //
         // Check if some services are using the shared communicator in which
         // case we create the shared communicator now with a property set which
@@ -462,14 +391,36 @@ IceBox::ServiceManagerI::start()
                 {
                     initData.properties->setProperty(r->first, r->second);
                 }
-            
+
                 //
-                // Parse <service>.* command line options (the Ice command line options 
+                // Parse <service>.* command line options (the Ice command line options
                 // were parsed by the createProperties above)
                 //
-                q->args = initData.properties->parseCommandLineOptions(q->name, q->args);                
+                q->args = initData.properties->parseCommandLineOptions(q->name, q->args);
             }
+
+            //
+            // If Ice metrics are enabled on the IceBox communicator, we also enable them on the 
+            // shared communicator.
+            // 
+            IceInternal::MetricsAdminIPtr metricsAdmin;
+            if(IceInternal::CommunicatorObserverIPtr::dynamicCast(_communicator->getObserver()))
+            {
+                metricsAdmin = new IceInternal::MetricsAdminI(initData.properties, getProcessLogger());
+                initData.observer = new IceInternal::CommunicatorObserverI(metricsAdmin);
+            }
+
             _sharedCommunicator = initialize(initData);
+
+            //
+            // Ensure the metrics admin plugin uses the same property set as the
+            // communicator. This is necessary to correctly deal with runtime 
+            // property updates.
+            //
+            if(metricsAdmin)
+            {
+                metricsAdmin->setProperties(_sharedCommunicator->getProperties());
+            }
         }
 
         //
@@ -505,18 +456,6 @@ IceBox::ServiceManagerI::start()
         try
         {
             _communicator->addAdminFacet(this, "IceBox.ServiceManager");
-
-            //
-            // Add a Properties facet for each service
-            // 
-            for(vector<ServiceInfo>::iterator r = _services.begin(); r != _services.end(); ++r)
-            {
-                const ServiceInfo& info = *r;
-                CommunicatorPtr communicator = info.communicator != 0 ? info.communicator : _sharedCommunicator;
-                _communicator->addAdminFacet(new PropertiesAdminI(communicator->getProperties()),
-                                             "IceBox.Service." + info.name + ".Properties");
-            }
-          
             _communicator->getAdmin();
         }
         catch(const ObjectAdapterDeactivatedException&)
@@ -554,6 +493,20 @@ IceBox::ServiceManagerI::start()
         stopAll();
         return false;
     }
+    catch(const std::exception& ex)
+    {
+        Error out(_logger);
+        out << "ServiceManager: " << ex.what();
+        stopAll();
+        return false;
+    }
+    catch(...)
+    {
+        Error out(_logger);
+        out << "ServiceManager: unknown exception";
+        stopAll();
+        return false;
+    }
 
     return true;
 }
@@ -569,15 +522,11 @@ IceBox::ServiceManagerI::start(const string& service, const string& entryPoint,
 {
     IceUtil::Monitor<IceUtil::Mutex>::Lock lock(*this);
 
-    ServiceInfo info;
-    info.name = service;
-    info.status = Stopped;
-    info.args = args;
-
     //
     // Load the entry point.
     //
-    IceInternal::DynamicLibraryPtr library = new IceInternal::DynamicLibrary();
+    IceInternal::DynamicLibraryPtr library = 
+        new IceInternal::DynamicLibrary(IceInternal::getInstance(_communicator)->initializationData().stringConverter);
     IceInternal::DynamicLibrary::symbol_type sym = library->loadEntryPoint(entryPoint, false);
     if(sym == 0)
     {
@@ -591,64 +540,41 @@ IceBox::ServiceManagerI::start(const string& service, const string& entryPoint,
         throw ex;
     }
 
-    //
-    // Invoke the factory function.
-    //
-    SERVICE_FACTORY factory = (SERVICE_FACTORY)sym;
-    try
-    {
-        info.service = factory(_communicator);
-    }
-    catch(const FailureException&)
-    {
-        throw;
-    }
-    catch(const Exception& ex)
-    {
-        ostringstream s;
-        s << "ServiceManager: exception in entry point `" + entryPoint + "' for service " << service << ":\n";
-        s << ex;
-
-        FailureException e(__FILE__, __LINE__);
-        e.reason = s.str();
-        throw e;
-    }
-    catch(...)
-    {
-        ostringstream s;
-        s << "ServiceManager: unknown exception in entry point `" + entryPoint + "' for service " << service;
-
-        FailureException e(__FILE__, __LINE__);
-        e.reason = s.str();
-        throw e;
-    }
+    ServiceInfo info;
+    info.name = service;
+    info.status = Stopped;
+    info.args = args;
 
     //
-    // Invoke Service::start().
+    // If Ice.UseSharedCommunicator.<name> is not defined, create
+    // a communicator for the service. The communicator inherits
+    // from the shared communicator properties. If it's defined
+    // add the service properties to the shared commnunicator
+    // property set.
     //
-    try
+    Ice::CommunicatorPtr communicator;
+    IceInternal::MetricsAdminIPtr metricsAdmin;
+    if(_communicator->getProperties()->getPropertyAsInt("IceBox.UseSharedCommunicator." + service) > 0)
     {
-        //
-        // If Ice.UseSharedCommunicator.<name> is not defined, create
-        // a communicator for the service. The communicator inherits
-        // from the shared communicator properties. If it's defined
-        // add the service properties to the shared commnunicator
-        // property set.
-        //
-        Ice::CommunicatorPtr communicator;
-        if(_communicator->getProperties()->getPropertyAsInt("IceBox.UseSharedCommunicator." + service) > 0)
+        assert(_sharedCommunicator);
+        communicator = _sharedCommunicator;
+        Ice::Instrumentation::CommunicatorObserverPtr o = communicator->getObserver();
+        if(o)
         {
-            assert(_sharedCommunicator);
-            communicator = _sharedCommunicator;
+            metricsAdmin = IceInternal::CommunicatorObserverIPtr::dynamicCast(o)->getMetricsAdmin();
         }
-        else
-        {       
+    }
+    else
+    {
+        try
+        {
             //
             // Create the service properties. We use the communicator properties as the default
             // properties if IceBox.InheritProperties is set.
             //
             InitializationData initData;
             initData.properties = createServiceProperties(service);
+
             if(!info.args.empty())
             {
                 //
@@ -656,108 +582,206 @@ IceBox::ServiceManagerI::start(const string& service, const string& entryPoint,
                 // read the service config file if it's specified with --Ice.Config.
                 //
                 initData.properties = createProperties(info.args, initData.properties);
-
+                
                 //
-                // Next, parse the service "<service>.*" command line options (the Ice command 
+                // Next, parse the service "<service>.*" command line options (the Ice command
                 // line options were parsed by the createProperties above)
                 //
                 info.args = initData.properties->parseCommandLineOptions(service, info.args);
             }
             
             //
-            // Clone the logger to assign a new prefix.
+            // Clone the logger to assign a new prefix. If one of the built-in loggers is configured
+            // don't set any logger.
             //
-            initData.logger = _logger->cloneWithPrefix(initData.properties->getProperty("Ice.ProgramName"));
-            
+            if(initData.properties->getProperty("Ice.LogFile").empty() 
+#ifndef _WIN32
+               && initData.properties->getPropertyAsInt("Ice.UseSyslog") == 0
+#endif
+               )
+            {
+                initData.logger = _logger->cloneWithPrefix(initData.properties->getProperty("Ice.ProgramName"));
+            }
+
             //
-            // Remaining command line options are passed to the communicator. This is 
+            // If Ice metrics are enabled on the IceBox communicator, we also enable them on
+            // the service communicator.
+            // 
+            if(IceInternal::CommunicatorObserverIPtr::dynamicCast(_communicator->getObserver()))
+            {
+                metricsAdmin = new IceInternal::MetricsAdminI(initData.properties, initData.logger);
+                initData.observer = new IceInternal::CommunicatorObserverI(metricsAdmin);
+            }
+
+            //
+            // Remaining command line options are passed to the communicator. This is
             // necessary for Ice plug-in properties (e.g.: IceSSL).
             //
             info.communicator = initialize(info.args, initData);
             communicator = info.communicator;
+
+            //
+            // Ensure the metrics admin plugin uses the same property set as the
+            // communicator. This is necessary to correctly deal with runtime 
+            // property updates.
+            //
+            if(metricsAdmin)
+            {
+                metricsAdmin->setProperties(communicator->getProperties());
+            }
         }
-        
+        catch(const Exception& ex)
+        {
+            LoggerOutputBase s;
+            s << "ServiceManager: exception while starting service " << service << ":\n";
+            s << ex;
+
+            FailureException e(__FILE__, __LINE__);
+            e.reason = s.str();
+            throw e;
+        }
+    }
+
+    try
+    {
+        //
+        // Add a PropertiesAdmin facet to the service manager's communicator that provides
+        // access to this service's property set. We do this prior to instantiating the
+        // service so that the service's constructor is able to access the facet (e.g.,
+        // in case it wants to set a callback).
+        //
+        string facetName = "IceBox.Service." + info.name + ".Properties";
+        PropertiesAdminIPtr propAdmin = 
+            new PropertiesAdminI(facetName, communicator->getProperties(), communicator->getLogger());
+        _communicator->addAdminFacet(propAdmin, facetName);
+
+        //
+        // If a metrics admin facet is setup for the service, register
+        // it with the IceBox communicator.
+        //
+        if(metricsAdmin)
+        {
+            _communicator->addAdminFacet(metricsAdmin, "IceBox.Service." + info.name + ".Metrics");
+
+            // Ensure the metrics admin facet is notified of property updates.
+            propAdmin->addUpdateCallback(metricsAdmin);
+        }
+
+        //
+        // Invoke the factory function.
+        //
+        SERVICE_FACTORY factory = (SERVICE_FACTORY)sym;
+        try
+        {
+            info.service = factory(_communicator);
+        }
+        catch(const FailureException&)
+        {
+            throw;
+        }
+        catch(const Exception& ex)
+        {
+            LoggerOutputBase s;
+            s << "ServiceManager: exception in entry point `" + entryPoint + "' for service " << info.name << ":\n";
+            s << ex;
+            
+            FailureException e(__FILE__, __LINE__);
+            e.reason = s.str();
+            throw e;
+        }
+        catch(...)
+        {
+            ostringstream s;
+            s << "ServiceManager: unknown exception in entry point `" + entryPoint + "' for service " << info.name;
+            
+            FailureException e(__FILE__, __LINE__);
+            e.reason = s.str();
+            throw e;
+        }
+            
         //
         // Start the service.
         //
         try
         {
-            info.service->start(service, communicator, info.args);
-            info.status = Started;
-
+            info.service->start(info.name, communicator, info.args);
+            
             //
             // There is no need to notify the observers since the 'start all'
-            // (that indirectly calls this function) occurs before the creation of 
-            // the Server Admin object, and before the activation of the main 
+            // (that indirectly calls this function) occurs before the creation of
+            // the Server Admin object, and before the activation of the main
             // object adapter (so before any observer can be registered)
             //
         }
+        catch(const FailureException&)
+        {
+            throw;
+        }
+        catch(const Exception& ex)
+        {
+            LoggerOutputBase s;
+            s << "ServiceManager: exception while starting service " << info.name << ":\n";
+            s << ex;
+
+            FailureException e(__FILE__, __LINE__);
+            e.reason = s.str();
+            throw e;
+        }
         catch(...)
         {
-            if(info.communicator)
-            {
-                try
-                {
-                    info.communicator->shutdown();
-                    info.communicator->waitForShutdown();
-                }
-                catch(const Ice::CommunicatorDestroyedException&)
-                {
-                    //
-                    // Ignore, the service might have already destroyed
-                    // the communicator for its own reasons.
-                    //
-                }
-                catch(const Ice::Exception& ex)
-                {
-                    Warning out(_logger);
-                    out << "ServiceManager: exception while shutting down communicator for service " << service
-                        << ":\n";
-                    out << ex;
-                }
+            ostringstream s;
+            s << "ServiceManager: unknown exception while starting service " << info.name;
 
-                try
-                {
-                    info.communicator->destroy();
-                    info.communicator = 0;
-                }
-                catch(const Exception& ex)
-                {
-                    Warning out(_logger);
-                    out << "ServiceManager: exception while destroying communicator for service " << service
-                        << ":\n";
-                    out << ex;
-                }
-            }
-            throw;
+            FailureException e(__FILE__, __LINE__);
+            e.reason = s.str();
+            throw e;
         }
 
         info.library = library;
+        info.status = Started;
         _services.push_back(info);
     }
-    catch(const FailureException&)
+    catch(const ObjectAdapterDeactivatedException&)
     {
+        //
+        // Can be raised by addAdminFacet if the service manager communicator has been shut down.
+        //
+        if(info.communicator)
+        {
+            destroyServiceCommunicator(info.name, info.communicator);
+        }
+    }
+    catch(const Exception&)
+    {
+        try
+        {
+            _communicator->removeAdminFacet("IceBox.Service." + info.name + ".Properties");
+        }
+        catch(const LocalException&)
+        {
+            // Ignored
+        }
+
+        try
+        {
+            Ice::ObjectPtr admin = _communicator->removeAdminFacet("IceBox.Service." + info.name + ".Metrics");
+            if(admin && communicator != _sharedCommunicator)
+            {
+                IceInternal::MetricsAdminIPtr::dynamicCast(admin)->destroy();
+            }
+        }
+        catch(const LocalException&)
+        {
+            // Ignored
+        }
+
+        if(info.communicator)
+        {
+            destroyServiceCommunicator(info.name, info.communicator);
+        }
+
         throw;
     }
-    catch(const Exception& ex)
-    {
-        ostringstream s;
-        s << "ServiceManager: exception while starting service " << service << ":\n";
-        s << ex;
-
-        FailureException e(__FILE__, __LINE__);
-        e.reason = s.str();
-        throw e;
-    }
-    catch(...)
-    {
-        ostringstream s;
-        s << "ServiceManager: unknown exception while starting service " << service;
-
-        FailureException e(__FILE__, __LINE__);
-        e.reason = s.str();
-        throw e;
-    }
 }
 
 void
@@ -776,15 +800,14 @@ IceBox::ServiceManagerI::stopAll()
     //
     // Services are stopped in the reverse order from which they are started.
     //
-    vector<ServiceInfo>::reverse_iterator p;
 
     vector<string> stoppedServices;
 
     //
-    // First, for each service, we call stop on the service and flush its database environment to 
+    // First, for each service, we call stop on the service and flush its database environment to
     // the disk.
     //
-    for(p = _services.rbegin(); p != _services.rend(); ++p)
+    for(vector<ServiceInfo>::reverse_iterator p = _services.rbegin(); p != _services.rend(); ++p)
     {
         ServiceInfo& info = *p;
 
@@ -796,7 +819,7 @@ IceBox::ServiceManagerI::stopAll()
                 info.status = Stopped;
                 stoppedServices.push_back(info.name);
             }
-            catch(const Ice::Exception& ex)
+            catch(const Exception& ex)
             {
                 Warning out(_logger);
                 out << "ServiceManager: exception while stopping service " << info.name << ":\n";
@@ -810,10 +833,10 @@ IceBox::ServiceManagerI::stopAll()
         }
     }
 
-    for(p = _services.rbegin(); p != _services.rend(); ++p)
+    for(vector<ServiceInfo>::reverse_iterator p = _services.rbegin(); p != _services.rend(); ++p)
     {
         ServiceInfo& info = *p;
-        
+
         try
         {
             _communicator->removeAdminFacet("IceBox.Service." + info.name + ".Properties");
@@ -822,6 +845,19 @@ IceBox::ServiceManagerI::stopAll()
         {
             // Ignored
         }
+        
+        try
+        {
+            Ice::ObjectPtr admin = _communicator->removeAdminFacet("IceBox.Service." + info.name + ".Metrics");
+            if(admin && info.communicator != _sharedCommunicator)
+            {
+                IceInternal::MetricsAdminIPtr::dynamicCast(admin)->destroy();
+            }
+        }
+        catch(const LocalException&)
+        {
+            // Ignored
+        }
 
         if(info.communicator)
         {
@@ -830,14 +866,14 @@ IceBox::ServiceManagerI::stopAll()
                 info.communicator->shutdown();
                 info.communicator->waitForShutdown();
             }
-            catch(const Ice::CommunicatorDestroyedException&)
+            catch(const CommunicatorDestroyedException&)
             {
                 //
                 // Ignore, the service might have already destroyed
                 // the communicator for its own reasons.
                 //
             }
-            catch(const Ice::Exception& ex)
+            catch(const Exception& ex)
             {
                 Warning out(_logger);
                 out << "ServiceManager: exception while stopping service " << info.name << ":\n";
@@ -846,9 +882,9 @@ IceBox::ServiceManagerI::stopAll()
         }
 
         //
-        // Release the service, the service communicator and then the library. The order is important, 
+        // Release the service, the service communicator and then the library. The order is important,
         // the service must be released before destroying the communicator so that the communicator
-        // leak detector doesn't report potential leaks, and the communicator must be destroyed before 
+        // leak detector doesn't report potential leaks, and the communicator must be destroyed before
         // the library is released since the library will destroy its global state.
         //
         try
@@ -881,7 +917,7 @@ IceBox::ServiceManagerI::stopAll()
                 out << ex;
             }
         }
-        
+
         try
         {
             info.library = 0;
@@ -901,6 +937,12 @@ IceBox::ServiceManagerI::stopAll()
 
     if(_sharedCommunicator)
     {
+        Ice::Instrumentation::CommunicatorObserverPtr o = _sharedCommunicator->getObserver();
+        if(o)
+        {
+            IceInternal::CommunicatorObserverIPtr::dynamicCast(o)->getMetricsAdmin()->destroy();
+        }
+
         try
         {
             _sharedCommunicator->destroy();
@@ -916,6 +958,8 @@ IceBox::ServiceManagerI::stopAll()
     _services.clear();
 
     servicesStopped(stoppedServices, _observers);
+
+    _observerCompletedCB = 0; // Break cyclic reference count.
 }
 
 void
@@ -925,9 +969,7 @@ IceBox::ServiceManagerI::servicesStarted(const vector<string>& services, const s
     {
         for(set<ServiceObserverPrx>::const_iterator p = observers.begin(); p != observers.end(); ++p)
         {
-            ServiceObserverPrx observer = *p;
-            observer->servicesStarted_async(new AMICallbackT<AMI_ServiceObserver_servicesStarted>(this, observer),
-                                            services);
+            (*p)->begin_servicesStarted(services, _observerCompletedCB);
         }
     }
 }
@@ -939,16 +981,14 @@ IceBox::ServiceManagerI::servicesStopped(const vector<string>& services, const s
     {
         for(set<ServiceObserverPrx>::const_iterator p = observers.begin(); p != observers.end(); ++p)
         {
-            ServiceObserverPrx observer = *p;
-            observer->servicesStopped_async(new AMICallbackT<AMI_ServiceObserver_servicesStopped>(this, observer),
-                                            services);
+            (*p)->begin_servicesStopped(services, _observerCompletedCB);
         }
     }
 }
 
 void
 IceBox::ServiceManagerI::observerRemoved(const ServiceObserverPrx& observer, const std::exception& ex)
-{ 
+{
     if(_traceServiceObserver >= 1)
     {
         //
@@ -962,8 +1002,8 @@ IceBox::ServiceManagerI::observerRemoved(const ServiceObserverPrx& observer, con
             out << "Removed service observer " << _communicator->proxyToString(observer)
                 << "\nafter catching " << ex.what();
         }
-    } 
-} 
+    }
+}
 
 Ice::PropertiesPtr
 IceBox::ServiceManagerI::createServiceProperties(const string& service)
@@ -979,7 +1019,7 @@ IceBox::ServiceManagerI::createServiceProperties(const string& service)
     {
         properties = createProperties();
     }
-    
+
     string programName = communicatorProperties->getProperty("Ice.ProgramName");
     if(programName.empty())
     {
@@ -991,3 +1031,63 @@ IceBox::ServiceManagerI::createServiceProperties(const string& service)
     }
     return properties;
 }
+
+void
+ServiceManagerI::observerCompleted(const Ice::AsyncResultPtr& result)
+{
+     try 
+     {
+         result->throwLocalException();
+     }
+     catch(const Ice::LocalException& ex)
+     {
+        ServiceObserverPrx observer = ServiceObserverPrx::uncheckedCast(result->getProxy());
+        IceUtil::Monitor<IceUtil::Mutex>::Lock lock(*this);
+
+        //
+        // It's possible to remove several times the same observer, e.g. multiple concurrent
+        // requests that fail
+        //        
+        set<ServiceObserverPrx>::iterator p = _observers.find(observer);
+        if(p != _observers.end())
+        {
+            ServiceObserverPrx observer = *p;
+            _observers.erase(p);
+            observerRemoved(observer, ex);
+        }
+     }
+}
+
+void
+IceBox::ServiceManagerI::destroyServiceCommunicator(const string& service, const CommunicatorPtr& communicator)
+{
+    try
+    {
+        communicator->shutdown();
+        communicator->waitForShutdown();
+    }
+    catch(const CommunicatorDestroyedException&)
+    {
+        //
+        // Ignore, the service might have already destroyed
+        // the communicator for its own reasons.
+        //
+    }
+    catch(const Exception& ex)
+    {
+        Warning out(_logger);
+        out << "ServiceManager: exception in shutting down communicator for service " << service << ":\n";
+        out << ex;
+    }
+
+    try
+    {
+        communicator->destroy();
+    }
+    catch(const Exception& ex)
+    {
+        Warning out(_logger);
+        out << "ServiceManager: exception in shutting down communicator for service " << service << ":\n";
+        out << ex;
+    }
+}
