@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -79,7 +79,7 @@ private:
     IceUtil::TimerPtr _timer;
     RegistryIPtr _registry;
     NodeIPtr _node;
-    NodeSessionManager _sessions;
+    IceUtil::UniquePtr<NodeSessionManager> _sessions;
     Ice::ObjectAdapterPtr _adapter;
 };
 
@@ -87,7 +87,7 @@ class CollocatedRegistry : public RegistryI
 {
 public:
 
-    CollocatedRegistry(const CommunicatorPtr&, const ActivatorPtr&, bool, bool);
+    CollocatedRegistry(const CommunicatorPtr&, const ActivatorPtr&, bool, bool, const std::string&);
     virtual void shutdown();
 
 private:
@@ -105,7 +105,7 @@ public:
     {
     }
 
-    virtual ObjectPtr locate(const Current& c, LocalObjectPtr&)
+    virtual ObjectPtr locate(const Current&, LocalObjectPtr&)
     {
         return _servant;
     }
@@ -151,8 +151,9 @@ setNoIndexingAttribute(const string& pa)
 CollocatedRegistry::CollocatedRegistry(const CommunicatorPtr& com, 
                                        const ActivatorPtr& activator, 
                                        bool nowarn,
-                                       bool readonly) :
-    RegistryI(com, new TraceLevels(com, "IceGrid.Registry"), nowarn, readonly), 
+                                       bool readonly,
+                                       const string& initFromReplica) :
+    RegistryI(com, new TraceLevels(com, "IceGrid.Registry"), nowarn, readonly, initFromReplica), 
     _activator(activator)
 {
 }
@@ -195,7 +196,10 @@ NodeService::shutdown()
 {
     assert(_activator);
     _activator->shutdown();
-    _sessions.terminate(); // Unblock the main thread if it's blocked on waitForCreate()
+    if(_sessions.get())
+    {
+        _sessions->terminate(); // Unblock the main thread if it's blocked on waitForCreate()
+    }
     return true;
 }
 
@@ -223,8 +227,11 @@ NodeService::startImpl(int argc, char* argv[], int& status)
 {
     bool nowarn = false;
     bool readonly = false;
+    string initFromReplica;
     string desc;
     vector<string> targets;
+
+
     for(int i = 1; i < argc; ++i)
     {
         if(strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
@@ -247,6 +254,17 @@ NodeService::startImpl(int argc, char* argv[], int& status)
         {
             readonly = true;
         }
+        else if(strcmp(argv[i], "--initdb-from-replica") == 0)
+        {
+            if(i + 1 >= argc)
+            {
+                error("missing replica argument for option `" + string(argv[i]) + "'");
+                usage(argv[0]);
+                return false;
+            }
+
+            initFromReplica = argv[++i];
+        } 
         else if(strcmp(argv[i], "--deploy") == 0)
         {
             if(i + 1 >= argc)
@@ -307,7 +325,7 @@ NodeService::startImpl(int argc, char* argv[], int& status)
     //
     if(properties->getPropertyAsInt("IceGrid.Node.CollocateRegistry") > 0)
     {
-        _registry = new CollocatedRegistry(communicator(), _activator, nowarn, readonly);
+        _registry = new CollocatedRegistry(communicator(), _activator, nowarn, readonly, initFromReplica);
         if(!_registry->start())
         {
             return false;
@@ -325,9 +343,9 @@ NodeService::startImpl(int argc, char* argv[], int& status)
             locatorId.category = properties->getPropertyWithDefault("IceGrid.InstanceName", "IceGrid");
             locatorId.name = "Locator";
             string endpoints = properties->getProperty("IceGrid.Registry.Client.Endpoints");
-            string locatorPrx = "\"" + communicator()->identityToString(locatorId) + "\" :" + endpoints;
-            communicator()->setDefaultLocator(Ice::LocatorPrx::uncheckedCast(communicator()->stringToProxy(locatorPrx)));
-            properties->setProperty("Ice.Default.Locator", locatorPrx);
+            string locPrx = "\"" + communicator()->identityToString(locatorId) + "\" :" + endpoints;
+            communicator()->setDefaultLocator(Ice::LocatorPrx::uncheckedCast(communicator()->stringToProxy(locPrx)));
+            properties->setProperty("Ice.Default.Locator", locPrx);
         }
     }
     else if(properties->getProperty("Ice.Default.Locator").empty())
@@ -350,12 +368,12 @@ NodeService::startImpl(int argc, char* argv[], int& status)
     {
         if(!IceUtilInternal::directoryExists(dataPath))
         {
-            ostringstream os;
             FileException ex(__FILE__, __LINE__);
             ex.path = dataPath;
             ex.error = IceInternal::getSystemErrno();
-            os << ex;
-            error("property `IceGrid.Node.Data' is set to an invalid path:\n" + os.str());
+      
+            ServiceError err(this);
+            err << "property `IceGrid.Node.Data' is set to an invalid path:\n" << ex;
             return false;
         }
 
@@ -434,11 +452,10 @@ NodeService::startImpl(int argc, char* argv[], int& status)
         {
             mapper = UserAccountMapperPrx::uncheckedCast(communicator()->propertyToProxy(mapperProperty));
         }
-        catch(const Ice::LocalException& ex)
+        catch(const std::exception& ex)
         {
-            ostringstream os;
-            os << "user account mapper `" << mapperProperty << "' is invalid:\n" << ex;
-            error(os.str());
+            ServiceError err(this);
+            err << "user account mapper `" << mapperProperty << "' is invalid:\n" << ex;
             return false;
         }
     }
@@ -471,6 +488,8 @@ NodeService::startImpl(int argc, char* argv[], int& status)
     //
     const string instanceName = communicator()->getDefaultLocator()->ice_getIdentity().category;
 
+    _sessions.reset(new NodeSessionManager(communicator()));
+
     //
     // Create the server factory. The server factory creates persistent objects
     // for the server and server adapter. It also takes care of installing the
@@ -478,10 +497,11 @@ NodeService::startImpl(int argc, char* argv[], int& status)
     //
     Identity id = communicator()->stringToIdentity(instanceName + "/Node-" + name);
     NodePrx nodeProxy = NodePrx::uncheckedCast(_adapter->createProxy(id));
-    _node = new NodeI(_adapter, _sessions, _activator, _timer, traceLevels, nodeProxy, name, mapper);
+    _node = new NodeI(_adapter, *_sessions, _activator, _timer, traceLevels, nodeProxy, name, mapper);
     _adapter->add(_node, nodeProxy->ice_getIdentity());
 
-    _adapter->addServantLocator(new DefaultServantLocator(new NodeServerAdminRouter(_node)), _node->getServerAdminCategory());
+    _adapter->addServantLocator(new DefaultServantLocator(new NodeServerAdminRouter(_node)), 
+                                _node->getServerAdminCategory());
 
     //
     // Start the platform info thread if needed.
@@ -509,7 +529,7 @@ NodeService::startImpl(int argc, char* argv[], int& status)
     //
     // Create the node sessions with the registries.
     //
-    _sessions.create(_node);
+    _sessions->create(_node);
 
     //
     // In some tests, we deploy icegridnodes using IceGrid:
@@ -548,13 +568,13 @@ NodeService::startImpl(int argc, char* argv[], int& status)
     // Notify the node session manager that the node can start
     // accepting incoming connections.
     //
-    _sessions.activate();
+    _sessions->activate();
 
     string bundleName = properties->getProperty("IceGrid.Node.PrintServersReady");
     if(!bundleName.empty() || !desc.empty())
     {
         enableInterrupt();
-        if(!_sessions.waitForCreate())
+        if(!_sessions->waitForCreate())
         {
             //
             // Create was interrupted, return true as if the service was
@@ -572,12 +592,11 @@ NodeService::startImpl(int argc, char* argv[], int& status)
     {
         try
         {
-            Ice::Identity registryId;
-            registryId.category = instanceName;
-            registryId.name = "Registry";
+            Ice::Identity regId;
+            regId.category = instanceName;
+            regId.name = "Registry";
             
-            RegistryPrx registry = RegistryPrx::checkedCast(
-                communicator()->stringToProxy("\"" + communicator()->identityToString(registryId) + "\""));
+            RegistryPrx registry = RegistryPrx::checkedCast(communicator()->getDefaultLocator()->findObjectById(regId));
             if(!registry)
             {
                 throw "invalid registry";
@@ -627,28 +646,24 @@ NodeService::startImpl(int argc, char* argv[], int& status)
         }
         catch(const DeploymentException& ex)
         {
-            ostringstream ostr;
-            ostr << "failed to deploy application `" << desc << "':\n" << ex << ": " << ex.reason;
-            warning(ostr.str());
+            ServiceWarning warn(this);
+            warn << "failed to deploy application `" << desc << "':\n" << ex;
         }
         catch(const AccessDeniedException& ex)
         {
-            ostringstream ostr;
-            ostr << "failed to deploy application `" << desc << "':\n" 
+            ServiceWarning warn(this);
+            warn << "failed to deploy application `" << desc << "':\n" 
                  << "registry database is locked by `" << ex.lockUserId << "'";
-            warning(ostr.str());
         }
         catch(const std::exception& ex)
         {
-            ostringstream ostr;
-            ostr << "failed to deploy application `" << desc << "':\n" << ex.what();
-            warning(ostr.str());
+            ServiceWarning warn(this);
+            warn << "failed to deploy application `" << desc << "':\n" << ex;
         }
         catch(const string& reason)
         {
-            ostringstream ostr;
-            ostr << "failed to deploy application `" << desc << "':\n" << reason;
-            warning(ostr.str());
+            ServiceWarning warn(this);
+            warn << "failed to deploy application `" << desc << "':\n" << reason;
         }
     }
 
@@ -716,18 +731,20 @@ NodeService::stop()
             _adapter->deactivate();
             _adapter = 0;
         }
-        catch(const Ice::LocalException& ex)
+        catch(const std::exception& ex)
         {
-            ostringstream ostr;
-            ostr << "unexpected exception while shutting down node:\n" << ex;
-            warning(ostr.str());
+            ServiceWarning warn(this);
+            warn << "unexpected exception while shutting down node:\n" << ex;
         }
     }
 
     //
     // Terminate the node sessions with the registries.
     //
-    _sessions.destroy();
+    if(_sessions.get())
+    {
+        _sessions->destroy();
+    }
 
     //
     // Stop the platform info thread.
@@ -745,11 +762,10 @@ NodeService::stop()
         communicator()->shutdown();
         communicator()->waitForShutdown();
     }
-    catch(const Ice::LocalException& ex)
+    catch(const std::exception& ex)
     {
-        ostringstream ostr;
-        ostr << "unexpected exception while shutting down node:\n" << ex;
-        warning(ostr.str());
+        ServiceWarning warn(this);
+        warn << "unexpected exception while shutting down node:\n" << ex;
     }
 
     //
@@ -815,8 +831,10 @@ NodeService::usage(const string& appName)
         "-h, --help           Show this message.\n"
         "-v, --version        Display the Ice version.\n"
         "--nowarn             Don't print any security warnings.\n"
-        "--readonly           Start the collocated master registry in read-only mode."
-        "\n"
+        "--readonly           Start the collocated master registry in read-only mode.\n"
+        "--initdb-from-replica=<replica>\n"
+        "                     Initialize the collocated registry database from the\n"
+        "                     given replica.\n"
         "--deploy DESCRIPTOR [TARGET1 [TARGET2 ...]]\n"
         "                     Add or update descriptor in file DESCRIPTOR, with\n"
         "                     optional targets.\n";
@@ -833,8 +851,7 @@ NodeService::usage(const string& appName)
     print("Usage: " + appName + " [options]\n" + options);
 }
 
-//COMPILERFIX: Borland C++ 2010 doesn't support wmain for console applications.
-#if defined(_WIN32 ) && !defined(__BCPLUSPLUS__)
+#ifdef _WIN32
 
 int
 wmain(int argc, wchar_t* argv[])
