@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -38,7 +38,7 @@ NodeSessionPrx
 NodeSessionKeepAliveThread::createSession(InternalRegistryPrx& registry, IceUtil::Time& timeout)
 {
     NodeSessionPrx session;
-    auto_ptr<Ice::Exception> exception;
+    IceUtil::UniquePtr<Ice::Exception> exception;
     TraceLevelsPtr traceLevels = _node->getTraceLevels();
     try
     {
@@ -110,6 +110,14 @@ NodeSessionKeepAliveThread::createSession(InternalRegistryPrx& registry, IceUtil
         }
         exception.reset(ex.ice_clone());
     }
+    catch(const PermissionDeniedException& ex)
+    {
+        if(traceLevels)
+        { 
+            traceLevels->logger->error("connection to the registry `" + _name + "' was denied:\n" + ex.reason);
+        }
+        exception.reset(ex.ice_clone());
+    }
     catch(const Ice::Exception& ex)
     {
         exception.reset(ex.ice_clone());
@@ -146,14 +154,23 @@ NodeSessionKeepAliveThread::createSession(InternalRegistryPrx& registry, IceUtil
 NodeSessionPrx
 NodeSessionKeepAliveThread::createSessionImpl(const InternalRegistryPrx& registry, IceUtil::Time& timeout)
 {
-    NodeSessionPrx session = _node->registerWithRegistry(registry);
-    int t = session->getTimeout();
-    if(t > 0)
+    NodeSessionPrx session;
+    try
     {
-        timeout = IceUtil::Time::seconds(t / 2);
+        session = _node->registerWithRegistry(registry);
+        int t = session->getTimeout();
+        if(t > 0)
+        {
+            timeout = IceUtil::Time::seconds(t / 2);
+        }
+        _node->addObserver(session, session->getObserver());
+        return session;
+    }
+    catch(const Ice::LocalException&)
+    {
+        destroySession(session);
+        throw;
     }
-    _node->addObserver(session, session->getObserver());
-    return session;
 }
 
 void 
@@ -161,22 +178,25 @@ NodeSessionKeepAliveThread::destroySession(const NodeSessionPrx& session)
 {
     _node->removeObserver(session);
 
-    try
+    if(session)
     {
-        session->destroy();
-
-        if(_node->getTraceLevels() && _node->getTraceLevels()->replica > 0)
+        try
         {
-            Ice::Trace out(_node->getTraceLevels()->logger, _node->getTraceLevels()->replicaCat);
-            out << "destroyed replica `" << _name << "' session";
+            session->destroy();
+            
+            if(_node->getTraceLevels() && _node->getTraceLevels()->replica > 0)
+            {
+                Ice::Trace out(_node->getTraceLevels()->logger, _node->getTraceLevels()->replicaCat);
+                out << "destroyed replica `" << _name << "' session";
+            }
         }
-    }
-    catch(const Ice::LocalException& ex)
-    {
-        if(_node->getTraceLevels() && _node->getTraceLevels()->replica > 1)
+        catch(const Ice::LocalException& ex)
         {
-            Ice::Trace out(_node->getTraceLevels()->logger, _node->getTraceLevels()->replicaCat);
-            out << "couldn't destroy replica `" << _name << "' session:\n" << ex;
+            if(_node->getTraceLevels() && _node->getTraceLevels()->replica > 1)
+            {
+                Ice::Trace out(_node->getTraceLevels()->logger, _node->getTraceLevels()->replicaCat);
+                out << "couldn't destroy replica `" << _name << "' session:\n" << ex;
+            }
         }
     }
 }
@@ -207,8 +227,8 @@ NodeSessionKeepAliveThread::keepAlive(const NodeSessionPrx& session)
     }
 }
 
-NodeSessionManager::NodeSessionManager() : 
-    _serial(1), 
+NodeSessionManager::NodeSessionManager(const Ice::CommunicatorPtr& communicator) : 
+    SessionManager(communicator),
     _destroyed(false),
     _activated(false)
 {
@@ -220,33 +240,7 @@ NodeSessionManager::create(const NodeIPtr& node)
     {
         Lock sync(*this);
         assert(!_node);
-
         const_cast<NodeIPtr&>(_node) = node;
-
-        Ice::CommunicatorPtr communicator = _node->getCommunicator();
-        assert(communicator->getDefaultLocator());
-        Ice::Identity id = communicator->getDefaultLocator()->ice_getIdentity();
-
-        //
-        // Initialize the IceGrid::Query objects. The IceGrid::Query
-        // interface is used to lookup the registry proxy in case it
-        // becomes unavailable. Since replicas might not always have
-        // an up to date registry proxy, we need to query all the
-        // replicas.
-        //
-        Ice::EndpointSeq endpoints = communicator->getDefaultLocator()->ice_getEndpoints();
-        id.name = "Query";
-        QueryPrx query = QueryPrx::uncheckedCast(communicator->stringToProxy(communicator->identityToString(id)));
-        for(Ice::EndpointSeq::const_iterator p = endpoints.begin(); p != endpoints.end(); ++p)
-        {
-            Ice::EndpointSeq singleEndpoint;
-            singleEndpoint.push_back(*p);
-            _queryObjects.push_back(QueryPrx::uncheckedCast(query->ice_endpoints(singleEndpoint)));
-        }
-
-        id.name = "InternalRegistry-Master";
-        _master = InternalRegistryPrx::uncheckedCast(communicator->stringToProxy(communicator->identityToString(id)));
-
         _thread = new Thread(*this);
         _thread->start();
     }
@@ -303,8 +297,10 @@ NodeSessionManager::activate()
             session->setReplicaObserver(_node->getProxy());
             syncServers(session);
         }
-        catch(const Ice::LocalException&)
+        catch(const Ice::LocalException& ex)
         {
+            Ice::Warning out(_node->getTraceLevels()->logger);
+            out << "failed to set replica observer:\n" << ex;
         }
     }
 }
@@ -342,8 +338,8 @@ NodeSessionManager::destroy()
     {
         _thread->terminate();
     }
-    NodeSessionMap::const_iterator p;
-    for(p = sessions.begin(); p != sessions.end(); ++p)
+
+    for(NodeSessionMap::const_iterator p = sessions.begin(); p != sessions.end(); ++p)
     {
         p->second->terminate();
     }
@@ -352,7 +348,7 @@ NodeSessionManager::destroy()
     {
         _thread->getThreadControl().join();
     }
-    for(p = sessions.begin(); p != sessions.end(); ++p)
+    for(NodeSessionMap::const_iterator p = sessions.begin(); p != sessions.end(); ++p)
     {
         p->second->getThreadControl().join();
     }
@@ -505,8 +501,10 @@ NodeSessionManager::createdSession(const NodeSessionPrx& session)
             session->setReplicaObserver(_node->getProxy());
             syncServers(session);
         }
-        catch(const Ice::LocalException&)
+        catch(const Ice::LocalException& ex)
         {
+            Ice::Warning out(_node->getTraceLevels()->logger);
+            out << "failed to set replica observer:\n" << ex;
         }
         return;
     }
@@ -535,14 +533,32 @@ NodeSessionManager::createdSession(const NodeSessionPrx& session)
     }
     else
     {
-        vector<Ice::AsyncResultPtr> results;
-        for(vector<QueryPrx>::const_iterator q = _queryObjects.begin(); q != _queryObjects.end(); ++q)
+        vector<Ice::AsyncResultPtr> results1;
+        vector<Ice::AsyncResultPtr> results2;
+        vector<QueryPrx> queryObjects = findAllQueryObjects();
+
+        //
+        // Below we try to retrieve internal registry proxies either
+        // directly by querying for the internal registry type or
+        // indirectly by querying registry proxies.
+        //
+        // IceGrid registries <= 3.5.0 kept internal registry proxies
+        // while earlier version now keep registry proxies. Registry
+        // proxies have fixed endpoints (no random port) so they are
+        // more reliable.
+        //
+        
+        for(vector<QueryPrx>::const_iterator q = queryObjects.begin(); q != queryObjects.end(); ++q)
         {
-            results.push_back((*q)->begin_findAllObjectsByType(InternalRegistry::ice_staticId()));
+            results1.push_back((*q)->begin_findAllObjectsByType(InternalRegistry::ice_staticId()));
+        }
+        for(vector<QueryPrx>::const_iterator q = queryObjects.begin(); q != queryObjects.end(); ++q)
+        {
+            results2.push_back((*q)->begin_findAllObjectsByType(Registry::ice_staticId()));
         }
 
         map<Ice::Identity, Ice::ObjectPrx> proxies;
-        for(vector<Ice::AsyncResultPtr>::const_iterator p = results.begin(); p != results.end(); ++p)
+        for(vector<Ice::AsyncResultPtr>::const_iterator p = results1.begin(); p != results1.end(); ++p)
         {
             QueryPrx query = QueryPrx::uncheckedCast((*p)->getProxy());
             if(isDestroyed())
@@ -555,11 +571,6 @@ NodeSessionManager::createdSession(const NodeSessionPrx& session)
                 Ice::ObjectProxySeq prxs = query->end_findAllObjectsByType(*p);
                 for(Ice::ObjectProxySeq::const_iterator q = prxs.begin(); q != prxs.end(); ++q)
                 {
-                    //
-                    // NOTE: We might override a good proxy here! We could improve this to make
-                    // sure that we don't override the proxy for replica N if that proxy was
-                    // obtained from replica N.
-                    //
                     proxies[(*q)->ice_getIdentity()] = *q;
                 }
             }
@@ -568,6 +579,39 @@ NodeSessionManager::createdSession(const NodeSessionPrx& session)
                 // IGNORE
             }
         }
+        for(vector<Ice::AsyncResultPtr>::const_iterator p = results2.begin(); p != results2.end(); ++p)
+        {
+            QueryPrx query = QueryPrx::uncheckedCast((*p)->getProxy());
+            if(isDestroyed())
+            {
+                return;
+            }
+
+            try
+            {
+                Ice::ObjectProxySeq prxs = query->end_findAllObjectsByType(*p);
+                for(Ice::ObjectProxySeq::const_iterator q = prxs.begin(); q != prxs.end(); ++q)
+                {
+                    Ice::Identity id = (*q)->ice_getIdentity();
+                    const string prefix("Registry-");
+                    string::size_type pos = id.name.find(prefix);
+                    if(pos == string::npos)
+                    {
+                        continue; // Ignore the master registry proxy.
+                    }
+                    id.name = "InternalRegistry-" + id.name.substr(prefix.size());
+        
+                    Ice::ObjectPrx prx = (*q)->ice_identity(id)->ice_endpoints(Ice::EndpointSeq());
+                    id.name = "Locator";
+                    prx = prx->ice_locator(Ice::LocatorPrx::uncheckedCast((*q)->ice_identity(id)));
+                    proxies[id] = prx;
+                }
+            }
+            catch(const Ice::LocalException&)
+            {
+                // IGNORE
+            }
+        }
 
         for(map<Ice::Identity, Ice::ObjectPrx>::const_iterator q = proxies.begin(); q != proxies.end(); ++q)
         {
