@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -35,7 +35,10 @@ using namespace IceUtilInternal;
 using namespace Ice;
 using namespace IceGrid;
 
-static const char* _commandsHelp[][3] = {
+namespace
+{
+
+const char* _commandsHelp[][3] = {
 { "application", "add", 
 "application add [-n | --no-patch] DESC [TARGET ... ] [NAME=VALUE ... ]\n"
 "                          Add application described in DESC. If specified\n"
@@ -48,13 +51,17 @@ static const char* _commandsHelp[][3] = {
 "application describe NAME Describe application NAME.\n" 
 },
 { "application", "diff",
-"application diff DESC [TARGET ... ] [NAME=VALUE ... ]\n"
+"application diff [-s | --servers] DESC [TARGET ... ] [NAME=VALUE ... ]\n"
 "                          Print the differences betwen the application\n"
 "                          described in DESC and the current deployment.\n" 
+"                          If -s or --servers is specified, print the\n"
+"                          the list of servers affected by the differences.\n" 
 },
 { "application", "update",
-"application update DESC [TARGET ... ] [NAME=VALUE ... ]\n"
-"                          Update the application described in DESC.\n"
+"application update [-n | --no-restart] DESC [TARGET ... ] [NAME=VALUE ... ]\n"
+"                          Update the application described in DESC. If -n or\n"
+"                          --no-restart is specified, the update will fail if\n"
+"                          it is necessary to stop some servers.\n"
 },
 { "application", "patch",
 "application patch [-f | --force] NAME\n"
@@ -89,8 +96,8 @@ static const char* _commandsHelp[][3] = {
 { "node", "load",
 "node load NAME            Print the load of the node NAME.\n" 
 },
-{ "node", "processors",
-"node processors [NAME]    Print the number of processor sockets of the\n"
+{ "node", "sockets",
+"node sockets [NAME]       Print the number of CPU sockets of the\n"
 "                          node NAME or all the nodes if NAME is omitted.\n" 
 },
 { "node", "show",
@@ -236,6 +243,8 @@ static const char* _commandsHelp[][3] = {
 { 0, 0, 0 }
 };
 
+}
+
 namespace IceGrid
 {
 
@@ -470,8 +479,28 @@ Parser::describeApplication(const list<string>& args)
 }
 
 void
-Parser::diffApplication(const list<string>& args)
+Parser::diffApplication(const list<string>& origArgs)
 {
+    list<string> copyArgs = origArgs;
+    copyArgs.push_front("icegridadmin");
+
+    IceUtilInternal::Options opts;
+    opts.addOpt("s", "servers");
+    vector<string> args;
+    try
+    {
+        for(list<string>::const_iterator p = copyArgs.begin(); p != copyArgs.end(); ++p)
+        {
+            args.push_back(*p);
+        }
+        args = opts.parse(args);
+    }
+    catch(const IceUtilInternal::BadOptException& e)
+    {
+        error(e.reason);
+        return;
+    }
+
     if(args.size() < 1)
     {
         invalidCommand("application diff" , "requires at least one argument");
@@ -483,7 +512,7 @@ Parser::diffApplication(const list<string>& args)
         StringSeq targets;
         map<string, string> vars;
 
-        list<string>::const_iterator p = args.begin();
+        vector<string>::const_iterator p = args.begin();
         string desc = *p++;
 
         for(; p != args.end(); ++p)
@@ -506,7 +535,55 @@ Parser::diffApplication(const list<string>& args)
         ApplicationHelper oldAppHelper(_communicator, origApp.descriptor);
         
         Output out(cout);
-        newAppHelper.printDiff(out, oldAppHelper);
+        if(opts.isSet("servers"))
+        {
+            map<string, ServerInfo> oldServers = oldAppHelper.getServerInfos(origApp.uuid, origApp.revision);
+            map<string, ServerInfo> newServers = newAppHelper.getServerInfos(origApp.uuid, origApp.revision);
+
+            vector<string> messages;
+            map<string, ServerInfo>::const_iterator p;
+            for(p = oldServers.begin(); p != oldServers.end(); ++p)
+            {
+                map<string, ServerInfo>::const_iterator q = newServers.find(p->first);
+                if(q == newServers.end())
+                {
+                    messages.push_back("server `" + p->first + "': removed");
+                }
+            }
+
+            for(p = newServers.begin(); p != newServers.end(); ++p)
+            {
+                map<string, ServerInfo>::const_iterator q = oldServers.find(p->first);
+                if(q == oldServers.end())
+                {
+                    messages.push_back("server `" + p->first + "': added");
+                }
+                else if(isServerUpdated(p->second, q->second))
+                {
+                    if(isServerUpdated(p->second, q->second, true)) // Ignore properties
+                    {
+                        messages.push_back("server `" + p->first + "': updated (restart required)");
+                    }
+                    else
+                    {
+                        messages.push_back("server `" + p->first + "': properties updated (no restart required)");
+                    }
+                }
+            }
+
+            out << "application `" << origApp.descriptor.name << "'";
+            out << sb;
+            sort(messages.begin(), messages.end());
+            for(vector<string>::const_iterator r = messages.begin(); r != messages.end(); ++r)
+            {
+                out << nl << *r;
+            }
+            out << eb;
+        }
+        else
+        {
+            newAppHelper.printDiff(out, oldAppHelper);
+        }
         out << nl;  
     }
     catch(const Ice::Exception& ex)
@@ -516,8 +593,28 @@ Parser::diffApplication(const list<string>& args)
 }
 
 void
-Parser::updateApplication(const list<string>& args)
+Parser::updateApplication(const list<string>& origArgs)
 {
+    list<string> copyArgs = origArgs;
+    copyArgs.push_front("icegridadmin");
+
+    IceUtilInternal::Options opts;
+    opts.addOpt("n", "no-restart");
+    vector<string> args;
+    try
+    {
+        for(list<string>::const_iterator p = copyArgs.begin(); p != copyArgs.end(); ++p)
+        {
+            args.push_back(*p);
+        }
+        args = opts.parse(args);
+    }
+    catch(const IceUtilInternal::BadOptException& e)
+    {
+        error(e.reason);
+        return;
+    }
+
     if(args.size() < 1)
     {
         invalidCommand("application update", "requires at least one argument");
@@ -529,8 +626,8 @@ Parser::updateApplication(const list<string>& args)
         StringSeq targets;
         map<string, string> vars;
 
-        list<string>::const_iterator p = args.begin();
-        string desc = *p++;
+        vector<string>::const_iterator p = args.begin();
+        string xml = *p++;
 
         for(; p != args.end(); ++p)
         {
@@ -545,7 +642,19 @@ Parser::updateApplication(const list<string>& args)
             }
         }
 
-        _admin->syncApplication(DescriptorParser::parseDescriptor(desc, targets, vars, _communicator, _admin));
+        ApplicationDescriptor desc = DescriptorParser::parseDescriptor(xml, targets, vars, _communicator, _admin);
+        if(opts.isSet("no-restart"))
+        {
+            _admin->syncApplicationWithoutRestart(desc);
+        }
+        else
+        {
+            _admin->syncApplication(desc);
+        }
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+        error("registry doesn't support updates without restart");
     }
     catch(const Ice::Exception& ex)
     {
@@ -767,7 +876,7 @@ Parser::describeNode(const list<string>& args)
         out << nl << "release = `" << info.release << "'";
         out << nl << "version = `" << info.version << "'";
         out << nl << "machine type = `" << info.machine << "'";
-        out << nl << "number of processors = `" << info.nProcessors << "'";
+        out << nl << "number of threads = `" << info.nProcessors << "'";
         out << eb;
         out << nl;
     }
@@ -824,11 +933,11 @@ Parser::printLoadNode(const list<string>& args)
 }
 
 void
-Parser::printNodeProcessors(const list<string>& args)
+Parser::printNodeProcessorSockets(const list<string>& args)
 {
     if(args.size() > 1)
     {
-        invalidCommand("node processors", "requires no more than one argument");
+        invalidCommand("node sockets", "requires no more than one argument");
         return;
     }
 
@@ -2063,7 +2172,7 @@ Parser::showFile(const string& reader, const list<string>& origArgs)
 void
 Parser::showBanner()
 {
-    cout << "Ice " << ICE_STRING_VERSION << "  Copyright 2003-2011 ZeroC, Inc." << endl;
+    cout << "Ice " << ICE_STRING_VERSION << "  Copyright (c) 2003-2013 ZeroC, Inc." << endl;
 }
 
 void
@@ -2089,12 +2198,7 @@ Parser::getInput(char* buf, int& result, int maxSize)
         }
         else
         {
-#if defined(_MSC_VER) && _MSC_VER < 1500 && !defined(_STLP_MSVC)
-            // COMPILERBUG: Visual C++ defines min and max as macros
-            result = _MIN(maxSize, static_cast<int>(_commands.length()));
-#else
             result = min(maxSize, static_cast<int>(_commands.length()));
-#endif
             strncpy(buf, _commands.c_str(), result);
             _commands.erase(0, result);
             if(_commands.empty())
@@ -2159,7 +2263,7 @@ Parser::getInput(char* buf, int& result, int maxSize)
             }
         }
         
-        result = (int) line.length();
+        result = static_cast<int>(line.length());
         if(result > maxSize)
         {
             error("input line too long");
