@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -82,7 +82,7 @@ public:
     void describeNode(const std::list<std::string>&);
     void pingNode(const std::list<std::string>&);
     void printLoadNode(const std::list<std::string>&);
-    void printNodeProcessors(const std::list<std::string>&);
+    void printNodeProcessorSockets(const std::list<std::string>&);
     void shutdownNode(const std::list<std::string>&);
     void listAllNodes(const std::list<std::string>&);
 
