@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -51,12 +51,11 @@ using namespace IceGrid;
 namespace
 {
 
-
 class NullPermissionsVerifierI : public Glacier2::PermissionsVerifier
 {
 public:
 
-    bool checkPermissions(const string& userId, const string& password, string&, const Current&) const
+    bool checkPermissions(const string& /*userId*/, const string& /*password*/, string&, const Current&) const
     {
         return true;
     }
@@ -109,7 +108,6 @@ private:
     const std::map<std::string, std::string> _passwords;
 };
 
-
 class DefaultServantLocator : public Ice::ServantLocator
 {
 public:
@@ -119,7 +117,7 @@ public:
     {
     }
 
-    virtual ObjectPtr locate(const Current& c, LocalObjectPtr&)
+    virtual ObjectPtr locate(const Current&, LocalObjectPtr&)
     {
         return _servant;
     }
@@ -141,11 +139,14 @@ private:
 RegistryI::RegistryI(const CommunicatorPtr& communicator, 
                      const TraceLevelsPtr& traceLevels, 
                      bool nowarn, 
-                     bool readonly) : 
+                     bool readonly,
+                     const string& initFromReplica) : 
     _communicator(communicator),
     _traceLevels(traceLevels),
     _nowarn(nowarn),
     _readonly(readonly),
+    _initFromReplica(initFromReplica),
+    _session(communicator),
     _platform("IceGrid.Registry", communicator, traceLevels)
 {
 }
@@ -234,17 +235,55 @@ RegistryI::startImpl()
     properties->setProperty("IceGrid.Registry.Client.AdapterId", "");
     properties->setProperty("IceGrid.Registry.Server.AdapterId", "");
     properties->setProperty("IceGrid.Registry.SessionManager.AdapterId", "");
+    properties->setProperty("IceGrid.Registry.AdminSessionManager.AdapterId", "");
     properties->setProperty("IceGrid.Registry.Internal.AdapterId", "");
 
     setupThreadPool(properties, "IceGrid.Registry.Client.ThreadPool", 1, 10);
     setupThreadPool(properties, "IceGrid.Registry.Server.ThreadPool", 1, 10, true); // Serialize for admin callbacks
     setupThreadPool(properties, "IceGrid.Registry.SessionManager.ThreadPool", 1, 10);
+    setupThreadPool(properties, "IceGrid.Registry.AdminSessionManager.ThreadPool", 1, 10);
     setupThreadPool(properties, "IceGrid.Registry.Internal.ThreadPool", 1, 100);
 
     _replicaName = properties->getPropertyWithDefault("IceGrid.Registry.ReplicaName", "Master");
     _master = _replicaName == "Master";
     _sessionTimeout = properties->getPropertyAsIntWithDefault("IceGrid.Registry.SessionTimeout", 30);
 
+    if(!_initFromReplica.empty() && (_initFromReplica == _replicaName || (_master && _initFromReplica == "Master")))
+    {
+        Error out(_communicator->getLogger());
+        out << "invalid --initdb-from-replica option: identical replica";
+        return false; 
+    }
+
+    if(properties->getProperty("IceGrid.Registry.Client.ACM").empty())
+    {
+        //
+        // Set the client object adapter ACM timeout to the session
+        // timeout * 2. If no session timeout is set, ACM is disabled.
+        //
+        ostringstream os;
+        os << _sessionTimeout * 2;
+        properties->setProperty("IceGrid.Registry.Client.ACM", os.str());
+    }
+    if(properties->getProperty("IceGrid.Registry.Server.ACM").empty())
+    {
+        properties->setProperty("IceGrid.Registry.Server.ACM", "30");
+    }
+    if(properties->getProperty("IceGrid.Registry.Internal.ACM").empty())
+    {
+        int nt = properties->getPropertyAsIntWithDefault("IceGrid.Registry.NodeSessionTimeout", 30);
+        int rt = properties->getPropertyAsIntWithDefault("IceGrid.Registry.ReplicaSessionTimeout", 30);
+
+        //
+        // Set the internal object adapter ACM timeout to the replica
+        // or node session timeout * 2. If no session timeout is set,
+        // ACM is disabled.
+        //
+        ostringstream os;
+        os << std::max(nt, rt) * 2;
+        properties->setProperty("IceGrid.Registry.Internal.ACM", os.str());
+    }
+    
     if(!_master && properties->getProperty("Ice.Default.Locator").empty())
     {
         if(properties->getProperty("Ice.Default.Locator").empty())
@@ -278,6 +317,28 @@ RegistryI::startImpl()
         _instanceName = _communicator->getDefaultLocator()->ice_getIdentity().category;
     }
 
+    //
+    // Create the registry database.
+    //
+    DatabasePluginPtr plugin;
+    try
+    {
+        plugin = DatabasePluginPtr::dynamicCast(_communicator->getPluginManager()->getPlugin("DB"));
+    }
+    catch(const NotRegisteredException&)
+    {
+    }
+    if(!plugin)
+    {
+        Error out(_communicator->getLogger());
+        out << "no database plugin configured with `Ice.Plugin.DB' or plugin is not a database plugin";
+        return false;
+    }
+    if(!plugin->initDB())
+    {
+        return false;
+    }
+
     //
     // Ensure that nothing is running on this port. This is also
     // useful to ensure that we don't run twice the same instance of
@@ -297,7 +358,7 @@ RegistryI::startImpl()
     catch(const Ice::LocalException&)
     {
     }
-    
+
     //
     // Create the reaper thread.
     //
@@ -317,52 +378,105 @@ RegistryI::startImpl()
     registryTopicManagerId.category = _instanceName;
     registryTopicManagerId.name = "RegistryTopicManager";
     _iceStorm = IceStormInternal::Service::create(_communicator, 
-					          registryAdapter, 
-                                          	  registryAdapter, 
-                                          	  "IceGrid.Registry", 
-                                          	  registryTopicManagerId,
-                                          	  "Registry");
+                                                  registryAdapter, 
+                                                  registryAdapter, 
+                                                  "IceGrid.Registry", 
+                                                  registryTopicManagerId,
+                                                  "Registry");
     const IceStorm::TopicManagerPrx topicManager = _iceStorm->getTopicManager();
 
-    //
-    // Create the registry database.
-    //
-    DatabasePluginPtr plugin;
-    try
-    {
-        plugin = DatabasePluginPtr::dynamicCast(_communicator->getPluginManager()->getPlugin("DB"));
-    }
-    catch(const NotRegisteredException&)
-    {
-    }
-    if(!plugin)
-    {
-        Error out(_communicator->getLogger());
-        out << "no database plugin configured with `Ice.Plugin.DB' or plugin is not a database plugin";
-        return false;
-    }
-    
     _database = new Database(registryAdapter, topicManager, _instanceName, _traceLevels, getInfo(), plugin, _readonly);
     _wellKnownObjects = new WellKnownObjectsManager(_database);
 
-    //
-    // Get the saved replica/node proxies.
-    //
-    ObjectProxySeq proxies;
-    ObjectProxySeq::const_iterator p;
-
-    NodePrxSeq nodes;
-    proxies = _database->getInternalObjectsByType(Node::ice_staticId());
-    for(p = proxies.begin(); p != proxies.end(); ++p)
+    if(!_initFromReplica.empty())
     {
-        nodes.push_back(NodePrx::uncheckedCast(*p));
+        Ice::Identity id;
+        id.category = _instanceName;
+        id.name = (_initFromReplica == "Master") ? "Registry" : "Registry-" + _initFromReplica;
+        Ice::ObjectPrx proxy;
+        try
+        {
+            proxy = _database->getObjectProxy(id);
+            id.name = "Query";
+            IceGrid::QueryPrx query = IceGrid::QueryPrx::uncheckedCast(proxy->ice_identity(id));
+            if(query)
+            {
+                id.name = "InternalRegistry-" + _initFromReplica;
+                try
+                {
+                    proxy = query->findObjectById(id);
+                }
+                catch(const Ice::Exception&)
+                {
+                }
+            }
+        }
+        catch(const ObjectNotRegisteredException&)
+        {
+            id.name = "InternalRegistry-" + _initFromReplica;
+            try
+            {
+                proxy = _database->getObjectProxy(id);
+            }
+            catch(const ObjectNotRegisteredException&)
+            {
+            }
+        }
+
+        // If we still didn't find the replica proxy, check with the
+        // locator or the IceGrid.Registry.ReplicaEndpoints properties
+        // if we can find it.
+        if(!proxy)
+        {
+            id.name = "InternalRegistry-" + (_initFromReplica.empty() ? "Master" : _initFromReplica);
+            proxy = _session.findInternalRegistryForReplica(id);
+        }
+
+        if(!proxy)
+        {
+            Error out(_communicator->getLogger());
+            out << "couldn't find replica `" << _initFromReplica << "' to\n";
+            out << "initialize the database (specify the replica endpoints in the endpoints of\n";
+            out << "the `Ice.Default.Locator' proxy property to allow finding the replica)";
+            return false;
+        }
+
+        try
+        {
+            Ice::Long serial;
+            IceGrid::InternalRegistryPrx registry = IceGrid::InternalRegistryPrx::checkedCast(proxy);
+	    ApplicationInfoSeq applications = registry->getApplications(serial);
+            _database->syncApplications(applications, serial);
+	    AdapterInfoSeq adapters = registry->getAdapters(serial);
+            _database->syncAdapters(adapters, serial);
+	    ObjectInfoSeq objects = registry->getObjects(serial);
+            _database->syncObjects(objects, serial);
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+            Error out(_communicator->getLogger());
+            out << "couldn't initialize database from replica `" << _initFromReplica << "':\n";
+            out << "replica doesn't support this functionality (IceGrid < 3.5.1)";
+            return false;
+        }
+        catch(const Ice::Exception& ex)
+        {
+            Error out(_communicator->getLogger());
+            out << "couldn't initialize database from replica `" << _initFromReplica << "':\n";
+            out << ex;
+            return false;
+        }
     }
 
-    InternalRegistryPrxSeq replicas;
-    proxies = _database->getObjectsByType(InternalRegistry::ice_staticId());
-    for(p = proxies.begin(); p != proxies.end(); ++p)
+    //
+    // Get proxies for nodes that we were connected with on last
+    // shutdown.
+    //
+    NodePrxSeq nodes;
+    ObjectProxySeq proxies = _database->getInternalObjectsByType(Node::ice_staticId());
+    for(ObjectProxySeq::const_iterator p = proxies.begin(); p != proxies.end(); ++p)
     {
-        replicas.push_back(InternalRegistryPrx::uncheckedCast(*p));
+        nodes.push_back(NodePrx::uncheckedCast(*p));
     }
 
     //
@@ -374,7 +488,7 @@ RegistryI::startImpl()
     InternalRegistryPrx internalRegistry = setupInternalRegistry(registryAdapter);
     if(_master)
     {
-        nodes = registerReplicas(internalRegistry, replicas, nodes);
+        nodes = registerReplicas(internalRegistry, nodes);
         registerNodes(internalRegistry, nodes);
     }
     else
@@ -586,7 +700,7 @@ RegistryI::setupClientSessionFactory(const Ice::ObjectAdapterPtr& registryAdapte
     }
 
     assert(_reaper);
-    _timer = new IceUtil::Timer();  // Used for for session allocation timeout.
+    _timer = new IceUtil::Timer();  // Used for session allocation timeout.
     _clientSessionFactory = new ClientSessionFactory(servantManager, _database, _timer, _reaper);
 
     if(servantManager && _master) // Slaves don't support client session manager objects.
@@ -768,6 +882,12 @@ RegistryI::createSession(const string& user, const string& password, const Curre
             throw exc;
         }
     }
+    catch(const Glacier2::PermissionDeniedException& ex)
+    {
+        PermissionDeniedException exc;
+        exc.reason = ex.reason;
+        throw exc;
+    }
     catch(const LocalException& ex)
     {
         if(_traceLevels && _traceLevels->session > 0)
@@ -818,6 +938,12 @@ RegistryI::createAdminSession(const string& user, const string& password, const
             throw exc;
         }
     }
+    catch(const Glacier2::PermissionDeniedException& ex)
+    {
+        PermissionDeniedException exc;
+        exc.reason = ex.reason;
+        throw exc;
+    }
     catch(const LocalException& ex)
     {
         if(_traceLevels && _traceLevels->session > 0)
@@ -877,6 +1003,12 @@ RegistryI::createSessionFromSecureConnection(const Current& current)
             throw exc;
         }
     }
+    catch(const Glacier2::PermissionDeniedException& ex)
+    {
+        PermissionDeniedException exc;
+        exc.reason = ex.reason;
+        throw exc;
+    }
     catch(const LocalException& ex)
     {
         if(_traceLevels && _traceLevels->session > 0)
@@ -922,6 +1054,12 @@ RegistryI::createAdminSessionFromSecureConnection(const Current& current)
             throw exc;
         }
     }
+    catch(const Glacier2::PermissionDeniedException& ex)
+    {
+        PermissionDeniedException exc;
+        exc.reason = ex.reason;
+        throw exc;
+    }
     catch(const LocalException& ex)
     {
         if(_traceLevels && _traceLevels->session > 0)
@@ -945,7 +1083,7 @@ RegistryI::createAdminSessionFromSecureConnection(const Current& current)
 }
 
 int
-RegistryI::getSessionTimeout(const Ice::Current& current) const
+RegistryI::getSessionTimeout(const Ice::Current& /*current*/) const
 {
     return _sessionTimeout;
 }
@@ -1239,18 +1377,63 @@ RegistryI::getSSLInfo(const ConnectionPtr& connection, string& userDN)
 }
 
 NodePrxSeq
-RegistryI::registerReplicas(const InternalRegistryPrx& internalRegistry, 
-                            const InternalRegistryPrxSeq& replicas,
-                            const NodePrxSeq& dbNodes)
+RegistryI::registerReplicas(const InternalRegistryPrx& internalRegistry, const NodePrxSeq& dbNodes)
 {
+    //
+    // Get proxies for slaves that we we connected with on last
+    // shutdown.
+    //
+    // We first get the internal registry proxies and then also check
+    // the public registry proxies. If we find public registry
+    // proxies, we use indirect proxies setup with a locator using the
+    // public proxy in preference over the internal proxy which might
+    // contain stale endpoints if the slave was restarted. IceGrid
+    // version <= 3.5.0 also kept the internal proxy in the database
+    // instead of the public proxy.
+    //
+    map<InternalRegistryPrx, RegistryPrx> replicas;
+    Ice::ObjectProxySeq proxies = _database->getObjectsByType(InternalRegistry::ice_staticId());
+    for(ObjectProxySeq::const_iterator p = proxies.begin(); p != proxies.end(); ++p)
+    {
+        replicas.insert(pair<InternalRegistryPrx, RegistryPrx>(InternalRegistryPrx::uncheckedCast(*p), RegistryPrx()));
+    }
+
+    proxies = _database->getObjectsByType(Registry::ice_staticId());
+    for(ObjectProxySeq::const_iterator p = proxies.begin(); p != proxies.end(); ++p)
+    {
+        Ice::Identity id = (*p)->ice_getIdentity();
+        const string prefix("Registry-");
+        string::size_type pos = id.name.find(prefix);
+        if(pos == string::npos)
+        {
+            continue; // Ignore the master registry proxy.
+        }
+        id.name = "InternalRegistry-" + id.name.substr(prefix.size());
+        
+        Ice::ObjectPrx prx = (*p)->ice_identity(id)->ice_endpoints(Ice::EndpointSeq());
+        id.name = "Locator";
+        prx = prx->ice_locator(Ice::LocatorPrx::uncheckedCast((*p)->ice_identity(id)));
+
+        for(map<InternalRegistryPrx, RegistryPrx>::iterator q = replicas.begin(); q != replicas.end(); ++q)
+        {
+            if(q->first->ice_getIdentity() == prx->ice_getIdentity()) 
+            {
+                replicas.erase(q);
+                break;
+            }
+        }
+        replicas.insert(pair<InternalRegistryPrx, RegistryPrx>(InternalRegistryPrx::uncheckedCast(prx), 
+                                                               RegistryPrx::uncheckedCast(*p)));
+    }
+
     set<NodePrx> nodes;
     nodes.insert(dbNodes.begin(), dbNodes.end());
     vector<Ice::AsyncResultPtr> results;
-    for(InternalRegistryPrxSeq::const_iterator r = replicas.begin(); r != replicas.end(); ++r)
+    for(map<InternalRegistryPrx, RegistryPrx>::const_iterator r = replicas.begin(); r != replicas.end(); ++r)
     {
-        if((*r)->ice_getIdentity() != internalRegistry->ice_getIdentity())
+        if(r->first->ice_getIdentity() != internalRegistry->ice_getIdentity())
         {
-            results.push_back((*r)->begin_registerWithReplica(internalRegistry));
+            results.push_back(r->first->begin_registerWithReplica(internalRegistry));
         }
     }
 
@@ -1293,14 +1476,35 @@ RegistryI::registerReplicas(const InternalRegistryPrx& internalRegistry,
             // Clear the proxy from the database if we can't
             // contact the replica.
             //
+            RegistryPrx registry;
+            for(map<InternalRegistryPrx, RegistryPrx>::const_iterator q = replicas.begin(); q != replicas.end(); ++q)
+            {
+                if(q->first->ice_getIdentity() == replica->ice_getIdentity()) 
+                {
+                    registry = q->second;
+                    break;
+                }
+            }
+	    ObjectInfoSeq infos;
+            if(registry)
+            {
+                try
+                {
+		    infos.push_back(_database->getObjectInfo(registry->ice_getIdentity()));
+                }
+                catch(const ObjectNotRegisteredException&)
+                {
+                }
+            }
             try
             {
-                _database->removeObject(replica->ice_getIdentity());
+	        infos.push_back(_database->getObjectInfo(replica->ice_getIdentity()));
             }
             catch(const ObjectNotRegisteredException&)
             {
             }
-            
+	    _database->removeRegistryWellKnownObjects(infos);
+
             if(_traceLevels && _traceLevels->replica > 1)
             {
                 Ice::Trace out(_traceLevels->logger, _traceLevels->replicaCat);
@@ -1318,7 +1522,7 @@ RegistryI::registerReplicas(const InternalRegistryPrx& internalRegistry,
 }
 
 void
-RegistryI::registerNodes(const InternalRegistryPrx& internalRegistry, const NodePrxSeq& nodes)
+RegistryI::registerNodes(const InternalRegistryPrx& /*internalRegistry*/, const NodePrxSeq& nodes)
 {
     const string prefix("Node-");
 
