@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -35,17 +35,19 @@ public:
     virtual void 
     applicationInit(int, const ApplicationInfoSeq& applications, const Ice::Current& current)
     {
-        _database->syncApplications(applications);
-        receivedUpdate(ApplicationObserverTopicName, getSerial(current.ctx));
+        int serial;
+        _database->syncApplications(applications, getSerials(current.ctx, serial));
+        receivedUpdate(ApplicationObserverTopicName, serial);
     }
 
     virtual void 
     applicationAdded(int, const ApplicationInfo& application, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->addApplication(application);
+            _database->addApplication(application, 0, getSerials(current.ctx, serial));
         }
         catch(const DeploymentException& ex)
         {
@@ -53,16 +55,17 @@ public:
             os << ex << ":\n" << ex.reason;
             failure = os.str();
         }
-        receivedUpdate(ApplicationObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(ApplicationObserverTopicName, serial, failure);
     }
 
     virtual void 
     applicationRemoved(int, const std::string& name, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->removeApplication(name);
+            _database->removeApplication(name, 0, getSerials(current.ctx, serial));
         }
         catch(const ApplicationNotExistException& ex)
         {
@@ -70,16 +73,17 @@ public:
             os << ex << ":\napplication: " << ex.name;
             failure = os.str();
         }
-        receivedUpdate(ApplicationObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(ApplicationObserverTopicName, serial, failure);
     }
 
     virtual void 
     applicationUpdated(int, const ApplicationUpdateInfo& update, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->updateApplication(update);
+            _database->updateApplication(update, false, 0, getSerials(current.ctx, serial));
         }
         catch(const DeploymentException& ex)
         {
@@ -93,75 +97,81 @@ public:
             os << ex << ":\napplication: " << ex.name;
             failure = os.str();
         }
-        receivedUpdate(ApplicationObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(ApplicationObserverTopicName, serial, failure);
     }
 
     virtual void
     adapterInit(const AdapterInfoSeq& adapters, const Ice::Current& current)
     {
-        _database->syncAdapters(adapters);
-        receivedUpdate(AdapterObserverTopicName, getSerial(current.ctx));
+        int serial;
+        _database->syncAdapters(adapters, getSerials(current.ctx, serial));
+        receivedUpdate(AdapterObserverTopicName, serial);
     }
 
     virtual void 
     adapterAdded(const AdapterInfo& info, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->setAdapterDirectProxy(info.id, info.replicaGroupId, info.proxy);
+            _database->setAdapterDirectProxy(info.id, info.replicaGroupId, info.proxy, getSerials(current.ctx, serial));
         }
         catch(const AdapterExistsException&)
         {
             failure = "adapter `" + info.id + "' already exists and belongs to an application";
         }
-        receivedUpdate(AdapterObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(AdapterObserverTopicName, serial, failure);
     }
 
     virtual void 
     adapterUpdated(const AdapterInfo& info, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->setAdapterDirectProxy(info.id, info.replicaGroupId, info.proxy);
+            _database->setAdapterDirectProxy(info.id, info.replicaGroupId, info.proxy, getSerials(current.ctx, serial));
         }
         catch(const AdapterExistsException&)
         {
             failure = "adapter `" + info.id + "' already exists and belongs to an application";
         }
-        receivedUpdate(AdapterObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(AdapterObserverTopicName, serial, failure);
     }
 
     virtual void 
     adapterRemoved(const std::string& id, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->setAdapterDirectProxy(id, "", 0);
+            _database->setAdapterDirectProxy(id, "", 0, getSerials(current.ctx, serial));
         }
         catch(const AdapterExistsException&)
         {
             failure = "adapter `" + id + "' already exists and belongs to an application";
         }
-        receivedUpdate(AdapterObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(AdapterObserverTopicName, serial, failure);
     }
 
     virtual void
     objectInit(const ObjectInfoSeq& objects, const Ice::Current& current)
     {
-        _database->syncObjects(objects);
-        receivedUpdate(ObjectObserverTopicName, getSerial(current.ctx));
+        int serial;
+        _database->syncObjects(objects, getSerials(current.ctx, serial));
+        receivedUpdate(ObjectObserverTopicName, serial);
     }
 
     virtual void 
     objectAdded(const ObjectInfo& info, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->addOrUpdateObject(info);
+            _database->addOrUpdateObject(info, getSerials(current.ctx, serial));
         }
         catch(const ObjectExistsException& ex)
         {
@@ -170,16 +180,17 @@ public:
             os << "id: " << info.proxy->ice_getCommunicator()->identityToString(info.proxy->ice_getIdentity());
             failure = os.str();
         }
-        receivedUpdate(ObjectObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(ObjectObserverTopicName, serial, failure);
     }
 
     virtual void 
     objectUpdated(const ObjectInfo& info, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->addOrUpdateObject(info);
+            _database->addOrUpdateObject(info, getSerials(current.ctx, serial));
         }
         catch(const ObjectExistsException& ex)
         {
@@ -194,16 +205,17 @@ public:
             os << ex << ":\n" << ex.reason;
             failure = os.str();
         }
-        receivedUpdate(ObjectObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(ObjectObserverTopicName, serial, failure);
     }
 
     virtual void 
     objectRemoved(const Ice::Identity& id, const Ice::Current& current)
     {
+        int serial;
         string failure;
         try
         {
-            _database->removeObject(id);
+            _database->removeObject(id, getSerials(current.ctx, serial));
         }
         catch(const DeploymentException& ex)
         {
@@ -214,23 +226,37 @@ public:
         catch(const ObjectNotRegisteredException&)
         {
         }
-        receivedUpdate(ObjectObserverTopicName, getSerial(current.ctx), failure);
+        receivedUpdate(ObjectObserverTopicName, serial, failure);
     }
 
 private:
 
-    int 
-    getSerial(const Ice::Context& context)
+    Ice::Long
+    getSerials(const Ice::Context& context, int& serial)
     {
         Ice::Context::const_iterator p = context.find("serial");
         if(p != context.end())
         {
-            int serial;
             istringstream is(p->second);
             is >> serial;
-            return serial;
         }
-        return -1;
+        else 
+        {
+            serial = -1;
+        }
+
+        p = context.find("dbSerial");
+        if(p != context.end())
+        {
+            Ice::Long dbSerial;
+            istringstream is(p->second);
+            is >> dbSerial;
+            return dbSerial;
+        }
+        else
+        {
+            return -1;
+        }
     }
     
     void 
@@ -256,7 +282,7 @@ private:
 
 };
 
-ReplicaSessionManager::ReplicaSessionManager()
+ReplicaSessionManager::ReplicaSessionManager(const Ice::CommunicatorPtr& communicator) : SessionManager(communicator) 
 {
 }
 
@@ -267,15 +293,9 @@ ReplicaSessionManager::create(const string& name,
                               const WellKnownObjectsManagerPtr& wellKnownObjects,
                               const InternalRegistryPrx& internalRegistry)
 {
-    Ice::CommunicatorPtr comm = database->getCommunicator();
     {
         Lock sync(*this);
 
-        Ice::Identity id;
-        id.category = comm->getDefaultLocator()->ice_getIdentity().category;
-        id.name = "InternalRegistry-Master";
-        
-        _master = InternalRegistryPrx::uncheckedCast(comm->stringToProxy(comm->identityToString(id)));
         _name = name;
         _info = info;
         _internalRegistry = internalRegistry;
@@ -283,23 +303,6 @@ ReplicaSessionManager::create(const string& name,
         _wellKnownObjects = wellKnownObjects;
         _traceLevels = _database->getTraceLevels();
 
-        //
-        // Initialize the IceGrid::Query objects. The IceGrid::Query
-        // interface is used to lookup the registry proxy in case it
-        // becomes unavailable. Since replicas might not always have
-        // an up to date registry proxy, we need to query all the
-        // replicas.
-        //
-        Ice::EndpointSeq endpoints = comm->getDefaultLocator()->ice_getEndpoints();
-        id.name = "Query";
-        QueryPrx query = QueryPrx::uncheckedCast(comm->stringToProxy(comm->identityToString(id)));
-        for(Ice::EndpointSeq::const_iterator p = endpoints.begin(); p != endpoints.end(); ++p)
-        {
-            Ice::EndpointSeq singleEndpoint;
-            singleEndpoint.push_back(*p);
-            _queryObjects.push_back(QueryPrx::uncheckedCast(query->ice_endpoints(singleEndpoint)));
-        }
-
         _thread = new Thread(*this, _master, _traceLevels->logger);
         _thread->start();
         notifyAll();
@@ -356,16 +359,21 @@ ReplicaSessionManager::destroy()
     ThreadPtr thread;
     {
         Lock sync(*this);
-        if(!_thread)
+        if(!_communicator)
         {
             return;
         }
         thread = _thread;
         _thread = 0;
+
+        _communicator = 0;
     }
 
-    thread->terminate();
-    thread->getThreadControl().join();
+    if(thread)
+    {
+        thread->terminate();
+        thread->getThreadControl().join();
+    }
 
     _database = 0;
     _wellKnownObjects = 0;
@@ -400,6 +408,31 @@ ReplicaSessionManager::registerAllWellKnownObjects()
     }
 }
 
+IceGrid::InternalRegistryPrx
+ReplicaSessionManager::findInternalRegistryForReplica(const Ice::Identity& id)
+{
+    vector<Ice::AsyncResultPtr> results;
+    vector<QueryPrx> queryObjects = findAllQueryObjects();
+    for(vector<QueryPrx>::const_iterator q = queryObjects.begin(); q != queryObjects.end(); ++q)
+    {
+        results.push_back((*q)->begin_findObjectById(id));
+    }
+    
+    for(vector<Ice::AsyncResultPtr>::const_iterator p = results.begin(); p != results.end(); ++p)
+    {
+        QueryPrx query = QueryPrx::uncheckedCast((*p)->getProxy());
+        try
+        {
+            return InternalRegistryPrx::checkedCast(query->end_findObjectById(*p));
+        }
+        catch(const Ice::Exception&)
+        {
+        }
+    }
+        
+    return 0;
+}
+
 bool
 ReplicaSessionManager::keepAlive(const ReplicaSessionPrx& session)
 {
@@ -429,7 +462,7 @@ ReplicaSessionPrx
 ReplicaSessionManager::createSession(InternalRegistryPrx& registry, IceUtil::Time& timeout)
 {
     ReplicaSessionPrx session;
-    auto_ptr<Ice::Exception> exception;
+    IceUtil::UniquePtr<Ice::Exception> exception;
     try
     {
         if(_traceLevels && _traceLevels->replica > 1)
@@ -456,7 +489,8 @@ ReplicaSessionManager::createSession(InternalRegistryPrx& registry, IceUtil::Tim
         if(!session)
         {
             vector<Ice::AsyncResultPtr> results;
-            for(vector<QueryPrx>::const_iterator q = _queryObjects.begin(); q != _queryObjects.end(); ++q)
+            vector<QueryPrx> queryObjects = findAllQueryObjects();
+            for(vector<QueryPrx>::const_iterator q = queryObjects.begin(); q != queryObjects.end(); ++q)
             {
                 results.push_back((*q)->begin_findObjectById(registry->ice_getIdentity()));
             }
@@ -500,6 +534,22 @@ ReplicaSessionManager::createSession(InternalRegistryPrx& registry, IceUtil::Tim
         }
         exception.reset(ex.ice_clone());
     }
+    catch(const DeploymentException& ex)
+    {
+        if(_traceLevels)
+        {
+            _traceLevels->logger->error("database synchronization with master failed:\n" + ex.reason);
+        }
+        exception.reset(ex.ice_clone());
+    }
+    catch(const PermissionDeniedException& ex)
+    {
+        if(_traceLevels)
+        {
+            _traceLevels->logger->error("connection to master was denied:\n" + ex.reason);
+        }
+        exception.reset(ex.ice_clone());
+    }
     catch(const Ice::Exception& ex)
     {
         exception.reset(ex.ice_clone());
@@ -546,9 +596,10 @@ ReplicaSessionManager::createSession(InternalRegistryPrx& registry, IceUtil::Tim
 ReplicaSessionPrx
 ReplicaSessionManager::createSessionImpl(const InternalRegistryPrx& registry, IceUtil::Time& timeout)
 {           
+    ReplicaSessionPrx session;
     try
     {
-        ReplicaSessionPrx session = registry->registerReplica(_info, _internalRegistry);
+        session = registry->registerReplica(_info, _internalRegistry);
         int t = session->getTimeout();
         if(t > 0)
         {
@@ -562,22 +613,18 @@ ReplicaSessionManager::createSessionImpl(const InternalRegistryPrx& registry, Ic
         //
         DatabaseObserverPtr servant = new MasterDatabaseObserverI(_thread, _database, session);
         _observer = DatabaseObserverPrx::uncheckedCast(_database->getInternalAdapter()->addWithUUID(servant));
-        session->setDatabaseObserver(_observer);
+        StringLongDict serials = _database->getSerials();
+        IceUtil::Optional<StringLongDict> serialsOpt;
+        if(!serials.empty())
+        {
+            serialsOpt = serials; // Don't provide serials parameter if serials aren't supported.
+        }
+        session->setDatabaseObserver(_observer, serialsOpt);
         return session;
     }
-    catch(const Ice::LocalException&)
+    catch(const Ice::Exception&)
     {
-        if(_observer)
-        {
-            try
-            {
-                _database->getInternalAdapter()->remove(_observer->ice_getIdentity());
-            }
-            catch(const Ice::LocalException&)
-            {
-            }
-            _observer = 0;
-        }
+        destroySession(session);
         throw;
     }
 }
@@ -585,22 +632,25 @@ ReplicaSessionManager::createSessionImpl(const InternalRegistryPrx& registry, Ic
 void
 ReplicaSessionManager::destroySession(const ReplicaSessionPrx& session)
 {
-    try
+    if(session)
     {
-        session->destroy();
-
-        if(_traceLevels && _traceLevels->replica > 0)
+        try
         {
-            Ice::Trace out(_traceLevels->logger, _traceLevels->replicaCat);
-            out << "destroyed master replica session";
+            session->destroy();
+            
+            if(_traceLevels && _traceLevels->replica > 0)
+            {
+                Ice::Trace out(_traceLevels->logger, _traceLevels->replicaCat);
+                out << "destroyed master replica session";
+            }
         }
-    }
-    catch(const Ice::LocalException& ex)
-    {
-        if(_traceLevels && _traceLevels->replica > 1)
+        catch(const Ice::LocalException& ex)
         {
-            Ice::Trace out(_traceLevels->logger, _traceLevels->replicaCat);
-            out << "couldn't destroy master replica session:\n" << ex;
+            if(_traceLevels && _traceLevels->replica > 1)
+            {
+                Ice::Trace out(_traceLevels->logger, _traceLevels->replicaCat);
+                out << "couldn't destroy master replica session:\n" << ex;
+            }
         }
     }
 
