@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,28 +14,47 @@
 using namespace std;
 using namespace IceGrid;
 
-
-ObserverTopic::ObserverTopic(const IceStorm::TopicManagerPrx& topicManager, const string& name) :
-    _logger(topicManager->ice_getCommunicator()->getLogger()),
-    _serial(0)
+namespace
 {
-    IceStorm::TopicPrx t;
-    try
-    {
-        t = topicManager->create(name);
-    }
-    catch(const IceStorm::TopicExists&)
-    {
-        t = topicManager->retrieve(name);
-    }
 
-    //
-    // NOTE: collocation optimization needs to be turned on for the
-    // topic because the subscribe() method is given a fixed proxy
-    // which can't be marshalled.
-    //
-    _topic = IceStorm::TopicPrx::uncheckedCast(t->ice_collocationOptimized(true));
-    _basePublisher = _topic->getPublisher()->ice_collocationOptimized(false);
+//
+// Encodings supported by the observers. We create one topic per
+// encoding version and subscribe the observer to the appropriate
+// topic depending on its encoding.
+//
+Ice::EncodingVersion encodings[] = {
+    { 1, 0 },
+    { 1, 1 }
+};
+
+}
+
+ObserverTopic::ObserverTopic(const IceStorm::TopicManagerPrx& topicManager, const string& name, Ice::Long dbSerial) :
+    _logger(topicManager->ice_getCommunicator()->getLogger()), _serial(0), _dbSerial(dbSerial)
+{
+    for(int i = 0; i < static_cast<int>(sizeof(encodings) / sizeof(Ice::EncodingVersion)); ++i)
+    {
+        ostringstream os;
+        os << name << "-" << Ice::encodingVersionToString(encodings[i]);
+        IceStorm::TopicPrx t;
+        try
+        {
+            t = topicManager->create(os.str());
+        }
+        catch(const IceStorm::TopicExists&)
+        {
+            t = topicManager->retrieve(os.str());
+        }
+
+        //
+        // NOTE: collocation optimization needs to be turned on for the
+        // topic because the subscribe() method is given a fixed proxy
+        // which can't be marshalled.
+        //
+        _topics[encodings[i]] = t->ice_collocationOptimized(true);
+        _basePublishers.push_back(
+            t->getPublisher()->ice_collocationOptimized(false)->ice_encodingVersion(encodings[i]));
+    }
 }
 
 ObserverTopic::~ObserverTopic()
@@ -46,7 +65,7 @@ int
 ObserverTopic::subscribe(const Ice::ObjectPrx& obsv, const string& name)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
@@ -56,7 +75,15 @@ ObserverTopic::subscribe(const Ice::ObjectPrx& obsv, const string& name)
     {
         IceStorm::QoS qos;
         qos["reliability"] = "ordered";
-        initObserver(_topic->subscribeAndGetPublisher(qos, obsv->ice_twoway()));
+        Ice::EncodingVersion v = IceInternal::getCompatibleEncoding(obsv->ice_getEncodingVersion());
+        map<Ice::EncodingVersion, IceStorm::TopicPrx>::const_iterator p = _topics.find(v);
+        if(p == _topics.end())
+        {
+            Ice::Warning out(_logger);
+            out << "unsupported encoding version for observer `" << obsv << "'";
+            return -1;
+        }
+        initObserver(p->second->subscribeAndGetPublisher(qos, obsv->ice_twoway()));
     }
     catch(const IceStorm::AlreadySubscribed&)
     {
@@ -77,10 +104,13 @@ void
 ObserverTopic::unsubscribe(const Ice::ObjectPrx& observer, const string& name)
 {
     Lock sync(*this);
-    if(_topic)
+    Ice::EncodingVersion v = IceInternal::getCompatibleEncoding(observer->ice_getEncodingVersion());
+    map<Ice::EncodingVersion, IceStorm::TopicPrx>::const_iterator p = _topics.find(v);
+    if(p == _topics.end())
     {
-        _topic->unsubscribe(observer);
+        return;
     }
+    p->second->unsubscribe(observer);
 
     assert(observer);
 
@@ -116,7 +146,7 @@ void
 ObserverTopic::destroy()
 {
     Lock sync(*this);
-    _topic = 0;
+    _topics.clear();
     notifyAll();
 }
 
@@ -155,6 +185,13 @@ ObserverTopic::waitForSyncedSubscribers(int serial, const string& name)
     waitForSyncedSubscribersNoSync(serial, name);
 }
 
+int
+ObserverTopic::getSerial() const
+{
+    Lock sync(*this);
+    return _serial;
+}
+
 void
 ObserverTopic::addExpectedUpdate(int serial, const string& name)
 {
@@ -186,7 +223,7 @@ ObserverTopic::waitForSyncedSubscribersNoSync(int serial, const string& name)
     //
     // Wait until all the updates are received or the service shutdown.
     //
-    while(_topic)
+    while(!_topics.empty())
     {
         map<int, set<string> >::const_iterator p = _waitForUpdates.find(serial);
         if(p == _waitForUpdates.end())
@@ -219,42 +256,55 @@ ObserverTopic::waitForSyncedSubscribersNoSync(int serial, const string& name)
 }
 
 void
-ObserverTopic::updateSerial(int serial)
+ObserverTopic::updateSerial(Ice::Long dbSerial)
 {
-    assert(_serial + 1 == serial);
-    _serial = serial;
+    ++_serial;
+    if(dbSerial > 0)
+    {
+        _dbSerial = dbSerial;
+    }
 }
 
 Ice::Context
-ObserverTopic::getContext(int serial) const
+ObserverTopic::getContext(int serial, Ice::Long dbSerial) const
 {
-    ostringstream os;
-    os << serial;
-
     Ice::Context context;
-    context["serial"] = os.str();
+    {
+        ostringstream os;
+        os << serial;
+        context["serial"] = os.str();
+    }
+    if(dbSerial > 0)
+    {
+        ostringstream os;
+        os << dbSerial;
+        context["dbSerial"] = os.str();
+    }
     return context;
 }
 
 RegistryObserverTopic::RegistryObserverTopic(const IceStorm::TopicManagerPrx& topicManager) : 
     ObserverTopic(topicManager, "RegistryObserver")
 {
-    const_cast<RegistryObserverPrx&>(_publisher) = RegistryObserverPrx::uncheckedCast(_basePublisher);
+    _publishers = getPublishers<RegistryObserverPrx>();
 }
 
 void
 RegistryObserverTopic::registryUp(const RegistryInfo& info)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return;
     }
-    updateSerial(_serial + 1);
+    updateSerial();
     _registries.insert(make_pair(info.name, info));
     try
     {
-        _publisher->registryUp(info);
+        for(vector<RegistryObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->registryUp(info);
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -267,7 +317,7 @@ void
 RegistryObserverTopic::registryDown(const string& name)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return;
     }
@@ -277,11 +327,14 @@ RegistryObserverTopic::registryDown(const string& name)
         return;
     }
 
-    updateSerial(_serial + 1);
+    updateSerial();
     _registries.erase(name);
     try
     {
-        _publisher->registryDown(name);
+        for(vector<RegistryObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->registryDown(name);
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -307,7 +360,7 @@ NodeObserverTopic::NodeObserverTopic(const IceStorm::TopicManagerPrx& topicManag
                                      const Ice::ObjectAdapterPtr& adapter) : 
     ObserverTopic(topicManager, "NodeObserver")
 {
-    const_cast<NodeObserverPrx&>(_publisher) = NodeObserverPrx::uncheckedCast(_basePublisher);
+    _publishers = getPublishers<NodeObserverPrx>();
     try
     {
         const_cast<NodeObserverPrx&>(_externalPublisher) = NodeObserverPrx::uncheckedCast(adapter->addWithUUID(this));
@@ -324,18 +377,21 @@ NodeObserverTopic::nodeInit(const NodeDynamicInfoSeq&, const Ice::Current&)
 }
 
 void
-NodeObserverTopic::nodeUp(const NodeDynamicInfo& info, const Ice::Current& current)
+NodeObserverTopic::nodeUp(const NodeDynamicInfo& info, const Ice::Current&)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return;
     }
-    updateSerial(_serial + 1);
+    updateSerial();
     _nodes.insert(make_pair(info.info.name, info));
     try
     {
-        _publisher->nodeUp(info);
+        for(vector<NodeObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->nodeUp(info);
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -345,7 +401,7 @@ NodeObserverTopic::nodeUp(const NodeDynamicInfo& info, const Ice::Current& curre
 }
 
 void 
-NodeObserverTopic::nodeDown(const string& name, const Ice::Current&)
+NodeObserverTopic::nodeDown(const string& /*name*/, const Ice::Current&)
 {
     assert(false);
 }
@@ -354,7 +410,7 @@ void
 NodeObserverTopic::updateServer(const string& node, const ServerDynamicInfo& server, const Ice::Current&)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return;
     }
@@ -367,7 +423,7 @@ NodeObserverTopic::updateServer(const string& node, const ServerDynamicInfo& ser
         return;
     }
     
-    updateSerial(_serial + 1);
+    updateSerial();
 
     ServerDynamicInfoSeq& servers = _nodes[node].servers;
     ServerDynamicInfoSeq::iterator p = servers.begin();
@@ -394,7 +450,10 @@ NodeObserverTopic::updateServer(const string& node, const ServerDynamicInfo& ser
 
     try
     {
-        _publisher->updateServer(node, server);
+        for(vector<NodeObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->updateServer(node, server);
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -407,7 +466,7 @@ void
 NodeObserverTopic::updateAdapter(const string& node, const AdapterDynamicInfo& adapter, const Ice::Current&)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return;
     }
@@ -420,7 +479,7 @@ NodeObserverTopic::updateAdapter(const string& node, const AdapterDynamicInfo& a
         return;
     }
 
-    updateSerial(_serial + 1);
+    updateSerial();
 
     AdapterDynamicInfoSeq& adapters = _nodes[node].adapters;
     AdapterDynamicInfoSeq::iterator p = adapters.begin();
@@ -447,7 +506,10 @@ NodeObserverTopic::updateAdapter(const string& node, const AdapterDynamicInfo& a
     
     try
     {
-        _publisher->updateAdapter(node, adapter);
+        for(vector<NodeObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->updateAdapter(node, adapter);
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -460,19 +522,22 @@ void
 NodeObserverTopic::nodeDown(const string& name)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return;
     }
 
-    updateSerial(_serial + 1);
+    updateSerial();
 
     if(_nodes.find(name) != _nodes.end())
     {
         _nodes.erase(name);
         try
         {
-            _publisher->nodeDown(name);
+            for(vector<NodeObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+            {
+                (*p)->nodeDown(name);
+            }
         }
         catch(const Ice::LocalException& ex)
         {
@@ -496,22 +561,22 @@ NodeObserverTopic::initObserver(const Ice::ObjectPrx& obsv)
 }
 
 ApplicationObserverTopic::ApplicationObserverTopic(const IceStorm::TopicManagerPrx& topicManager,
-                                                   const map<string, ApplicationInfo>& applications) :
-    ObserverTopic(topicManager, "ApplicationObserver"),
-    _applications(applications)
+                                                   const ApplicationsWrapperPtr& wrapper) :
+    ObserverTopic(topicManager, "ApplicationObserver", wrapper->getSerial()),
+    _applications(wrapper->getMap())
 {
-    const_cast<ApplicationObserverPrx&>(_publisher) = ApplicationObserverPrx::uncheckedCast(_basePublisher);
+    _publishers = getPublishers<ApplicationObserverPrx>();
 }
 
 int
-ApplicationObserverTopic::applicationInit(int serial, const ApplicationInfoSeq& apps)
+ApplicationObserverTopic::applicationInit(Ice::Long dbSerial, const ApplicationInfoSeq& apps)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(serial);
+    updateSerial(dbSerial);
     _applications.clear();
     for(ApplicationInfoSeq::const_iterator p = apps.begin(); p != apps.end(); ++p)
     {
@@ -519,80 +584,89 @@ ApplicationObserverTopic::applicationInit(int serial, const ApplicationInfoSeq&
     }
     try
     {
-        _publisher->applicationInit(serial, apps, getContext(serial));
+        for(vector<ApplicationObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->applicationInit(_serial, apps, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
         Ice::Warning out(_logger);
         out << "unexpected exception while publishing `applicationInit' update:\n" << ex;    
     }
-    addExpectedUpdate(serial);
-    return serial;
+    addExpectedUpdate(_serial);
+    return _serial;
 }
 
 int 
-ApplicationObserverTopic::applicationAdded(int serial, const ApplicationInfo& info)
+ApplicationObserverTopic::applicationAdded(Ice::Long dbSerial, const ApplicationInfo& info)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
 
-    updateSerial(serial);
+    updateSerial(dbSerial);
     _applications.insert(make_pair(info.descriptor.name, info));
     try
     {
-        _publisher->applicationAdded(serial, info, getContext(serial));
+        for(vector<ApplicationObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->applicationAdded(_serial, info, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
         Ice::Warning out(_logger);
         out << "unexpected exception while publishing `applicationAdded' update:\n" << ex;    
     }
-    addExpectedUpdate(serial);
-    return serial;
+    addExpectedUpdate(_serial);
+    return _serial;
 }
 
 int 
-ApplicationObserverTopic::applicationRemoved(int serial, const string& name)
+ApplicationObserverTopic::applicationRemoved(Ice::Long dbSerial, const string& name)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(serial);
+    updateSerial(dbSerial);
     _applications.erase(name);
     try
     {
-        _publisher->applicationRemoved(serial, name, getContext(serial));
+        for(vector<ApplicationObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->applicationRemoved(_serial, name, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
         Ice::Warning out(_logger);
         out << "unexpected exception while publishing `applicationRemoved' update:\n" << ex;    
     }
-    addExpectedUpdate(serial);
-    return serial;
+    addExpectedUpdate(_serial);
+    return _serial;
 }
 
 int 
-ApplicationObserverTopic::applicationUpdated(int serial, const ApplicationUpdateInfo& info)
+ApplicationObserverTopic::applicationUpdated(Ice::Long dbSerial, const ApplicationUpdateInfo& info)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
 
-    updateSerial(serial);
+    updateSerial(dbSerial);
     try
     {
         map<string, ApplicationInfo>::iterator p = _applications.find(info.descriptor.name);
         if(p != _applications.end())
         {
-            ApplicationHelper helper(_publisher->ice_getCommunicator(), p->second.descriptor);
+            ApplicationHelper helper(_publishers[0]->ice_getCommunicator(), p->second.descriptor);
             p->second.descriptor = helper.update(info.descriptor);
             p->second.updateTime = info.updateTime;
             p->second.updateUser = info.updateUser;
@@ -601,34 +675,42 @@ ApplicationObserverTopic::applicationUpdated(int serial, const ApplicationUpdate
     }
     catch(const DeploymentException& ex)
     {
-        cerr << ex.reason << endl;
+        Ice::Error out(_logger);
+        out << "unexpected exception while instantiating application `" << info.descriptor.name << "':\n" << ex.reason;
         assert(false);
     }
     catch(const std::string& msg)
     {
-        cerr << msg << endl;
+        Ice::Error out(_logger);
+        out << "unexpected exception while instantiating application `" << info.descriptor.name << "':\n" << msg;
         assert(false);
     }
     catch(const char* msg)
     {
-        cerr << msg << endl;
+        Ice::Error out(_logger);
+        out << "unexpected exception while instantiating application `" << info.descriptor.name << "':\n" << msg;
         assert(false);
     }
     catch(...)
     {
+        Ice::Error out(_logger);
+        out << "unexpected exception while instantiating application `" << info.descriptor.name << "'";
         assert(false);
     }
     try
     {
-        _publisher->applicationUpdated(serial, info, getContext(serial));
+        for(vector<ApplicationObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->applicationUpdated(_serial, info, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
         Ice::Warning out(_logger);
         out << "unexpected exception while publishing `applicationUpdated' update:\n" << ex;    
     }
-    addExpectedUpdate(serial);
-    return serial;
+    addExpectedUpdate(_serial);
+    return _serial;
 }
 
 void 
@@ -640,26 +722,26 @@ ApplicationObserverTopic::initObserver(const Ice::ObjectPrx& obsv)
     {
         applications.push_back(p->second);
     }
-    observer->applicationInit(_serial, applications, getContext(_serial));
+    observer->applicationInit(_serial, applications, getContext(_serial, _dbSerial));
 }
 
 AdapterObserverTopic::AdapterObserverTopic(const IceStorm::TopicManagerPrx& topicManager,
-                                           const map<string, AdapterInfo>& adapters) :
-    ObserverTopic(topicManager, "AdapterObserver"),
-    _adapters(adapters)
+                                           const AdaptersWrapperPtr& wrapper) :
+    ObserverTopic(topicManager, "AdapterObserver", wrapper->getSerial()),
+    _adapters(wrapper->getMap())
 {
-    const_cast<AdapterObserverPrx&>(_publisher) = AdapterObserverPrx::uncheckedCast(_basePublisher);
+    _publishers = getPublishers<AdapterObserverPrx>();
 }
 
 int 
-AdapterObserverTopic::adapterInit(const AdapterInfoSeq& adpts)
+AdapterObserverTopic::adapterInit(Ice::Long dbSerial, const AdapterInfoSeq& adpts)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _adapters.clear();
     for(AdapterInfoSeq::const_iterator q = adpts.begin(); q != adpts.end(); ++q)
     {
@@ -667,7 +749,10 @@ AdapterObserverTopic::adapterInit(const AdapterInfoSeq& adpts)
     }
     try
     {
-        _publisher->adapterInit(adpts, getContext(_serial));
+        for(vector<AdapterObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->adapterInit(adpts, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -679,18 +764,21 @@ AdapterObserverTopic::adapterInit(const AdapterInfoSeq& adpts)
 }
 
 int 
-AdapterObserverTopic::adapterAdded(const AdapterInfo& info)
+AdapterObserverTopic::adapterAdded(Ice::Long dbSerial, const AdapterInfo& info)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _adapters.insert(make_pair(info.id, info));
     try
     {
-        _publisher->adapterAdded(info, getContext(_serial));
+        for(vector<AdapterObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->adapterAdded(info, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -702,18 +790,21 @@ AdapterObserverTopic::adapterAdded(const AdapterInfo& info)
 }
 
 int 
-AdapterObserverTopic::adapterUpdated(const AdapterInfo& info)
+AdapterObserverTopic::adapterUpdated(Ice::Long dbSerial, const AdapterInfo& info)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _adapters[info.id] = info;
     try
     {
-        _publisher->adapterUpdated(info, getContext(_serial));
+        for(vector<AdapterObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->adapterUpdated(info, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -725,18 +816,21 @@ AdapterObserverTopic::adapterUpdated(const AdapterInfo& info)
 }
 
 int
-AdapterObserverTopic::adapterRemoved(const string& id)
+AdapterObserverTopic::adapterRemoved(Ice::Long dbSerial, const string& id)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _adapters.erase(id);
     try
     {
-        _publisher->adapterRemoved(id, getContext(_serial));
+        for(vector<AdapterObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->adapterRemoved(id, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -756,26 +850,26 @@ AdapterObserverTopic::initObserver(const Ice::ObjectPrx& obsv)
     {
         adapters.push_back(p->second);
     }   
-    observer->adapterInit(adapters, getContext(_serial));
+    observer->adapterInit(adapters, getContext(_serial, _dbSerial));
 }
 
 ObjectObserverTopic::ObjectObserverTopic(const IceStorm::TopicManagerPrx& topicManager,
-                                         const map<Ice::Identity, ObjectInfo>& objects) :
-    ObserverTopic(topicManager, "ObjectObserver"),
-    _objects(objects)
+                                         const ObjectsWrapperPtr& wrapper) :
+    ObserverTopic(topicManager, "ObjectObserver", wrapper->getSerial()),
+    _objects(wrapper->getMap())
 {
-    const_cast<ObjectObserverPrx&>(_publisher) = ObjectObserverPrx::uncheckedCast(_basePublisher);
+    _publishers = getPublishers<ObjectObserverPrx>();
 }
 
 int 
-ObjectObserverTopic::objectInit(const ObjectInfoSeq& objects)
+ObjectObserverTopic::objectInit(Ice::Long dbSerial, const ObjectInfoSeq& objects)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _objects.clear();
     for(ObjectInfoSeq::const_iterator r = objects.begin(); r != objects.end(); ++r)
     {
@@ -783,7 +877,10 @@ ObjectObserverTopic::objectInit(const ObjectInfoSeq& objects)
     }
     try
     {
-        _publisher->objectInit(objects, getContext(_serial));
+        for(vector<ObjectObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->objectInit(objects, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -795,18 +892,21 @@ ObjectObserverTopic::objectInit(const ObjectInfoSeq& objects)
 }
 
 int 
-ObjectObserverTopic::objectAdded(const ObjectInfo& info)
+ObjectObserverTopic::objectAdded(Ice::Long dbSerial, const ObjectInfo& info)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _objects.insert(make_pair(info.proxy->ice_getIdentity(), info));
     try
     {
-        _publisher->objectAdded(info, getContext(_serial));
+        for(vector<ObjectObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->objectAdded(info, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -818,18 +918,21 @@ ObjectObserverTopic::objectAdded(const ObjectInfo& info)
 }
 
 int 
-ObjectObserverTopic::objectUpdated(const ObjectInfo& info)
+ObjectObserverTopic::objectUpdated(Ice::Long dbSerial, const ObjectInfo& info)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _objects[info.proxy->ice_getIdentity()] = info;
     try
     {
-        _publisher->objectUpdated(info, getContext(_serial));
+        for(vector<ObjectObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->objectUpdated(info, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -841,18 +944,21 @@ ObjectObserverTopic::objectUpdated(const ObjectInfo& info)
 }
 
 int
-ObjectObserverTopic::objectRemoved(const Ice::Identity& id)
+ObjectObserverTopic::objectRemoved(Ice::Long dbSerial, const Ice::Identity& id)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
+    updateSerial(dbSerial);
     _objects.erase(id);
     try
     {
-        _publisher->objectRemoved(id, getContext(_serial));
+        for(vector<ObjectObserverPrx>::const_iterator p = _publishers.begin(); p != _publishers.end(); ++p)
+        {
+            (*p)->objectRemoved(id, getContext(_serial, dbSerial));
+        }
     }
     catch(const Ice::LocalException& ex)
     {
@@ -864,24 +970,27 @@ ObjectObserverTopic::objectRemoved(const Ice::Identity& id)
 }
 
 int
-ObjectObserverTopic::objectsAddedOrUpdated(const ObjectInfoSeq& infos)
+ObjectObserverTopic::wellKnownObjectsAddedOrUpdated(const ObjectInfoSeq& infos)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
 
     for(ObjectInfoSeq::const_iterator p = infos.begin(); p != infos.end(); ++p)
     {
+        updateSerial();
         map<Ice::Identity, ObjectInfo>::iterator q = _objects.find(p->proxy->ice_getIdentity());
         if(q != _objects.end())
         {
             q->second = *p;
             try
             {
-                _publisher->objectUpdated(*p, getContext(_serial));
+                for(vector<ObjectObserverPrx>::const_iterator q = _publishers.begin(); q != _publishers.end(); ++q)
+                {
+                    (*q)->objectUpdated(*p, getContext(_serial));
+                }
             }
             catch(const Ice::LocalException& ex)
             {
@@ -894,7 +1003,10 @@ ObjectObserverTopic::objectsAddedOrUpdated(const ObjectInfoSeq& infos)
             _objects.insert(make_pair(p->proxy->ice_getIdentity(), *p));
             try
             {
-                _publisher->objectAdded(*p, getContext(_serial));
+                for(vector<ObjectObserverPrx>::const_iterator q = _publishers.begin(); q != _publishers.end(); ++q)
+                {
+                    (*q)->objectAdded(*p, getContext(_serial));
+                }
             }
             catch(const Ice::LocalException& ex)
             {
@@ -914,21 +1026,24 @@ ObjectObserverTopic::objectsAddedOrUpdated(const ObjectInfoSeq& infos)
 }
 
 int
-ObjectObserverTopic::objectsRemoved(const ObjectInfoSeq& infos)
+ObjectObserverTopic::wellKnownObjectsRemoved(const ObjectInfoSeq& infos)
 {
     Lock sync(*this);
-    if(!_topic)
+    if(_topics.empty())
     {
         return -1;
     }
-    updateSerial(_serial + 1);
 
     for(ObjectInfoSeq::const_iterator p = infos.begin(); p != infos.end(); ++p)
     {
+        updateSerial();
         _objects.erase(p->proxy->ice_getIdentity());
         try
         {
-            _publisher->objectRemoved(p->proxy->ice_getIdentity(), getContext(_serial));
+            for(vector<ObjectObserverPrx>::const_iterator q = _publishers.begin(); q != _publishers.end(); ++q)
+            {
+                (*q)->objectRemoved(p->proxy->ice_getIdentity(), getContext(_serial));
+            }
         }
         catch(const Ice::LocalException& ex)
         {
@@ -956,5 +1071,5 @@ ObjectObserverTopic::initObserver(const Ice::ObjectPrx& obsv)
     {
         objects.push_back(p->second);
     }
-    observer->objectInit(objects, getContext(_serial));
+    observer->objectInit(objects, getContext(_serial, _dbSerial));
 }
