@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -38,7 +38,7 @@ using namespace std;
 using namespace Ice;
 using namespace IceSSL;
 
-IceUtil::Shared* IceInternal::upCast(IceSSL::Instance* p) { return p; }
+IceUtil::Shared* IceSSL::upCast(IceSSL::Instance* p) { return p; }
 
 namespace
 {
@@ -80,7 +80,7 @@ extern "C"
 // OpenSSL mutex callback.
 //
 void
-IceSSL_opensslLockCallback(int mode, int n, const char* file, int line)
+IceSSL_opensslLockCallback(int mode, int n, const char* /*file*/, int /*line*/)
 {
     assert(locks);
     if(mode & CRYPTO_LOCK)
@@ -184,96 +184,121 @@ IceSSL::Instance::Instance(const CommunicatorPtr& communicator) :
         PropertiesPtr properties = communicator->getProperties();
 
         //
-        // Create the mutexes and set the callbacks.
+        // The IceSSL.InitOpenSSL property specifies whether we should perform the global
+        // startup (and shutdown) tasks for the OpenSSL library.
         //
-        if(!locks)
+        // If an application uses multiple components that each depend on OpenSSL, the
+        // application should disable OpenSSL initialization in those components and
+        // perform the initialization itself.
+        //
+        _initOpenSSL = properties->getPropertyAsIntWithDefault("IceSSL.InitOpenSSL", 1) > 0;
+        if(_initOpenSSL)
         {
-            locks = new IceUtil::Mutex[CRYPTO_num_locks()];
-            CRYPTO_set_locking_callback(IceSSL_opensslLockCallback);
-            CRYPTO_set_id_callback(IceSSL_opensslThreadIdCallback);
-        }
+            //
+            // Create the mutexes and set the callbacks.
+            //
+            if(!locks)
+            {
+                locks = new IceUtil::Mutex[CRYPTO_num_locks()];
+                CRYPTO_set_locking_callback(IceSSL_opensslLockCallback);
+                CRYPTO_set_id_callback(IceSSL_opensslThreadIdCallback);
+            }
 
-        //
-        // Load human-readable error messages.
-        //
-        SSL_load_error_strings();
+            //
+            // Load human-readable error messages.
+            //
+            SSL_load_error_strings();
 
-        //
-        // Initialize the SSL library.
-        //
-        SSL_library_init();
+            //
+            // Initialize the SSL library.
+            //
+            SSL_library_init();
 
-        //
-        // This is necessary to allow programs that use OpenSSL 0.9.x to
-        // load private key files generated by OpenSSL 1.x.
-        //
-        OpenSSL_add_all_algorithms();
+            //
+            // This is necessary to allow programs that use OpenSSL 0.9.x to
+            // load private key files generated by OpenSSL 1.x.
+            //
+            OpenSSL_add_all_algorithms();
 
-        //
-        // Initialize the PRNG.
-        //
+            //
+            // Initialize the PRNG.
+            //
 #ifdef WINDOWS
-        RAND_screen(); // Uses data from the screen if possible.
+            RAND_screen(); // Uses data from the screen if possible.
 #endif
-        char randFile[1024];
-        if(RAND_file_name(randFile, sizeof(randFile))) // Gets the name of a default seed file.
-        {
-            RAND_load_file(randFile, 1024);
-        }
-        string randFiles = Ice::nativeToUTF8(communicator, properties->getProperty("IceSSL.Random"));
+            char randFile[1024];
+            if(RAND_file_name(randFile, sizeof(randFile))) // Gets the name of a default seed file.
+            {
+                RAND_load_file(randFile, 1024);
+            }
+            string randFiles = Ice::nativeToUTF8(communicator, properties->getProperty("IceSSL.Random"));
 
-        if(!randFiles.empty())
-        {
-            vector<string> files;
+            if(!randFiles.empty())
+            {
+                vector<string> files;
 #ifdef _WIN32
-            const string sep = ";";
+                const string sep = ";";
 #else
-            const string sep = ":";
+                const string sep = ":";
 #endif
-            string defaultDir = Ice::nativeToUTF8(communicator, properties->getProperty("IceSSL.DefaultDir"));
+                string defaultDir = Ice::nativeToUTF8(communicator, properties->getProperty("IceSSL.DefaultDir"));
 
-            if(!IceUtilInternal::splitString(randFiles, sep, files))
-            {
-                PluginInitializationException ex(__FILE__, __LINE__);
-                ex.reason = "IceSSL: invalid value for IceSSL.Random:\n" + randFiles;
-                throw ex;
-            }
-            for(vector<string>::iterator p = files.begin(); p != files.end(); ++p)
-            {
-                string file = *p;
-                if(!checkPath(file, defaultDir, false))
+                if(!IceUtilInternal::splitString(randFiles, sep, files))
                 {
                     PluginInitializationException ex(__FILE__, __LINE__);
-                    ex.reason = "IceSSL: entropy data file not found:\n" + file;
+                    ex.reason = "IceSSL: invalid value for IceSSL.Random:\n" + randFiles;
                     throw ex;
                 }
-                if(!RAND_load_file(file.c_str(), 1024))
+                for(vector<string>::iterator p = files.begin(); p != files.end(); ++p)
                 {
-                    PluginInitializationException ex(__FILE__, __LINE__);
-                    ex.reason = "IceSSL: unable to load entropy data from " + file;
-                    throw ex;
+                    string file = *p;
+                    if(!checkPath(file, defaultDir, false))
+                    {
+                        PluginInitializationException ex(__FILE__, __LINE__);
+                        ex.reason = "IceSSL: entropy data file not found:\n" + file;
+                        throw ex;
+                    }
+                    if(!RAND_load_file(file.c_str(), 1024))
+                    {
+                        PluginInitializationException ex(__FILE__, __LINE__);
+                        ex.reason = "IceSSL: unable to load entropy data from " + file;
+                        throw ex;
+                    }
                 }
             }
-        }
 #ifndef _WIN32
-        //
-        // The Entropy Gathering Daemon (EGD) is not available on Windows.
-        // The file should be a Unix domain socket for the daemon.
-        //
-        string entropyDaemon = properties->getProperty("IceSSL.EntropyDaemon");
-        if(!entropyDaemon.empty())
-        {
-            if(RAND_egd(entropyDaemon.c_str()) <= 0)
+            //
+            // The Entropy Gathering Daemon (EGD) is not available on Windows.
+            // The file should be a Unix domain socket for the daemon.
+            //
+            string entropyDaemon = properties->getProperty("IceSSL.EntropyDaemon");
+            if(!entropyDaemon.empty())
             {
-                PluginInitializationException ex(__FILE__, __LINE__);
-                ex.reason = "IceSSL: EGD failure using file " + entropyDaemon;
-                throw ex;
+                if(RAND_egd(entropyDaemon.c_str()) <= 0)
+                {
+                    PluginInitializationException ex(__FILE__, __LINE__);
+                    ex.reason = "IceSSL: EGD failure using file " + entropyDaemon;
+                    throw ex;
+                }
+            }
+#endif
+            if(!RAND_status())
+            {
+                communicator->getLogger()->warning("IceSSL: insufficient data to initialize PRNG");
             }
         }
-#endif
-        if(!RAND_status())
+        else
         {
-            communicator->getLogger()->warning("IceSSL: insufficient data to initialize PRNG");
+            if(!properties->getProperty("IceSSL.Random").empty())
+            {
+                _logger->warning("IceSSL: ignoring IceSSL.Random because OpenSSL initialization is disabled");
+            }
+#ifndef _WIN32
+            else if(!properties->getProperty("IceSSL.EntropyDaemon").empty())
+            {
+                _logger->warning("IceSSL: ignoring IceSSL.EntropyDaemon because OpenSSL initialization is disabled");
+            }
+#endif
         }
     }
 
@@ -299,7 +324,7 @@ IceSSL::Instance::~Instance()
     //
     IceUtilInternal::MutexPtrLock<IceUtil::Mutex> sync(staticMutex);
 
-    if(--instanceCount == 0)
+    if(--instanceCount == 0 && _initOpenSSL)
     {
         //
         // NOTE: We can't destroy the locks here: threads which might have called openssl methods
@@ -349,12 +374,17 @@ IceSSL::Instance::initialize()
         //
         _verifyPeer = properties->getPropertyAsIntWithDefault(propPrefix + "VerifyPeer", 2);
 
+        //
+        // Protocols selects which protocols to enable.
+        //
+        const int protocols = parseProtocols(properties->getPropertyAsList(propPrefix + "Protocols"));
+
         //
         // Create an SSL context if the application hasn't supplied one.
         //
         if(!_ctx)
         {
-            _ctx = SSL_CTX_new(SSLv23_method());
+            _ctx = SSL_CTX_new(getMethod(protocols));
             if(!_ctx)
             {
                 PluginInitializationException ex(__FILE__, __LINE__);
@@ -656,13 +686,20 @@ IceSSL::Instance::initialize()
         //
         SSL_CTX_set_session_cache_mode(_ctx, SSL_SESS_CACHE_OFF);
 
+        //
+        // Although we disable session caching, we still need to set a session ID
+        // context (ICE-5103). The value can be anything; here we just use the
+        // pointer to this Instance object.
+        //
+        SSL_CTX_set_session_id_context(_ctx, reinterpret_cast<unsigned char*>(this),
+                                       static_cast<unsigned int>(sizeof(this)));
+
         //
         // Select protocols.
         //
-        StringSeq protocols = properties->getPropertyAsList(propPrefix + "Protocols");
-        if(!protocols.empty())
+        if(protocols != 0)
         {
-            parseProtocols(protocols);
+            setOptions(protocols);
         }
 
         //
@@ -769,12 +806,30 @@ IceSSL::Instance::protocolSupport() const
     return _facade->getProtocolSupport();
 }
 
+bool
+IceSSL::Instance::preferIPv6() const
+{
+    return _facade->preferIPv6();
+}
+
+IceInternal::NetworkProxyPtr
+IceSSL::Instance::networkProxy() const
+{
+    return _facade->getNetworkProxy();
+}
+
 string
 IceSSL::Instance::defaultHost() const
 {
     return _facade->getDefaultHost();
 }
 
+Ice::EncodingVersion
+IceSSL::Instance::defaultEncoding() const
+{
+    return _facade->getDefaultEncoding();
+}
+
 int
 IceSSL::Instance::networkTraceLevel() const
 {
@@ -882,7 +937,7 @@ IceSSL::Instance::verifyPeer(SSL* ssl, SOCKET fd, const string& address, const N
         }
 
         //
-        // Compare the peer's address against the the dnsName and ipAddress
+        // Compare the peer's address against the dnsName and ipAddress
         // values in the subject alternative name.
         //
         if(!certNameOK)
@@ -1088,21 +1143,30 @@ IceSSL::Instance::traceConnection(SSL* ssl, bool incoming)
     out << IceInternal::fdToString(SSL_get_fd(ssl));
 }
 
-void
+int
 IceSSL::Instance::parseProtocols(const StringSeq& protocols)
 {
-    bool sslv3 = false, tlsv1 = false;
+    int v = 0;
+
     for(Ice::StringSeq::const_iterator p = protocols.begin(); p != protocols.end(); ++p)
     {
         string prot = *p;
 
         if(prot == "ssl3" || prot == "sslv3")
         {
-            sslv3 = true;
+            v |= SSLv3;
         }
-        else if(prot == "tls" || prot == "tls1" || prot == "tlsv1")
+        else if(prot == "tls" || prot == "tls1" || prot == "tlsv1" || prot == "tls1_0" || prot == "tlsv1_0")
         {
-            tlsv1 = true;
+            v |= TLSv1_0;
+        }
+        else if(prot == "tls1_1" || prot == "tlsv1_1")
+        {
+            v |= TLSv1_1;
+        }
+        else if(prot == "tls1_2" || prot == "tlsv1_2")
+        {
+            v |= TLSv1_2;
         }
         else
         {
@@ -1112,14 +1176,60 @@ IceSSL::Instance::parseProtocols(const StringSeq& protocols)
         }
     }
 
+    return v;
+}
+
+SSL_METHOD*
+IceSSL::Instance::getMethod(int /*protocols*/)
+{
+    //
+    // Despite its name, the SSLv23 method can negotiate SSL3, TLS1.0, TLS1.1, and TLS1.2.
+    // We use the const_cast for backward compatibility with older OpenSSL releases.
+    //
+    SSL_METHOD* meth = const_cast<SSL_METHOD*>(SSLv23_method());
+
+    /*
+     * Early versions of OpenSSL 1.0.1 would not negotiate a TLS1.2 connection using
+     * the SSLv23 method. You can enable the code below to override the method.
+    if(protocols & TLSv1_2)
+    {
+        meth = const_cast<SSL_METHOD*>(TLSv1_2_method());
+    }
+    */
+
+    return meth;
+}
+
+void
+IceSSL::Instance::setOptions(int protocols)
+{
     long opts = SSL_OP_NO_SSLv2; // SSLv2 is not supported.
-    if(!sslv3)
+    if(!(protocols & SSLv3))
     {
         opts |= SSL_OP_NO_SSLv3;
     }
-    if(!tlsv1)
+    if(!(protocols & TLSv1_0))
     {
         opts |= SSL_OP_NO_TLSv1;
     }
+#ifdef SSL_OP_NO_TLSv1_1
+    if(!(protocols & TLSv1_1))
+    {
+        opts |= SSL_OP_NO_TLSv1_1;
+        //
+        // The value of SSL_OP_NO_TLSv1_1 changed between 1.0.1a and 1.0.1b.
+        //
+        if(SSL_OP_NO_TLSv1_1 == 0x00000400L)
+        {
+            opts |= 0x10000000L; // New value of SSL_OP_NO_TLSv1_1.
+        }
+    }
+#endif
+#ifdef SSL_OP_NO_TLSv1_2
+    if(!(protocols & TLSv1_2))
+    {
+        opts |= SSL_OP_NO_TLSv1_2;
+    }
+#endif
     SSL_CTX_set_options(_ctx, opts);
 }
