@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -23,6 +23,12 @@ struct GroupNodeInfo
     GroupNodeInfo(int i, LogUpdate l, const Ice::ObjectPrx& o = Ice::ObjectPrx());
     bool operator<(const GroupNodeInfo& rhs) const;
     bool operator==(const GroupNodeInfo& rhs) const;
+    //
+    // COMPILER FIX: Clang using libc++ requires to define operator=
+    //
+#if defined(__clang__) && defined(_LIBCPP_VERSION)
+        GroupNodeInfo& operator=(const GroupNodeInfo&);
+#endif
     const int id;
     const LogUpdate llu;
     const Ice::ObjectPrx observer;
