@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -44,11 +44,7 @@ IceXML::ParserException::ice_name() const
 void
 IceXML::ParserException::ice_print(std::ostream& out) const
 {
-#ifdef __BCPLUSPLUS__
-    IceUtil::Exception::ice_print(out);
-#else
     Exception::ice_print(out);
-#endif
     if(!_reason.empty())
     {
         out << "\n" << _reason;
@@ -59,7 +55,7 @@ IceXML::ParserException::ice_print(std::ostream& out) const
     }
 }
 
-IceUtil::Exception*
+IceXML::ParserException*
 IceXML::ParserException::ice_clone() const
 {
     return new ParserException(*this);
@@ -284,7 +280,7 @@ IceXML::DocumentBuilder::startElement(const string& name, const Attributes& attr
 }
 
 void
-IceXML::DocumentBuilder::endElement(const string& name, int, int)
+IceXML::DocumentBuilder::endElement(const string&, int, int)
 {
     assert(!_nodeStack.empty());
     _nodeStack.pop_front();
@@ -327,8 +323,8 @@ startElementHandler(void* data, const XML_Char* name, const XML_Char** attr)
         attributes[attr[i]] = attr[i + 1];
     }
 
-    int line = XML_GetCurrentLineNumber(cb->parser);
-    int column = XML_GetCurrentColumnNumber(cb->parser);
+    int line = static_cast<int>(XML_GetCurrentLineNumber(cb->parser));
+    int column = static_cast<int>(XML_GetCurrentColumnNumber(cb->parser));
     cb->handler->startElement(name, attributes, line, column);
 }
 
@@ -336,8 +332,8 @@ static void
 endElementHandler(void* data, const XML_Char* name)
 {
     CallbackData* cb = static_cast<CallbackData*>(data);
-    int line = XML_GetCurrentLineNumber(cb->parser);
-    int column = XML_GetCurrentColumnNumber(cb->parser);
+    int line = static_cast<int>(XML_GetCurrentLineNumber(cb->parser));
+    int column = static_cast<int>(XML_GetCurrentColumnNumber(cb->parser));
     cb->handler->endElement(name, line, column);
 }
 
@@ -347,8 +343,8 @@ characterDataHandler(void* data, const XML_Char* s, int len)
     CallbackData* cb = static_cast<CallbackData*>(data);
 
     string str(s, len);
-    int line = XML_GetCurrentLineNumber(cb->parser);
-    int column = XML_GetCurrentColumnNumber(cb->parser);
+    int line = static_cast<int>(XML_GetCurrentLineNumber(cb->parser));
+    int column = static_cast<int>(XML_GetCurrentColumnNumber(cb->parser));
     cb->handler->characters(str, line, column);
 }
 
@@ -410,8 +406,9 @@ IceXML::Parser::parse(istream& in, Handler& handler)
             }
             if(XML_Parse(parser, buff, static_cast<int>(in.gcount()), isFinal) != 1)
             {
-                handler.error(XML_ErrorString(XML_GetErrorCode(parser)), XML_GetCurrentLineNumber(parser),
-                              XML_GetCurrentColumnNumber(parser));
+                handler.error(XML_ErrorString(XML_GetErrorCode(parser)), 
+                              static_cast<int>(XML_GetCurrentLineNumber(parser)),
+                              static_cast<int>(XML_GetCurrentColumnNumber(parser)));
                 return;
             }
         }
