@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -136,8 +136,43 @@ Slice::Preprocessor::normalizeIncludePath(const string& path)
     return result;
 }
 
+namespace
+{
+
+vector<string>
+baseArgs(vector<string> args, bool keepComments, const string& extraArgs, const string& fileName)
+{
+    if(keepComments)
+    {
+        args.push_back("-C");
+    }
+    args.push_back("-e");
+    args.push_back("en_us.utf8");
+    
+    //
+    // Define version macros __ICE_VERSION__ is preferred. We keep
+    // ICE_VERSION for backward compatibility with 3.5.0.
+    //
+    const string version[2] = {"ICE_VERSION", "__ICE_VERSION__"};
+    for(int i = 0; i < 2; ++i)
+    {
+        ostringstream os;
+        os << "-D" << version[i] << "=" << ICE_INT_VERSION;
+        args.push_back(os.str());
+    }
+    
+    if(!extraArgs.empty())
+    {
+        args.push_back(extraArgs);
+    }
+    args.push_back(fileName);
+    return args;
+}
+
+}
+
 FILE*
-Slice::Preprocessor::preprocess(bool keepComments)
+Slice::Preprocessor::preprocess(bool keepComments, const string& extraArgs)
 {
     if(!checkInputFile())
     {
@@ -147,15 +182,7 @@ Slice::Preprocessor::preprocess(bool keepComments)
     //
     // Build arguments list.
     //
-    vector<string> args = _args;
-    if(keepComments)
-    {
-        args.push_back("-C");
-    }
-    args.push_back("-e");
-    args.push_back("en_us.utf8");
-    args.push_back(_fileName);
-
+    vector<string> args = baseArgs(_args, keepComments, extraArgs, _fileName);
     const char** argv = new const char*[args.size() + 1];
     argv[0] = "mcpp";
     for(unsigned int i = 0; i < args.size(); ++i)
@@ -249,22 +276,32 @@ Slice::Preprocessor::preprocess(bool keepComments)
 
 bool
 Slice::Preprocessor::printMakefileDependencies(Language lang, const vector<string>& includePaths,
-                                               const string& cppSourceExt, const string& pyPrefix)
+                                               const std::string& extraArgs, const string& cppSourceExt,
+                                               const string& optValue)
 {
     if(!checkInputFile())
     {
         return false;
     }
+    
+    string cppHeaderExt;
+    string pyPrefix;
+    if(lang == CPlusPlus)
+    {
+        cppHeaderExt = optValue;
+    }
+    else if(lang == Python)
+    {
+        pyPrefix = optValue;
+    }
 
     //
     // Build arguments list.
     //
     vector<string> args = _args;
     args.push_back("-M");
-    args.push_back("-e");
-    args.push_back("en_us.utf8");
-    args.push_back(_fileName);
-
+    args = baseArgs(args, false, extraArgs, _fileName);
+   
     const char** argv = new const char*[args.size() + 1];
     for(unsigned int i = 0; i < args.size(); ++i)
     {
@@ -341,8 +378,8 @@ Slice::Preprocessor::printMakefileDependencies(Language lang, const vector<strin
     }
 
     vector<string> fullIncludePaths;
-    vector<string>::const_iterator p;
-    for(p = includePaths.begin(); p != includePaths.end(); ++p)
+
+    for(vector<string>::const_iterator p = includePaths.begin(); p != includePaths.end(); ++p)
     {
         fullIncludePaths.push_back(fullPath(*p));
     }
@@ -452,10 +489,11 @@ Slice::Preprocessor::printMakefileDependencies(Language lang, const vector<strin
             //
             // Change .o[bj] suffix to the cpp source extension suffix.
             //
-            string::size_type pos;
-            while((pos = result.find(suffix)) != string::npos)
+            string::size_type pos = result.find(suffix);
+            if(pos != string::npos)
             {
-                result.replace(pos, suffix.size() - 1, "." + cppSourceExt);
+                string name = result.substr(0, pos);
+                result.replace(0, pos + suffix.size() - 1, name + "." + cppHeaderExt + " " + name + "." + cppSourceExt);
             }
             break;
         }
@@ -514,7 +552,7 @@ Slice::Preprocessor::printMakefileDependencies(Language lang, const vector<strin
             // Change .o[bj] suffix to .cs suffix.
             //
             string::size_type pos;
-            while((pos = result.find(suffix)) != string::npos)
+            if((pos = result.find(suffix)) != string::npos)
             {
                 result.replace(pos, suffix.size() - 1, ".cs");
             }
@@ -530,7 +568,7 @@ Slice::Preprocessor::printMakefileDependencies(Language lang, const vector<strin
                 result = pyPrefix + result;
             }
             string::size_type pos;
-            while((pos = result.find(suffix)) != string::npos)
+            if((pos = result.find(suffix)) != string::npos)
             {
                 result.replace(pos, suffix.size() - 1, "_ice.py");
             }
@@ -542,7 +580,7 @@ Slice::Preprocessor::printMakefileDependencies(Language lang, const vector<strin
             // Change .o[bj] suffix to .rb suffix.
             //
             string::size_type pos;
-            while((pos = result.find(suffix)) != string::npos)
+            if((pos = result.find(suffix)) != string::npos)
             {
                 result.replace(pos, suffix.size() - 1, ".rb");
             }
@@ -554,7 +592,7 @@ Slice::Preprocessor::printMakefileDependencies(Language lang, const vector<strin
             // Change .o[bj] suffix to .php suffix.
             //
             string::size_type pos;
-            while((pos = result.find(suffix)) != string::npos)
+            if((pos = result.find(suffix)) != string::npos)
             {
                 result.replace(pos, suffix.size() - 1, ".php");
             }
