@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2008 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,10 +12,7 @@
 #include <Slice/Util.h>
 #include <IceUtil/Functional.h>
 #include <IceUtil/InputUtil.h>
-
-#ifdef __BCPLUSPLUS__
 #include <iterator>
-#endif
 
 using namespace std;
 using namespace Slice;
@@ -65,24 +62,34 @@ private:
     void writeType(const TypePtr&);
 
     //
-    // Get a default value for a given type.
+    // Get an initializer value for a given type.
     //
-    string getDefaultValue(const TypePtr&);
+    string getInitializer(const TypePtr&);
 
     //
     // Add a value to a hash code.
     //
     void writeHash(const string&, const TypePtr&, int&);
 
+    //
+    // Write a constant value.
+    //
+    void writeConstantValue(const TypePtr&, const SyntaxTreeBasePtr&, const string&);
+
     struct MemberInfo
     {
         string lowerName; // Mapped name beginning with a lower-case letter for use as the name of a local variable.
         string fixedName;
-        TypePtr type;
         bool inherited;
+        DataMemberPtr dataMember;
     };
     typedef list<MemberInfo> MemberInfoList;
 
+    //
+    // Write constructor parameters with default values.
+    //
+    void writeConstructorParams(const MemberInfoList&);
+
     void collectClassMembers(const ClassDefPtr&, MemberInfoList&, bool);
     void collectExceptionMembers(const ExceptionPtr&, MemberInfoList&, bool);
 
@@ -109,9 +116,12 @@ lookupKwd(const string& name)
     static const string keywordList[] = 
     {       
         "BEGIN", "END", "alias", "and", "begin", "break", "case", "class", "clone", "def", "display", "do", "dup",
-        "else", "elsif", "end", "ensure", "extend", "false", "for", "freeze", "hash", "if", "in", "inspect", "method",
-        "methods", "module", "new", "next", "nil", "not", "or", "redo", "rescue", "retry", "return", "self", "send",
-        "super", "taint", "then", "true", "undef", "unless", "untaint", "until", "when", "while", "yield"
+        "else", "elsif", "end", "ensure", "extend", "false", "for", "freeze", "hash", "if", "in", "initialize_copy",
+        "inspect", "instance_eval", "instance_variable_get", "instance_variable_set", "instance_variables", "method",
+        "method_missing", "methods", "module", "new", "next", "nil", "not", "object_id", "or", "private_methods",
+        "protected_methods", "public_methods", "redo", "rescue", "retry", "return", "self", "send",
+        "singleton_methods", "super", "taint", "then", "to_a", "to_s", "true", "undef", "unless", "untaint", "until",
+        "when", "while", "yield"
     };
     bool found =  binary_search(&keywordList[0],
                                 &keywordList[sizeof(keywordList) / sizeof(*keywordList)],
@@ -167,7 +177,7 @@ Slice::Ruby::CodeVisitor::visitModuleStart(const ModulePtr& p)
 }
 
 void
-Slice::Ruby::CodeVisitor::visitModuleEnd(const ModulePtr& p)
+Slice::Ruby::CodeVisitor::visitModuleEnd(const ModulePtr&)
 {
     _out.dec();
     _out << nl << "end";
@@ -208,7 +218,6 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
     ClassList bases = p->bases();
     ClassDefPtr base;
     OperationList ops = p->operations();
-    OperationList::iterator oli;
 
     //
     // Define a mix-in module for the class.
@@ -286,7 +295,7 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
              << nl << "#"
              << nl << "# Operation signatures."
              << nl << "#";
-        for(oli = ops.begin(); oli != ops.end(); ++oli)
+        for(OperationList::iterator oli = ops.begin(); oli != ops.end(); ++oli)
         {
             string fixedOpName = fixIdent((*oli)->name(), IdentNormal);
 /* If AMI/AMD is ever implemented...
@@ -362,10 +371,9 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
     {
         bool prot = p->hasMetaData("protected");
         DataMemberList protectedMembers;
-        DataMemberList::iterator q;
 
         _out << sp << nl << "attr_accessor ";
-        for(q = members.begin(); q != members.end(); ++q)
+        for(DataMemberList::iterator q = members.begin(); q != members.end(); ++q)
         {
             if(q != members.begin())
             {
@@ -381,7 +389,7 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
         if(!protectedMembers.empty())
         {
             _out << nl << "protected ";
-            for(q = protectedMembers.begin(); q != protectedMembers.end(); ++q)
+            for(DataMemberList::iterator q = protectedMembers.begin(); q != protectedMembers.end(); ++q)
             {
                 if(q != protectedMembers.begin())
                 {
@@ -440,28 +448,27 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
         //
         MemberInfoList allMembers;
         collectClassMembers(p, allMembers, false);
-        bool inheritsMembers = false;
         if(!allMembers.empty())
         {
-            _out << sp << nl << "def initialize";
-            _out << spar;
-            MemberInfoList::iterator q;
-            for(q = allMembers.begin(); q != allMembers.end(); ++q)
+            _out << sp << nl << "def initialize(";
+            writeConstructorParams(allMembers);
+            _out << ')';
+            _out.inc();
+
+            bool inheritsMembers = false;
+            for(MemberInfoList::iterator q = allMembers.begin(); q != allMembers.end(); ++q)
             {
-                ostringstream ostr;
-                ostr << q->lowerName << '=' << getDefaultValue(q->type);
-                _out << ostr.str();
                 if(q->inherited)
                 {
                     inheritsMembers = true;
+                    break;
                 }
             }
-            _out << epar;
-            _out.inc();
+
             if(inheritsMembers)
             {
                 _out << nl << "super" << spar;
-                for(q = allMembers.begin(); q != allMembers.end(); ++q)
+                for(MemberInfoList::iterator q = allMembers.begin(); q != allMembers.end(); ++q)
                 {
                     if(q->inherited)
                     {
@@ -470,13 +477,15 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
                 }
                 _out << epar;
             }
-            for(q = allMembers.begin(); q != allMembers.end(); ++q)
+
+            for(MemberInfoList::iterator q = allMembers.begin(); q != allMembers.end(); ++q)
             {
                 if(!q->inherited)
                 {
                     _out << nl << '@' << q->fixedName << " = " << q->lowerName;
                 }
             }
+
             _out.dec();
             _out << nl << "end";
         }
@@ -497,7 +506,7 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
         {
             _out << nl << "include " << getAbsolute(*cli, IdentToUpper) << "Prx_mixin";
         }
-        for(oli = ops.begin(); oli != ops.end(); ++oli)
+        for(OperationList::iterator oli = ops.begin(); oli != ops.end(); ++oli)
         {
             string fixedOpName = fixIdent((*oli)->name(), IdentNormal);
             if(fixedOpName == "checkedCast" || fixedOpName == "uncheckedCast")
@@ -594,8 +603,10 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
     _out << nl << "end";
     _classHistory.insert(scoped); // Avoid redundant declarations.
 
-    _out << sp << nl << "T_" << name << ".defineClass(" << name << ", "
-         << (p->isAbstract() ? "true" : "false") << ", ";
+    bool isAbstract = p->isInterface() || p->allOperations().size() > 0; // Don't use isAbstract() here - see bug 3739
+    const bool preserved = p->hasMetaData("preserve-slice") || p->inheritsMetaData("preserve-slice");
+    _out << sp << nl << "T_" << name << ".defineClass(" << name << ", " << p->compactId() << ", "
+         << (isAbstract ? "true" : "false") << ", " << (preserved ? "true" : "false") << ", ";
     if(!base)
     {
         _out << "nil";
@@ -608,8 +619,6 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
     //
     // Interfaces
     //
-    // TODO: Necessary?
-    //
     {
         int interfaceCount = 0;
         for(ClassList::const_iterator q = bases.begin(); q != bases.end(); ++q)
@@ -630,7 +639,7 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
     //
     // Data members are represented as an array:
     //
-    //   ['MemberName', MemberType]
+    //   ['MemberName', MemberType, Optional, Tag]
     //
     // where MemberType is either a primitive type constant (T_INT, etc.) or the id of a constructed type.
     //
@@ -649,7 +658,8 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
             }
             _out << "['" << fixIdent((*q)->name(), IdentNormal) << "', ";
             writeType((*q)->type());
-            _out << ']';
+            _out << ", " << ((*q)->optional() ? "true" : "false") << ", " << ((*q)->optional() ? (*q)->tag() : 0)
+                 << ']';
         }
     }
     if(members.size() > 1)
@@ -663,7 +673,7 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
     //
     // Define each operation. The arguments to __defineOperation are:
     //
-    // 'opName', Mode, [InParams], [OutParams], ReturnType, [Exceptions]
+    // 'opName', Mode, IsAmd, FormatType, [InParams], [OutParams], ReturnParam, [Exceptions]
     //
     // where InParams and OutParams are arrays of type descriptions, and Exceptions
     // is an array of exception types.
@@ -682,6 +692,19 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
             ParamDeclList params = (*s)->parameters();
             ParamDeclList::iterator t;
             int count;
+            string format;
+            switch((*s)->format())
+            {
+            case DefaultFormat:
+                format = "nil";
+                break;
+            case CompactFormat:
+                format = "::Ice::FormatType::CompactFormat";
+                break;
+            case SlicedFormat:
+                format = "::Ice::FormatType::SlicedFormat";
+                break;
+            }
 
             _out << nl << name << "_mixin::OP_" << (*s)->name() << " = ::Ice::__defineOperation('"
                  << (*s)->name() << "', ";
@@ -710,7 +733,8 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
                 _out << "::Ice::OperationMode::Idempotent";
                 break;
             }
-            _out << ", " << ((p->hasMetaData("amd") || (*s)->hasMetaData("amd")) ? "true" : "false") << ", [";
+            _out << ", " << ((p->hasMetaData("amd") || (*s)->hasMetaData("amd")) ? "true" : "false") << ", " << format
+                 << ", [";
             for(t = params.begin(), count = 0; t != params.end(); ++t)
             {
                 if(!(*t)->isOutParam())
@@ -719,7 +743,10 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
                     {
                         _out << ", ";
                     }
+                    _out << '[';
                     writeType((*t)->type());
+                    _out << ", " << ((*t)->optional() ? "true" : "false") << ", "
+                         << ((*t)->optional() ? (*t)->tag() : 0) << ']';
                     ++count;
                 }
             }
@@ -732,7 +759,10 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
                     {
                         _out << ", ";
                     }
+                    _out << '[';
                     writeType((*t)->type());
+                    _out << ", " << ((*t)->optional() ? "true" : "false") << ", "
+                         << ((*t)->optional() ? (*t)->tag() : 0) << ']';
                     ++count;
                 }
             }
@@ -740,7 +770,15 @@ Slice::Ruby::CodeVisitor::visitClassDefStart(const ClassDefPtr& p)
             TypePtr returnType = (*s)->returnType();
             if(returnType)
             {
+                //
+                // The return type has the same format as an in/out parameter:
+                //
+                // Type, Optional?, OptionalTag
+                //
+                _out << '[';
                 writeType(returnType);
+                _out << ", " << ((*s)->returnIsOptional() ? "true" : "false") << ", "
+                     << ((*s)->returnIsOptional() ? (*s)->returnTag() : 0) << ']';
             }
             else
             {
@@ -805,7 +843,6 @@ Slice::Ruby::CodeVisitor::visitExceptionStart(const ExceptionPtr& p)
     _out.inc();
 
     DataMemberList members = p->dataMembers();
-    DataMemberList::iterator dmli;
 
     //
     // initialize
@@ -816,18 +853,16 @@ Slice::Ruby::CodeVisitor::visitExceptionStart(const ExceptionPtr& p)
     bool inheritsMembers = false;
     if(!allMembers.empty())
     {
-        _out << spar;
+        _out << '(';
+        writeConstructorParams(allMembers);
+        _out << ')';
         for(MemberInfoList::iterator q = allMembers.begin(); q != allMembers.end(); ++q)
         {
-            ostringstream ostr;
-            ostr << q->lowerName << '=' << getDefaultValue(q->type);
-            _out << ostr.str();
             if(q->inherited)
             {
                 inheritsMembers = true;
             }
         }
-        _out << epar;
     }
     _out.inc();
     if(!allMembers.empty())
@@ -864,22 +899,13 @@ Slice::Ruby::CodeVisitor::visitExceptionStart(const ExceptionPtr& p)
     _out.dec();
     _out << nl << "end";
 
-    //
-    // inspect
-    //
-    _out << sp << nl << "def inspect";
-    _out.inc();
-    _out << nl << "return ::Ice::__stringifyException(self)";
-    _out.dec();
-    _out << nl << "end";
-
     //
     // read/write accessors for data members.
     //
     if(!members.empty())
     {
         _out << sp << nl << "attr_accessor ";
-        for(dmli = members.begin(); dmli != members.end(); ++dmli)
+        for(DataMemberList::iterator dmli = members.begin(); dmli != members.end(); ++dmli)
         {
             if(dmli != members.begin())
             {
@@ -895,7 +921,9 @@ Slice::Ruby::CodeVisitor::visitExceptionStart(const ExceptionPtr& p)
     //
     // Emit the type information.
     //
-    _out << sp << nl << "T_" << name << " = ::Ice::__defineException('" << scoped << "', " << name << ", ";
+    const bool preserved = p->hasMetaData("preserve-slice") || p->inheritsMetaData("preserve-slice");
+    _out << sp << nl << "T_" << name << " = ::Ice::__defineException('" << scoped << "', " << name << ", "
+         << (preserved ? "true" : "false") << ", ";
     if(!base)
     {
         _out << "nil";
@@ -913,11 +941,11 @@ Slice::Ruby::CodeVisitor::visitExceptionStart(const ExceptionPtr& p)
     //
     // Data members are represented as an array:
     //
-    //   ['MemberName', MemberType]
+    //   ['MemberName', MemberType, Optional, Tag]
     //
     // where MemberType is either a primitive type constant (T_INT, etc.) or the id of a constructed type.
     //
-    for(dmli = members.begin(); dmli != members.end(); ++dmli)
+    for(DataMemberList::iterator dmli = members.begin(); dmli != members.end(); ++dmli)
     {
         if(dmli != members.begin())
         {
@@ -925,7 +953,8 @@ Slice::Ruby::CodeVisitor::visitExceptionStart(const ExceptionPtr& p)
         }
         _out << "[\"" << fixIdent((*dmli)->name(), IdentNormal) << "\", ";
         writeType((*dmli)->type());
-        _out << ']';
+        _out << ", " << ((*dmli)->optional() ? "true" : "false") << ", " << ((*dmli)->optional() ? (*dmli)->tag() : 0)
+             << ']';
     }
     if(members.size() > 1)
     {
@@ -947,7 +976,6 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
     string scoped = p->scoped();
     string name = fixIdent(p->name(), IdentToUpper);
     MemberInfoList memberList;
-    MemberInfoList::iterator r;
 
     {
         DataMemberList members = p->dataMembers();
@@ -956,7 +984,8 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
             memberList.push_back(MemberInfo());
             memberList.back().lowerName = fixIdent((*q)->name(), IdentToLower);
             memberList.back().fixedName = fixIdent((*q)->name(), IdentNormal);
-            memberList.back().type = (*q)->type();
+            memberList.back().inherited = false;
+            memberList.back().dataMember = *q;
         }
     }
 
@@ -967,17 +996,10 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
     if(!memberList.empty())
     {
         _out << nl << "def initialize(";
-        for(r = memberList.begin(); r != memberList.end(); ++r)
-        {
-            if(r != memberList.begin())
-            {
-                _out << ", ";
-            }
-            _out << r->lowerName << '=' << getDefaultValue(r->type);
-        }
+        writeConstructorParams(memberList);
         _out << ")";
         _out.inc();
-        for(r = memberList.begin(); r != memberList.end(); ++r)
+        for(MemberInfoList::iterator r = memberList.begin(); r != memberList.end(); ++r)
         {
             _out << nl << '@' << r->fixedName << " = " << r->lowerName;
         }
@@ -992,9 +1014,9 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
     _out.inc();
     _out << nl << "_h = 0";
     int iter = 0;
-    for(r = memberList.begin(); r != memberList.end(); ++r)
+    for(MemberInfoList::iterator r = memberList.begin(); r != memberList.end(); ++r)
     {
-        writeHash("@" + r->fixedName, r->type, iter);
+        writeHash("@" + r->fixedName, r->dataMember->type(), iter);
     }
     _out << nl << "_h % 0x7fffffff";
     _out.dec();
@@ -1007,7 +1029,7 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
     _out.inc();
     _out << nl << "return false if";
     _out.inc();
-    for(r = memberList.begin(); r != memberList.end(); ++r)
+    for(MemberInfoList::iterator r = memberList.begin(); r != memberList.end(); ++r)
     {
         if(r != memberList.begin())
         {
@@ -1020,6 +1042,17 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
     _out.dec();
     _out << nl << "end";
 
+    //
+    // eql?
+    //
+    // This method is used to determine the equality of keys in a Hash object.
+    //
+    _out << sp << nl << "def eql?(other)";
+    _out.inc();
+    _out << nl << "return other.class == self.class && other == self";
+    _out.dec();
+    _out << nl << "end";
+
     //
     // inspect
     //
@@ -1035,7 +1068,7 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
     if(!memberList.empty())
     {
         _out << sp << nl << "attr_accessor ";
-        for(r = memberList.begin(); r != memberList.end(); ++r)
+        for(MemberInfoList::iterator r = memberList.begin(); r != memberList.end(); ++r)
         {
             if(r != memberList.begin())
             {
@@ -1064,14 +1097,14 @@ Slice::Ruby::CodeVisitor::visitStructStart(const StructPtr& p)
         _out.inc();
         _out << nl;
     }
-    for(r = memberList.begin(); r != memberList.end(); ++r)
+    for(MemberInfoList::iterator r = memberList.begin(); r != memberList.end(); ++r)
     {
         if(r != memberList.begin())
         {
             _out << ',' << nl;
         }
         _out << "[\"" << r->fixedName << "\", ";
-        writeType(r->type);
+        writeType(r->dataMember->type());
         _out << ']';
     }
     if(memberList.size() > 1)
@@ -1129,23 +1162,16 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     string scoped = p->scoped();
     string name = fixIdent(p->name(), IdentToUpper);
     EnumeratorList enums = p->getEnumerators();
-    EnumeratorList::iterator q;
-    int i;
 
     _out << sp << nl << "if not defined?(" << getAbsolute(p, IdentToUpper) << ')';
     _out.inc();
     _out << nl << "class " << name;
     _out.inc();
     _out << nl << "include Comparable";
-    _out << sp << nl << "def initialize(val)";
+    _out << sp << nl << "def initialize(name, value)";
     _out.inc();
-    {
-        ostringstream assertion;
-        assertion << "fail(\"invalid value #{val} for " << name << "\") unless(val >= 0 and val < "
-                  << enums.size() << ')';
-        _out << nl << assertion.str();
-    }
-    _out << nl << "@val = val";
+    _out << nl << "@name = name";
+    _out << nl << "@value = value";
     _out.dec();
     _out << nl << "end";
 
@@ -1157,9 +1183,7 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
         ostringstream sz;
         sz << enums.size() - 1;
         _out.inc();
-        _out << nl << "raise IndexError, \"#{val} is out of range 0.." << sz.str() << "\" if(val < 0 || val > "
-             << sz.str() << ')';
-        _out << nl << "@@_values[val]";
+        _out << nl << "@@_enumerators[val]"; // Evaluates to nil if the key is not found
         _out.dec();
         _out << nl << "end";
     }
@@ -1169,7 +1193,7 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     //
     _out << sp << nl << "def to_s";
     _out.inc();
-    _out << nl << "@@_names[@val]";
+    _out << nl << "@name";
     _out.dec();
     _out << nl << "end";
 
@@ -1178,7 +1202,7 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     //
     _out << sp << nl << "def to_i";
     _out.inc();
-    _out << nl << "@val";
+    _out << nl << "@value";
     _out.dec();
     _out << nl << "end";
 
@@ -1188,7 +1212,7 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     _out << sp << nl << "def <=>(other)";
     _out.inc();
     _out << nl << "other.is_a?(" << name << ") or raise ArgumentError, \"value must be a " << name << "\"";
-    _out << nl << "@val <=> other.to_i";
+    _out << nl << "@value <=> other.to_i";
     _out.dec();
     _out << nl << "end";
 
@@ -1197,7 +1221,7 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     //
     _out << sp << nl << "def hash";
     _out.inc();
-    _out << nl << "@val.hash";
+    _out << nl << "@value.hash";
     _out.dec();
     _out << nl << "end";
 
@@ -1206,7 +1230,7 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     //
     _out << sp << nl << "def inspect";
     _out.inc();
-    _out << nl << "@@_names[@val] + \"(#{@val})\"";
+    _out << nl << "@name + \"(#{@value})\"";
     _out.dec();
     _out << nl << "end";
 
@@ -1215,45 +1239,40 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     //
     _out << sp << nl << "def " << name << ".each(&block)";
     _out.inc();
-    _out << nl << "@@_values.each(&block)";
+    _out << nl << "@@_enumerators.each_value(&block)";
     _out.dec();
     _out << nl << "end";
 
-    _out << sp << nl << "@@_names = [";
-    for(q = enums.begin(); q != enums.end(); ++q)
-    {
-        if(q != enums.begin())
-        {
-            _out << ", ";
-        }
-        _out << "'" << (*q)->name() << "'";
-    }
-    _out << ']';
-
-    _out << nl << "@@_values = [";
-    for(EnumeratorList::size_type j = 0; j < enums.size(); ++j)
-    {
-        if(j > 0)
-        {
-            _out << ", ";
-        }
-        ostringstream idx;
-        idx << j;
-        _out << name << ".new(" << idx.str() << ')';
-    }
-    _out << ']';
-
     //
     // Constant for each enumerator.
     //
     _out << sp;
-    for(q = enums.begin(), i = 0; q != enums.end(); ++q, ++i)
+    int i = 0;
+    for(EnumeratorList::iterator q = enums.begin(); q != enums.end(); ++q, ++i)
     {
         ostringstream idx;
         idx << i;
-        _out << nl << fixIdent((*q)->name(), IdentToUpper) << " = @@_values[" << idx.str() << "]";
+        _out << nl << fixIdent((*q)->name(), IdentToUpper) << " = " << name << ".new(\"" << (*q)->name()
+             << "\", " << (*q)->value() << ')';
     }
 
+    _out << sp << nl << "@@_enumerators = {";
+    for(EnumeratorList::iterator q = enums.begin(); q != enums.end(); ++q)
+    {
+        if(q != enums.begin())
+        {
+            _out << ", ";
+        }
+        _out << (*q)->value() << "=>" << fixIdent((*q)->name(), IdentToUpper);
+    }
+    _out << '}';
+
+    _out << sp << nl << "def " << name << "._enumerators";
+    _out.inc();
+    _out << nl << "@@_enumerators";
+    _out.dec();
+    _out << nl << "end";
+
     _out << sp << nl << "private_class_method :new";
 
     _out.dec();
@@ -1262,16 +1281,8 @@ Slice::Ruby::CodeVisitor::visitEnum(const EnumPtr& p)
     //
     // Emit the type information.
     //
-    _out << sp << nl << "T_" << name << " = ::Ice::__defineEnum('" << scoped << "', " << name << ", [";
-    for(q = enums.begin(); q != enums.end(); ++q)
-    {
-        if(q != enums.begin())
-        {
-            _out << ", ";
-        }
-        _out << name << "::" << fixIdent((*q)->name(), IdentToUpper);
-    }
-    _out << "])";
+    _out << sp << nl << "T_" << name << " = ::Ice::__defineEnum('" << scoped << "', " << name << ", " << name
+         << "::_enumerators)";
 
     _out.dec();
     _out << nl << "end"; // if not defined?()
@@ -1281,141 +1292,10 @@ void
 Slice::Ruby::CodeVisitor::visitConst(const ConstPtr& p)
 {
     Slice::TypePtr type = p->type();
-    string value = p->value();
     string name = fixIdent(p->name(), IdentToUpper);
 
     _out << sp << nl << name << " = ";
-
-    Slice::BuiltinPtr b = Slice::BuiltinPtr::dynamicCast(type);
-    Slice::EnumPtr en = Slice::EnumPtr::dynamicCast(type);
-    if(b)
-    {
-        switch(b->kind())
-        {
-        case Slice::Builtin::KindBool:
-        {
-            _out << value;
-            break;
-        }
-        case Slice::Builtin::KindByte:
-        case Slice::Builtin::KindShort:
-        case Slice::Builtin::KindInt:
-        case Slice::Builtin::KindFloat:
-        case Slice::Builtin::KindDouble:
-        {
-            _out << value;
-            break;
-        }
-        case Slice::Builtin::KindLong:
-        {
-            IceUtil::Int64 l;
-            IceUtilInternal::stringToInt64(value, l);
-            _out << value;
-            break;
-        }
-
-        case Slice::Builtin::KindString:
-        {
-            //
-            // Expand strings into the basic source character set. We can't use isalpha() and the like
-            // here because they are sensitive to the current locale.
-            //
-            static const string basicSourceChars = "abcdefghijklmnopqrstuvwxyz"
-                                                   "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-                                                   "0123456789"
-                                                   "_{}[]#()<>%:;.?*+-/^&|~!=, '";
-            static const set<char> charSet(basicSourceChars.begin(), basicSourceChars.end());
-
-            _out << "\"";                                      // Opening "
-
-            for(string::const_iterator c = value.begin(); c != value.end(); ++c)
-            {
-                switch(*c)
-                {
-                case '"':
-                {
-                    _out << "\\\"";
-                    break;
-                }
-                case '\\':
-                {
-                    _out << "\\\\";
-                    break;
-                }
-                case '\r':
-                {
-                    _out << "\\r";
-                    break;
-                }
-                case '\n':
-                {
-                    _out << "\\n";
-                    break;
-                }
-                case '\t':
-                {
-                    _out << "\\t";
-                    break;
-                }
-                case '\b':
-                {
-                    _out << "\\b";
-                    break;
-                }
-                case '\f':
-                {
-                    _out << "\\f";
-                    break;
-                }
-                default:
-                {
-                    if(charSet.find(*c) == charSet.end())
-                    {
-                        unsigned char uc = *c;            // Char may be signed, so make it positive.
-                        stringstream s;
-                        s << "\\";                            // Print as octal if not in basic source character set.
-                        s.flags(ios_base::oct);
-                        s.width(3);
-                        s.fill('0');
-                        s << static_cast<unsigned>(uc);
-                        _out << s.str();
-                    }
-                    else
-                    {
-                        _out << *c;                          // Print normally if in basic source character set.
-                    }
-                    break;
-                }
-                }
-            }
-
-            _out << "\"";                                      // Closing "
-            break;
-        }
-
-        case Slice::Builtin::KindObject:
-        case Slice::Builtin::KindObjectProxy:
-        case Slice::Builtin::KindLocalObject:
-            assert(false);
-        }
-    }
-    else if(en)
-    {
-        _out << getAbsolute(en, IdentToUpper) << "::";
-        string::size_type colon = value.rfind(':');
-        if(colon != string::npos)
-        {
-            _out << fixIdent(value.substr(colon + 1), IdentToUpper);
-        }
-        else
-        {
-            _out << fixIdent(value, IdentToUpper);
-        }
-    }
-    else
-    {
-        assert(false); // Unknown const type.
-    }
+    writeConstantValue(type, p->valueType(), p->value());
 }
 
 void
@@ -1498,7 +1378,7 @@ Slice::Ruby::CodeVisitor::writeType(const TypePtr& p)
 }
 
 string
-Slice::Ruby::CodeVisitor::getDefaultValue(const TypePtr& p)
+Slice::Ruby::CodeVisitor::getInitializer(const TypePtr& p)
 {
     BuiltinPtr builtin = BuiltinPtr::dynamicCast(p);
     if(builtin)
@@ -1551,11 +1431,177 @@ Slice::Ruby::CodeVisitor::getDefaultValue(const TypePtr& p)
 }
 
 void
-Slice::Ruby::CodeVisitor::writeHash(const string& name, const TypePtr& p, int& iter)
+Slice::Ruby::CodeVisitor::writeHash(const string& name, const TypePtr&, int&)
 {
     _out << nl << "_h = 5 * _h + " << name << ".hash";
 }
 
+void
+Slice::Ruby::CodeVisitor::writeConstantValue(const TypePtr& type, const SyntaxTreeBasePtr& valueType,
+                                             const string& value)
+{
+    ConstPtr constant = ConstPtr::dynamicCast(valueType);
+    if(constant)
+    {
+        _out << fixIdent(constant->scoped(), IdentToUpper);
+    }
+    else
+    {
+        Slice::BuiltinPtr b = Slice::BuiltinPtr::dynamicCast(type);
+        Slice::EnumPtr en = Slice::EnumPtr::dynamicCast(type);
+        if(b)
+        {
+            switch(b->kind())
+            {
+            case Slice::Builtin::KindBool:
+            case Slice::Builtin::KindByte:
+            case Slice::Builtin::KindShort:
+            case Slice::Builtin::KindInt:
+            case Slice::Builtin::KindFloat:
+            case Slice::Builtin::KindDouble:
+            {
+                _out << value;
+                break;
+            }
+            case Slice::Builtin::KindLong:
+            {
+                IceUtil::Int64 l;
+                IceUtilInternal::stringToInt64(value, l);
+                _out << value;
+                break;
+            }
+            case Slice::Builtin::KindString:
+            {
+                //
+                // Expand strings into the basic source character set. We can't use isalpha() and the like
+                // here because they are sensitive to the current locale.
+                //
+                static const string basicSourceChars = "abcdefghijklmnopqrstuvwxyz"
+                                                       "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                                                       "0123456789"
+                                                       "_{}[]#()<>%:;.?*+-/^&|~!=, '";
+                static const set<char> charSet(basicSourceChars.begin(), basicSourceChars.end());
+
+                _out << "\"";                                      // Opening "
+
+                for(string::const_iterator c = value.begin(); c != value.end(); ++c)
+                {
+                    switch(*c)
+                    {
+                    case '"':
+                    {
+                        _out << "\\\"";
+                        break;
+                    }
+                    case '\\':
+                    {
+                        _out << "\\\\";
+                        break;
+                    }
+                    case '\r':
+                    {
+                        _out << "\\r";
+                        break;
+                    }
+                    case '\n':
+                    {
+                        _out << "\\n";
+                        break;
+                    }
+                    case '\t':
+                    {
+                        _out << "\\t";
+                        break;
+                    }
+                    case '\b':
+                    {
+                        _out << "\\b";
+                        break;
+                    }
+                    case '\f':
+                    {
+                        _out << "\\f";
+                        break;
+                    }
+                    default:
+                    {
+                        if(charSet.find(*c) == charSet.end())
+                        {
+                            unsigned char uc = *c;              // Char may be signed, so make it positive.
+                            stringstream s;
+                            s << "\\";                          // Print as octal if not in basic source character set.
+                            s.flags(ios_base::oct);
+                            s.width(3);
+                            s.fill('0');
+                            s << static_cast<unsigned>(uc);
+                            _out << s.str();
+                        }
+                        else
+                        {
+                            _out << *c;                         // Print normally if in basic source character set.
+                        }
+                        break;
+                    }
+                    }
+                }
+
+                _out << "\"";                                   // Closing "
+                break;
+            }
+
+            case Slice::Builtin::KindObject:
+            case Slice::Builtin::KindObjectProxy:
+            case Slice::Builtin::KindLocalObject:
+                assert(false);
+            }
+        }
+        else if(en)
+        {
+            _out << getAbsolute(en, IdentToUpper) << "::";
+            string::size_type colon = value.rfind(':');
+            if(colon != string::npos)
+            {
+                _out << fixIdent(value.substr(colon + 1), IdentToUpper);
+            }
+            else
+            {
+                _out << fixIdent(value, IdentToUpper);
+            }
+        }
+        else
+        {
+            assert(false); // Unknown const type.
+        }
+    }
+}
+
+void
+Slice::Ruby::CodeVisitor::writeConstructorParams(const MemberInfoList& members)
+{
+    for(MemberInfoList::const_iterator p = members.begin(); p != members.end(); ++p)
+    {
+        if(p != members.begin())
+        {
+            _out << ", ";
+        }
+        _out << p->lowerName << "=";
+
+        const DataMemberPtr member = p->dataMember;
+        if(member->defaultValueType())
+        {
+            writeConstantValue(member->type(), member->defaultValueType(), member->defaultValue());
+        }
+        else if(member->optional())
+        {
+            _out << "::Ice::Unset";
+        }
+        else
+        {
+            _out << getInitializer(member->type());
+        }
+    }
+}
+
 void
 Slice::Ruby::CodeVisitor::collectClassMembers(const ClassDefPtr& p, MemberInfoList& allMembers, bool inherited)
 {
@@ -1572,8 +1618,8 @@ Slice::Ruby::CodeVisitor::collectClassMembers(const ClassDefPtr& p, MemberInfoLi
         MemberInfo m;
         m.lowerName = fixIdent((*q)->name(), IdentToLower);
         m.fixedName = fixIdent((*q)->name(), IdentNormal);
-        m.type = (*q)->type();
         m.inherited = inherited;
+        m.dataMember = *q;
         allMembers.push_back(m);
     }
 }
@@ -1594,8 +1640,8 @@ Slice::Ruby::CodeVisitor::collectExceptionMembers(const ExceptionPtr& p, MemberI
         MemberInfo m;
         m.lowerName = fixIdent((*q)->name(), IdentToLower);
         m.fixedName = fixIdent((*q)->name(), IdentNormal);
-        m.type = (*q)->type();
         m.inherited = inherited;
+        m.dataMember = *q;
         allMembers.push_back(m);
     }
 }
@@ -1728,7 +1774,7 @@ Slice::Ruby::printHeader(IceUtilInternal::Output& out)
     static const char* header =
 "# **********************************************************************\n"
 "#\n"
-"# Copyright (c) 2003-2008 ZeroC, Inc. All rights reserved.\n"
+"# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.\n"
 "#\n"
 "# This copy of Ice is licensed to you under the terms described in the\n"
 "# ICE_LICENSE file included in this distribution.\n"
@@ -1737,5 +1783,7 @@ Slice::Ruby::printHeader(IceUtilInternal::Output& out)
         ;
 
     out << header;
-    out << "\n# Ice version " << ICE_STRING_VERSION;
+    out << "#\n";
+    out << "# Ice version " << ICE_STRING_VERSION << "\n";
+    out << "#\n";
 }
