@@ -1,31 +1,19 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-#define _WIN32_WINNT 0x0500
-
-#if defined(_MSC_VER) && _MSC_VER >= 1400
-#    define _CRT_SECURE_NO_DEPRECATE 1  // C4996 '<C function>' was declared deprecated
-#endif
-
+#include <IceUtil/DisableWarnings.h>
 #include <ServiceInstaller.h>
 #include <IceUtil/StringUtil.h>
 #include <IceUtil/FileUtil.h>
 
-
 #include <Aclapi.h>
-
-#if defined(_MSC_VER) && _MSC_VER >= 1300
-//
-// The VC6 headers don't include Sddl.h
-//
 #include <Sddl.h>
-#endif
 
 using namespace std;
 using namespace Ice;
@@ -67,8 +55,7 @@ IceServiceInstaller::IceServiceInstaller(int serviceType, const string& configFi
     //
     // Compute _serviceName
     //
-    _defaultLocator = LocatorPrx::uncheckedCast(
-        _communicator->stringToProxy(_serviceProperties->getProperty("Ice.Default.Locator")));
+    
 
     if(_serviceType == icegridregistry)
     {
@@ -77,9 +64,11 @@ IceServiceInstaller::IceServiceInstaller(int serviceType, const string& configFi
     }
     else
     {
-        if(_defaultLocator != 0)
+        Ice::LocatorPrx defaultLocator = LocatorPrx::uncheckedCast(
+            _communicator->stringToProxy(_serviceProperties->getProperty("Ice.Default.Locator")));
+        if(defaultLocator != 0)
         {
-            _icegridInstanceName = _defaultLocator->ice_getIdentity().category;
+            _icegridInstanceName = defaultLocator->ice_getIdentity().category;
         }
 
         if(_serviceType == icegridnode)
@@ -418,25 +407,26 @@ IceServiceInstaller::initializeSid(const string& name)
 {
     {
         DWORD sidSize = 32;
-        _sid = auto_ptr<SID>(new SID[sidSize]);
+        _sidBuffer.reset(new IceUtil::Byte[sidSize]);
 
         DWORD domainNameSize = 32;
-        auto_ptr<wchar_t> domainName(new wchar_t[domainNameSize]);
+        IceUtil::ScopedArray<wchar_t> domainName(new wchar_t[domainNameSize]);
 
         SID_NAME_USE nameUse;
-        while(LookupAccountNameW(0, IceUtil::stringToWstring(name).c_str(), _sid.get(), &sidSize, domainName.get(),
+        while(LookupAccountNameW(0, IceUtil::stringToWstring(name).c_str(), _sidBuffer.get(), &sidSize, domainName.get(),
               &domainNameSize, &nameUse) == false)
         {
             DWORD res = GetLastError();
 
             if(res == ERROR_INSUFFICIENT_BUFFER)
             {
-                _sid =  auto_ptr<SID>(new SID[sidSize]);
-                domainName  = auto_ptr<wchar_t>(new wchar_t[domainNameSize]);
+                _sidBuffer.reset(new IceUtil::Byte[sidSize]);
+                domainName.reset(new wchar_t[domainNameSize]);
                 continue;
             }
             throw "Could not retrieve Security ID for " + name + ": " + IceUtilInternal::errorToString(res);
         }
+        _sid = reinterpret_cast<SID*>(_sidBuffer.get());
     }
 
     //
@@ -462,7 +452,7 @@ IceServiceInstaller::initializeSid(const string& name)
         DWORD domainLen = 1024;
 
         SID_NAME_USE nameUse;
-        if(LookupAccountSidW(0, _sid.get(), accountName, &accountNameLen, domainName, &domainLen, &nameUse) == false)
+        if(LookupAccountSidW(0, _sid, accountName, &accountNameLen, domainName, &domainLen, &nameUse) == false)
         {
             DWORD res = GetLastError();
             throw "Could not retrieve full account name for " + name + ": " + IceUtilInternal::errorToString(res);
@@ -474,13 +464,10 @@ IceServiceInstaller::initializeSid(const string& name)
     if(_debug)
     {
         Trace trace(_communicator->getLogger(), "IceServiceInstaller");
-
-#if defined(_MSC_VER) && _MSC_VER >= 1300
         wchar_t* sidString = 0;
-        ConvertSidToStringSidW(_sid.get(), &sidString);
+        ConvertSidToStringSidW(_sid, &sidString);
         trace << "SID: " << IceUtil::wstringToString(sidString) << "; ";
         LocalFree(sidString);
-#endif
         trace << "Full name: " << _sidName;
     }
 }
@@ -535,7 +522,7 @@ IceServiceInstaller::grantPermissions(const string& path, SE_OBJECT_TYPE type, b
         // Now check if _sid can read this file/dir/key
         //
         TRUSTEE_W trustee;
-        BuildTrusteeWithSidW(&trustee, _sid.get());
+        BuildTrusteeWithSidW(&trustee, _sid);
 
         ACCESS_MASK accessMask = 0;
         res = GetEffectiveRightsFromAclW(acl, &trustee, &accessMask);
