@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -40,7 +40,7 @@ using namespace Slice::Python;
 namespace
 {
 
-IceUtil::Mutex* mutex = 0;
+IceUtil::Mutex* globalMutex = 0;
 bool interrupted = false;
 
 class Init
@@ -49,13 +49,13 @@ public:
 
     Init()
     {
-        mutex = new IceUtil::Mutex;
+        globalMutex = new IceUtil::Mutex;
     }
 
     ~Init()
     {
-        delete mutex;
-        mutex = 0;
+        delete globalMutex;
+        globalMutex = 0;
     }
 };
 
@@ -64,9 +64,9 @@ Init init;
 }
 
 void
-interruptedCallback(int signal)
+interruptedCallback(int /*signal*/)
 {
-    IceUtilInternal::MutexPtrLock<IceUtil::Mutex> sync(mutex);
+    IceUtilInternal::MutexPtrLock<IceUtil::Mutex> sync(globalMutex);
 
     interrupted = true;
 }
@@ -335,8 +335,6 @@ PackageVisitor::writeInit(const string& dir, const string& name, const StringLis
     }
     FileTracker::instance()->addFile(initPath);
 
-    StringList::const_iterator p;
-
     os << "# Generated by slice2py - DO NOT EDIT!" << endl
        << "#" << endl;
     os << endl
@@ -344,14 +342,14 @@ PackageVisitor::writeInit(const string& dir, const string& name, const StringLis
        << "Ice.updateModule(\"" << name << "\")" << endl
        << endl;
     os << _moduleTag << endl;
-    for(p = modules.begin(); p != modules.end(); ++p)
+    for(StringList::const_iterator p = modules.begin(); p != modules.end(); ++p)
     {
         os << "import " << *p << endl;
     }
 
     os << endl;
     os << _submoduleTag << endl;
-    for(p = submodules.begin(); p != submodules.end(); ++p)
+    for(StringList::const_iterator p = submodules.begin(); p != submodules.end(); ++p)
     {
         os << "import " << *p << endl;
     }
@@ -427,20 +425,19 @@ compile(int argc, char* argv[])
 
     vector<string> cppArgs;
     vector<string> optargs = opts.argVec("D");
-    vector<string>::const_iterator i;
-    for(i = optargs.begin(); i != optargs.end(); ++i)
+    for(vector<string>::const_iterator i = optargs.begin(); i != optargs.end(); ++i)
     {
         cppArgs.push_back("-D" + *i);
     }
 
     optargs = opts.argVec("U");
-    for(i = optargs.begin(); i != optargs.end(); ++i)
+    for(vector<string>::const_iterator i = optargs.begin(); i != optargs.end(); ++i)
     {
         cppArgs.push_back("-U" + *i);
     }
 
     vector<string> includePaths = opts.argVec("I");
-    for(i = includePaths.begin(); i != includePaths.end(); ++i)
+    for(vector<string>::const_iterator i = includePaths.begin(); i != includePaths.end(); ++i)
     {
         cppArgs.push_back("-I" + Preprocessor::normalizeIncludePath(*i));
     }
@@ -479,7 +476,7 @@ compile(int argc, char* argv[])
 
     bool keepComments = true;
 
-    for(i = args.begin(); i != args.end(); ++i)
+    for(vector<string>::const_iterator i = args.begin(); i != args.end(); ++i)
     {
         //
         // Ignore duplicates.
@@ -493,7 +490,7 @@ compile(int argc, char* argv[])
         if(depend)
         {
             PreprocessorPtr icecpp = Preprocessor::create(argv[0], *i, cppArgs);
-            FILE* cppHandle = icecpp->preprocess(false);
+            FILE* cppHandle = icecpp->preprocess(false, "-D__SLICE2PY__");
 
             if(cppHandle == 0)
             {
@@ -509,7 +506,8 @@ compile(int argc, char* argv[])
                 return EXIT_FAILURE;
             }
 
-            if(!icecpp->printMakefileDependencies(Preprocessor::Python, includePaths, "", prefix))
+            if(!icecpp->printMakefileDependencies(Preprocessor::Python, includePaths, 
+                                                  "-D__SLICE2PY__", "", prefix))
             {
                 return EXIT_FAILURE;
             }
@@ -522,7 +520,7 @@ compile(int argc, char* argv[])
         else
         {
             PreprocessorPtr icecpp = Preprocessor::create(argv[0], *i, cppArgs);
-            FILE* cppHandle = icecpp->preprocess(keepComments);
+            FILE* cppHandle = icecpp->preprocess(keepComments, "-D__SLICE2PY__");
 
             if(cppHandle == 0)
             {
@@ -631,7 +629,7 @@ compile(int argc, char* argv[])
         }
 
         {
-            IceUtilInternal::MutexPtrLock<IceUtil::Mutex> sync(mutex);
+            IceUtilInternal::MutexPtrLock<IceUtil::Mutex> sync(globalMutex);
 
             if(interrupted)
             {
