@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -35,7 +35,7 @@ SessionI::SessionI(const Glacier2::SessionControlPrx& sessionControl, const Test
 }
 
 void
-SessionI::destroySession(const Ice::Current& current)
+SessionI::destroySession(const Ice::Current&)
 {
     _sessionControl->destroy();
 }
