@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -16,9 +16,9 @@ if len(head) > 0:
     path = [os.path.join(head, p) for p in path]
 path = [os.path.abspath(p) for p in path if os.path.exists(os.path.join(p, "scripts", "TestUtil.py")) ]
 if len(path) == 0:
-    raise "can't find toplevel directory!"
-sys.path.append(os.path.join(path[0]))
-from scripts import *
+    raise RuntimeError("can't find toplevel directory!")
+sys.path.append(os.path.join(path[0], "scripts"))
+import TestUtil
 
 server = os.path.join(os.getcwd(), "server")
 client = os.path.join(os.getcwd(), "client")
@@ -29,9 +29,10 @@ if TestUtil.appverifier:
     targets = [server, client, router]
     TestUtil.setAppVerifierSettings(targets)
 
-print "starting server...",
+sys.stdout.write("starting server... ")
+sys.stdout.flush()
 serverProc = TestUtil.startServer(server)
-print "ok"
+print("ok")
 
 args = ' --Ice.Warn.Dispatch=0' + \
        ' --Glacier2.AddSSLContext=1' + \
@@ -47,16 +48,18 @@ args = ' --Ice.Warn.Dispatch=0' + \
 
 routerCfg = TestUtil.DriverConfig("server")
 routerCfg.protocol = "ssl"
-print "starting router...",
+sys.stdout.write("starting router... ")
+sys.stdout.flush()
 starterProc = TestUtil.startServer(router, args, routerCfg, count = 2)
-print "ok"
+print("ok")
 
 clientCfg = TestUtil.DriverConfig("client")
 clientCfg.protocol = "ssl"
 
-print "starting client...",
+sys.stdout.write("starting client... ")
+sys.stdout.flush()
 clientProc = TestUtil.startClient(client, "", clientCfg, startReader = False)
-print "ok"
+print("ok")
 clientProc.startReader()
 
 clientProc.waitTestSuccess()
