@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -16,9 +16,9 @@ if len(head) > 0:
     path = [os.path.join(head, p) for p in path]
 path = [os.path.abspath(p) for p in path if os.path.exists(os.path.join(p, "scripts", "TestUtil.py")) ]
 if len(path) == 0:
-    raise "can't find toplevel directory!"
-sys.path.append(os.path.join(path[0]))
-from scripts import *
+    raise RuntimeError("can't find toplevel directory!")
+sys.path.append(os.path.join(path[0], "scripts"))
+import TestUtil
 
 hostname = socket.gethostname()
 fqdn = socket.getfqdn()
@@ -218,19 +218,19 @@ if not limitedTests:
                 [(False, 'hello:tcp -h %s -p 12010:tcp -h 127.0.0.1 -p 12010' % fqdn),
                 (True, 'bar:tcp -h 127.0.0.1 -p 12010')], []),
             ('testing maximum proxy length rule',
-                ('', '', '40', '', '', ''),
+                ('', '', '41', '', '', ''),
                 [(True, 'hello:tcp -h 127.0.0.1 -p 12010'),
                 (False, '012345678901234567890123456789012345678901234567890123456789:tcp -h 127.0.0.1 -p 12010')], []),
             ])
 
 if len(testcases) == 0:
-    print "WARNING: You are running this test with SSL disabled and the network "
-    print "         configuration for this host does not permit the other tests "
-    print "         to run correctly."
+    print("WARNING: You are running this test with SSL disabled and the network ")
+    print("         configuration for this host does not permit the other tests ")
+    print("         to run correctly.")
     sys.exit(0)
 elif len(testcases) < 6:
-    print "WARNING: The network configuration for this host does not permit all "
-    print "         tests to run correctly, some tests have been disabled."
+    print("WARNING: The network configuration for this host does not permit all ")
+    print("         tests to run correctly, some tests have been disabled.")
 
 def pingProgress():
     sys.stdout.write('.')
@@ -244,7 +244,7 @@ for testcase in testcases:
     # use command line arguments to pass the test cases in, but a
     # configuration file is easier.
     #
-    attackcfg = file(os.path.join(os.getcwd(), 'attack.cfg'), 'w')
+    attackcfg = open(os.path.join(os.getcwd(), 'attack.cfg'), 'w')
     accepts=0
     rejects=0
     sys.stdout.write(description)
@@ -285,7 +285,7 @@ for testcase in testcases:
           ' --Ice.Admin.InstanceName=Glacier2' + \
           ' --Glacier2.CryptPasswords="'  + os.path.join(os.getcwd(), "passwords") + '"'
 
-    routerConfig = file(os.path.join(os.getcwd(), "router.cfg"), "w")
+    routerConfig = open(os.path.join(os.getcwd(), "router.cfg"), "w")
 
     routerConfig.write("Ice.Default.Locator=locator:tcp -h %s -p 12010\n" % hostname)
     routerConfig.write("Glacier2.Client.Trace.Reject=0\n")
@@ -319,15 +319,15 @@ for testcase in testcases:
     # it is set to loopback.
     #
     routerDriver = TestUtil.DriverConfig("server")
-    if routerDriver.host == "127.0.0.1":
-        routerDriver.host = None
+    if routerDriver.host == None:
+        routerDriver.host = ""
     routerDriver.overrides = commonServerOptions + routerArgs
     
     starterProc = TestUtil.startServer(router, config=routerDriver, count=2)
     pingProgress()
 
     if TestUtil.protocol != "ssl":
-        serverConfig = file(os.path.join(os.getcwd(), "server.cfg"), "w")
+        serverConfig = open(os.path.join(os.getcwd(), "server.cfg"), "w")
         serverOptions = ' --Ice.Config="' + os.path.join(os.getcwd(), "server.cfg") + '" ' 
         serverConfig.write("BackendAdapter.Endpoints=tcp -p 12010\n")
         serverConfig.close()
@@ -335,8 +335,8 @@ for testcase in testcases:
         serverOptions = ""
 
     serverDriver = TestUtil.DriverConfig("server")
-    if serverDriver.host == "127.0.0.1":
-        serverDriver.host = None
+    if serverDriver.host == None:
+        serverDriver.host = ""
     serverDriver.overrides = commonServerOptions
     serverProc = TestUtil.startServer(serverCmd, serverOptions, serverDriver)
     pingProgress()
@@ -349,8 +349,8 @@ for testcase in testcases:
     # failure will result in an assertion and the test will abort.
     #
     clientDriver = TestUtil.DriverConfig("client")
-    if clientDriver.host == "127.0.0.1":
-        clientDriver.host = None
+    if clientDriver.host == None:
+        clientDriver.host = ""
     clientDriver.host = commonClientOptions
     clientArgs = ' --Ice.Config="%s"' % os.path.join(os.getcwd(), 'attack.cfg')
     clientProc = TestUtil.startClient(clientCmd, clientArgs, clientDriver)
