@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -39,7 +39,7 @@ ServantLocatorI::locate(const Ice::Current& current, Ice::LocalObjectPtr& cookie
 }
 
 void
-ServantLocatorI::finished(const Ice::Current& current, const Ice::ObjectPtr& servant,
+ServantLocatorI::finished(const Ice::Current&, const Ice::ObjectPtr&,
                           const Ice::LocalObjectPtr& cookie)
 {
     test(!_deactivated);
