@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -87,7 +87,7 @@ public:
     }
 
     void
-    sent(bool ss)
+    sent(bool)
     {
         _sent.called();
     }
@@ -1158,7 +1158,7 @@ readWriteTests(const ConfigurationPtr& configuration,
     ctl->holdAdapter(); // Hold to block in request send.
 
     Ice::ByteSeq seq;
-    seq.resize(512 * 1024); // Make sure the request doesn't compress too well.
+    seq.resize(1024 * 1024); // Make sure the request doesn't compress too well.
     for(Ice::ByteSeq::iterator p = seq.begin(); p != seq.end(); ++p)
     {
         *p = static_cast<Ice::Byte>(IceUtilInternal::random(255));
