@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,8 +14,10 @@
 using namespace std;
 using namespace Test;
 
+DEFINE_TEST("client")
+
 int
-run(int argc, char* argv[])
+run(int, char**)
 {
     void allTests();
     allTests();
