@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,10 +12,12 @@
 #include <Test.h>
 #include <Dispatcher.h>
 
+DEFINE_TEST("client")
+
 using namespace std;
 
 int
-run(int argc, char* argv[], const Ice::CommunicatorPtr& communicator)
+run(int, char**, const Ice::CommunicatorPtr& communicator)
 {
     void allTests(const Ice::CommunicatorPtr&);
     allTests(communicator);
@@ -27,13 +29,24 @@ main(int argc, char* argv[])
 {
     int status;
     Ice::CommunicatorPtr communicator;
-
+    
     try
     {
         Ice::InitializationData initData;
         initData.properties = Ice::createProperties(argc, argv);
+#ifdef ICE_CPP11
+        Ice::DispatcherPtr dispatcher = new Dispatcher();
+        initData.dispatcher = Ice::newDispatcher(
+            [=](const Ice::DispatcherCallPtr& call, const Ice::ConnectionPtr& conn)
+                {
+                    dispatcher->dispatch(call, conn);
+                });
+#else
         initData.dispatcher = new Dispatcher();
+#endif
         communicator = Ice::initialize(argc, argv, initData);
+        
+
         status = run(argc, argv, communicator);
     }
     catch(const Ice::Exception& ex)
