@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -15,13 +15,13 @@
 using namespace std;
 
 void
-TestIntfI::op(const Ice::Current& current)
+TestIntfI::op(const Ice::Current&)
 {
     test(Dispatcher::isDispatcherThread());
 }
 
 void
-TestIntfI::opWithPayload(const Ice::ByteSeq&, const Ice::Current& current)
+TestIntfI::opWithPayload(const Ice::ByteSeq&, const Ice::Current&)
 {
     test(Dispatcher::isDispatcherThread());
 }
