@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -33,7 +33,7 @@ private:
 
     Ice::DispatchInterceptorAsyncCallbackPtr _defaultCb;
     Ice::DispatchStatus _actualStatus;
-    std::auto_ptr<IceUtil::Exception> _exception;
+    IceUtil::UniquePtr<IceUtil::Exception> _exception;
   
     IceUtil::Mutex _mutex;
 };
