@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -11,12 +11,14 @@
 #include <TestCommon.h>
 #include <Test.h>
 
+DEFINE_TEST("client")
+
 using namespace std;
 
 int
-run(int argc, char* argv[], 
+run(int, char**, 
     const Ice::CommunicatorPtr& communicator,
-    const Ice::InitializationData& initData)
+    const Ice::InitializationData&)
 {
     Test::MyClassPrx allTests(const Ice::CommunicatorPtr&);
     Test::MyClassPrx myClass = allTests(communicator);
