@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,7 +22,7 @@ class HelloI : virtual public Hello
 public:
     
     virtual void
-    sayHello(const Ice::Current& foo)
+    sayHello(const Ice::Current&)
     {
         // Do nothing, this is just a dummy servant.
     }
@@ -577,6 +577,19 @@ allTests(const Ice::CommunicatorPtr& communicator, const string& ref)
     hello->sayHello();
     cout << "ok" << endl;
 
+    cout << "testing locator encoding resolution... " << flush;
+
+    hello = HelloPrx::checkedCast(communicator->stringToProxy("hello"));
+    count = locator->getRequestCount();
+    communicator->stringToProxy("test@TestAdapter")->ice_encodingVersion(Ice::Encoding_1_1)->ice_ping();
+    test(count == locator->getRequestCount());
+    communicator->stringToProxy("test@TestAdapter10")->ice_encodingVersion(Ice::Encoding_1_0)->ice_ping();
+    test(++count == locator->getRequestCount());
+    communicator->stringToProxy("test -e 1.0@TestAdapter10-2")->ice_ping();
+    test(++count == locator->getRequestCount());
+
+    cout << "ok" << endl;
+
     cout << "shutdown server... " << flush;
     obj->shutdown();
     cout << "ok" << endl;
