@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -9,7 +9,7 @@
 
 #include <Ice/Ice.h>
 #include <TestCommon.h>
-#include <Test.h>
+#include <ClientPrivate.h>
 
 using namespace std;
 using namespace Test;
@@ -52,7 +52,6 @@ private:
     bool _called;
 };
 
-
 class Callback : public CallbackBase, public IceUtil::Shared
 {
 public:
@@ -322,9 +321,51 @@ public:
         called();
     }
 };
-
 typedef IceUtil::Handle<Callback> CallbackPtr;
 
+class RelayI : public Relay
+{
+    virtual void knownPreservedAsBase(const ::Ice::Current&)
+    {
+        KnownPreservedDerived ex;
+        ex.b = "base";
+        ex.kp = "preserved";
+        ex.kpd = "derived";
+        throw ex;
+    }
+
+    virtual void knownPreservedAsKnownPreserved(const ::Ice::Current&)
+    {
+        KnownPreservedDerived ex;
+        ex.b = "base";
+        ex.kp = "preserved";
+        ex.kpd = "derived";
+        throw ex;
+    }
+
+    virtual void unknownPreservedAsBase(const ::Ice::Current&)
+    {
+        Preserved2 ex;
+        ex.b = "base";
+        ex.kp = "preserved";
+        ex.kpd = "derived";
+        ex.p1 = new PreservedClass("bc", "pc");
+        ex.p2 = ex.p1;
+        throw ex;
+    }
+
+    virtual void unknownPreservedAsKnownPreserved(const ::Ice::Current&)
+    {
+        Preserved2 ex;
+        ex.b = "base";
+        ex.kp = "preserved";
+        ex.kpd = "derived";
+        ex.p1 = new PreservedClass("bc", "pc");
+        ex.p2 = ex.p1;
+        throw ex;
+    }
+};
+
 TestIntfPrx
 allTests(const Ice::CommunicatorPtr& communicator)
 {
@@ -718,5 +759,153 @@ allTests(const Ice::CommunicatorPtr& communicator)
     }
     cout << "ok" << endl;
 
+    cout << "unknown most derived in compact format... " << flush;
+    {
+        try
+        {
+            test->unknownMostDerived2AsBaseCompact();
+            test(false);
+        }
+        catch(const Base&)
+        {
+            //
+            // For the 1.0 encoding, the unknown exception is sliced to Base.
+            //
+            test(test->ice_getEncodingVersion() == Ice::Encoding_1_0);
+        }
+        catch(const Ice::UnknownUserException&)
+        {
+            //
+            // An UnknownUserException is raised for the compact format because the
+            // most-derived type is unknown and the exception cannot be sliced.
+            //
+            test(test->ice_getEncodingVersion() != Ice::Encoding_1_0);
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+        }
+        catch(...)
+        {
+            test(false);
+        }
+    }
+    cout << "ok" << endl;
+
+    cout << "preserved exceptions... " << flush;
+    {
+        Ice::ObjectAdapterPtr adapter = communicator->createObjectAdapterWithEndpoints("Relay", "default");
+        RelayPrx relay = RelayPrx::uncheckedCast(adapter->addWithUUID(new RelayI));
+        adapter->activate();
+
+        try
+        {
+            test->relayKnownPreservedAsBase(relay);
+            test(false);
+        }
+        catch(const KnownPreservedDerived& ex)
+        {
+            test(ex.b == "base");
+            test(ex.kp == "preserved");
+            test(ex.kpd == "derived");
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+        }
+        catch(...)
+        {
+            test(false);
+        }
+
+        try
+        {
+            test->relayKnownPreservedAsKnownPreserved(relay);
+            test(false);
+        }
+        catch(const KnownPreservedDerived& ex)
+        {
+            test(ex.b == "base");
+            test(ex.kp == "preserved");
+            test(ex.kpd == "derived");
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+        }
+        catch(...)
+        {
+            test(false);
+        }
+
+        try
+        {
+            test->relayUnknownPreservedAsBase(relay);
+            test(false);
+        }
+        catch(const Preserved2& ex)
+        {
+            test(ex.b == "base");
+            test(ex.kp == "preserved");
+            test(ex.kpd == "derived");
+            test(ex.p1->ice_id() == PreservedClass::ice_staticId());
+            PreservedClassPtr pc = PreservedClassPtr::dynamicCast(ex.p1);
+            test(pc->bc == "bc");
+            test(pc->pc == "pc");
+            test(ex.p2 == ex.p1);
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+        }
+        catch(const KnownPreservedDerived& ex)
+        {
+            //
+            // For the 1.0 encoding, the unknown exception is sliced to KnownPreserved.
+            //
+            test(test->ice_getEncodingVersion() == Ice::Encoding_1_0);
+            test(ex.b == "base");
+            test(ex.kp == "preserved");
+            test(ex.kpd == "derived");
+        }
+        catch(...)
+        {
+            test(false);
+        }
+
+        try
+        {
+            test->relayUnknownPreservedAsKnownPreserved(relay);
+            test(false);
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+        }
+        catch(const Preserved2& ex)
+        {
+            test(ex.b == "base");
+            test(ex.kp == "preserved");
+            test(ex.kpd == "derived");
+            test(ex.p1->ice_id() == PreservedClass::ice_staticId());
+            PreservedClassPtr pc = PreservedClassPtr::dynamicCast(ex.p1);
+            test(pc->bc == "bc");
+            test(pc->pc == "pc");
+            test(ex.p2 == ex.p1);
+        }
+        catch(const KnownPreservedDerived& ex)
+        {
+            //
+            // For the 1.0 encoding, the unknown exception is sliced to KnownPreserved.
+            //
+            test(test->ice_getEncodingVersion() == Ice::Encoding_1_0);
+            test(ex.b == "base");
+            test(ex.kp == "preserved");
+            test(ex.kpd == "derived");
+        }
+        catch(...)
+        {
+            test(false);
+        }
+
+        adapter->destroy();
+    }
+    cout << "ok" << endl;
+
     return test;
 }
