@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -10,7 +10,6 @@
 #include <Ice/Ice.h>
 #include <TestCommon.h>
 #include <ClientPrivate.h>
-#include <sstream>
 
 using namespace std;
 using namespace Test;
@@ -99,18 +98,45 @@ public:
     }
 
     void
-    response_SUnknownAsObject(const Ice::ObjectPtr& o)
+    response_SBSUnknownDerivedAsSBaseCompact(const SBasePtr&)
     {
         test(false);
     }
 
     void
-    exception_SUnknownAsObject(const Ice::Exception& exc)
+    exception_SBSUnknownDerivedAsSBaseCompact(const Ice::Exception& exc)
     {
         test(exc.ice_name() == "Ice::NoObjectFactoryException");
         called();
     }
 
+    void
+    response_SUnknownAsObject10(const Ice::ObjectPtr&)
+    {
+        test(false);
+    }
+
+    void
+    exception_SUnknownAsObject10(const Ice::Exception& exc)
+    {
+        test(exc.ice_name() == "Ice::NoObjectFactoryException");
+        called();
+    }
+
+    void
+    response_SUnknownAsObject11(const Ice::ObjectPtr& o)
+    {
+        test(Ice::UnknownSlicedObjectPtr::dynamicCast(o));
+        test(Ice::UnknownSlicedObjectPtr::dynamicCast(o)->getUnknownTypeId() == "::Test::SUnknown");
+        called();
+    }
+
+    void
+    exception_SUnknownAsObject11(const Ice::Exception&)
+    {
+        test(false);
+    }
+
     void
     response_oneElementCycle(const BPtr& b)
     {
@@ -217,14 +243,14 @@ public:
     }
 
     void
-    response_returnTest1(const BPtr& r, const BPtr& p1, const BPtr& p2)
+    response_returnTest1(const BPtr& r, const BPtr& p1, const BPtr&)
     {
         test(r == p1);
         called();
     }
 
     void
-    response_returnTest2(const BPtr& r, const BPtr& p1, const BPtr& p2)
+    response_returnTest2(const BPtr& r, const BPtr& p1, const BPtr&)
     {
         test(r == p1);
         called();
@@ -355,15 +381,112 @@ public:
         called();
     }
 
+    void
+    response_preserved1(const PBasePtr& r)
+    {
+        PDerivedPtr pd = PDerivedPtr::dynamicCast(r);
+        test(pd);
+        test(pd->pi == 3);
+        test(pd->ps == "preserved");
+        test(pd->pb == pd);
+        called();
+    }
+
+    void
+    response_preserved2(const PBasePtr& r)
+    {
+        PCUnknownPtr p2 = PCUnknownPtr::dynamicCast(r);
+        test(!p2);
+        test(r->pi == 3);
+        called();
+    }
+
+    void
+    response_preserved3(const PBasePtr& r)
+    {
+        //
+        // Encoding 1.0
+        //
+        PCDerivedPtr p2 = PCDerivedPtr::dynamicCast(r);
+        test(!p2);
+        test(r->pi == 3);
+        called();
+    }
+
+    void
+    response_preserved4(const PBasePtr& r)
+    {
+        //
+        // Encoding > 1.0
+        //
+        PCDerivedPtr p2 = PCDerivedPtr::dynamicCast(r);
+        test(p2);
+        test(p2->pi == 3);
+        test(p2->pbs[0] == p2);
+        called();
+    }
+
+    void
+    response_preserved5(const PBasePtr& r)
+    {
+        PCDerived3Ptr p3 = PCDerived3Ptr::dynamicCast(r);
+        test(p3);
+        test(p3->pi == 3);
+        for(int i = 0; i < 300; ++i)
+        {
+            PCDerived2Ptr p2 = PCDerived2Ptr::dynamicCast(p3->pbs[i]);
+            test(p2->pi == i);
+            test(p2->pbs.size() == 1);
+            test(!p2->pbs[0]);
+            test(p2->pcd2 == i);
+        }
+        test(p3->pcd2 == p3->pi);
+        test(p3->pcd3 == p3->pbs[10]);
+        called();
+    }
+
+    void
+    response_compactPreserved1(const PBasePtr& r)
+    {
+        //
+        // Encoding 1.0
+        //
+        CompactPCDerivedPtr p2 = CompactPCDerivedPtr::dynamicCast(r);
+        test(!p2);
+        test(r->pi == 3);
+        called();
+    }
+
+    void
+    response_compactPreserved2(const PBasePtr& r)
+    {
+        //
+        // Encoding > 1.0
+        //
+        CompactPCDerivedPtr p2 = CompactPCDerivedPtr::dynamicCast(r);
+        test(p2);
+        test(p2->pi == 3);
+        test(p2->pbs[0] == p2);
+        called();
+    }
+
     void 
     response()
     {
         test(false);
     }
+
     void 
-    exception(const ::Ice::Exception&)
+    exception(const ::Ice::Exception& ex)
     {
-        test(false);
+        if(!dynamic_cast<const Ice::OperationNotExistException*>(&ex))
+        {
+            test(false);
+        }
+        else
+        {
+            called();
+        }
     }
  
     BPtr rb;
@@ -371,9 +494,82 @@ public:
     BDict rbdict;
     BDict obdict;
 };
-
 typedef IceUtil::Handle<Callback> CallbackPtr;
 
+class PNodeI : virtual public PNode
+{
+public:
+
+    PNodeI()
+    {
+        ++counter;
+    }
+
+    virtual ~PNodeI()
+    {
+        --counter;
+    }
+
+    static int counter;
+};
+
+int PNodeI::counter = 0;
+
+class NodeFactoryI : public Ice::ObjectFactory
+{
+public:
+
+    virtual Ice::ObjectPtr create(const string& id)
+    {
+        if(id == PNode::ice_staticId())
+        {
+            return new PNodeI;
+        }
+        return 0;
+    }
+
+    virtual void destroy()
+    {
+    }
+};
+
+class PreservedI : virtual public Preserved
+{
+public:
+
+    PreservedI()
+    {
+        ++counter;
+    }
+
+    virtual ~PreservedI()
+    {
+        --counter;
+    }
+
+    static int counter;
+};
+
+int PreservedI::counter = 0;
+
+class PreservedFactoryI : public Ice::ObjectFactory
+{
+public:
+
+    virtual Ice::ObjectPtr create(const string& id)
+    {
+        if(id == Preserved::ice_staticId())
+        {
+            return new PreservedI;
+        }
+        return 0;
+    }
+
+    virtual void destroy()
+    {
+    }
+};
+
 void
 testUOO(const TestIntfPrx& test)
 {
@@ -381,10 +577,19 @@ testUOO(const TestIntfPrx& test)
     try
     {
         o = test->SUnknownAsObject();
-        test(false);
+        test(test->ice_getEncodingVersion() != Ice::Encoding_1_0);
+        test(Ice::UnknownSlicedObjectPtr::dynamicCast(o));
+        test(Ice::UnknownSlicedObjectPtr::dynamicCast(o)->getUnknownTypeId() == "::Test::SUnknown");
+        test->checkSUnknown(o);
     }
     catch(const Ice::NoObjectFactoryException&)
     {
+        test(test->ice_getEncodingVersion() == Ice::Encoding_1_0);
+    }
+    catch(const std::exception& ex)
+    {
+        cout << ex.what() << endl;
+        test(false);
     }
     catch(...)
     {
@@ -516,6 +721,47 @@ allTests(const Ice::CommunicatorPtr& communicator)
             test(false);
         }
     }
+    if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+    {
+        try
+        {
+            //
+            // This test succeeds for the 1.0 encoding.
+            //
+            SBasePtr sb = test->SBSUnknownDerivedAsSBaseCompact();
+            test(sb->sb == "SBSUnknownDerived.sb");
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+        }
+        catch(...)
+        {
+            test(false);
+        }
+    }
+    else
+    {
+        try
+        {
+            //
+            // This test fails when using the compact format because the instance cannot
+            // be sliced to a known type.
+            //
+            SBasePtr sb = test->SBSUnknownDerivedAsSBaseCompact();
+            test(false);
+        }
+        catch(const Ice::OperationNotExistException&)
+        {
+        }
+        catch(const Ice::NoObjectFactoryException&)
+        {
+            // Expected.
+        }
+        catch(...)
+        {
+            test(false);
+        }
+    }
     cout << "ok" << endl;
 
     cout << "base with unknown derived as base (AMI)... " << flush;
@@ -526,6 +772,30 @@ allTests(const Ice::CommunicatorPtr& communicator)
                 cb, &Callback::response_SBSUnknownDerivedAsSBase, &Callback::exception));
         cb->check();
     }
+    if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+    {
+        //
+        // This test succeeds for the 1.0 encoding.
+        //
+        CallbackPtr cb = new Callback;
+        test->begin_SBSUnknownDerivedAsSBaseCompact(
+            newCallback_TestIntf_SBSUnknownDerivedAsSBaseCompact(
+                cb, &Callback::response_SBSUnknownDerivedAsSBase, &Callback::exception));
+        cb->check();
+    }
+    else
+    {
+        //
+        // This test fails when using the compact format because the instance cannot
+        // be sliced to a known type.
+        //
+        CallbackPtr cb = new Callback;
+        test->begin_SBSUnknownDerivedAsSBaseCompact(
+            newCallback_TestIntf_SBSUnknownDerivedAsSBaseCompact(
+                cb, &Callback::response_SBSUnknownDerivedAsSBaseCompact,
+                &Callback::exception_SBSUnknownDerivedAsSBaseCompact));
+        cb->check();
+    }
     cout << "ok" << endl;
 
     cout << "unknown with Object as Object... " << flush;
@@ -539,9 +809,18 @@ allTests(const Ice::CommunicatorPtr& communicator)
         try
         {
             CallbackPtr cb = new Callback;
-            test->begin_SUnknownAsObject(
-                newCallback_TestIntf_SUnknownAsObject(
-                    cb, &Callback::response_SUnknownAsObject, &Callback::exception_SUnknownAsObject));
+            if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+            {
+                test->begin_SUnknownAsObject(
+                    newCallback_TestIntf_SUnknownAsObject(
+                        cb, &Callback::response_SUnknownAsObject10, &Callback::exception_SUnknownAsObject10));
+            }
+            else
+            {
+                test->begin_SUnknownAsObject(
+                    newCallback_TestIntf_SUnknownAsObject(
+                        cb, &Callback::response_SUnknownAsObject11, &Callback::exception_SUnknownAsObject11));
+            }
             cb->check();
         }
         catch(...)
@@ -1697,5 +1976,423 @@ allTests(const Ice::CommunicatorPtr& communicator)
     }
     cout << "ok" << endl;
 
+    cout << "preserved classes... " << flush;
+    try
+    {
+        //
+        // Server knows the most-derived class PDerived.
+        //
+        PDerivedPtr pd = new PDerived;
+        pd->pi = 3;
+        pd->ps = "preserved";
+        pd->pb = pd;
+
+        PBasePtr r = test->exchangePBase(pd);
+        PDerivedPtr p2 = PDerivedPtr::dynamicCast(r);
+        test(p2);
+        test(p2->pi == 3);
+        test(p2->ps == "preserved");
+        test(p2->pb == p2);
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+    }
+
+    try
+    {
+        //
+        // Server only knows the base (non-preserved) type, so the object is sliced.
+        //
+        PCUnknownPtr pu = new PCUnknown;
+        pu->pi = 3;
+        pu->pu = "preserved";
+
+        PBasePtr r = test->exchangePBase(pu);
+        PCUnknownPtr p2 = PCUnknownPtr::dynamicCast(r);
+        test(!p2);
+        test(r->pi == 3);
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+    }
+
+    try
+    {
+        //
+        // Server only knows the intermediate type Preserved. The object will be sliced to
+        // Preserved for the 1.0 encoding; otherwise it should be returned intact.
+        //
+        PCDerivedPtr pcd = new PCDerived;
+        pcd->pi = 3;
+        pcd->pbs.push_back(pcd);
+
+        PBasePtr r = test->exchangePBase(pcd);
+        PCDerivedPtr p2 = PCDerivedPtr::dynamicCast(r);
+        if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+        {
+            test(!p2);
+            test(r->pi == 3);
+        }
+        else
+        {
+            test(p2);
+            test(p2->pi == 3);
+            test(p2->pbs[0] == p2);
+        }
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+    }
+
+    try
+    {
+        //
+        // Server only knows the intermediate type CompactPDerived. The object will be sliced to
+        // CompactPDerived for the 1.0 encoding; otherwise it should be returned intact.
+        //
+        CompactPCDerivedPtr pcd = new CompactPCDerived;
+        pcd->pi = 3;
+        pcd->pbs.push_back(pcd);
+
+        PBasePtr r = test->exchangePBase(pcd);
+        CompactPCDerivedPtr p2 = CompactPCDerivedPtr::dynamicCast(r);
+        if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+        {
+            test(!p2);
+            test(r->pi == 3);
+        }
+        else
+        {
+            test(p2);
+            test(p2->pi == 3);
+            test(p2->pbs[0] == p2);
+        }
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+    }
+
+    try
+    {
+        //
+        // Send an object that will have multiple preserved slices in the server.
+        // The object will be sliced to Preserved for the 1.0 encoding.
+        //
+        PCDerived3Ptr pcd = new PCDerived3;
+        pcd->pi = 3;
+        //
+        // Sending more than 254 objects exercises the encoding for object ids.
+        //
+        int i;
+        for(i = 0; i < 300; ++i)
+        {
+            PCDerived2Ptr p2 = new PCDerived2;
+            p2->pi = i;
+            p2->pbs.push_back(0); // Nil reference. This slice should not have an indirection table.
+            p2->pcd2 = i;
+            pcd->pbs.push_back(p2);
+        }
+        pcd->pcd2 = pcd->pi;
+        pcd->pcd3 = pcd->pbs[10];
+
+        PBasePtr r = test->exchangePBase(pcd);
+        PCDerived3Ptr p3 = PCDerived3Ptr::dynamicCast(r);
+        if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+        {
+            test(!p3);
+            test(PreservedPtr::dynamicCast(r));
+            test(r->pi == 3);
+        }
+        else
+        {
+            test(p3);
+            test(p3->pi == 3);
+            for(i = 0; i < 300; ++i)
+            {
+                PCDerived2Ptr p2 = PCDerived2Ptr::dynamicCast(p3->pbs[i]);
+                test(p2->pi == i);
+                test(p2->pbs.size() == 1);
+                test(!p2->pbs[0]);
+                test(p2->pcd2 == i);
+            }
+            test(p3->pcd2 == p3->pi);
+            test(p3->pcd3 == p3->pbs[10]);
+        }
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+    }
+
+    try
+    {
+        //
+        // Obtain an object with preserved slices and send it back to the server.
+        // The preserved slices should be excluded for the 1.0 encoding, otherwise
+        // they should be included.
+        //
+        PreservedPtr p = test->PBSUnknownAsPreserved();
+        test->checkPBSUnknown(p);
+        if(test->ice_getEncodingVersion() != Ice::Encoding_1_0)
+        {
+            test->ice_encodingVersion(Ice::Encoding_1_0)->checkPBSUnknown(p);
+        }
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+    }
+
+    cout << "ok" << endl;
+
+    cout << "preserved classes (AMI)... " << flush;
+    {
+        //
+        // Server knows the most-derived class PDerived.
+        //
+        PDerivedPtr pd = new PDerived;
+        pd->pi = 3;
+        pd->ps = "preserved";
+        pd->ps = "preserved";
+        pd->pb = pd;
+
+        CallbackPtr cb = new Callback;
+        test->begin_exchangePBase(
+            pd, newCallback_TestIntf_exchangePBase(cb, &Callback::response_preserved1, &Callback::exception));
+        cb->check();
+    }
+
+    {
+        //
+        // Server only knows the base (non-preserved) type, so the object is sliced.
+        //
+        PCUnknownPtr pu = new PCUnknown;
+        pu->pi = 3;
+        pu->pu = "preserved";
+
+        CallbackPtr cb = new Callback;
+        test->begin_exchangePBase(
+            pu, newCallback_TestIntf_exchangePBase(cb, &Callback::response_preserved2, &Callback::exception));
+        cb->check();
+    }
+
+    {
+        //
+        // Server only knows the intermediate type Preserved. The object will be sliced to
+        // Preserved for the 1.0 encoding; otherwise it should be returned intact.
+        //
+        PCDerivedPtr pcd = new PCDerived;
+        pcd->pi = 3;
+        pcd->pbs.push_back(pcd);
+
+        CallbackPtr cb = new Callback;
+        if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+        {
+            test->begin_exchangePBase(
+                pcd, newCallback_TestIntf_exchangePBase(cb, &Callback::response_preserved3, &Callback::exception));
+        }
+        else
+        {
+            test->begin_exchangePBase(
+                pcd, newCallback_TestIntf_exchangePBase(cb, &Callback::response_preserved4, &Callback::exception));
+        }
+        cb->check();
+    }
+
+    {
+        //
+        // Server only knows the intermediate type CompactPDerived. The object will be sliced to
+        // CompactPDerived for the 1.0 encoding; otherwise it should be returned intact.
+        //
+        CompactPCDerivedPtr pcd = new CompactPCDerived;
+        pcd->pi = 3;
+        pcd->pbs.push_back(pcd);
+
+        CallbackPtr cb = new Callback;
+        if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+        {
+            test->begin_exchangePBase(pcd, newCallback_TestIntf_exchangePBase(cb,
+                                                                              &Callback::response_compactPreserved1,
+                                                                              &Callback::exception));
+        }
+        else
+        {
+            test->begin_exchangePBase(pcd, newCallback_TestIntf_exchangePBase(cb,
+                                                                              &Callback::response_compactPreserved2, 
+                                                                              &Callback::exception));
+        }
+        cb->check();
+    }
+
+    {
+        //
+        // Send an object that will have multiple preserved slices in the server.
+        // The object will be sliced to Preserved for the 1.0 encoding.
+        //
+        PCDerived3Ptr pcd = new PCDerived3;
+        pcd->pi = 3;
+        //
+        // Sending more than 254 objects exercises the encoding for object ids.
+        //
+        int i;
+        for(i = 0; i < 300; ++i)
+        {
+            PCDerived2Ptr p2 = new PCDerived2;
+            p2->pi = i;
+            p2->pbs.push_back(0); // Nil reference. This slice should not have an indirection table.
+            p2->pcd2 = i;
+            pcd->pbs.push_back(p2);
+        }
+        pcd->pcd2 = pcd->pi;
+        pcd->pcd3 = pcd->pbs[10];
+
+        CallbackPtr cb = new Callback;
+        if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+        {
+            test->begin_exchangePBase(
+                pcd, newCallback_TestIntf_exchangePBase(cb, &Callback::response_preserved3, &Callback::exception));
+        }
+        else
+        {
+            test->begin_exchangePBase(
+                pcd, newCallback_TestIntf_exchangePBase(cb, &Callback::response_preserved5, &Callback::exception));
+        }
+        cb->check();
+    }
+    cout << "ok" << endl;
+
+    cout << "garbage collection for preserved classes... " << flush;
+    try
+    {
+        //
+        // Register a factory in order to substitute our own subclass of PNode. This provides
+        // an easy way to determine how many unmarshaled instances currently exist.
+        //
+        communicator->addObjectFactory(new NodeFactoryI, PNode::ice_staticId());
+
+        //
+        // Relay a graph through the server. This test uses a preserved class
+        // with a class member, so the preserved class already supports GC.
+        //
+        {
+            PNodePtr c = new PNode;
+            c->next = new PNode;
+            c->next->next = new PNode;
+            c->next->next->next = c;
+
+            test(PNodeI::counter == 0);
+            PNodePtr n = test->exchangePNode(c);
+
+            c->next->next->next = 0; // Break the cycle.
+
+            test(PNodeI::counter == 3);
+            Ice::collectGarbage();          // No effect.
+            test(PNodeI::counter == 3);
+            n = 0;                          // Release reference.
+            Ice::collectGarbage();
+            test(PNodeI::counter == 0);
+        }
+
+        //
+        // Obtain a preserved object from the server where the most-derived
+        // type is unknown. The preserved slice refers to a graph of PNode
+        // objects.
+        //
+        {
+            test(PNodeI::counter == 0);
+            PreservedPtr p = test->PBSUnknownAsPreservedWithGraph();
+            test->checkPBSUnknownWithGraph(p);
+            if(test->ice_getEncodingVersion() == Ice::Encoding_1_0)
+            {
+                //
+                // Although the unknown slice is ignored, the PNode instances are still
+                // unmarshaled and their graph is not destroyed until we collect garbage.
+                //
+                test(PNodeI::counter == 3);
+                Ice::collectGarbage();
+                test(PNodeI::counter == 0);
+            }
+            else
+            {
+                test(PNodeI::counter == 3);
+                Ice::collectGarbage();          // No effect.
+                test(PNodeI::counter == 3);
+                p = 0;                          // Release reference.
+                Ice::collectGarbage();
+                test(PNodeI::counter == 0);
+            }
+        }
+
+        //
+        // Register a factory in order to substitute our own subclass of Preserved. This provides
+        // an easy way to determine how many unmarshaled instances currently exist.
+        //
+        communicator->addObjectFactory(new PreservedFactoryI, Preserved::ice_staticId());
+
+        //
+        // Obtain a preserved object from the server where the most-derived
+        // type is unknown. A data member in the preserved slice refers to the
+        // outer object, so the chain of references looks like this:
+        //
+        // outer->slicedData->outer
+        //
+        {
+            test(PreservedI::counter == 0);
+            PreservedPtr p = test->PBSUnknown2AsPreservedWithGraph();
+            test->checkPBSUnknown2WithGraph(p);
+            test(PreservedI::counter == 1);
+            Ice::collectGarbage();          // No effect.
+            test(PreservedI::counter == 1);
+            p = 0;                          // Release reference.
+            Ice::collectGarbage();
+            test(PreservedI::counter == 0);
+        }
+
+        //
+        // Throw a preserved exception where the most-derived type is unknown.
+        // The preserved exception slice contains a class data member. This
+        // object is also preserved, and its most-derived type is also unknown.
+        // The preserved slice of the object contains a class data member that
+        // refers to itself.
+        //
+        // The chain of references looks like this:
+        //
+        // ex->slicedData->obj->slicedData->obj
+        //
+        try
+        {
+            test(PreservedI::counter == 0);
+
+            try
+            {
+                test->throwPreservedException();
+            }
+            catch(const PreservedException&)
+            {
+                //
+                // The class instance is only retained when the encoding is > 1.0.
+                //
+                if(test->ice_getEncodingVersion() != Ice::Encoding_1_0)
+                {
+                    test(PreservedI::counter == 1);
+                    Ice::collectGarbage();          // No effect.
+                    test(PreservedI::counter == 1);
+                }
+            }
+
+            //
+            // Exception has gone out of scope.
+            //
+            Ice::collectGarbage();
+            test(PreservedI::counter == 0);
+        }
+        catch(const Ice::Exception&)
+        {
+            test(false);
+        }
+    }
+    catch(const Ice::OperationNotExistException&)
+    {
+    }
+    cout << "ok" << endl;
+
     return test;
 }
