@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -23,7 +23,7 @@ TestI::getProperty(const std::string& name, const Ice::Current& current)
 }
 
 Ice::StringSeq
-TestI::getArgs(const Ice::Current& current)
+TestI::getArgs(const Ice::Current&)
 {
     return _args;
 }
