@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,14 +16,13 @@ class TestI : public ::Test::TestIntf
 {
 public:
 
-    TestI(const Ice::ObjectAdapterPtr&, const Ice::PropertiesPtr&);
+    TestI(const Ice::PropertiesPtr&);
 
     virtual void shutdown(const Ice::Current&);
     virtual std::string getProperty(const std::string&, const Ice::Current&);
 
 private:
 
-    Ice::ObjectAdapterPtr _adapter;
     Ice::PropertiesPtr _properties;
 };
 
