@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,6 +22,13 @@ using namespace std;
 using namespace Test;
 using namespace IceGrid;
 
+namespace 
+{
+
+const int sleepTime = 100; // 100ms
+const int maxRetry = 240000 / sleepTime; // 4 minutes
+
+
 class SessionKeepAliveThread : public IceUtil::Thread, public IceUtil::Monitor<IceUtil::Mutex>
 {
 public:
@@ -84,24 +91,70 @@ void
 waitForServerState(const IceGrid::AdminPrx& admin, const std::string& server, bool up)
 {
     int nRetry = 0;
-    while(nRetry < 30)
+    while(nRetry < maxRetry) // One minute
     {
         if(admin->getServerState(server) == (up ? Active : Inactive))
         {
             return;
         } 
 
-        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(500));
+        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
         ++nRetry;
     }
     test(false);
 }
 
+void
+waitForReplicaState(const IceGrid::AdminPrx& admin, const std::string& replica, bool up)
+{
+    int nRetry = 0;
+    while(nRetry < maxRetry)
+    {
+        try
+        {
+            if(admin->pingRegistry(replica) == up)
+            {
+                return;
+            }
+        }
+        catch(const RegistryNotExistException&)
+        {
+            if(!up)
+            {
+                return;
+            }
+        }
+
+        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
+        ++nRetry;
+    }
+
+    try
+    {
+        if(admin->pingRegistry(replica) != up)
+        {
+            cerr << "replica state change timed out:" << endl;
+            cerr << "replica: " << replica << endl;
+            cerr << "state: " << up << endl;
+        }
+    }
+    catch(const RegistryNotExistException&)
+    {
+        if(up)
+        {
+            cerr << "replica state change timed out:" << endl;
+            cerr << "replica: " << replica << endl;
+            cerr << "state: " << up << endl;
+        }
+    }
+
+}
+
 void
 waitForNodeState(const IceGrid::AdminPrx& admin, const std::string& node, bool up)
 {
     int nRetry = 0;
-    while(nRetry < 30)
+    while(nRetry < maxRetry)
     {
         try
         {
@@ -118,7 +171,7 @@ waitForNodeState(const IceGrid::AdminPrx& admin, const std::string& node, bool u
             }
         }
         
-        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(500));
+        IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
         ++nRetry;
     }
     try
@@ -209,7 +262,7 @@ bool
 waitAndPing(const Ice::ObjectPrx& obj)
 {
     int nRetry = 0;
-    while(nRetry < 100)
+    while(nRetry < maxRetry)
     {
         try
         {
@@ -218,7 +271,7 @@ waitAndPing(const Ice::ObjectPrx& obj)
         }
         catch(const Ice::LocalException&)
         {
-            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(500));
+            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
             ++nRetry;
         }
     }
@@ -244,6 +297,8 @@ createAdminSession(const Ice::LocatorPrx& locator, const string& replica)
     return session->getAdmin();
 }
 
+}
+
 void
 allTests(const Ice::CommunicatorPtr& comm)
 {
@@ -334,9 +389,9 @@ allTests(const Ice::CommunicatorPtr& comm)
         info = masterAdmin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator"));
         // We eventually need to wait here for the update of the replicated objects to propagate to the replica.
         int nRetry = 0;
-        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator")) != info && nRetry < 30)
+        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator")) != info && nRetry < maxRetry)
         {
-            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(500));
+            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
             ++nRetry;
         }
         test(slave2Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator")) == info);
@@ -350,9 +405,9 @@ allTests(const Ice::CommunicatorPtr& comm)
         info = masterAdmin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query"));
         // We eventually need to wait here for the update of the replicated objects to propagate to the replica.
         nRetry = 0;
-        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query")) != info && nRetry < 30)
+        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query")) != info && nRetry < maxRetry)
         {
-            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(500));
+            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
             ++nRetry;
         }
         test(slave2Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query")) == info);
@@ -369,9 +424,9 @@ allTests(const Ice::CommunicatorPtr& comm)
         info = masterAdmin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator"));
         // We eventually need to wait here for the update of the replicated objects to propagate to the replica.
         nRetry = 0;
-        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator")) != info && nRetry < 30)
+        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator")) != info && nRetry < maxRetry)
         {
-            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(500));
+            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
             ++nRetry;
         }
         test(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Locator")) == info);
@@ -383,9 +438,9 @@ allTests(const Ice::CommunicatorPtr& comm)
 
         info = masterAdmin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query"));
         nRetry = 0;
-        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query")) != info && nRetry < 30)
+        while(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query")) != info && nRetry < maxRetry)
         {
-            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(500));
+            IceUtil::ThreadControl::sleep(IceUtil::Time::milliSeconds(sleepTime));
             ++nRetry;
         }
         test(slave1Admin->getObjectInfo(comm->stringToIdentity("TestIceGrid/Query")) == info);
@@ -1187,6 +1242,7 @@ allTests(const Ice::CommunicatorPtr& comm)
         slave1Admin->shutdown();
         waitForServerState(admin, "Slave1", false);
         
+        params.clear();
         params["id"] = "Slave1";
         params["port"] = "12051";
         params["replicaName"] = "Master";
@@ -1197,6 +1253,8 @@ allTests(const Ice::CommunicatorPtr& comm)
             Ice::LocatorPrx::uncheckedCast(comm->stringToProxy("TestIceGrid/Locator-Master:default -p 12051"));
         slave1Admin = createAdminSession(slave1Locator, "");
 
+        waitForReplicaState(slave1Admin, "Slave2", true);
+
         ApplicationUpdateDescriptor update;
         update.name = "TestApp";
         NodeUpdateDescriptor node;
@@ -1214,11 +1272,19 @@ allTests(const Ice::CommunicatorPtr& comm)
         slave1Admin->shutdown();
         waitForServerState(admin, "Slave1", false);
 
+        params.clear();
         params["id"] = "Slave1";
         params["replicaName"] = "Slave1";
         params["port"] = "12051";
         instantiateServer(admin, "IceGridRegistry", params);
 
+        params.clear();
+        params["id"] = "Master";
+        params["replicaName"] = "";
+        params["port"] = "12050";
+        params["arg"] = "--initdb-from-replica=Slave2";
+        instantiateServer(admin, "IceGridRegistry", params);
+
         admin->startServer("Master");
         masterAdmin = createAdminSession(masterLocator, "");
 
@@ -1232,24 +1298,91 @@ allTests(const Ice::CommunicatorPtr& comm)
         comm->stringToProxy("test")->ice_locator(slave2Locator)->ice_locatorCacheTimeout(0)->ice_ping();
 
         masterAdmin->stopServer("Server");
-        
+
+        waitForReplicaState(masterAdmin, "Slave1", true);
+        waitForReplicaState(masterAdmin, "Slave2", true);
+
+        ApplicationInfo info = masterAdmin->getApplicationInfo("TestApp");
+        test(info.revision == 2);
+
         masterAdmin->removeApplication("TestApp");
     }
     cout << "ok" << endl;
+
+    cout << "testing interop with registry and node using the 1.0 encoding... " << flush;
+    {
+        params.clear();
+        params["id"] = "Slave3";
+        params["replicaName"] = "Slave3";
+        params["port"] = "12053";
+        params["encoding"] = "1.0";
+        instantiateServer(admin, "IceGridRegistry", params);
+
+        params.clear();
+        params["id"] = "Node2";
+        params["encoding"] = "1.0";
+        instantiateServer(admin, "IceGridNode", params);
+
+        admin->startServer("Slave3");
+        waitForServerState(admin, "Slave3", true);
+        waitForReplicaState(masterAdmin, "Slave3", true);
+
+        admin->startServer("Node2");
+        waitForNodeState(masterAdmin, "Node2", true);
+
+        Ice::LocatorPrx slave3Locator = 
+            Ice::LocatorPrx::uncheckedCast(comm->stringToProxy("TestIceGrid/Locator-Slave3 -e 1.0:default -p 12053"));
+        IceGrid::AdminPrx slave3Admin = createAdminSession(slave3Locator, "Slave3");
+        waitForNodeState(slave3Admin, "Node2", true);
+
+        ApplicationDescriptor app;
+        app.name = "TestApp";
+        app.description = "added application";
+
+        ServerDescriptorPtr server = new ServerDescriptor();
+        server->id = "Server";
+        server->exe = comm->getProperties()->getProperty("TestDir") + "/server";
+        server->pwd = ".";
+        server->applicationDistrib = false;
+        server->allocatable = false;
+        addProperty(server, "Ice.Admin.Endpoints", "tcp -h 127.0.0.1");
+        server->activation = "on-demand";
+        AdapterDescriptor adapter;
+        adapter.name = "TestAdapter";
+        adapter.id = "TestAdapter.Server";
+        adapter.serverLifetime = true;
+        adapter.registerProcess = false;
+        PropertyDescriptor property;
+        property.name = "TestAdapter.Endpoints";
+        property.value = "default";
+        server->propertySet.properties.push_back(property);
+        property.name = "Identity";
+        property.value = "test";
+        server->propertySet.properties.push_back(property);
+        ObjectDescriptor object;
+        object.id = comm->stringToIdentity("test");
+        object.type = "::Test::TestIntf";
+        adapter.objects.push_back(object);
+        server->adapters.push_back(adapter);
+        app.nodes["Node2"].servers.push_back(server);
+
+        masterAdmin->addApplication(app);
+
+        comm->stringToProxy("test -e 1.0")->ice_locator(
+            masterLocator->ice_encodingVersion(Ice::Encoding_1_0))->ice_locatorCacheTimeout(0)->ice_ping();
+        comm->stringToProxy("test -e 1.0")->ice_locator(
+            slave1Locator->ice_encodingVersion(Ice::Encoding_1_0))->ice_locatorCacheTimeout(0)->ice_ping();
+        comm->stringToProxy("test -e 1.0")->ice_locator(slave3Locator)->ice_locatorCacheTimeout(0)->ice_ping();
+        masterAdmin->stopServer("Server");
+
+    }
+    cout << "ok" << endl;
     
     
-    cout << "shutting down Node1... " << flush;
     slave1Admin->shutdownNode("Node1");
-    cout << "ok" << endl;
-
-    cout << "removing Node1 server... " << flush;
     removeServer(admin, "Node1");
-    cout << "ok" << endl;
 
-    cout << "removing Slave2 server..." << flush;
     removeServer(admin, "Slave2");
-    cout << "ok" << endl;
-
 
     slave1Admin->shutdown();
     removeServer(admin, "Slave1");
