@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -29,7 +29,8 @@ Server::run(int argc, char* argv[])
 
     Ice::ObjectAdapterPtr adapter = communicator()->createObjectAdapter("TestAdapter");
     Ice::ObjectPtr object = new TestI();
-    adapter->add(object, communicator()->stringToIdentity("test"));
+    string id = communicator()->getProperties()->getPropertyWithDefault("Identity", "test");
+    adapter->add(object, communicator()->stringToIdentity(id));
 
     shutdownOnInterrupt();
     try
