@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -16,9 +16,9 @@ if len(head) > 0:
     path = [os.path.join(head, p) for p in path]
 path = [os.path.abspath(p) for p in path if os.path.exists(os.path.join(p, "scripts", "TestUtil.py")) ]
 if len(path) == 0:
-    raise "can't find toplevel directory!"
-sys.path.append(os.path.join(path[0]))
-from scripts import *
+    raise RuntimeError("can't find toplevel directory!")
+sys.path.append(os.path.join(path[0], "scripts"))
+import TestUtil, IceStormUtil, IceGridAdmin
 
 targets = []
 if TestUtil.appverifier:
@@ -38,6 +38,11 @@ if TestUtil.sqlType != None:
 else:
     variables += " db-plugin=IceStormFreezeDB:createFreezeDB"
 
+#
+# Don't use a slave registry for this test.
+#
+IceGridAdmin.nreplicas=0
+
 #
 # Test client/server without on demand activation.
 #
