@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -15,7 +15,7 @@ using namespace IceUtil;
 using namespace std;
 
 int
-main(int argc, char** argv)
+main(int, char**)
 {
     {
         IceUtilInternal::FileLockPtr lock;
