@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -42,6 +42,12 @@ class InitialI : InitialDisp_
         return _derivedPrinter;
     }
     
+    public override Printer updatePrinterMessage(Printer printer, Ice.Current current)
+    {
+	printer.message = "a modified message 4 u";
+        return printer;
+    }
+
     public override void throwDerivedPrinter(Ice.Current current)
     {
         DerivedPrinterException ex = new DerivedPrinterException();
