@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -23,6 +23,11 @@ class ObjectFactory : Ice.ObjectFactory
             return new DerivedPrinterI();
         }
         
+        if(type.Equals("::Demo::ClientPrinter"))
+        {
+            return new ClientPrinterI();
+        }
+        
         Debug.Assert(false);
         return null;
     }
