@@ -1,12 +1,14 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+#if !SILVERLIGHT
+
 namespace Ice
 {
     using System;
@@ -18,7 +20,7 @@ namespace Ice
 
     internal static class NativeMethods
     {
-#if !COMPACT
+#if !COMPACT && !UNITY
         //
         // Technically it's not necessary to wrap DllImport in conditional compilation because
         // the binding occurs at run time and it will never be executed on Mono. However, it
@@ -50,7 +52,7 @@ namespace Ice
     }
 
     /// <summary>
-    /// Utility base class that makes it easy to to correctly initialize and finalize
+    /// Utility base class that makes it easy to correctly initialize and finalize
     /// the Ice run time, as well as handle signals. Unless the application specifies
     /// a logger, Application installs a per-process logger that logs to the standard
     /// error output.
@@ -201,15 +203,28 @@ namespace Ice
             {
                 initData = new InitializationData();
             }
-            initData.properties = Util.createProperties(ref args, initData.properties);
-
+            
+            try
+            {
+                initData.properties = Util.createProperties(ref args, initData.properties);
+            }
+            catch(Ice.Exception ex)
+            {
+                Util.getProcessLogger().error(ex.ToString());
+                return 1;
+            }
+            catch(System.Exception ex)
+            {
+                Util.getProcessLogger().error("unknown exception:\n" + ex);
+                return 1;
+            }
             appName__ = initData.properties.getPropertyWithDefault("Ice.ProgramName", appName__);
 
             nohup__ = initData.properties.getPropertyAsInt("Ice.Nohup") > 0;
             _application = this;
 
             int status;
-#if COMPACT
+#if COMPACT || UNITY
             status = doMain(args, initData);
 #else
 
@@ -227,6 +242,7 @@ namespace Ice
 
                 status = doMain(args, initData);
 
+                _signals.destroy();
                 _signals = null;
             }
             else
@@ -815,18 +831,22 @@ namespace Ice
 
         private delegate void SignalHandler(int sig);
         private static readonly SignalHandler _handler = new SignalHandler(signalHandler);
-#if !COMPACT
+#if !COMPACT && !UNITY
         private Signals _signals;
 
         private interface Signals
         {
             void register(SignalHandler handler);
+            void destroy();
         }
 
         private class MonoSignals : Signals
         {
             public void register(SignalHandler handler)
             {
+                _handler = handler;
+                _destroyed = false;
+
                 try
                 {
                     //
@@ -848,18 +868,28 @@ namespace Ice
                     object SIGTERM = Enum.Parse(sigs, "SIGTERM");
                     Type stdlib = a.GetType("Mono.Unix.Native.Stdlib");
                     MethodInfo method = stdlib.GetMethod("signal", BindingFlags.Static | BindingFlags.Public);
-                    Type del = a.GetType("Mono.Unix.Native.SignalHandler");
-                    _delegate = Delegate.CreateDelegate(del, handler.Target, handler.Method);
+                    Type delType = a.GetType("Mono.Unix.Native.SignalHandler");
+                    Delegate del = Delegate.CreateDelegate(delType, this, "callback");
                     object[] args = new object[2];
                     args[0] = SIGHUP;
-                    args[1] = _delegate;
+                    args[1] = del;
                     method.Invoke(null, args);
                     args[0] = SIGINT;
-                    args[1] = _delegate;
+                    args[1] = del;
                     method.Invoke(null, args);
                     args[0] = SIGTERM;
-                    args[1] = _delegate;
+                    args[1] = del;
                     method.Invoke(null, args);
+
+                    //
+                    // Doing certain activities within Mono's signal dispatch thread
+                    // can cause the VM to crash, so we use a separate thread to invoke
+                    // the handler.
+                    //
+                    _thread = new Thread(new ThreadStart(run));
+                    _thread.IsBackground = true;
+                    _thread.Name = "Ice.Application.SignalThread";
+                    _thread.Start();
                 }
                 catch(System.DllNotFoundException)
                 {
@@ -869,13 +899,90 @@ namespace Ice
                     //
                     Util.getProcessLogger().warning("unable to initialize signals");
                 }
-                catch(System.Exception)
+            }
+
+            public void destroy()
+            {
+                _m.Lock();
+                try
                 {
-                    Debug.Assert(false);
+                    _destroyed = true;
+                    _m.Notify();
+                }
+                finally
+                {
+                    _m.Unlock();
+                }
+
+                if(_thread != null)
+                {
+                    _thread.Join();
+                    _thread = null;
                 }
             }
 
-            private Delegate _delegate;
+            private void callback(int sig)
+            {
+                _m.Lock();
+                try
+                {
+                    _signals.Add(sig);
+                    _m.Notify();
+                }
+                finally
+                {
+                    _m.Unlock();
+                }
+            }
+
+            private void run()
+            {
+                while(true)
+                {
+                    List<int> signals = null;
+                    bool destroyed = false;
+
+                    _m.Lock();
+                    try
+                    {
+                        if(!_destroyed && _signals.Count == 0)
+                        {
+                            _m.Wait();
+                        }
+
+                        if(_signals.Count > 0)
+                        {
+                            signals = _signals;
+                            _signals = new List<int>();
+                        }
+
+                        destroyed = _destroyed;
+                    }
+                    finally
+                    {
+                        _m.Unlock();
+                    }
+
+                    if(signals != null)
+                    {
+                        foreach(int sig in signals)
+                        {
+                            _handler(sig);
+                        }
+                    }
+
+                    if(destroyed)
+                    {
+                        break;
+                    }
+                }
+            }
+
+            private SignalHandler _handler;
+            private bool _destroyed;
+            private readonly IceUtilInternal.Monitor _m = new IceUtilInternal.Monitor();
+            private Thread _thread;
+            private List<int> _signals = new List<int>();
         }
 
         private class WindowsSignals : Signals
@@ -897,6 +1004,10 @@ namespace Ice
                 Debug.Assert(rc);
             }
 
+            public void destroy()
+            {
+            }
+
             private CtrlCEventHandler _callback;
             private SignalHandler _handler;
 
@@ -912,3 +1023,4 @@ namespace Ice
     
     delegate bool CtrlCEventHandler(int sig);
 }
+#endif
