@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,6 +22,8 @@ namespace IceInternal
             _instance = instance;
 
             _thread = new HelperThread(this);
+            updateObserver();
+#if !SILVERLIGHT
             if(instance.initializationData().properties.getProperty("Ice.ThreadPriority").Length > 0)
             {
                 ThreadPriority priority = IceInternal.Util.stringToThreadPriority(
@@ -32,7 +34,34 @@ namespace IceInternal
             {
                 _thread.Start(ThreadPriority.Normal);
             }
+#else
+            _thread.Start();
+#endif
+        }
 
+        public void
+        updateObserver()
+        {
+            _m.Lock();
+            try
+            {
+                Ice.Instrumentation.CommunicatorObserver obsv = _instance.getObserver();
+                if(obsv != null)
+                {
+                    _observer = obsv.getThreadObserver("Communicator", 
+                                                       _thread.getName(), 
+                                                       Ice.Instrumentation.ThreadState.ThreadStateIdle, 
+                                                       _observer);
+                    if(_observer != null)
+                    {
+                        _observer.attach();
+                    }
+                }
+            } 
+            finally
+            {
+                _m.Unlock();
+            }
         }
 
         public void queue(ThreadPoolWorkItem callback)
@@ -76,11 +105,19 @@ namespace IceInternal
         public void run()
         {
             LinkedList<ThreadPoolWorkItem> queue = new LinkedList<ThreadPoolWorkItem>();
+            bool inUse = false;
             while(true)
             {
                 _m.Lock();
                 try
                 {
+                    if(_observer != null && inUse)
+                    {
+                        _observer.stateChanged(Ice.Instrumentation.ThreadState.ThreadStateInUseForIO, 
+                                               Ice.Instrumentation.ThreadState.ThreadStateIdle);
+                        inUse = false;
+                    }
+
                     if(_destroyed && _queue.Count == 0)
                     {
                         break;
@@ -94,6 +131,13 @@ namespace IceInternal
                     LinkedList<ThreadPoolWorkItem> tmp = queue;
                     queue = _queue;
                     _queue = tmp;
+
+                    if(_observer != null)
+                    {
+                        _observer.stateChanged(Ice.Instrumentation.ThreadState.ThreadStateIdle, 
+                                               Ice.Instrumentation.ThreadState.ThreadStateInUseForIO);
+                        inUse = true;
+                    }
                 }
                 finally
                 {
@@ -119,11 +163,17 @@ namespace IceInternal
                 }
                 queue.Clear();
             }
+
+            if(_observer != null)
+            {
+                    _observer.detach();
+            }
         }
 
         private Instance _instance;
         private bool _destroyed;
         private LinkedList<ThreadPoolWorkItem> _queue = new LinkedList<ThreadPoolWorkItem>();
+        private Ice.Instrumentation.ThreadObserver _observer;
 
         private sealed class HelperThread
         {
@@ -143,12 +193,23 @@ namespace IceInternal
                 _thread.Join();
             }
 
+            public string getName()
+            {
+                return _name;
+            }
+
+#if !SILVERLIGHT
             public void Start(ThreadPriority priority)
+#else
+            public void Start()
+#endif
             {
                 _thread = new Thread(new ThreadStart(Run));
                 _thread.IsBackground = true;
                 _thread.Name = _name;
+#if !SILVERLIGHT
                 _thread.Priority = priority;
+#endif
                 _thread.Start();
             }
 
