@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,6 +16,7 @@ namespace IceInternal
     using System.Diagnostics;
     using System.Net.Sockets;
     using System.Threading;
+    using System.Text;
     using IceUtilInternal;
 
     public class MultiDictionary<K, V> : Dictionary<K, ICollection<V>>
@@ -71,6 +72,7 @@ namespace IceInternal
                 }
 
                 _destroyed = true;
+                _communicator = null;
                 _m.NotifyAll();
             }
             finally
@@ -79,6 +81,25 @@ namespace IceInternal
             }
         }
 
+        public void updateConnectionObservers()
+        {
+            _m.Lock();
+            try
+            {
+                foreach(ICollection<Ice.ConnectionI> connections in _connections.Values)
+                {
+                    foreach(Ice.ConnectionI c in connections)
+                    {
+                        c.updateObserver();
+                    }
+                }
+            }
+            finally
+            {
+                _m.Unlock();
+            }
+        }
+
         public void waitUntilFinished()
         {
             Dictionary<Connector, ICollection<Ice.ConnectionI>> connections = null;
@@ -181,31 +202,8 @@ namespace IceInternal
                     //
                     // Create connectors for the endpoint.
                     //
-                    List<Connector> cons = endpoint.connectors();
+                    List<Connector> cons = endpoint.connectors(selType);
                     Debug.Assert(cons.Count > 0);
-
-                    //
-                    // Shuffle connectors if endpoint selection type is Random.
-                    //
-                    if(selType == Ice.EndpointSelectionType.Random)
-                    {
-                        lock(rand_)
-                        {       
-                            for(int j = 0; j < cons.Count - 1; ++j)
-                            {
-                                int r = rand_.Next(cons.Count - j) + j;
-                                Debug.Assert(r >= j && r < cons.Count);
-                                if(r != j)
-                                {
-                                    Connector tmp = cons[j];
-                                    cons[j] = cons[r];
-                                    cons[r] = tmp;
-                                }
-                            }
-
-                        }
-                    }
-
                     foreach(Connector conn in cons)
                     {
                         connectors.Add(new ConnectorInfo(conn, endpoint));
@@ -239,16 +237,33 @@ namespace IceInternal
             //
             // Try to establish the connection to the connectors.
             //
-            DefaultsAndOverrides defaultsAndOverrides = instance_.defaultsAndOverrides();
+            DefaultsAndOverrides defaultsAndOverrides = _instance.defaultsAndOverrides();
+            Ice.Instrumentation.CommunicatorObserver obsv = _instance.getObserver();
             ConnectorInfo ci = null;
             for(int i = 0; i < connectors.Count; ++i)
             {
                 ci = connectors[i];
+
+                Ice.Instrumentation.Observer observer = null;
+                if(obsv != null)
+                {
+                    observer = obsv.getConnectionEstablishmentObserver(ci.endpoint, ci.connector.ToString());
+                    if(observer != null)
+                    {
+                        observer.attach();
+                    }
+                }
+
                 try
                 {
                     connection = createConnection(ci.connector.connect(), ci);
                     connection.start(null);
 
+                    if(observer != null)
+                    {
+                        observer.detach();
+                    }
+
                     if(defaultsAndOverrides.overrideCompress)
                     {
                         compress = defaultsAndOverrides.overrideCompressValue;
@@ -262,6 +277,11 @@ namespace IceInternal
                 }
                 catch(Ice.CommunicatorDestroyedException ex)
                 {
+                    if(observer != null)
+                    {
+                        observer.failed(ex.ice_name());
+                        observer.detach();
+                    }
                     exception = ex;
                     handleConnectionException(exception, hasMore || i < connectors.Count - 1);
                     connection = null;
@@ -269,6 +289,11 @@ namespace IceInternal
                 }
                 catch(Ice.LocalException ex)
                 {
+                    if(observer != null)
+                    {
+                        observer.failed(ex.ice_name());
+                        observer.detach();
+                    }
                     exception = ex;
                     handleConnectionException(exception, hasMore || i < connectors.Count - 1);
                     connection = null;
@@ -349,7 +374,7 @@ namespace IceInternal
                 // received over such connections.
                 //
                 Ice.ObjectAdapter adapter = routerInfo.getAdapter();
-                DefaultsAndOverrides defaultsAndOverrides = instance_.defaultsAndOverrides();
+                DefaultsAndOverrides defaultsAndOverrides = _instance.defaultsAndOverrides();
                 EndpointI[] endpoints = routerInfo.getClientEndpoints();
                 for(int i = 0; i < endpoints.Length; i++)
                 {
@@ -398,7 +423,7 @@ namespace IceInternal
             try
             {
                 if(_destroyed)
-                { 
+                {
                     return;
                 }
 
@@ -461,16 +486,17 @@ namespace IceInternal
         //
         // Only for use by Instance.
         //
-        internal OutgoingConnectionFactory(Instance instance)
+        internal OutgoingConnectionFactory(Ice.Communicator communicator, Instance instance)
         {
-            instance_ = instance;
+            _communicator = communicator;
+            _instance = instance;
             _destroyed = false;
             _pendingConnectCount = 0;
         }
 
         private List<EndpointI> applyOverrides(EndpointI[] endpts)
         {
-            DefaultsAndOverrides defaultsAndOverrides = instance_.defaultsAndOverrides();
+            DefaultsAndOverrides defaultsAndOverrides = _instance.defaultsAndOverrides();
             List<EndpointI> endpoints = new List<EndpointI>();
             for(int i = 0; i < endpts.Length; i++)
             {
@@ -500,7 +526,7 @@ namespace IceInternal
                     throw new Ice.CommunicatorDestroyedException();
                 }
 
-                DefaultsAndOverrides defaultsAndOverrides = instance_.defaultsAndOverrides();
+                DefaultsAndOverrides defaultsAndOverrides = _instance.defaultsAndOverrides();
                 Debug.Assert(endpoints.Count > 0);
 
                 foreach(EndpointI endpoint in endpoints)
@@ -542,7 +568,7 @@ namespace IceInternal
         //
         private Ice.ConnectionI findConnection(List<ConnectorInfo> connectors, out bool compress)
         {
-            DefaultsAndOverrides defaultsAndOverrides = instance_.defaultsAndOverrides();
+            DefaultsAndOverrides defaultsAndOverrides = _instance.defaultsAndOverrides();
             foreach(ConnectorInfo ci in connectors)
             {
                 if(_pending.ContainsKey(ci.connector))
@@ -580,13 +606,13 @@ namespace IceInternal
         internal void incPendingConnectCount()
         {
             //
-            // Keep track of the number of pending connects. The outgoing connection factory 
+            // Keep track of the number of pending connects. The outgoing connection factory
             // waitUntilFinished() method waits for all the pending connects to terminate before
             // to return. This ensures that the communicator client thread pool isn't destroyed
             // too soon and will still be available to execute the ice_exception() callbacks for
             // the asynchronous requests waiting on a connection to be established.
             //
-            
+
             _m.Lock();
             try
             {
@@ -668,9 +694,9 @@ namespace IceInternal
                     if(addToPending(cb, connectors))
                     {
                         //
-                        // If a callback is not specified we wait until another thread notifies us about a 
-                        // change to the pending list. Otherwise, if a callback is provided we're done: 
-                        // when the pending list changes the callback will be notified and will try to 
+                        // If a callback is not specified we wait until another thread notifies us about a
+                        // change to the pending list. Otherwise, if a callback is provided we're done:
+                        // when the pending list changes the callback will be notified and will try to
                         // get the connection again.
                         //
                         if(cb == null)
@@ -699,7 +725,7 @@ namespace IceInternal
             }
 
             //
-            // At this point, we're responsible for establishing the connection to one of 
+            // At this point, we're responsible for establishing the connection to one of
             // the given connectors. If it's a non-blocking connect, calling nextConnector
             // will start the connection establishment. Otherwise, we return null to get
             // the caller to establish the connection.
@@ -733,7 +759,7 @@ namespace IceInternal
                         throw new Ice.CommunicatorDestroyedException();
                     }
 
-                    connection = new Ice.ConnectionI(instance_, _reaper, transceiver, ci.connector, 
+                    connection = new Ice.ConnectionI(_communicator, _instance, _reaper, transceiver, ci.connector,
                                                      ci.endpoint.compress(false), null);
                 }
                 catch(Ice.LocalException)
@@ -760,8 +786,8 @@ namespace IceInternal
             }
         }
 
-        private void finishGetConnection(List<ConnectorInfo> connectors, 
-                                         ConnectorInfo ci, 
+        private void finishGetConnection(List<ConnectorInfo> connectors,
+                                         ConnectorInfo ci,
                                          Ice.ConnectionI connection,
                                          ConnectCallback cb)
         {
@@ -812,7 +838,7 @@ namespace IceInternal
             }
 
             bool compress;
-            DefaultsAndOverrides defaultsAndOverrides = instance_.defaultsAndOverrides();
+            DefaultsAndOverrides defaultsAndOverrides = _instance.defaultsAndOverrides();
             if(defaultsAndOverrides.overrideCompress)
             {
                 compress = defaultsAndOverrides.overrideCompressValue;
@@ -839,7 +865,7 @@ namespace IceInternal
             {
                 failedCallbacks.Add(cb);
             }
-            
+
             HashSet<ConnectCallback> callbacks = new HashSet<ConnectCallback>();
             _m.Lock();
             try
@@ -888,7 +914,7 @@ namespace IceInternal
 
         private void handleConnectionException(Ice.LocalException ex, bool hasMore)
         {
-            TraceLevels traceLevels = instance_.traceLevels();
+            TraceLevels traceLevels = _instance.traceLevels();
             if(traceLevels.retry >= 2)
             {
                 System.Text.StringBuilder s = new System.Text.StringBuilder();
@@ -909,7 +935,7 @@ namespace IceInternal
                     }
                 }
                 s.Append(ex);
-                instance_.initializationData().logger.trace(traceLevels.retryCat, s.ToString());
+                _instance.initializationData().logger.trace(traceLevels.retryCat, s.ToString());
             }
         }
 
@@ -939,8 +965,8 @@ namespace IceInternal
             }
 
             //
-            // If there's no pending connection for the given connectors, we're 
-            // responsible for its establishment. We add empty pending lists, 
+            // If there's no pending connection for the given connectors, we're
+            // responsible for its establishment. We add empty pending lists,
             // other callbacks to the same connectors will be queued.
             //
             foreach(ConnectorInfo ci in connectors)
@@ -968,7 +994,7 @@ namespace IceInternal
 
         internal void handleException(Ice.LocalException ex, bool hasMore)
         {
-            TraceLevels traceLevels = instance_.traceLevels();
+            TraceLevels traceLevels = _instance.traceLevels();
             if(traceLevels.retry >= 2)
             {
                 System.Text.StringBuilder s = new System.Text.StringBuilder();
@@ -989,7 +1015,7 @@ namespace IceInternal
                     }
                 }
                 s.Append(ex);
-                instance_.initializationData().logger.trace(traceLevels.retryCat, s.ToString());
+                _instance.initializationData().logger.trace(traceLevels.retryCat, s.ToString());
             }
         }
 
@@ -1034,12 +1060,21 @@ namespace IceInternal
             //
             public void connectionStartCompleted(Ice.ConnectionI connection)
             {
+                if(_observer != null)
+                {
+                    _observer.detach();
+                }
                 connection.activate();
                 _factory.finishGetConnection(_connectors, _current, connection, this);
             }
 
             public void connectionStartFailed(Ice.ConnectionI connection, Ice.LocalException ex)
             {
+                if(_observer != null)
+                {
+                    _observer.failed(ex.ice_name());
+                    _observer.detach();
+                }
                 _factory.handleConnectionException(ex, _hasMore || _iter < _connectors.Count);
                 if(ex is Ice.CommunicatorDestroyedException) // No need to continue.
                 {
@@ -1060,27 +1095,6 @@ namespace IceInternal
             //
             public void connectors(List<Connector> cons)
             {
-                //
-                // Shuffle connectors if endpoint selection type is Random.
-                //
-                if(_selType == Ice.EndpointSelectionType.Random)
-                {
-                    lock(rand_)
-                    {
-                        for(int j = 0; j < cons.Count - 1; ++j)
-                        {
-                            int r = OutgoingConnectionFactory.rand_.Next(cons.Count - j) + j;
-                            Debug.Assert(r >= j && r < cons.Count);
-                            if(r != j)
-                            {
-                                Connector tmp = cons[j];
-                                cons[j] = cons[r];
-                                cons[r] = tmp;
-                            }
-                        }
-                    }
-                }
-
                 foreach(Connector connector in cons)
                 {
                     _connectors.Add(new ConnectorInfo(connector, _currentEndpoint));
@@ -1190,7 +1204,7 @@ namespace IceInternal
                 {
                     Debug.Assert(_endpointsIter < _endpoints.Count);
                     _currentEndpoint = _endpoints[_endpointsIter++];
-                    _currentEndpoint.connectors_async(this);
+                    _currentEndpoint.connectors_async(_selType, this);
                 }
                 catch(Ice.LocalException ex)
                 {
@@ -1213,9 +1227,9 @@ namespace IceInternal
                         //
                         // A null return value from getConnection indicates that the connection
                         // is being established and that everthing has been done to ensure that
-                        // the callback will be notified when the connection establishment is 
+                        // the callback will be notified when the connection establishment is
                         // done.
-                        // 
+                        //
                         return;
                     }
 
@@ -1236,6 +1250,18 @@ namespace IceInternal
                 {
                     Debug.Assert(_iter < _connectors.Count);
                     _current = _connectors[_iter++];
+
+                    Ice.Instrumentation.CommunicatorObserver obsv = _factory._instance.getObserver();
+                    if(obsv != null)
+                    {
+                        _observer = obsv.getConnectionEstablishmentObserver(_current.endpoint, 
+                                                                            _current.connector.ToString());
+                        if(_observer != null)
+                        {
+                            _observer.attach();
+                        }
+                    }
+
                     connection = _factory.createConnection(_current.connector.connect(), _current);
                     connection.start(this);
                 }
@@ -1255,22 +1281,22 @@ namespace IceInternal
             private List<ConnectorInfo> _connectors = new List<ConnectorInfo>();
             private int _iter;
             private ConnectorInfo _current;
+            private Ice.Instrumentation.Observer _observer;
         }
 
-        private readonly Instance instance_;
+        private Ice.Communicator _communicator;
+        private readonly Instance _instance;
         private ConnectionReaper _reaper = new ConnectionReaper();
         private bool _destroyed;
 
-        private MultiDictionary<Connector, Ice.ConnectionI> _connections = 
+        private MultiDictionary<Connector, Ice.ConnectionI> _connections =
             new MultiDictionary<Connector, Ice.ConnectionI>();
-        private MultiDictionary<EndpointI, Ice.ConnectionI> _connectionsByEndpoint = 
+        private MultiDictionary<EndpointI, Ice.ConnectionI> _connectionsByEndpoint =
             new MultiDictionary<EndpointI, Ice.ConnectionI>();
-        private Dictionary<Connector, HashSet<ConnectCallback>> _pending = 
+        private Dictionary<Connector, HashSet<ConnectCallback>> _pending =
             new Dictionary<Connector, HashSet<ConnectCallback>>();
         private int _pendingConnectCount;
 
-        private static System.Random rand_ = new System.Random(unchecked((int)System.DateTime.Now.Ticks));
-
         private readonly IceUtilInternal.Monitor _m = new IceUtilInternal.Monitor();
     }
 
@@ -1315,6 +1341,22 @@ namespace IceInternal
             }
         }
 
+        public void updateConnectionObservers()
+        {
+            _m.Lock();
+            try
+            {
+                foreach(Ice.ConnectionI connection in _connections)
+                {
+                    connection.updateObserver();
+                }
+            }
+            finally
+            {
+                _m.Unlock();
+            }
+        }
+
         public void waitUntilHolding()
         {
             ICollection<Ice.ConnectionI> connections;
@@ -1459,7 +1501,7 @@ namespace IceInternal
         //
         // Operations from EventHandler.
         //
-        public override bool startAsync(int unused, AsyncCallback callback, ref bool completedSynchronously)
+        public override bool startAsync(int operation, AsyncCallback callback, ref bool completedSynchronously)
         {
             if(_state >= StateClosed)
             {
@@ -1480,7 +1522,7 @@ namespace IceInternal
                 }
                 finally
                 {
-#if !COMPACT
+#if !COMPACT && !SILVERLIGHT
                     System.Environment.FailFast(s);
 #endif
                 }
@@ -1507,7 +1549,7 @@ namespace IceInternal
                     }
                     finally
                     {
-#if !COMPACT
+#if !COMPACT && !SILVERLIGHT
                         System.Environment.FailFast(s);
 #endif
                     }
@@ -1536,7 +1578,7 @@ namespace IceInternal
                 {
                     return;
                 }
-                
+
                 try
                 {
                     if(_state >= StateClosed)
@@ -1547,7 +1589,7 @@ namespace IceInternal
                     {
                         return;
                     }
-                
+
                     //
                     // Reap closed connections
                     //
@@ -1579,7 +1621,7 @@ namespace IceInternal
                             }
                             finally
                             {
-#if !COMPACT
+#if !COMPACT && !SILVERLIGHT
                                 System.Environment.FailFast(s);
 #endif
                             }
@@ -1602,7 +1644,8 @@ namespace IceInternal
 
                     try
                     {
-                        connection = new Ice.ConnectionI(_instance, _reaper, transceiver, null, _endpoint, _adapter);
+                        connection = new Ice.ConnectionI(_adapter.getCommunicator(), _instance, _reaper, transceiver,
+                                                         null, _endpoint, _adapter);
                     }
                     catch(Ice.LocalException ex)
                     {
@@ -1696,10 +1739,9 @@ namespace IceInternal
                     return;
                 }
 
-                if(_warn)
-                {
-                    warning(ex);
-                }
+                //
+                // Do not warn about connection exceptions here. The connection is not yet validated.
+                //
             }
             finally
             {
@@ -1733,8 +1775,8 @@ namespace IceInternal
                 _transceiver = _endpoint.transceiver(ref _endpoint);
                 if(_transceiver != null)
                 {
-                    Ice.ConnectionI connection = 
-                        new Ice.ConnectionI(_instance, _reaper, _transceiver, null, _endpoint, _adapter);
+                    Ice.ConnectionI connection = new Ice.ConnectionI(_adapter.getCommunicator(), _instance, _reaper, 
+                                                                     _transceiver, null, _endpoint, _adapter);
                     connection.start(null);
                     _connections.Add(connection);
                 }
@@ -1772,7 +1814,7 @@ namespace IceInternal
                     }
                     catch(Ice.LocalException)
                     {
-                        // Here we ignore any exceptions in close().                    
+                        // Here we ignore any exceptions in close().
                     }
                 }
 
@@ -1804,7 +1846,7 @@ namespace IceInternal
 
             switch (state)
             {
-                case StateActive: 
+                case StateActive:
                 {
                     if(_state != StateHolding) // Can only switch from holding to active.
                     {
@@ -1812,6 +1854,15 @@ namespace IceInternal
                     }
                     if(_acceptor != null)
                     {
+                        if(_instance.traceLevels().network >= 1)
+                        {
+                            StringBuilder s = new StringBuilder("accepting ");
+                            s.Append(_endpoint.protocol());
+                            s.Append(" connections at ");
+                            s.Append(_acceptor.ToString());
+                            _instance.initializationData().logger.trace(_instance.traceLevels().networkCat, 
+                                                                        s.ToString());
+                        }                
                         ((Ice.ObjectAdapterI)_adapter).getThreadPool().register(this, SocketOperation.Read);
                     }
 
@@ -1822,7 +1873,7 @@ namespace IceInternal
                     break;
                 }
 
-                case StateHolding: 
+                case StateHolding:
                 {
                     if(_state != StateActive) // Can only switch from active to holding.
                     {
@@ -1830,6 +1881,15 @@ namespace IceInternal
                     }
                     if(_acceptor != null)
                     {
+                        if(_instance.traceLevels().network >= 1)
+                        {
+                            StringBuilder s = new StringBuilder("holding ");
+                            s.Append(_endpoint.protocol());
+                            s.Append(" connections at ");
+                            s.Append(_acceptor.ToString());
+                            _instance.initializationData().logger.trace(_instance.traceLevels().networkCat, 
+                                                                        s.ToString());
+                        }                
                         ((Ice.ObjectAdapterI)_adapter).getThreadPool().unregister(this, SocketOperation.Read);
                     }
 
@@ -1840,7 +1900,7 @@ namespace IceInternal
                     break;
                 }
 
-                case StateClosed: 
+                case StateClosed:
                 {
                     if(_acceptor != null)
                     {
@@ -1851,7 +1911,7 @@ namespace IceInternal
                     {
                         state = StateFinished;
                     }
-                
+
                     foreach(Ice.ConnectionI connection in _connections)
                     {
                         connection.destroy(Ice.ConnectionI.ObjectAdapterDeactivated);
