@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,6 +19,26 @@ namespace IceInternal
 
     public sealed class Instance
     {
+        private class ObserverUpdaterI : Ice.Instrumentation.ObserverUpdater
+        {
+            public ObserverUpdaterI(Instance instance)
+            {
+                _instance = instance;
+            }
+
+            public void updateConnectionObservers()
+            {
+                _instance.updateConnectionObservers();
+            }
+
+            public void updateThreadObservers()
+            {
+                _instance.updateThreadObservers();
+            }
+
+            private Instance _instance;
+        }
+
         public bool destroyed()
         {
             return _state == StateDestroyed;
@@ -34,14 +54,14 @@ namespace IceInternal
             //
             return _initData;
         }
-        
+
         public TraceLevels traceLevels()
         {
             // No mutex lock, immutable.
             Debug.Assert(_traceLevels != null);
             return _traceLevels;
         }
-        
+
         public DefaultsAndOverrides defaultsAndOverrides()
         {
             // No mutex lock, immutable.
@@ -49,7 +69,7 @@ namespace IceInternal
             return _defaultsAndOverrides;
         }
 
-#if COMPACT
+#if COMPACT || SILVERLIGHT
         public string[] factoryAssemblies()
         {
             return _factoryAssemblies;
@@ -64,12 +84,12 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_routerManager != null);
                 return _routerManager;
             }
         }
-        
+
         public LocatorManager locatorManager()
         {
             lock(this)
@@ -83,7 +103,7 @@ namespace IceInternal
                 return _locatorManager;
             }
         }
-        
+
         public ReferenceFactory referenceFactory()
         {
             lock(this)
@@ -92,12 +112,12 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_referenceFactory != null);
                 return _referenceFactory;
             }
         }
-        
+
         public ProxyFactory proxyFactory()
         {
             lock(this)
@@ -106,12 +126,12 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_proxyFactory != null);
                 return _proxyFactory;
             }
         }
-        
+
         public OutgoingConnectionFactory outgoingConnectionFactory()
         {
             lock(this)
@@ -120,12 +140,12 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_outgoingConnectionFactory != null);
                 return _outgoingConnectionFactory;
             }
         }
-        
+
         public ConnectionMonitor connectionMonitor()
         {
             lock(this)
@@ -134,12 +154,12 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_connectionMonitor != null);
                 return _connectionMonitor;
             }
         }
-        
+
         public ObjectFactoryManager servantFactoryManager()
         {
             lock(this)
@@ -148,12 +168,12 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_servantFactoryManager != null);
                 return _servantFactoryManager;
             }
         }
-        
+
         public ObjectAdapterFactory objectAdapterFactory()
         {
             lock(this)
@@ -162,7 +182,7 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_objectAdapterFactory != null);
                 return _objectAdapterFactory;
             }
@@ -170,17 +190,19 @@ namespace IceInternal
 
         public int protocolSupport()
         {
-            lock(this)
-            {
-                if(_state == StateDestroyed)
-                {
-                    throw new Ice.CommunicatorDestroyedException();
-                }
-                
-                return _protocolSupport;
-            }
+            return _protocolSupport;
         }
-        
+
+        public bool preferIPv6()
+        {
+            return _preferIPv6;
+        }
+
+        public NetworkProxy networkProxy()
+        {
+            return _networkProxy;
+        }
+
         public ThreadPool clientThreadPool()
         {
             lock(this)
@@ -189,13 +211,13 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_clientThreadPool != null);
                 return _clientThreadPool;
             }
         }
-        
-        public ThreadPool serverThreadPool()
+
+        public ThreadPool serverThreadPool(bool create)
         {
             lock(this)
             {
@@ -203,13 +225,13 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
-                if(_serverThreadPool == null) // Lazy initialization.
+
+                if(_serverThreadPool == null && create) // Lazy initialization.
                 {
                     int timeout = _initData.properties.getPropertyAsInt("Ice.ServerIdleTime");
                     _serverThreadPool = new ThreadPool(this, "Ice.ThreadPool.Server", timeout);
                 }
-                
+
                 return _serverThreadPool;
             }
         }
@@ -222,17 +244,18 @@ namespace IceInternal
                 if(_state == StateDestroyed)
                 {
                     throw new Ice.CommunicatorDestroyedException();
-                }        
-                
+                }
+
                 if(_asyncIOThread == null) // Lazy initialization.
                 {
                     _asyncIOThread = new AsyncIOThread(this);
                 }
-            
+
                 return _asyncIOThread;
             }
         }
 
+#if !SILVERLIGHT
         public EndpointHostResolver endpointHostResolver()
         {
             lock(this)
@@ -246,7 +269,7 @@ namespace IceInternal
                 return _endpointHostResolver;
             }
         }
-
+#endif
         public RetryQueue
         retryQueue()
         {
@@ -256,7 +279,7 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_retryQueue != null);
                 return _retryQueue;
             }
@@ -285,12 +308,12 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_endpointFactoryManager != null);
                 return _endpointFactoryManager;
             }
         }
-        
+
         public Ice.PluginManager pluginManager()
         {
             lock(this)
@@ -299,30 +322,30 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Debug.Assert(_pluginManager != null);
                 return _pluginManager;
             }
         }
-        
+
         public int messageSizeMax()
         {
             // No mutex lock, immutable.
             return _messageSizeMax;
         }
-        
+
         public int clientACM()
         {
             // No mutex lock, immutable.
             return _clientACM;
         }
-        
+
         public int serverACM()
         {
             // No mutex lock, immutable.
             return _serverACM;
         }
-        
+
         public Ice.ImplicitContextI getImplicitContext()
         {
             return _implicitContext;
@@ -338,7 +361,7 @@ namespace IceInternal
             return Ice.Util.identityToString(ident);
         }
 
-        public Ice.ObjectPrx 
+        public Ice.ObjectPrx
         getAdmin()
         {
             Ice.ObjectAdapter adapter = null;
@@ -351,9 +374,9 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 string adminOA = "Ice.Admin";
-                
+
                 if(_adminAdapter != null)
                 {
                     return _adminAdapter.createProxy(_adminIdentity);
@@ -366,9 +389,9 @@ namespace IceInternal
                 {
                     serverId = _initData.properties.getProperty("Ice.Admin.ServerId");
                     string instanceName = _initData.properties.getProperty("Ice.Admin.InstanceName");
-                    
+
                     defaultLocator = _referenceFactory.getDefaultLocator();
-                    
+
                     if((defaultLocator != null && serverId.Length > 0) || instanceName.Length > 0)
                     {
                         if(_adminIdentity == null)
@@ -382,12 +405,12 @@ namespace IceInternal
                             // Afterwards, _adminIdentity is read-only
                             //
                         }
-                        
+
                         //
                         // Create OA
                         //
                         _adminAdapter = _objectAdapterFactory.createObjectAdapter(adminOA, null);
-                
+
                         //
                         // Add all facets to OA
                         //
@@ -438,12 +461,12 @@ namespace IceInternal
 
                 Ice.ObjectPrx admin = adapter.createProxy(_adminIdentity);
                 if(defaultLocator != null && serverId.Length > 0)
-                {    
+                {
                     Ice.ProcessPrx process = Ice.ProcessPrxHelper.uncheckedCast(admin.ice_facet("Process"));
                     try
                     {
                         //
-                        // Note that as soon as the process proxy is registered, the communicator might be 
+                        // Note that as soon as the process proxy is registered, the communicator might be
                         // shutdown by a remote client and admin facets might start receiving calls.
                         //
                         defaultLocator.getRegistry().setServerProcessProxy(serverId, process);
@@ -457,7 +480,7 @@ namespace IceInternal
                             s.Append("the server is not known to the locator registry");
                             _initData.logger.trace(_traceLevels.locationCat, s.ToString());
                         }
-                        
+
                         throw new Ice.InitializationException("Locator knows nothing about server '" + serverId +
                                                               "'");
                     }
@@ -471,7 +494,7 @@ namespace IceInternal
                         }
                         throw; // TODO: Shall we raise a special exception instead of a non obvious local exception?
                     }
-            
+
                     if(_traceLevels.location >= 1)
                     {
                         System.Text.StringBuilder s = new System.Text.StringBuilder();
@@ -480,10 +503,10 @@ namespace IceInternal
                     }
                 }
                 return admin;
-            }    
+            }
         }
-        
-        public void 
+
+        public void
         addAdminFacet(Ice.Object servant, string facet)
         {
             lock(this)
@@ -492,8 +515,8 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-            
-                if(_adminAdapter == null || (_adminFacetFilter.Count == 0 && !_adminFacetFilter.Contains(facet)))
+
+                if(_adminAdapter == null || (_adminFacetFilter.Count > 0 && !_adminFacetFilter.Contains(facet)))
                 {
                     if(_adminFacets.ContainsKey(facet))
                     {
@@ -508,8 +531,7 @@ namespace IceInternal
             }
         }
 
-
-        public Ice.Object 
+        public Ice.Object
         removeAdminFacet(string facet)
         {
             lock(this)
@@ -518,9 +540,9 @@ namespace IceInternal
                 {
                     throw new Ice.CommunicatorDestroyedException();
                 }
-                
+
                 Ice.Object result = null;
-                if(_adminAdapter == null || (_adminFacetFilter.Count == 0 && !_adminFacetFilter.Contains(facet)))
+                if(_adminAdapter == null || (_adminFacetFilter.Count > 0 && !_adminFacetFilter.Contains(facet)))
                 {
                     try
                     {
@@ -530,7 +552,7 @@ namespace IceInternal
                     {
                         throw new Ice.NotRegisteredException("facet", facet);
                     }
-                    
+
                     _adminFacets.Remove(facet);
                 }
                 else
@@ -541,6 +563,41 @@ namespace IceInternal
             }
         }
 
+        public Ice.Object
+        findAdminFacet(string facet)
+        {
+            lock(this)
+            {
+                if(_state == StateDestroyed)
+                {
+                    throw new Ice.CommunicatorDestroyedException();
+                }
+
+                Ice.Object result = null;
+                if(_adminAdapter == null || (_adminFacetFilter.Count > 0 && !_adminFacetFilter.Contains(facet)))
+                {
+                    try
+                    {
+                        result = _adminFacets[facet];
+                    }
+                    catch(KeyNotFoundException)
+                    {
+                    }
+                }
+                else
+                {
+                    result = _adminAdapter.findFacet(_adminIdentity, facet);
+                }
+                return result;
+            }
+        }
+
+        public Ice.Instrumentation.CommunicatorObserver
+        getObserver()
+        {
+            return _observer;
+        }
+
         public void
         setDefaultLocator(Ice.LocatorPrx locator)
         {
@@ -594,34 +651,34 @@ namespace IceInternal
         {
             _state = StateActive;
             _initData = initData;
-                
+
             try
             {
                 if(_initData.properties == null)
                 {
                     _initData.properties = Ice.Util.createProperties();
                 }
-
+#if !SILVERLIGHT && !UNITY
                 lock(_staticLock)
                 {
                     if(!_oneOffDone)
                     {
                         string stdOut = _initData.properties.getProperty("Ice.StdOut");
                         string stdErr = _initData.properties.getProperty("Ice.StdErr");
-                        
+
                         System.IO.StreamWriter outStream = null;
-                        
+
                         if(stdOut.Length > 0)
                         {
                             try
-                            {                       
+                            {
                                 outStream = System.IO.File.AppendText(stdOut);
                             }
                             catch(System.IO.IOException ex)
                             {
                                 Ice.FileException fe = new Ice.FileException(ex);
                                 fe.path = stdOut;
-                                throw fe;       
+                                throw fe;
                             }
                             outStream.AutoFlush = true;
                             System.Console.Out.Close();
@@ -631,7 +688,7 @@ namespace IceInternal
                         {
                             if(stdErr.Equals(stdOut))
                             {
-                                System.Console.SetError(outStream); 
+                                System.Console.SetError(outStream);
                             }
                             else
                             {
@@ -644,7 +701,7 @@ namespace IceInternal
                                 {
                                     Ice.FileException fe = new Ice.FileException(ex);
                                     fe.path = stdErr;
-                                    throw fe;   
+                                    throw fe;
                                 }
                                 errStream.AutoFlush = true;
                                 System.Console.Error.Close();
@@ -655,11 +712,14 @@ namespace IceInternal
                         _oneOffDone = true;
                     }
                 }
-                
+#endif
+
                 if(_initData.logger == null)
                 {
+#if !SILVERLIGHT && !UNITY
                     string logfile = _initData.properties.getProperty("Ice.LogFile");
-                    if(_initData.properties.getPropertyAsInt("Ice.UseSyslog") > 0)
+                    if(_initData.properties.getPropertyAsInt("Ice.UseSyslog") > 0 &&
+                       AssemblyUtil.platform_ != AssemblyUtil.Platform.Windows)
                     {
                         if(logfile.Length != 0)
                         {
@@ -668,34 +728,51 @@ namespace IceInternal
                         _initData.logger = new Ice.SysLoggerI(_initData.properties.getProperty("Ice.ProgramName"),
                             _initData.properties.getPropertyWithDefault("Ice.SyslogFacility", "LOG_USER"));
                     }
-                    else if(logfile.Length != 0 || Ice.Util.getProcessLogger() is Ice.LoggerI) 
+                    else if(logfile.Length != 0)
+                    {
+
+                        _initData.logger =
+                            new Ice.FileLoggerI(_initData.properties.getProperty("Ice.ProgramName"), logfile);
+                    }
+                    else if(Ice.Util.getProcessLogger() is Ice.LoggerI)
                     {
                         //
-                        // Ice.ConsoleListener is enabled by default unless Ice.LogFile is set.
+                        // Ice.ConsoleListener is enabled by default.
                         //
-                        bool console = 
-                            _initData.properties.getPropertyAsIntWithDefault("Ice.ConsoleListener",
-                                                                             logfile.Length == 0 ? 1 : 0) > 0;
-                        _initData.logger = 
-                            new Ice.TraceLoggerI(_initData.properties.getProperty("Ice.ProgramName"), logfile, console);
+#  if COMPACT
+                        _initData.logger =
+                            new Ice.ConsoleLoggerI(_initData.properties.getProperty("Ice.ProgramName"));
+#  else
+                        bool console =
+                            _initData.properties.getPropertyAsIntWithDefault("Ice.ConsoleListener", 1) == 1;
+                        _initData.logger =
+                            new Ice.TraceLoggerI(_initData.properties.getProperty("Ice.ProgramName"), console);
+#  endif
                     }
+#else
+                    if(Ice.Util.getProcessLogger() is Ice.LoggerI)
+                    {
+                        _initData.logger =
+                            new Ice.ConsoleLoggerI(_initData.properties.getProperty("Ice.ProgramName"));
+                    }
+#endif
                     else
                     {
                         _initData.logger = Ice.Util.getProcessLogger();
                     }
                 }
-                
+
                 _traceLevels = new TraceLevels(_initData.properties);
-                
+
                 _defaultsAndOverrides = new DefaultsAndOverrides(_initData.properties);
 
-#if COMPACT
-                _factoryAssemblies = _initData.properties.getPropertyAsList("Ice.FactoryAssemblies");
+#if COMPACT || SILVERLIGHT
+                char[] separators = { ' ', '\t', '\n', '\r' };
+                _factoryAssemblies = _initData.properties.getProperty("Ice.FactoryAssemblies").Split(separators);
 #endif
-
                 {
                     const int defaultMessageSizeMax = 1024;
-                    int num = 
+                    int num =
                         _initData.properties.getPropertyAsIntWithDefault("Ice.MessageSizeMax", defaultMessageSizeMax);
                     if(num < 1)
                     {
@@ -710,7 +787,7 @@ namespace IceInternal
                         _messageSizeMax = num * 1024; // Property is in kilobytes, _messageSizeMax in bytes
                     }
                 }
-                
+
                 //
                 // Client ACM enabled by default. Server ACM disabled by default.
                 //
@@ -719,15 +796,24 @@ namespace IceInternal
 
                 _implicitContext = Ice.ImplicitContextI.create(_initData.properties.getProperty("Ice.ImplicitContext"));
                 _routerManager = new RouterManager();
-                
+
                 _locatorManager = new LocatorManager(_initData.properties);
-                
+
                 _referenceFactory = new ReferenceFactory(this, communicator);
-                
+
                 _proxyFactory = new ProxyFactory(this);
-                
+
+                string proxyHost = _initData.properties.getProperty("Ice.SOCKSProxyHost");
+                int defaultIPv6 = 1; // IPv6 enabled by default.
+                if(proxyHost.Length > 0)
+                {
+                    int proxyPort = _initData.properties.getPropertyAsIntWithDefault("Ice.SOCKSProxyPort", 1080);
+                    _networkProxy = new SOCKSNetworkProxy(proxyHost, proxyPort);
+                    defaultIPv6 = 0; // IPv6 is not supported with SOCKS
+                }
+
                 bool ipv4 = _initData.properties.getPropertyAsIntWithDefault("Ice.IPv4", 1) > 0;
-                bool ipv6 = _initData.properties.getPropertyAsIntWithDefault("Ice.IPv6", 0) > 0;
+                bool ipv6 = _initData.properties.getPropertyAsIntWithDefault("Ice.IPv6", defaultIPv6) > 0;
                 if(!ipv4 && !ipv6)
                 {
                     throw new Ice.InitializationException("Both IPV4 and IPv6 support cannot be disabled.");
@@ -744,20 +830,29 @@ namespace IceInternal
                 {
                     _protocolSupport = Network.EnableIPv6;
                 }
+                _preferIPv6 = _initData.properties.getPropertyAsInt("Ice.PreferIPv6Address") > 0;
+
+                if(ipv6 && _networkProxy is SOCKSNetworkProxy)
+                {
+                    throw new Ice.InitializationException("IPv6 is not supported with SOCKS4 proxies");
+                }
+
                 _endpointFactoryManager = new EndpointFactoryManager(this);
                 EndpointFactory tcpEndpointFactory = new TcpEndpointFactory(this);
                 _endpointFactoryManager.add(tcpEndpointFactory);
                 EndpointFactory udpEndpointFactory = new UdpEndpointFactory(this);
                 _endpointFactoryManager.add(udpEndpointFactory);
-                
-                _pluginManager = new Ice.PluginManagerI(communicator);
 
-                _outgoingConnectionFactory = new OutgoingConnectionFactory(this);
-                
+#if !SILVERLIGHT
+                _pluginManager = new Ice.PluginManagerI(communicator);
+#endif
+
+                _outgoingConnectionFactory = new OutgoingConnectionFactory(communicator, this);
+
                 _servantFactoryManager = new ObjectFactoryManager();
-                
+
                 _objectAdapterFactory = new ObjectAdapterFactory(this, communicator);
-                
+
                 _retryQueue = new RetryQueue(this);
 
                 string[] facetFilter = _initData.properties.getPropertyAsList("Ice.Admin.Facets");
@@ -768,8 +863,33 @@ namespace IceInternal
                         _adminFacetFilter.Add(s);
                     }
                 }
-                _adminFacets.Add("Properties", new PropertiesAdminI(_initData.properties));
+
                 _adminFacets.Add("Process", new ProcessI(communicator));
+
+                MetricsAdminI admin = new MetricsAdminI(_initData.properties, _initData.logger);
+                _adminFacets.Add("Metrics", admin);
+
+                PropertiesAdminI props = new PropertiesAdminI("Properties", _initData.properties, _initData.logger);
+                _adminFacets.Add("Properties", props);
+
+                //
+                // Setup the communicator observer only if the user didn't already set an
+                // Ice observer resolver and if the admininistrative endpoints are set.
+                //
+                if((_adminFacetFilter.Count == 0 || _adminFacetFilter.Contains("Metrics")) &&
+                   _initData.properties.getProperty("Ice.Admin.Endpoints").Length > 0)
+                {
+                    _observer = new CommunicatorObserverI(admin, _initData.observer);
+
+                    //
+                    // Make sure the admin plugin receives property updates.
+                    //
+                    props.addUpdateCallback(admin);
+                }
+                else
+                {
+                    _observer = initData.observer;
+                }
             }
             catch(Ice.LocalException)
             {
@@ -777,21 +897,32 @@ namespace IceInternal
                 throw;
             }
         }
-        
+
         public void finishSetup(ref string[] args)
         {
             //
             // Load plug-ins.
             //
             Debug.Assert(_serverThreadPool == null);
+#if !SILVERLIGHT
             Ice.PluginManagerI pluginManagerImpl = (Ice.PluginManagerI)_pluginManager;
             pluginManagerImpl.loadPlugins(ref args);
-            
+#endif
+
+            //
+            // Set observer updater
+            //
+            if(_observer != null)
+            {
+                _observer.setObserverUpdater(new ObserverUpdaterI(this));
+            }
+
             //
             // Create threads.
             //
             try
             {
+#if !SILVERLIGHT
                 if(initializationData().properties.getProperty("Ice.ThreadPriority").Length > 0)
                 {
                     ThreadPriority priority = IceInternal.Util.stringToThreadPriority(
@@ -802,6 +933,9 @@ namespace IceInternal
                 {
                     _timer = new Timer(this);
                 }
+#else
+                _timer = new Timer(this);
+#endif
             }
             catch(System.Exception ex)
             {
@@ -809,7 +943,8 @@ namespace IceInternal
                 _initData.logger.error(s);
                 throw;
             }
-          
+
+#if !SILVERLIGHT
             try
             {
                 _endpointHostResolver = new EndpointHostResolver(this);
@@ -820,7 +955,7 @@ namespace IceInternal
                 _initData.logger.error(s);
                 throw;
             }
-
+#endif
             _clientThreadPool = new ThreadPool(this, "Ice.ThreadPool.Client", 0);
 
             //
@@ -839,10 +974,11 @@ namespace IceInternal
             {
                 _referenceFactory = _referenceFactory.setDefaultLocator(l);
             }
-            
+
             //
             // Show process id if requested (but only once).
             //
+#if !SILVERLIGHT
             lock(this)
             {
                 if(!_printProcessIdDone && _initData.properties.getPropertyAsInt("Ice.PrintProcessId") > 0)
@@ -854,7 +990,7 @@ namespace IceInternal
                     _printProcessIdDone = true;
                 }
             }
-             
+#endif
             //
             // Create the connection monitor and ensure the interval for
             // monitoring connections is appropriate for client & server
@@ -869,19 +1005,20 @@ namespace IceInternal
             // Server thread pool initialization is lazy in serverThreadPool().
             //
 
-            //      
+            //
             // An application can set Ice.InitPlugins=0 if it wants to postpone
             // initialization until after it has interacted directly with the
             // plug-ins.
-            //      
+            //
+#if !SILVERLIGHT
             if(_initData.properties.getPropertyAsIntWithDefault("Ice.InitPlugins", 1) > 0)
             {
                 pluginManagerImpl.initializePlugins();
             }
-
+#endif
             //
             // This must be done last as this call creates the Ice.Admin object adapter
-            // and eventually registers a process proxy with the Ice locator (allowing 
+            // and eventually registers a process proxy with the Ice locator (allowing
             // remote clients to invoke on Ice.Admin facets as soon as it's registered).
             //
             if(_initData.properties.getPropertyAsIntWithDefault("Ice.Admin.DelayCreation", 0) <= 0)
@@ -889,7 +1026,7 @@ namespace IceInternal
                 getAdmin();
             }
         }
-        
+
         //
         // Only for use by Ice.CommunicatorI
         //
@@ -905,7 +1042,7 @@ namespace IceInternal
                 {
                     return false;
                 }
-            
+
                 //
                 // We cannot set state to StateDestroyed otherwise instance
                 // methods called during the destroy process (such as
@@ -914,27 +1051,27 @@ namespace IceInternal
                 //
                 _state = StateDestroyInProgress;
             }
-            
+
             if(_objectAdapterFactory != null)
             {
                 _objectAdapterFactory.shutdown();
             }
-            
+
             if(_outgoingConnectionFactory != null)
             {
                 _outgoingConnectionFactory.destroy();
             }
-            
+
             if(_objectAdapterFactory != null)
             {
                 _objectAdapterFactory.destroy();
             }
-            
+
             if(_outgoingConnectionFactory != null)
             {
                 _outgoingConnectionFactory.waitUntilFinished();
             }
-            
+
             if(_retryQueue != null)
             {
                 _retryQueue.destroy();
@@ -943,8 +1080,10 @@ namespace IceInternal
             ThreadPool serverThreadPool = null;
             ThreadPool clientThreadPool = null;
             AsyncIOThread asyncIOThread = null;
-            EndpointHostResolver endpointHostResolver = null;
 
+#if !SILVERLIGHT
+            EndpointHostResolver endpointHostResolver = null;
+#endif
             lock(this)
             {
                 _objectAdapterFactory = null;
@@ -956,7 +1095,7 @@ namespace IceInternal
                     _connectionMonitor.destroy();
                     _connectionMonitor = null;
                 }
-                
+
                 if(_serverThreadPool != null)
                 {
                     _serverThreadPool.destroy();
@@ -978,53 +1117,53 @@ namespace IceInternal
                     _asyncIOThread = null;
                 }
 
+#if !SILVERLIGHT
                 if(_endpointHostResolver != null)
                 {
                     _endpointHostResolver.destroy();
                     endpointHostResolver = _endpointHostResolver;
                     _endpointHostResolver = null;
                 }
+#endif
 
                 if(_timer != null)
                 {
                     _timer.destroy();
                     _timer = null;
                 }
-                
+
                 if(_servantFactoryManager != null)
                 {
                     _servantFactoryManager.destroy();
                     _servantFactoryManager = null;
                 }
-                
-                if(_referenceFactory != null)
-                {
-                    _referenceFactory.destroy();
-                    _referenceFactory = null;
-                }
-                
+
+                // No destroy function defined.
+                //_referenceFactory.destroy();
+                _referenceFactory = null;
+
                 // No destroy function defined.
                 // _proxyFactory.destroy();
                 _proxyFactory = null;
-                
+
                 if(_routerManager != null)
                 {
                     _routerManager.destroy();
                     _routerManager = null;
                 }
-                
+
                 if(_locatorManager != null)
                 {
                     _locatorManager.destroy();
                     _locatorManager = null;
                 }
-                
+
                 if(_endpointFactoryManager != null)
                 {
                     _endpointFactoryManager.destroy();
                     _endpointFactoryManager = null;
                 }
-                
+
                 if(_pluginManager != null)
                 {
                     _pluginManager.destroy();
@@ -1033,10 +1172,10 @@ namespace IceInternal
 
                 _adminAdapter = null;
                 _adminFacets.Clear();
-                
+
                 _state = StateDestroyed;
             }
-            
+
             //
             // Join with threads outside the synchronization.
             //
@@ -1052,18 +1191,19 @@ namespace IceInternal
             {
                 asyncIOThread.joinWithThread();
             }
+#if !SILVERLIGHT
             if(endpointHostResolver != null)
             {
                 endpointHostResolver.joinWithThread();
             }
-
+#endif
             if(_initData.properties.getPropertyAsInt("Ice.Warn.UnusedProperties") > 0)
             {
-                ArrayList unusedProperties = ((Ice.PropertiesI)_initData.properties).getUnusedProperties();
-                if(unusedProperties.Count != 0)
+                List<string> unusedProperties = ((Ice.PropertiesI)_initData.properties).getUnusedProperties();
+                if (unusedProperties.Count != 0)
                 {
                     StringBuilder message = new StringBuilder("The following properties were set but never read:");
-                    foreach(string s in unusedProperties)
+                    foreach (string s in unusedProperties)
                     {
                         message.Append("\n    ");
                         message.Append(s);
@@ -1074,7 +1214,51 @@ namespace IceInternal
 
             return true;
         }
-        
+
+        internal void updateConnectionObservers()
+        {
+            try
+            {
+                Debug.Assert(_outgoingConnectionFactory != null);
+                _outgoingConnectionFactory.updateConnectionObservers();
+                Debug.Assert(_objectAdapterFactory != null);
+                _objectAdapterFactory.updateConnectionObservers();
+            }
+            catch(Ice.CommunicatorDestroyedException)
+            {
+            }
+        }
+
+        internal void updateThreadObservers()
+        {
+            try
+            {
+                if(_clientThreadPool != null)
+                {
+                    _clientThreadPool.updateObservers();
+                }
+                if(_serverThreadPool != null)
+                {
+                    _serverThreadPool.updateObservers();
+                }
+                Debug.Assert(_objectAdapterFactory != null);
+                _objectAdapterFactory.updateThreadObservers();
+#if !SILVERLIGHT
+                if(_endpointHostResolver != null)
+                {
+                    _endpointHostResolver.updateObserver();
+                }
+#endif
+                if(_asyncIOThread != null)
+                {
+                    _asyncIOThread.updateObserver();
+                }
+            }
+            catch(Ice.CommunicatorDestroyedException)
+            {
+            }
+        }
+
         private const int StateActive = 0;
         private const int StateDestroyInProgress = 1;
         private const int StateDestroyed = 2;
@@ -1082,13 +1266,14 @@ namespace IceInternal
         private Ice.InitializationData _initData; // Immutable, not reset by destroy().
         private TraceLevels _traceLevels; // Immutable, not reset by destroy().
         private DefaultsAndOverrides _defaultsAndOverrides; // Immutable, not reset by destroy().
-#if COMPACT
+#if COMPACT || SILVERLIGHT
         private string[] _factoryAssemblies; // Immutable, not reset by destroy().
 #endif
         private int _messageSizeMax; // Immutable, not reset by destroy().
         private int _clientACM; // Immutable, not reset by destroy().
         private int _serverACM; // Immutable, not reset by destroy().
         private Ice.ImplicitContextI _implicitContext; // Immutable
+        private Ice.Instrumentation.CommunicatorObserver _observer; // Immutable
         private RouterManager _routerManager;
         private LocatorManager _locatorManager;
         private ReferenceFactory _referenceFactory;
@@ -1098,10 +1283,14 @@ namespace IceInternal
         private ObjectFactoryManager _servantFactoryManager;
         private ObjectAdapterFactory _objectAdapterFactory;
         private int _protocolSupport;
+        private bool _preferIPv6;
+        private NetworkProxy _networkProxy;
         private ThreadPool _clientThreadPool;
         private ThreadPool _serverThreadPool;
         private AsyncIOThread _asyncIOThread;
+#if !SILVERLIGHT
         private EndpointHostResolver _endpointHostResolver;
+#endif
         private Timer _timer;
         private RetryQueue _retryQueue;
         private EndpointFactoryManager _endpointFactoryManager;
@@ -1111,9 +1300,14 @@ namespace IceInternal
         private HashSet<string> _adminFacetFilter = new HashSet<string>();
         private Ice.Identity _adminIdentity;
 
+#if !SILVERLIGHT
         private static bool _printProcessIdDone = false;
+#endif
 
+#if !SILVERLIGHT && !UNITY
         private static bool _oneOffDone = false;
+#endif
+
         private static System.Object _staticLock = new System.Object();
     }
 }
