@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -176,20 +176,55 @@ namespace Ice
         void readObject(ReadObjectCallback cb);
 
         /// <summary>
-        /// Extracts a Slice type ID from the stream.
+        /// Read an enumerated value.
         /// </summary>
-        /// <returns>The extracted type ID.</returns>
-        string readTypeId();
+        ///
+        /// <param name="maxValue">The maximum enumerator value in the definition.</param>
+        /// <returns>The enumerator.</returns>
+        int readEnum(int maxValue);
 
         /// <summary>
         /// Extracts a user exception from the stream and throws it.
         /// </summary>
         void throwException();
 
+        /// <summary>
+        /// Extracts a user exception from the stream and throws it.
+        /// Extracts a user exception from the stream and throws it, using the supplied
+        /// factory to instantiate a UserExceptionReader.
+        /// </summary>
+        /// <param name="factory">A factory that creates UserExceptionReader instances.</param>
+        void throwException(UserExceptionReaderFactory factory);
+
+        /// <summary>
+        /// Marks the start of an Ice object.
+        /// </summary>
+        void startObject();
+
+        /// <summary>
+        /// Marks the end of an Ice object.
+        /// </summary>
+        /// <param name="preserve">True if unknown slices should be preserved, false otherwise.</param>
+        /// <returns>A SlicedData object containing the preserved slices for unknown types.</returns>
+        SlicedData endObject(bool preserve);
+
+        /// <summary>
+        /// Marks the start of a user exception.
+        /// </summary>
+        void startException();
+
+        /// <summary>
+        /// Marks the end of a user exception.
+        /// </summary>
+        /// <param name="preserve">True if unknown slices should be preserved, false otherwise.</param>
+        /// <returns>A SlicedData object containing the preserved slices for unknown types.</returns>
+        SlicedData endException(bool preserve);
+
         /// <summary>
         /// Reads the start of an object or exception slice.
         /// </summary>
-        void startSlice();
+        /// <returns>The Slice type ID for this slice.</returns>
+        string startSlice();
 
         /// <summary>
         /// Indicates that the end of an object or exception slice has been reached.
@@ -204,7 +239,8 @@ namespace Ice
         /// <summary>
         /// Reads the start of an encapsulation.
         /// </summary>
-        void startEncapsulation();
+        /// <returns>The encapsulation encoding version.</returns>
+        EncodingVersion startEncapsulation();
 
         /// <summary>
         /// Indicates that the end of an encapsulation has been reached.
@@ -214,19 +250,14 @@ namespace Ice
         /// <summary>
         /// Skips over an encapsulation.
         /// </summary>
-        void skipEncapsulation();
+        /// <returns>The encapsulation encoding version.</returns>
+        EncodingVersion skipEncapsulation();
 
         /// <summary>
-        /// Returns the size of the current encapsulation.
+        /// Determines the current encoding version.
         /// </summary>
-        /// <returns>The size of the current encapsulation.</returns>
-        int getEncapsulationSize();
-
-        /// <summary>
-        /// Reads the specified number of bytes from the stream.
-        /// </summary>
-        /// <returns>The bytes read.</returns>
-        byte[] readBlob(int size);
+        /// <returns>The encoding version.</returns>
+        EncodingVersion getEncoding();
 
         /// <summary>
         /// Indicates that unmarshaling is complete, except for any Slice objects. The application must
@@ -241,6 +272,31 @@ namespace Ice
         /// </summary>
         void rewind();
 
+        /// <summary>
+        /// Skips ahead in the stream.
+        /// </summary>
+        /// <param name="sz">The number of bytes to skip.</param>
+        void skip(int sz);
+
+        /// <summary>
+        /// Skips over a size value.
+        /// </summary>
+        void skipSize();
+
+        /// <summary>
+        /// Determine if an optional value is available for reading.
+        /// </summary>
+        /// <param name="tag">The tag associated with the value.</param>
+        /// <param name="format">The optional format for the value.</param>
+        /// <returns>True if the value is present, false otherwise.</returns>
+        bool readOptional(int tag, OptionalFormat format);
+
+        /// <summary>
+        /// Determine the current position in the stream.
+        /// </summary>
+        /// <returns>The current position.</returns>
+        int pos();
+
         /// <summary>
         /// Destroys the stream and its associated resources. The application must call destroy prior
         /// to releasing the last reference to a stream; failure to do so may result in resource leaks.
@@ -390,10 +446,11 @@ namespace Ice
         void writeObject(Ice.Object v);
 
         /// <summary>
-        /// Writes a Slice type ID to the stream.
+        /// Write an enumerated value.
         /// </summary>
-        /// <param name="id">The Slice type ID to write.</param>
-        void writeTypeId(string id);
+        /// <param name="v">The enumerator.</param>
+        /// <param name="limit">The number of enumerators in the definition.</param>
+        void writeEnum(int v, int limit);
 
         /// <summary>
         /// Writes a user exception to the stream.
@@ -402,15 +459,47 @@ namespace Ice
         void writeException(UserException ex);
 
         /// <summary>
-        /// Writes the start of a slice to the stream.
+        /// Marks the start of an Ice object.
         /// </summary>
-        void startSlice();
+        /// <param name="slicedData">Preserved slices for this object, or null.</param>
+        void startObject(SlicedData slicedData);
 
         /// <summary>
-        /// Ends the previous slice.
+        /// Marks the end of an Ice object.
+        /// </summary>
+        void endObject();
+
+        /// <summary>
+        /// Marks the start of a user exception.
+        /// </summary>
+        /// <param name="slicedData">Preserved slices for this object, or null.</param>
+        void startException(SlicedData slicedData);
+
+        /// <summary>
+        /// Marks the end of a user exception.
+        /// </summary>
+        void endException();
+
+        /// <summary>
+        /// Marks the start of a new slice for an Ice object or user exception.
+        /// </summary>
+        /// <param name="typeId">The Slice type ID corresponding to this slice.</param>
+        /// <param name="compactId">The Slice compact type ID corresponding to this slice.</param>
+        /// <param name="last">True if this is the last slice, false otherwise.</param>
+        void startSlice(string typeId, int compactId, bool last);
+
+        /// <summary>
+        /// Marks the end of a slice for an Ice object or user exception.
         /// </summary>
         void endSlice();
 
+        /// <summary>
+        /// Writes the start of an encapsulation to the stream.
+        /// </summary>
+        /// <param name="encoding">The encoding version of the encapsulation.</param>
+        /// <param name="format">The format to use for encoding objects and user exceptions.</param>
+        void startEncapsulation(EncodingVersion encoding, FormatType format);
+
         /// <summary>
         /// Writes the start of an encapsulation to the stream.
         /// </summary>
@@ -422,9 +511,10 @@ namespace Ice
         void endEncapsulation();
 
         /// <summary>
-        /// Writes the specified bytes onto the stream.
+        /// Determines the current encoding version.
         /// </summary>
-        void writeBlob(byte[] data);
+        /// <returns>The encoding version.</returns>
+        EncodingVersion getEncoding();
 
         /// <summary>
         /// Writes the state of Slice classes whose index was previously
@@ -432,6 +522,39 @@ namespace Ice
         /// </summary>
         void writePendingObjects();
 
+        /// <summary>
+        /// Write the header information for an optional value.
+        /// </summary>
+        /// <param name="tag">The numeric tag associated with the value.</param>
+        /// <param name="format">The optional format of the value.</param>
+        /// <returns>True if the optional should be written, false otherwise.</returns>
+        bool writeOptional(int tag, OptionalFormat format);
+
+        /// <summary>
+        /// Determines the current position in the stream.
+        /// </summary>
+        /// <returns>The current position.</returns>
+        int pos();
+
+        /// <summary>
+        /// Inserts a fixed 32-bit size value into the stream at the given position.
+        /// </summary>
+        /// <param name="sz">The 32-bit size value.</param>
+        /// <param name="pos">The position at which to write the value.</param>
+        void rewrite(int sz, int pos);
+
+        /// <summary>
+        /// Records the current position and allocates four bytes for a fixed-length (32-bit)
+        /// size value.
+        /// </summary>
+        void startSize();
+
+        /// <summary>
+        /// Computes the amount of data written since the previous call to startSize and
+        /// writes that value at the saved position.
+        /// </summary>
+        void endSize();
+
         /// <summary>
         /// Indicates that the marshaling of a request or reply is finished.
         /// </summary>
@@ -462,23 +585,20 @@ namespace Ice
     public abstract class ObjectReader : ObjectImpl
     {
         /// <summary>
-        /// @param in The input stream to read from.
-        /// @param rid If <code>true</code>, extraction begins by reading a Slice type ID
-        /// first. If <code>false</code>, the leading type ID is not read. This is used
-        /// by the unmarshaling code in case the type ID has already been read as part
-        /// of other unmarshaling activities.
+        /// Read the object's data members.
         /// </summary>
-        public abstract void read(InputStream inStream, bool rid);
+        /// <param name="inStream">The input stream to read from.</param>
+        public abstract void read(InputStream inStream);
 
         public override void write__(IceInternal.BasicStream os)
         {
             Debug.Assert(false);
         }
 
-        public override void read__(IceInternal.BasicStream istr, bool rid)
+        public override void read__(IceInternal.BasicStream istr)
         {
             InputStream stream = (InputStream)istr.closure();
-            read(stream, rid);
+            read(stream);
         }
     }
 
@@ -499,12 +619,51 @@ namespace Ice
             write(stream);
         }
 
-        public override void read__(IceInternal.BasicStream istr, bool rid)
+        public override void read__(IceInternal.BasicStream istr)
         {
             Debug.Assert(false);
         }
     }
 
+    public abstract class UserExceptionReader : UserException
+    {
+        protected UserExceptionReader(Communicator communicator)
+        {
+            communicator_ = communicator;
+        }
+
+        public abstract void read(Ice.InputStream istr);
+
+        public override void write__(IceInternal.BasicStream ostr)
+        {
+            Debug.Assert(false);
+        }
+
+        public override void read__(IceInternal.BasicStream istr)
+        {
+            InputStream stream = (InputStream)istr.closure();
+            Debug.Assert(stream != null);
+            read(stream);
+        }
+
+        public override void write__(Ice.OutputStream ostr)
+        {
+            Debug.Assert(false);
+        }
+
+        public override void read__(Ice.InputStream istr)
+        {
+            read(istr);
+        }
+
+        protected Communicator communicator_;
+    }
+
+    public interface UserExceptionReaderFactory
+    {
+        void createAndThrow(string typeId);
+    }
+
     public abstract class UserExceptionWriter : UserException
     {
         public UserExceptionWriter(Communicator communicator)
@@ -513,7 +672,6 @@ namespace Ice
         }
 
         public abstract void write(OutputStream os);
-        public abstract bool usesClasses();
 
         public override void write__(IceInternal.BasicStream os)
         {
@@ -525,7 +683,7 @@ namespace Ice
             write(stream);
         }
 
-        public override void read__(IceInternal.BasicStream istr, bool rid)
+        public override void read__(IceInternal.BasicStream istr)
         {
             Debug.Assert(false);
         }
@@ -535,16 +693,11 @@ namespace Ice
             write(ostr);
         }
 
-        public override void read__(Ice.InputStream istr, bool rid)
+        public override void read__(Ice.InputStream istr)
         {
             Debug.Assert(false);
         }
 
-        public override bool usesClasses__()
-        {
-            return usesClasses();
-        }
-
         protected Communicator communicator_;
     }
 }
