@@ -1,15 +1,22 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+//
+// .NET and Silverlight use the new socket asynchronous APIs whereas
+// the compact framework and mono still use the old Begin/End APIs.
+//
+#if !COMPACT && !__MonoCS__ && !UNITY
+#define ICE_SOCKET_ASYNC_API
+#endif
+
 namespace IceInternal
 {
-
     using System;
     using System.ComponentModel;
     using System.Diagnostics;
@@ -20,42 +27,71 @@ namespace IceInternal
     {
         public int initialize()
         {
-            if(_state == StateNeedConnect)
+            try
             {
-                _state = StateConnectPending;
-                return SocketOperation.Connect;
-            }
-            else if(_state <= StateConnectPending)
-            {
-                try
+                if(_state == StateNeedConnect)
                 {
+                    _state = StateConnectPending;
+                    return SocketOperation.Connect;
+                }
+                else if(_state <= StateConnectPending)
+                {
+#if ICE_SOCKET_ASYNC_API
+                    if(_writeEventArgs.SocketError != SocketError.Success)
+                    {
+                        SocketException ex = new SocketException((int)_writeEventArgs.SocketError);
+                        if(Network.connectionRefused(ex))
+                        {
+                            throw new Ice.ConnectionRefusedException(ex);
+                        }
+                        else
+                        {
+                            throw new Ice.ConnectFailedException(ex);
+                        }
+                    }
+#else
                     Network.doFinishConnectAsync(_fd, _writeResult);
                     _writeResult = null;
-                    _state = StateConnected;
-                    _desc = Network.fdToString(_fd);
-                }
-                catch(Ice.LocalException ex)
-                {
-                    if(_traceLevels.network >= 2)
+#endif
+                    _desc = Network.fdToString(_fd, _proxy, _addr);
+                    if(_proxy != null)
                     {
-                        System.Text.StringBuilder s = new System.Text.StringBuilder();
-                        s.Append("failed to establish tcp connection\n");
-                        s.Append(Network.fdLocalAddressToString(_fd));
-                        Debug.Assert(_addr != null);
-                        s.Append("\nremote address = " + _addr.ToString() + "\n");
-                        s.Append(ex.ToString());
-                        _logger.trace(_traceLevels.networkCat, s.ToString());
+                        _state = StateProxyConnectRequest; // Send proxy connect request
+                        return SocketOperation.Write; 
                     }
-                    throw;
-                }
 
-                if(_traceLevels.network >= 1)
+                    _state = StateConnected;
+                }
+                else if(_state == StateProxyConnectRequest)
                 {
-                    string s = "tcp connection established\n" + _desc;
-                    _logger.trace(_traceLevels.networkCat, s);
+                    _state = StateProxyConnectRequestPending; // Wait for proxy response
+                    return SocketOperation.Read;
+                }
+                else if(_state == StateProxyConnectRequestPending)
+                {
+                    _state = StateConnected;
                 }
             }
+            catch(Ice.LocalException ex)
+            {
+                if(_traceLevels.network >= 2)
+                {
+                    System.Text.StringBuilder s = new System.Text.StringBuilder();
+                    s.Append("failed to establish tcp connection\n");
+                    s.Append(Network.fdToString(_fd, _proxy, _addr));
+                    s.Append("\n");
+                    s.Append(ex.ToString());
+                    _logger.trace(_traceLevels.networkCat, s.ToString());
+                }
+                throw;
+            }
+
             Debug.Assert(_state == StateConnected);
+            if(_traceLevels.network >= 1)
+            {
+                string s = "tcp connection established\n" + _desc;
+                _logger.trace(_traceLevels.networkCat, s);
+            }
             return SocketOperation.None;
         }
 
@@ -88,11 +124,11 @@ namespace IceInternal
 
         public bool write(Buffer buf)
         {
-#if COMPACT
+#if COMPACT || SILVERLIGHT
             //
-            // The Compact Framework does not support the use of synchronous socket
-            // operations on a non-blocking socket. Returning false here forces the
-            // caller to schedule an asynchronous operation.
+            // Silverlight and the Compact .NET Frameworks don't support the use of synchronous socket
+            // operations on a non-blocking socket. Returning false here forces the caller to schedule
+            // an asynchronous operation.
             //
             return false;
 #else
@@ -109,13 +145,6 @@ namespace IceInternal
                     packetSize = _maxSendPacketSize / 2;
                 }
             }
-            else
-            {
-                if(_blocking > 0)
-                {
-                    return false;
-                }
-            }
 
             while(buf.b.hasRemaining())
             {
@@ -128,7 +157,7 @@ namespace IceInternal
                     {
                         ret = _fd.Send(buf.b.rawBytes(), buf.b.position(), packetSize, SocketFlags.None);
                     }
-                    catch(Win32Exception e)
+                    catch(SocketException e)
                     {
                         if(Network.wouldBlock(e))
                         {
@@ -147,7 +176,9 @@ namespace IceInternal
 
                     if(_stats != null)
                     {
+#pragma warning disable 618                        
                         _stats.bytesSent(type(), ret);
+#pragma warning restore 618
                     }
 
                     buf.b.position(buf.b.position() + ret);
@@ -173,22 +204,13 @@ namespace IceInternal
 
         public bool read(Buffer buf)
         {
-#if COMPACT
+#if COMPACT || SILVERLIGHT
             //
-            // The .NET Compact Framework does not support the use of synchronous socket
+            // Silverlight and the Compact .NET Framework don't support the use of synchronous socket
             // operations on a non-blocking socket.
             //
             return false;
 #else
-            // COMPILERFIX: Workaround for Mac OS X broken poll(), see Mono bug #470120
-            if(AssemblyUtil.osx_)
-            {
-                if(_blocking > 0)
-                {
-                    return false;
-                }
-            }
-
             int remaining = buf.b.remaining();
             int position = buf.b.position();
 
@@ -211,7 +233,7 @@ namespace IceInternal
                             throw new Ice.ConnectionLostException();
                         }
                     }
-                    catch(Win32Exception e)
+                    catch(SocketException e)
                     {
                         if(Network.wouldBlock(e))
                         {
@@ -230,13 +252,15 @@ namespace IceInternal
 
                     if(_stats != null)
                     {
+#pragma warning disable 618
                         _stats.bytesReceived(type(), ret);
+#pragma warning restore 618
                     }
 
                     remaining -= ret;
                     buf.b.position(position += ret);
                 }
-                catch(Win32Exception ex)
+                catch(SocketException ex)
                 {
                     //
                     // On Mono, calling shutdownReadWrite() followed by read() causes Socket.Receive() to
@@ -268,15 +292,14 @@ namespace IceInternal
 
         public bool startRead(Buffer buf, AsyncCallback callback, object state)
         {
+#if ICE_SOCKET_ASYNC_API
+            Debug.Assert(_fd != null && _readEventArgs != null);
+#else
             Debug.Assert(_fd != null && _readResult == null);
-
-            // COMPILERFIX: Workaround for Mac OS X broken poll(), see Mono bug #470120
-            if(AssemblyUtil.osx_)
+#endif
+            if(_state == StateProxyConnectRequestPending)
             {
-                if(++_blocking == 1)
-                {
-                    Network.setBlock(_fd, true);
-                }
+                _proxy.beginReadConnectRequestResponse(buf);
             }
 
             int packetSize = buf.b.remaining();
@@ -287,10 +310,18 @@ namespace IceInternal
 
             try
             {
-                _readResult = _fd.BeginReceive(buf.b.rawBytes(), buf.b.position(), packetSize, SocketFlags.None, 
-                                               callback, state);
+                _readCallback = callback;
+#if ICE_SOCKET_ASYNC_API
+                _readEventArgs.UserToken = state;
+                _readEventArgs.SetBuffer(buf.b.rawBytes(), buf.b.position(), packetSize);
+                return !_fd.ReceiveAsync(_readEventArgs);
+#else
+                _readResult = _fd.BeginReceive(buf.b.rawBytes(), buf.b.position(), packetSize, SocketFlags.None,
+                                               readCompleted, state);
+                return _readResult.CompletedSynchronously;
+#endif
             }
-            catch(Win32Exception ex)
+            catch(SocketException ex)
             {
                 if(Network.connectionLost(ex))
                 {
@@ -299,37 +330,43 @@ namespace IceInternal
 
                 throw new Ice.SocketException(ex);
             }
-
-            return _readResult.CompletedSynchronously;
         }
 
         public void finishRead(Buffer buf)
         {
             if(_fd == null) // Transceiver was closed
             {
+#if ICE_SOCKET_ASYNC_API
+                _readEventArgs = null;
+#else
                 _readResult = null;
+#endif
                 return;
             }
 
+#if ICE_SOCKET_ASYNC_API
+            Debug.Assert(_fd != null && _readEventArgs != null);
+#else
             Debug.Assert(_fd != null && _readResult != null);
+#endif
 
             try
             {
+#if ICE_SOCKET_ASYNC_API
+                if(_readEventArgs.SocketError != SocketError.Success)
+                {
+                    throw new SocketException((int)_readEventArgs.SocketError);
+                }
+                int ret = _readEventArgs.BytesTransferred;
+#else
                 int ret = _fd.EndReceive(_readResult);
                 _readResult = null;
+#endif
                 if(ret == 0)
                 {
                     throw new Ice.ConnectionLostException();
                 }
 
-                // COMPILERFIX: Workaround for Mac OS X broken poll(), see Mono bug #470120
-                if(AssemblyUtil.osx_)
-                {
-                    if(--_blocking == 0)
-                    {
-                        Network.setBlock(_fd, false);
-                    }
-                }
                 Debug.Assert(ret > 0);
 
                 if(_traceLevels.network >= 3)
@@ -345,12 +382,19 @@ namespace IceInternal
 
                 if(_stats != null)
                 {
+#pragma warning disable 618
                     _stats.bytesReceived(type(), ret);
+#pragma warning restore 618
                 }
 
                 buf.b.position(buf.b.position() + ret);
+
+                if(_state == StateProxyConnectRequestPending)
+                {
+                    _proxy.endReadConnectRequestResponse(buf);
+                }
             }
-            catch(Win32Exception ex)
+            catch(SocketException ex)
             {
                 if(Network.connectionLost(ex))
                 {
@@ -367,22 +411,39 @@ namespace IceInternal
 
         public bool startWrite(Buffer buf, AsyncCallback callback, object state, out bool completed)
         {
+#if ICE_SOCKET_ASYNC_API
+            Debug.Assert(_fd != null && _writeEventArgs != null);
+#else
             Debug.Assert(_fd != null && _writeResult == null);
-
-            // COMPILERFIX: Workaround for Mac OS X broken poll(), see Mono bug #470120
-            if(AssemblyUtil.osx_)
-            {
-                if(++_blocking == 1)
-                {
-                    Network.setBlock(_fd, true);
-                }
-            }
+#endif
 
             if(_state < StateConnected)
             {
-                _writeResult = Network.doConnectAsync(_fd, _addr, callback, state);
                 completed = false;
-                return _writeResult.CompletedSynchronously;
+                if(_state == StateConnectPending)
+                {
+                    _writeCallback = callback;
+                    try
+                    {
+                        EndPoint addr = _proxy != null ? _proxy.getAddress() : _addr;
+#if ICE_SOCKET_ASYNC_API
+                        _writeEventArgs.RemoteEndPoint = addr;
+                        _writeEventArgs.UserToken = state;
+                        return !_fd.ConnectAsync(_writeEventArgs);
+#else
+                        _writeResult = Network.doConnectAsync(_fd, addr, callback, state);
+                        return _writeResult.CompletedSynchronously;
+#endif
+                    }
+                    catch(Exception ex)
+                    {
+                        throw new Ice.SocketException(ex);
+                    }
+                }
+                else if(_state == StateProxyConnectRequest)
+                {
+                    _proxy.beginWriteConnectRequest(_addr, buf);
+                }
             }
 
             //
@@ -397,10 +458,20 @@ namespace IceInternal
 
             try
             {
+                _writeCallback = callback;
+#if ICE_SOCKET_ASYNC_API
+                _writeEventArgs.UserToken = state;
+                _writeEventArgs.SetBuffer(buf.b.rawBytes(), buf.b.position(), packetSize);
+                bool completedSynchronously = !_fd.SendAsync(_writeEventArgs);
+#else
                 _writeResult = _fd.BeginSend(buf.b.rawBytes(), buf.b.position(), packetSize, SocketFlags.None, 
-                                             callback, state);
+                                             writeCompleted, state);
+                bool completedSynchronously = _writeResult.CompletedSynchronously;
+#endif
+                completed = packetSize == buf.b.remaining();
+                return completedSynchronously;
             }
-            catch(Win32Exception ex)
+            catch(SocketException ex)
             {
                 if(Network.connectionLost(ex))
                 {
@@ -413,9 +484,6 @@ namespace IceInternal
             {
                 throw new Ice.ConnectionLostException(ex);
             }
-            
-            completed = packetSize == buf.b.remaining();
-            return _writeResult.CompletedSynchronously;
         }
 
         public void finishWrite(Buffer buf)
@@ -426,36 +494,43 @@ namespace IceInternal
                 {
                     buf.b.position(buf.size()); // Assume all the data was sent for at-most-once semantics.
                 }
+#if ICE_SOCKET_ASYNC_API
+                _writeEventArgs = null;
+#else
                 _writeResult = null;
+#endif
                 return;
             }
 
+#if ICE_SOCKET_ASYNC_API
+            Debug.Assert(_fd != null && _writeEventArgs != null);
+#else
             Debug.Assert(_fd != null && _writeResult != null);
+#endif
 
-            if(_state < StateConnected)
+            if(_state < StateConnected && _state != StateProxyConnectRequest)
             {
                 return;
-            }
-
+            } 
+            
             try
             {
+#if ICE_SOCKET_ASYNC_API
+                if(_writeEventArgs.SocketError != SocketError.Success)
+                {
+                    throw new SocketException((int)_writeEventArgs.SocketError);
+                }
+                int ret = _writeEventArgs.BytesTransferred;
+#else
                 int ret = _fd.EndSend(_writeResult);
                 _writeResult = null;
+#endif
                 if(ret == 0)
                 {
                     throw new Ice.ConnectionLostException();
                 }
                 Debug.Assert(ret > 0);
 
-                // COMPILERFIX: Workaround for Mac OS X broken poll(), see Mono bug #470120
-                if(AssemblyUtil.osx_)
-                {
-                    if(--_blocking == 0)
-                    {
-                        Network.setBlock(_fd, false);
-                    }
-                }
-
                 if(_traceLevels.network >= 3)
                 {
                     int packetSize = buf.b.remaining();
@@ -469,12 +544,19 @@ namespace IceInternal
 
                 if(_stats != null)
                 {
+#pragma warning disable 618
                     _stats.bytesSent(type(), ret);
+#pragma warning restore 618
                 }
 
                 buf.b.position(buf.b.position() + ret);
+
+                if(_state == StateProxyConnectRequest)
+                {
+                    _proxy.endWriteConnectRequest(buf);
+                }
             }
-            catch(Win32Exception ex)
+            catch(SocketException ex)
             {
                 if(Network.connectionLost(ex))
                 {
@@ -497,21 +579,15 @@ namespace IceInternal
         public Ice.ConnectionInfo
         getInfo()
         {
-            Debug.Assert(_fd != null);
             Ice.TCPConnectionInfo info = new Ice.TCPConnectionInfo();
-            IPEndPoint localEndpoint = Network.getLocalAddress(_fd);
-            info.localAddress = localEndpoint.Address.ToString();
-            info.localPort = localEndpoint.Port;
-            IPEndPoint remoteEndpoint = Network.getRemoteAddress(_fd);
-            if(remoteEndpoint != null)
+            if(_fd != null)
             {
-                info.remoteAddress = remoteEndpoint.Address.ToString();
-                info.remotePort = remoteEndpoint.Port;
-            }
-            else
-            {
-                info.remoteAddress = "";
-                info.remotePort = -1;
+                EndPoint localEndpoint = Network.getLocalAddress(_fd);
+                info.localAddress = Network.endpointAddressToString(localEndpoint);
+                info.localPort = Network.endpointPort(localEndpoint);
+                EndPoint remoteEndpoint = Network.getRemoteAddress(_fd);
+                info.remoteAddress = Network.endpointAddressToString(remoteEndpoint);
+                info.remotePort = Network.endpointPort(remoteEndpoint);
             }
             return info;
         }
@@ -532,15 +608,37 @@ namespace IceInternal
         //
         // Only for use by TcpConnector, TcpAcceptor
         //
-        internal TcpTransceiver(Instance instance, Socket fd, IPEndPoint addr, bool connected)
+        internal TcpTransceiver(Instance instance, Socket fd, EndPoint addr, NetworkProxy proxy, bool connected)
         {
             _fd = fd;
             _addr = addr;
+            _proxy = proxy;
+            
             _traceLevels = instance.traceLevels();
             _logger = instance.initializationData().logger;
             _stats = instance.initializationData().stats;
             _state = connected ? StateConnected : StateNeedConnect;
-            _desc = connected ? Network.fdToString(_fd) : "<not connected>";
+            _desc = connected ? Network.fdToString(_fd, _proxy, _addr) : "<not connected>";
+            
+#if ICE_SOCKET_ASYNC_API
+            _readEventArgs = new SocketAsyncEventArgs();
+            _readEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(ioCompleted);
+
+            _writeEventArgs = new SocketAsyncEventArgs();
+            _writeEventArgs.Completed += new EventHandler<SocketAsyncEventArgs>(ioCompleted);
+#if SILVERLIGHT
+            String policy = instance.initializationData().properties.getProperty("Ice.ClientAccessPolicyProtocol");
+            if(policy.Equals("Http"))
+            {
+                _readEventArgs.SocketClientAccessPolicyProtocol = SocketClientAccessPolicyProtocol.Http;
+                _writeEventArgs.SocketClientAccessPolicyProtocol = SocketClientAccessPolicyProtocol.Http;
+            }
+            else if(!String.IsNullOrEmpty(policy))
+            {
+                _logger.warning("Ignoring invalid Ice.ClientAccessPolicyProtocol value `" + policy + "'");
+            }
+#endif
+#endif
 
             _maxSendPacketSize = Network.getSendBufferSize(fd);
             if(_maxSendPacketSize < 512)
@@ -555,8 +653,43 @@ namespace IceInternal
             }
         }
 
+#if ICE_SOCKET_ASYNC_API
+        internal void ioCompleted(object sender, SocketAsyncEventArgs e)
+        {
+            switch (e.LastOperation)
+            {
+            case SocketAsyncOperation.Receive:
+                _readCallback(e.UserToken);
+                break;
+            case SocketAsyncOperation.Send:
+            case SocketAsyncOperation.Connect:
+                _writeCallback(e.UserToken);
+                break;
+            default:
+                throw new ArgumentException("The last operation completed on the socket was not a receive or send");
+            }
+        }
+#else
+        internal void readCompleted(IAsyncResult result)
+        {
+            if(!result.CompletedSynchronously)
+            {
+                _readCallback(result.AsyncState);
+            }
+        }
+
+        internal void writeCompleted(IAsyncResult result)
+        {
+            if(!result.CompletedSynchronously)
+            {
+                _writeCallback(result.AsyncState);
+            }
+        }
+#endif
+
         private Socket _fd;
-        private IPEndPoint _addr;
+        private EndPoint _addr;
+        private NetworkProxy _proxy;
         private TraceLevels _traceLevels;
         private Ice.Logger _logger;
         private Ice.Stats _stats;
@@ -565,13 +698,21 @@ namespace IceInternal
         private int _maxSendPacketSize;
         private int _maxReceivePacketSize;
 
-        private int _blocking = 0;
+#if ICE_SOCKET_ASYNC_API
+        private SocketAsyncEventArgs _writeEventArgs;
+        private SocketAsyncEventArgs _readEventArgs;
+#else
         private IAsyncResult _writeResult;
         private IAsyncResult _readResult;
+#endif
+
+        AsyncCallback _writeCallback;
+        AsyncCallback _readCallback;
 
         private const int StateNeedConnect = 0;
         private const int StateConnectPending = 1;
-        private const int StateConnected = 2;
+        private const int StateProxyConnectRequest = 2;
+        private const int StateProxyConnectRequestPending = 3; 
+        private const int StateConnected = 4;
     }
-
 }
