@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,33 +19,25 @@ namespace IceInternal
 
     sealed class UdpEndpointI : EndpointI
     {
-        public UdpEndpointI(Instance instance, string ho, int po, string mif, int mttl, byte pma, byte pmi, byte ema,
-                            byte emi, bool conn, string conId, bool co)
+        public UdpEndpointI(Instance instance, string ho, int po, string mif, int mttl, bool conn, string conId, 
+                            bool co) : base(conId)
         {
             instance_ = instance;
             _host = ho;
             _port = po;
             _mcastInterface = mif;
             _mcastTtl = mttl;
-            _protocolMajor = pma;
-            _protocolMinor = pmi;
-            _encodingMajor = ema;
-            _encodingMinor = emi;
             _connect = conn;
-            _connectionId = conId;
+            connectionId_ = conId;
             _compress = co;
             calcHashValue();
         }
         
-        public UdpEndpointI(Instance instance, string str, bool oaEndpoint)
+        public UdpEndpointI(Instance instance, string str, bool oaEndpoint) : base("")
         {
             instance_ = instance;
             _host = null;
             _port = 0;
-            _protocolMajor = Protocol.protocolMajor;
-            _protocolMinor = Protocol.protocolMinor;
-            _encodingMajor = Protocol.encodingMajor;
-            _encodingMinor = Protocol.encodingMinor;
             _connect = false;
             _compress = false;
 
@@ -110,113 +102,7 @@ namespace IceInternal
                     }
                 }
                 
-                if(option.Equals("-v"))
-                {
-                    if(argument == null)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException();
-                        e.str = "no argument provided for -v option in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                    
-                    int pos = argument.IndexOf((System.Char) '.');
-                    if(pos == -1)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException();
-                        e.str = "malformed protocol version `" + argument + "' in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                    
-                    string majStr = argument.Substring(0, (pos) - (0));
-                    string minStr = argument.Substring(pos + 1, (argument.Length) - (pos + 1));
-                    int majVersion;
-                    int minVersion;
-                    try
-                    {
-                        majVersion = System.Int32.Parse(majStr, CultureInfo.InvariantCulture);
-                        minVersion = System.Int32.Parse(minStr, CultureInfo.InvariantCulture);
-                    }
-                    catch(System.FormatException ex)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException(ex);
-                        e.str = "invalid protocol version `" + argument + "' in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                    
-                    if(majVersion < 1 || majVersion > 255 || minVersion < 0 || minVersion > 255)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException();
-                        e.str = "range error in protocol version `" + argument + "' in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                    
-                    if(majVersion != Protocol.protocolMajor)
-                    {
-                        Ice.UnsupportedProtocolException e = new Ice.UnsupportedProtocolException();
-                        e.badMajor = majVersion < 0?majVersion + 255:majVersion;
-                        e.badMinor = minVersion < 0?minVersion + 255:minVersion;
-                        e.major = Protocol.protocolMajor;
-                        e.minor = Protocol.protocolMinor;
-                        throw e;
-                    }
-                    
-                    _protocolMajor = (byte)majVersion;
-                    _protocolMinor = (byte)minVersion;
-                }
-                else if(option.Equals("-e"))
-                {
-                    if(argument == null)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException();
-                        e.str = "no argument provided for -e option in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                
-                    int pos = argument.IndexOf((System.Char) '.');
-                    if(pos == -1)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException();
-                        e.str = "malformed encoding version `" + argument + "' in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                    
-                    string majStr = argument.Substring(0, (pos) - (0));
-                    string minStr = argument.Substring(pos + 1, (argument.Length) - (pos + 1));
-                    int majVersion;
-                    int minVersion;
-                    try
-                    {
-                        majVersion = System.Int32.Parse(majStr, CultureInfo.InvariantCulture);
-                        minVersion = System.Int32.Parse(minStr, CultureInfo.InvariantCulture);
-                    }
-                    catch(System.FormatException ex)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException(ex);
-                        e.str = "invalid encoding version `" + argument + "' in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                    
-                    if(majVersion < 1 || majVersion > 255 || minVersion < 0 || minVersion > 255)
-                    {
-                        Ice.EndpointParseException e = new Ice.EndpointParseException();
-                        e.str = "range error in encoding version `" + argument + "' in endpoint `udp " + str + "'";
-                        throw e;
-                    }
-                    
-                    if(majVersion != Protocol.encodingMajor)
-                    {
-                        Ice.UnsupportedEncodingException e = new Ice.UnsupportedEncodingException();
-                        e.badMajor = majVersion < 0?majVersion + 255:majVersion;
-                        e.badMinor = minVersion < 0?minVersion + 255:minVersion;
-                        e.major = Protocol.encodingMajor;
-                        e.minor = Protocol.encodingMinor;
-                        throw e;
-                    }
-                    
-                    _encodingMajor = (byte)majVersion;
-                    _encodingMinor = (byte)minVersion;
-                }
-                else if(option.Equals("-h"))
+                if(option.Equals("-h"))
                 {
                     if(argument == null)
                     {
@@ -276,6 +162,30 @@ namespace IceInternal
                     
                     _compress = true;
                 }
+                else if(option.Equals("-v") || option.Equals("-e"))
+                {
+                    if(argument == null)
+                    {
+                        Ice.EndpointParseException e = new Ice.EndpointParseException();
+                        e.str = "no argument provided for " + option + " option in endpoint " + "`udp " + str + "'";
+                        throw e;
+                    }
+
+                    try
+                    {
+                        Ice.EncodingVersion v = Ice.Util.stringToEncodingVersion(argument);
+                        if(v.major != 1 || v.minor != 0)
+                        {
+                            instance_.initializationData().logger.warning("deprecated udp endpoint option: " + option);
+                        }
+                    }
+                    catch(Ice.VersionParseException ex)
+                    {
+                        Ice.EndpointParseException e = new Ice.EndpointParseException();
+                        e.str = "invalid version `" + argument + "' in endpoint `udp " + str + "':\n" + ex.str;
+                        throw e;
+                    }
+                }
                 else if(option.Equals("--interface"))
                 {
                     if(argument == null)
@@ -316,9 +226,7 @@ namespace IceInternal
                 }
                 else
                 {
-                    Ice.EndpointParseException e = new Ice.EndpointParseException();
-                    e.str = "unknown option `" + option + "' in `udp " + str + "'";
-                    throw e;
+                    parseOption(option, argument, "udp", str);
                 }
             }
 
@@ -352,27 +260,12 @@ namespace IceInternal
             s.startReadEncaps();
             _host = s.readString();
             _port = s.readInt();
-            _protocolMajor = s.readByte();
-            _protocolMinor = s.readByte();
-            _encodingMajor = s.readByte();
-            _encodingMinor = s.readByte();
-            if(_protocolMajor != Protocol.protocolMajor)
+            if(s.getReadEncoding().Equals(Ice.Util.Encoding_1_0))
             {
-                Ice.UnsupportedProtocolException e = new Ice.UnsupportedProtocolException();
-                e.badMajor = _protocolMajor < 0?_protocolMajor + 255:_protocolMajor;
-                e.badMinor = _protocolMinor < 0?_protocolMinor + 255:_protocolMinor;
-                e.major = Protocol.protocolMajor;
-                e.minor = Protocol.protocolMinor;
-                throw e;
-            }
-            if(_encodingMajor != Protocol.encodingMajor)
-            {
-                Ice.UnsupportedEncodingException e = new Ice.UnsupportedEncodingException();
-                e.badMajor = _encodingMajor < 0?_encodingMajor + 255:_encodingMajor;
-                e.badMinor = _encodingMinor < 0?_encodingMinor + 255:_encodingMinor;
-                e.major = Protocol.encodingMajor;
-                e.minor = Protocol.encodingMinor;
-                throw e;
+                s.readByte();
+                s.readByte();
+                s.readByte();
+                s.readByte();
             }
             // Not transmitted.
             //_connect = s.readBool();
@@ -391,10 +284,11 @@ namespace IceInternal
             s.startWriteEncaps();
             s.writeString(_host);
             s.writeInt(_port);
-            s.writeByte(_protocolMajor);
-            s.writeByte(_protocolMinor);
-            s.writeByte(_encodingMajor);
-            s.writeByte(_encodingMinor);
+            if(s.getWriteEncoding().Equals(Ice.Util.Encoding_1_0))
+            {
+                Ice.Util.Protocol_1_0.write__(s);
+                Ice.Util.Encoding_1_0.write__(s);
+            }
             // Not transmitted.
             //s.writeBool(_connect);
             s.writeBool(_compress);
@@ -415,18 +309,6 @@ namespace IceInternal
             //
             string s = "udp";
 
-            if((int)_protocolMajor != 1 || (int)_protocolMinor != 0)
-            {
-                s += " -v " + (_protocolMajor < 0 ? (int)_protocolMajor + 255 : _protocolMajor);
-                s += "." + (_protocolMinor < 0 ? (int)_protocolMinor + 255 : _protocolMinor);
-            }
-
-            if((int)_encodingMajor != 1 || (int)_encodingMinor != 0)
-            {
-                s += " -e " + (_encodingMajor < 0 ? (int)_encodingMajor + 255 : _encodingMajor);
-                s += "." + (_encodingMinor < 0 ? (int)_encodingMinor + 255 : _encodingMinor);
-            }
-
             if(_host != null && _host.Length != 0)
             {
                 s += " -h ";
@@ -469,10 +351,8 @@ namespace IceInternal
         
         private sealed class InfoI : Ice.UDPEndpointInfo
         {
-            public InfoI(bool comp, string host, int port, byte protocolMajor, byte protocolMinor, 
-                         byte encodingMajor, byte encodingMinor, string mcastInterface, int mcastTtl) :
-                base(-1, comp, host, port, protocolMajor, protocolMinor, encodingMajor, encodingMinor, mcastInterface,
-                    mcastTtl)
+            public InfoI(bool comp, string host, int port, string mcastInterface, int mcastTtl) :
+                base(-1, comp, host, port, mcastInterface, mcastTtl)
             {
             }
 
@@ -497,8 +377,7 @@ namespace IceInternal
         //
         public override Ice.EndpointInfo getInfo()
         {
-            return new InfoI(_compress, _host, _port, _protocolMajor, _protocolMinor, _encodingMajor, 
-                             _encodingMinor, _mcastInterface, _mcastTtl);
+            return new InfoI(_compress, _host, _port, _mcastInterface, _mcastTtl);
         }
 
         //
@@ -509,6 +388,14 @@ namespace IceInternal
             return Ice.UDPEndpointType.value;
         }
         
+        //
+        // Return the protocol name;
+        //
+        public override string protocol()
+        {
+            return "udp";
+        }
+
         //
         // Return the timeout for the endpoint in milliseconds. 0 means
         // non-blocking, -1 means no timeout.
@@ -540,8 +427,7 @@ namespace IceInternal
             }
             else
             {
-                return new UdpEndpointI(instance_, _host, _port, _mcastInterface, _mcastTtl, _protocolMajor, 
-                                        _protocolMinor, _encodingMajor, _encodingMinor, _connect, _connectionId,
+                return new UdpEndpointI(instance_, _host, _port, _mcastInterface, _mcastTtl, _connect, connectionId_, 
                                         compress);
             }
         }
@@ -551,14 +437,13 @@ namespace IceInternal
         //
         public override EndpointI connectionId(string connectionId)
         {
-            if(connectionId == _connectionId)
+            if(connectionId == connectionId_)
             {
                 return this;
             }
             else
             {
-                return new UdpEndpointI(instance_, _host, _port, _mcastInterface, _mcastTtl, _protocolMajor, 
-                                        _protocolMinor, _encodingMajor, _encodingMinor, _connect, connectionId,
+                return new UdpEndpointI(instance_, _host, _port, _mcastInterface, _mcastTtl, _connect, connectionId, 
                                         _compress);
             }
         }
@@ -600,8 +485,7 @@ namespace IceInternal
         {
             UdpTransceiver p = new UdpTransceiver(instance_, _host, _port, _mcastInterface, _connect);
             endpoint = new UdpEndpointI(instance_, _host, p.effectivePort(), _mcastInterface, _mcastTtl, 
-                                        _protocolMajor, _protocolMinor, _encodingMajor, _encodingMinor, _connect,
-                                        _connectionId, _compress);
+                                        _connect, connectionId_, _compress);
             return p;
         }
 
@@ -609,16 +493,23 @@ namespace IceInternal
         // Return a connector for this endpoint, or empty list if no connector
         // is available.
         //
-        public override List<Connector> connectors()
+        public override List<Connector> connectors(Ice.EndpointSelectionType selType)
         {
-            return connectors(Network.getAddresses(_host, _port, instance_.protocolSupport()));
+            return connectors(Network.getAddresses(_host, _port, instance_.protocolSupport(), selType,
+                                                   instance_.preferIPv6(), true),
+                              instance_.networkProxy());
         }
 
-        public override void connectors_async(EndpointI_connectors callback)
-        {
-            instance_.endpointHostResolver().resolve(_host, _port, this, callback);
-        }
 
+        public override void connectors_async(Ice.EndpointSelectionType selType, EndpointI_connectors callback)
+        {
+#if SILVERLIGHT
+            callback.connectors(connectors(selType));
+#else
+            instance_.endpointHostResolver().resolve(_host, _port, selType, this, callback);
+#endif
+        }
+        
         //
         // Return an acceptor for this endpoint, or null if no acceptors
         // is available. In case an acceptor is created, this operation
@@ -649,9 +540,8 @@ namespace IceInternal
             {
                 foreach(string h in hosts)
                 {
-                    endps.Add(new UdpEndpointI(instance_, h, _port, _mcastInterface, _mcastTtl, _protocolMajor, 
-                                               _protocolMinor, _encodingMajor, _encodingMinor, _connect, _connectionId, 
-                                               _compress));
+                    endps.Add(new UdpEndpointI(instance_, h, _port, _mcastInterface, _mcastTtl, _connect, 
+                                               connectionId_, _compress));
                 }
             }
             return endps;
@@ -662,25 +552,21 @@ namespace IceInternal
         //
         public override bool equivalent(EndpointI endpoint)
         {
-            UdpEndpointI udpEndpointI = null;
-            try
-            {
-                udpEndpointI = (UdpEndpointI)endpoint;
-            }
-            catch(System.InvalidCastException)
+            if(!(endpoint is UdpEndpointI))
             {
                 return false;
             }
+
+            UdpEndpointI udpEndpointI = (UdpEndpointI)endpoint;
             return udpEndpointI._host.Equals(_host) && udpEndpointI._port == _port;
         }
 
-        public override List<Connector> connectors(List<IPEndPoint> addresses)
+        public override List<Connector> connectors(List<EndPoint> addresses, NetworkProxy networkProxy)
         {
             List<Connector> connectors = new List<Connector>();
-            foreach(IPEndPoint addr in addresses)
+            foreach(EndPoint addr in addresses)
             {
-                connectors.Add(new UdpConnector(instance_, addr, _mcastInterface, _mcastTtl, _protocolMajor,
-                                                _protocolMinor, _encodingMajor, _encodingMinor, _connectionId));
+                connectors.Add(new UdpConnector(instance_, addr, _mcastInterface, _mcastTtl, connectionId_));
             }
             return connectors;
         }
@@ -693,36 +579,26 @@ namespace IceInternal
         //
         // Compare endpoints for sorting purposes
         //
-        public override bool Equals(object obj)
+        public override int CompareTo(EndpointI obj)
         {
-            return CompareTo(obj) == 0;
-        }
-        
-        public override int CompareTo(object obj)
-        {
-            UdpEndpointI p = null;
-            
-            try
+            if(!(obj is UdpEndpointI))
             {
-                p = (UdpEndpointI) obj;
+                return type() < obj.type() ? -1 : 1;
             }
-            catch(System.InvalidCastException)
-            {
-                try
-                {
-                    EndpointI e = (EndpointI)obj;
-                    return type() < e.type() ? -1 : 1;
-                }
-                catch(System.InvalidCastException)
-                {
-                    Debug.Assert(false);
-                }
-            }
-            
+
+            UdpEndpointI p = (UdpEndpointI)obj;
             if(this == p)
             {
                 return 0;
             }
+            else
+            {
+                int r = base.CompareTo(p);
+                if(r != 0)
+                {
+                    return r;
+                }
+            }
             
             if(_port < p._port)
             {
@@ -742,9 +618,9 @@ namespace IceInternal
                 return 1;
             }
             
-            if(!_connectionId.Equals(p._connectionId))
+            if(!connectionId_.Equals(p.connectionId_))
             {
-                return string.Compare(_connectionId, p._connectionId, StringComparison.Ordinal);
+                return string.Compare(connectionId_, p.connectionId_, StringComparison.Ordinal);
             }
 
             if(!_compress && p._compress)
@@ -756,42 +632,6 @@ namespace IceInternal
                 return 1;
             }
             
-            if(_protocolMajor < p._protocolMajor)
-            {
-                return -1;
-            }
-            else if(p._protocolMajor < _protocolMajor)
-            {
-                return 1;
-            }
-            
-            if(_protocolMinor < p._protocolMinor)
-            {
-                return -1;
-            }
-            else if(p._protocolMinor < _protocolMinor)
-            {
-                return 1;
-            }
-            
-            if(_encodingMajor < p._encodingMajor)
-            {
-                return -1;
-            }
-            else if(p._encodingMajor < _encodingMajor)
-            {
-                return 1;
-            }
-            
-            if(_encodingMinor < p._encodingMinor)
-            {
-                return -1;
-            }
-            else if(p._encodingMinor < _encodingMinor)
-            {
-                return 1;
-            }
-
             int rc = string.Compare(_mcastInterface, p._mcastInterface, StringComparison.Ordinal);
             if(rc != 0)
             {
@@ -812,13 +652,16 @@ namespace IceInternal
         
         private void calcHashValue()
         {
-            _hashCode = _host.GetHashCode();
-            _hashCode = 5 * _hashCode + _port;
-            _hashCode = 5 * _hashCode + _mcastInterface.GetHashCode();
-            _hashCode = 5 * _hashCode + _mcastTtl.GetHashCode();
-            _hashCode = 5 * _hashCode + (_connect?1:0);
-            _hashCode = 5 * _hashCode + _connectionId.GetHashCode();
-            _hashCode = 5 * _hashCode + (_compress?1:0);
+            int h = 5381;
+            IceInternal.HashUtil.hashAdd(ref h, Ice.UDPEndpointType.value);
+            IceInternal.HashUtil.hashAdd(ref h, _host);
+            IceInternal.HashUtil.hashAdd(ref h, _port);
+            IceInternal.HashUtil.hashAdd(ref h, _mcastInterface);
+            IceInternal.HashUtil.hashAdd(ref h, _mcastTtl);
+            IceInternal.HashUtil.hashAdd(ref h, _connect);
+            IceInternal.HashUtil.hashAdd(ref h, connectionId_);
+            IceInternal.HashUtil.hashAdd(ref h, _compress);
+            _hashCode = h;
         }
         
         private Instance instance_;
@@ -826,12 +669,7 @@ namespace IceInternal
         private int _port;
         private string _mcastInterface = "";
         private int _mcastTtl = -1;
-        private byte _protocolMajor;
-        private byte _protocolMinor;
-        private byte _encodingMajor;
-        private byte _encodingMinor;
         private bool _connect;
-        private string _connectionId = "";
         private bool _compress;
         private int _hashCode;
     }
