@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -48,8 +48,8 @@ namespace IceSSL
 
             if(_instance.networkTraceLevel() >= 1)
             {
-                StringBuilder s = new StringBuilder("accepting ssl connections at ");
-		s.Append(ToString());
+                StringBuilder s = new StringBuilder("listening for ssl connections at ");
+                s.Append(ToString());
 
                 List<string> interfaces = 
                     IceInternal.Network.getHostsForEndpointExpand(_addr.Address.ToString(), 
@@ -63,7 +63,7 @@ namespace IceSSL
             }
         }
 
-        public bool startAccept(AsyncCallback callback, object state)
+        public bool startAccept(IceInternal.AsyncCallback callback, object state)
         {
             //
             // The plug-in may not be fully initialized.
@@ -77,7 +77,13 @@ namespace IceSSL
 
             try
             {
-                _result = _fd.BeginAccept(callback, state);
+                _result = _fd.BeginAccept(delegate(IAsyncResult result)
+                                          {
+                                              if(!result.CompletedSynchronously)
+                                              {
+                                                  callback(result.AsyncState);
+                                              }
+                                          }, state);
                 return _result.CompletedSynchronously;
             }
             catch(SocketException ex)
@@ -122,7 +128,7 @@ namespace IceSSL
             Socket acceptFd = _acceptFd;
             _acceptFd = null;
             _acceptError = null;
-            return new TransceiverI(_instance, acceptFd, null, "", true, _adapterName);
+            return new TransceiverI(_instance, acceptFd, "", true, true, _adapterName, null, null);
         }
 
         public override string ToString()
@@ -155,8 +161,9 @@ namespace IceSSL
 
             try
             {
-                _addr = IceInternal.Network.getAddressForServer(host, port, _instance.protocolSupport());
-                _fd = IceInternal.Network.createSocket(false, _addr.AddressFamily);
+                int protocol = instance.protocolSupport();
+                _addr = (IPEndPoint)IceInternal.Network.getAddressForServer(host, port, protocol, instance.preferIPv6());
+                _fd = IceInternal.Network.createServerSocket(false, _addr.AddressFamily, protocol);
                 IceInternal.Network.setBlock(_fd, false);
                 IceInternal.Network.setTcpBufSize(_fd, _instance.communicator().getProperties(), _logger);
                 if(IceInternal.AssemblyUtil.platform_ != IceInternal.AssemblyUtil.Platform.Windows)
