@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,7 +19,7 @@ namespace IceSSL
     using System.Security.Cryptography.X509Certificates;
     using System.Text;
     using System.Globalization;
-	
+        
     internal class Instance
     {
         internal Instance(Ice.Communicator communicator)
@@ -55,6 +55,28 @@ namespace IceSSL
             //
             _defaultDir = properties.getProperty(prefix + "DefaultDir");
 
+            string keySet = properties.getPropertyWithDefault(prefix + "KeySet", "DefaultKeySet");
+            if(!keySet.Equals("DefaultKeySet") && !keySet.Equals("UserKeySet") && !keySet.Equals("MachineKeySet"))
+            {
+                _logger.warning("Invalid IceSSL.KeySet value `" + keySet + "' adjusted to `DefaultKeySet'");
+                keySet = "DefaultKeySet";
+            }
+
+            X509KeyStorageFlags keyStorageFlags = X509KeyStorageFlags.DefaultKeySet;
+            if(keySet.Equals("UserKeySet"))
+            {
+                keyStorageFlags = X509KeyStorageFlags.UserKeySet;
+            }
+            else if(keySet.Equals("MachineKeySet"))
+            {
+                keyStorageFlags = X509KeyStorageFlags.MachineKeySet;
+            }
+
+            if(properties.getPropertyAsIntWithDefault(prefix + "PersistKeySet", 0) > 0)
+            {
+                keyStorageFlags |= X509KeyStorageFlags.PersistKeySet;
+            }
+
             //
             // Process IceSSL.ImportCert.* properties.
             //
@@ -65,7 +87,7 @@ namespace IceSSL
                 string val = entry.Value;
                 if(val.Length > 0)
                 {
-                    importCertificate(name, val);
+                    importCertificate(name, val, keyStorageFlags);
                 }
             }
 
@@ -186,6 +208,7 @@ namespace IceSSL
                 _certs = new X509Certificate2Collection();
                 string certFile = properties.getProperty(prefix + "CertFile");
                 string passwordStr = properties.getProperty(prefix + "Password");
+                
                 if(certFile.Length > 0)
                 {
                     if(!checkPath(ref certFile))
@@ -210,11 +233,11 @@ namespace IceSSL
                         X509Certificate2 cert;
                         if(password != null)
                         {
-                            cert = new X509Certificate2(certFile, password);
+                            cert = new X509Certificate2(certFile, password, keyStorageFlags);
                         }
                         else
                         {
-                            cert = new X509Certificate2(certFile);
+                            cert = new X509Certificate2(certFile, "", keyStorageFlags);
                         }
                         _certs.Add(cert);
                     }
@@ -305,6 +328,21 @@ namespace IceSSL
             return _facade.getProtocolSupport();
         }
 
+        internal bool preferIPv6()
+        {
+            return _facade.getPreferIPv6();
+        }
+
+        internal IceInternal.NetworkProxy networkProxy()
+        {
+            return _facade.getNetworkProxy();
+        }
+
+        internal Ice.EncodingVersion defaultEncoding()
+        {
+            return _facade.getDefaultEncoding();
+        }
+
         internal string defaultHost()
         {
             return _facade.getDefaultHost();
@@ -493,7 +531,7 @@ namespace IceSSL
                 }
 
                 //
-                // Compare the peer's address against the the dnsName and ipAddress
+                // Compare the peer's address against the dnsName and ipAddress
                 // values in the subject alternative name.
                 //
                 if(!certNameOK && ipAddresses != null)
@@ -679,7 +717,7 @@ namespace IceSSL
             return false;
         }
 
-        private void importCertificate(string propName, string propValue)
+        private void importCertificate(string propName, string propValue, X509KeyStorageFlags keyStorageFlags)
         {
             //
             // Expecting a property of the following form:
@@ -761,11 +799,11 @@ namespace IceSSL
                 X509Certificate2 cert;
                 if(password != null)
                 {
-                    cert = new X509Certificate2(file, password);
+                    cert = new X509Certificate2(file, password, keyStorageFlags);
                 }
                 else
                 {
-                    cert = new X509Certificate2(file);
+                    cert = new X509Certificate2(file, "", keyStorageFlags);
                 }
                 store.Add(cert);
             }
@@ -857,16 +895,47 @@ namespace IceSSL
                 result = 0;
                 for(int i = 0; i < arr.Length; ++i)
                 {
+                    string protocol = null;
                     string s = arr[i].ToUpperInvariant();
-                    if(s.Equals("SSL3") || s.Equals("SSLV3"))
+                    switch(s)
                     {
-                        result |= SslProtocols.Ssl3;
+                        case "SSL3":
+                        case "SSLV3":
+                        {
+                            protocol = "Ssl3";
+                            break;
+                        }
+                        case "TLS":
+                        case "TLS1":
+                        case "TLSV1":
+                        {
+                            protocol = "Tls";
+                            break;
+                        }
+                        case "TLS1_1": 
+                        case "TLSV1_1":
+                        {
+                            protocol = "Tls11";
+                            break;
+                        }
+                        case "TLS1_2": 
+                        case "TLSV1_2":
+                        {
+                            protocol = "Tls12";
+                            break;
+                        }
+                        default:
+                        {
+                            break;
+                        }
                     }
-                    else if(s.Equals("TLS") || s.Equals("TLS1") || s.Equals("TLSV1"))
+
+                    try
                     {
-                        result |= SslProtocols.Tls;
+                        SslProtocols value = (SslProtocols)Enum.Parse(typeof(SslProtocols), protocol);
+                        result |= value;
                     }
-                    else
+                    catch(Exception)
                     {
                         Ice.PluginInitializationException e = new Ice.PluginInitializationException();
                         e.reason = "IceSSL: unrecognized protocol `" + s + "'";
