@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -38,11 +38,28 @@ namespace IceSSL
                 {
                     IceInternal.Network.doFinishConnectAsync(_fd, _writeResult);
                     _writeResult = null;
+
+                    _desc = IceInternal.Network.fdToString(_fd, _proxy, _addr);
+                    if(_proxy != null)
+                    {
+                        _state = StateProxyConnectRequest; // Send proxy connect request
+                        return IceInternal.SocketOperation.Write; 
+                    }
+
                     _state = StateAuthenticatePending;
-                    _desc = IceInternal.Network.fdToString(_fd);
                     return IceInternal.SocketOperation.Connect;
                 }
-                if(_state == StateNeedAuthenticate)
+                else if(_state == StateProxyConnectRequest)
+                {
+                    _state = StateProxyConnectRequestPending; // Wait for proxy response
+                    return IceInternal.SocketOperation.Read;
+                }
+                else if(_state == StateProxyConnectRequestPending)
+                {
+                    _state = StateAuthenticatePending;
+                    return IceInternal.SocketOperation.Connect;
+                }
+                else if(_state == StateNeedAuthenticate)
                 {
                     _state = StateAuthenticatePending;
                     return IceInternal.SocketOperation.Connect;
@@ -61,9 +78,8 @@ namespace IceSSL
                 {
                     System.Text.StringBuilder s = new System.Text.StringBuilder();
                     s.Append("failed to establish ssl connection\n");
-                    s.Append(IceInternal.Network.fdLocalAddressToString(_fd));
-                    Debug.Assert(_addr != null);
-                    s.Append("\nremote address = " + _addr.ToString() + "\n");
+                    s.Append(IceInternal.Network.fdToString(_fd, _proxy, _addr));
+                    s.Append("\n");
                     s.Append(e.ToString());
                     _logger.trace(_instance.networkTraceCategory(), s.ToString());
                 }
@@ -117,10 +133,15 @@ namespace IceSSL
             return false; // Caller will use async read.
         }
 
-        public bool startRead(IceInternal.Buffer buf, AsyncCallback callback, object state)
+        public bool startRead(IceInternal.Buffer buf, IceInternal.AsyncCallback callback, object state)
         {
             Debug.Assert(_fd != null);
 
+            if(_state == StateProxyConnectRequestPending)
+            {
+                _proxy.beginReadConnectRequestResponse(buf);
+            }
+
             int packetSize = buf.b.remaining();
             if(_maxReceivePacketSize > 0 && packetSize > _maxReceivePacketSize)
             {
@@ -129,7 +150,17 @@ namespace IceSSL
 
             try
             {
-                _readResult = _stream.BeginRead(buf.b.rawBytes(), buf.b.position(), packetSize, callback, state);
+                _readCallback = callback;
+                if(_stream != null)
+                {
+                    _readResult = _stream.BeginRead(buf.b.rawBytes(), buf.b.position(), packetSize, readCompleted,
+                                                    state);
+                }
+                else
+                {
+                    _readResult = _fd.BeginReceive(buf.b.rawBytes(), buf.b.position(), packetSize, SocketFlags.None,
+                                                   readCompleted, state);
+                }
                 return _readResult.CompletedSynchronously;
             }
             catch(IOException ex)
@@ -170,7 +201,7 @@ namespace IceSSL
 
             try
             {
-                int ret = _stream.EndRead(_readResult);
+                int ret = _stream != null ? _stream.EndRead(_readResult) : _fd.EndReceive(_readResult);
                 _readResult = null;
                 if(ret == 0)
                 {
@@ -192,10 +223,17 @@ namespace IceSSL
 
                 if(_stats != null)
                 {
+#pragma warning disable 618
                     _stats.bytesReceived(type(), ret);
+#pragma warning restore 618
                 }
 
                 buf.b.position(buf.b.position() + ret);
+
+                if(_state == StateProxyConnectRequestPending)
+                {
+                    _proxy.endReadConnectRequestResponse(buf);
+                }
             }
             catch(IOException ex)
             {
@@ -223,21 +261,33 @@ namespace IceSSL
             }
         }
 
-        public bool startWrite(IceInternal.Buffer buf, AsyncCallback callback, object state, out bool completed)
+        public bool startWrite(IceInternal.Buffer buf, IceInternal.AsyncCallback callback, object state, 
+                               out bool completed)
         {
             Debug.Assert(_fd != null);
             
             if(_state < StateConnected)
             {
+                completed = false;
                 if(_state == StateConnectPending)
                 {
-                    _writeResult = IceInternal.Network.doConnectAsync(_fd, _addr, callback, state);
-                    completed = false;
-                    return _writeResult.CompletedSynchronously;
+                    try
+                    {
+                        EndPoint addr = _proxy != null ? _proxy.getAddress() : _addr;
+                        _writeResult = IceInternal.Network.doConnectAsync(_fd, addr, callback, state);
+                        return _writeResult.CompletedSynchronously;
+                    }
+                    catch(Exception ex)
+                    {
+                        throw new Ice.SocketException(ex);
+                    }
+                }
+                else if(_state == StateProxyConnectRequest)
+                {
+                    _proxy.beginWriteConnectRequest(_addr, buf);
                 }
                 else if(_state == StateAuthenticatePending)
                 {
-                    completed = false;
                     return beginAuthenticate(callback, state);
                 }
             }
@@ -254,7 +304,17 @@ namespace IceSSL
 
             try
             {
-                _writeResult = _stream.BeginWrite(buf.b.rawBytes(), buf.b.position(), packetSize, callback, state);
+                _writeCallback = callback;
+                if(_stream != null)
+                {
+                    _writeResult = _stream.BeginWrite(buf.b.rawBytes(), buf.b.position(), packetSize, writeCompleted, 
+                                                      state);
+                }
+                else
+                {
+                    _writeResult = _fd.BeginSend(buf.b.rawBytes(), buf.b.position(), packetSize, SocketFlags.None, 
+                                                 writeCompleted, state);
+                }
                 completed = packetSize == buf.b.remaining();
                 return _writeResult.CompletedSynchronously;
             }
@@ -296,7 +356,7 @@ namespace IceSSL
                 return; 
             }
             
-            if(_state < StateConnected)
+            if(_state < StateConnected && _state != StateProxyConnectRequest)
             {
                 return;
             }
@@ -305,7 +365,14 @@ namespace IceSSL
 
             try
             {
-                _stream.EndWrite(_writeResult);
+                if(_stream != null)
+                {
+                    _stream.EndWrite(_writeResult);
+                }
+                else
+                {
+                    _fd.EndSend(_writeResult);
+                }
                 _writeResult = null;
 
                 int packetSize = buf.b.remaining();
@@ -322,10 +389,17 @@ namespace IceSSL
 
                 if(_stats != null)
                 {
+#pragma warning disable 618
                     _stats.bytesSent(type(), packetSize);
+#pragma warning restore 618
                 }
 
                 buf.b.position(buf.b.position() + packetSize);
+
+                if(_state == StateProxyConnectRequest)
+                {
+                    _proxy.endWriteConnectRequest(buf);
+                }
             }
             catch(IOException ex)
             {
@@ -379,18 +453,20 @@ namespace IceSSL
         //
         // Only for use by ConnectorI, AcceptorI.
         //
-        internal TransceiverI(Instance instance, Socket fd, IPEndPoint addr, string host, bool connected,
-                              string adapterName)
+        internal TransceiverI(Instance instance, Socket fd, string host, bool connected,
+                              bool incoming, string adapterName, IPEndPoint addr, IceInternal.NetworkProxy proxy)
         {
             _instance = instance;
             _fd = fd;
-            _addr = addr;
             _host = host;
+            _incoming = incoming;
             _adapterName = adapterName;
+            _addr = addr;
+            _proxy = proxy;
             _stream = null;
             _logger = instance.communicator().getLogger();
             _stats = instance.communicator().getStats();
-            _desc = connected ? IceInternal.Network.fdToString(_fd) : "<not connected>";
+            _desc = connected ? IceInternal.Network.fdToString(_fd, _proxy, _addr) : "<not connected>";
             _state = connected ? StateNeedAuthenticate : StateNeedConnect;
 
             _maxSendPacketSize = IceInternal.Network.getSendBufferSize(fd);
@@ -422,43 +498,43 @@ namespace IceSSL
 
         private NativeConnectionInfo getNativeConnectionInfo()
         {
-            Debug.Assert(_fd != null && _stream != null);
             IceSSL.NativeConnectionInfo info = new IceSSL.NativeConnectionInfo();
-            IPEndPoint localEndpoint = IceInternal.Network.getLocalAddress(_fd);
-            info.localAddress = localEndpoint.Address.ToString();
-            info.localPort = localEndpoint.Port;
-            IPEndPoint remoteEndpoint = IceInternal.Network.getRemoteAddress(_fd);
-            if(remoteEndpoint != null)
+            if(_fd != null)
             {
-                info.remoteAddress = remoteEndpoint.Address.ToString();
-                info.remotePort = remoteEndpoint.Port;
-            }
-            else
-            {
-                info.remoteAddress = "";
-                info.remotePort = -1;
-            }
-            info.cipher = _stream.CipherAlgorithm.ToString();
-            info.nativeCerts = _chain;
-            List<string> certs = new List<string>();
-            if(info.nativeCerts != null)
-            {
-                foreach(X509Certificate2 cert in info.nativeCerts)
+                IPEndPoint localEndpoint = (IPEndPoint)IceInternal.Network.getLocalAddress(_fd);
+                info.localAddress = localEndpoint.Address.ToString();
+                info.localPort = localEndpoint.Port;
+                IPEndPoint remoteEndpoint = (IPEndPoint)IceInternal.Network.getRemoteAddress(_fd);
+                if(remoteEndpoint != null)
                 {
-                    StringBuilder s = new StringBuilder();
-                    s.Append("-----BEGIN CERTIFICATE-----\n");
-                    s.Append(Convert.ToBase64String(cert.Export(X509ContentType.Cert)));
-                    s.Append("\n-----END CERTIFICATE-----");
-                    certs.Add(s.ToString());
+                    info.remoteAddress = remoteEndpoint.Address.ToString();
+                    info.remotePort = remoteEndpoint.Port;
                 }
             }
-            info.certs = certs.ToArray();
+            if(_stream != null)
+            {
+                info.cipher = _stream.CipherAlgorithm.ToString();
+                info.nativeCerts = _chain;
+                List<string> certs = new List<string>();
+                if(info.nativeCerts != null)
+                {
+                    foreach(X509Certificate2 cert in info.nativeCerts)
+                    {
+                        StringBuilder s = new StringBuilder();
+                        s.Append("-----BEGIN CERTIFICATE-----\n");
+                        s.Append(Convert.ToBase64String(cert.Export(X509ContentType.Cert)));
+                        s.Append("\n-----END CERTIFICATE-----");
+                        certs.Add(s.ToString());
+                    }
+                }
+                info.certs = certs.ToArray();
+            }
             info.adapterName = _adapterName;
             info.incoming = _adapterName != null;
             return info;
         }
 
-        private bool beginAuthenticate(AsyncCallback callback, object state)
+        private bool beginAuthenticate(IceInternal.AsyncCallback callback, object state)
         {
             NetworkStream ns = new NetworkStream(_fd, true);
             _stream = new SslStream(ns, false, new RemoteCertificateValidationCallback(validationCallback), null);
@@ -473,7 +549,13 @@ namespace IceSSL
                     _writeResult = _stream.BeginAuthenticateAsClient(_host, _instance.certs(),
                                                                      _instance.protocols(),
                                                                      _instance.checkCRL() > 0,
-                                                                     callback, state);
+                                                                     delegate(IAsyncResult result)
+                                                                     {
+                                                                         if(!result.CompletedSynchronously)
+                                                                         {
+                                                                             callback(result.AsyncState);
+                                                                         }
+                                                                     }, state);
                 }
                 else
                 {
@@ -490,7 +572,14 @@ namespace IceSSL
                     }
 
                     _writeResult = _stream.BeginAuthenticateAsServer(cert, _verifyPeer > 1, _instance.protocols(),
-                                                                     _instance.checkCRL() > 0, callback, state);
+                                                                     _instance.checkCRL() > 0, 
+                                                                     delegate(IAsyncResult result)
+                                                                     {
+                                                                         if(!result.CompletedSynchronously)
+                                                                         {
+                                                                             callback(result.AsyncState);
+                                                                         }
+                                                                     }, state);
                 }
             }
             catch(IOException ex)
@@ -728,11 +817,29 @@ namespace IceSSL
             return true;
         }
 
+        internal void readCompleted(IAsyncResult result)
+        {
+            if(!result.CompletedSynchronously)
+            {
+                _readCallback(result.AsyncState);
+            }
+        }
+
+        internal void writeCompleted(IAsyncResult result)
+        {
+            if(!result.CompletedSynchronously)
+            {
+                _writeCallback(result.AsyncState);
+            }
+        }
+
         private Instance _instance;
         private Socket _fd;
-        private IPEndPoint _addr;
         private string _host;
+        private bool _incoming;
         private string _adapterName;
+        private IPEndPoint _addr;
+        private IceInternal.NetworkProxy _proxy;
         private SslStream _stream;
         private Ice.Logger _logger;
         private Ice.Stats _stats;
@@ -743,12 +850,16 @@ namespace IceSSL
         private int _state;
         private IAsyncResult _writeResult;
         private IAsyncResult _readResult;
+        private IceInternal.AsyncCallback _readCallback;
+        private IceInternal.AsyncCallback _writeCallback;
         private X509Certificate2[] _chain;
 
         private const int StateNeedConnect = 0;
         private const int StateConnectPending = 1;
-        private const int StateNeedAuthenticate = 2;
-        private const int StateAuthenticatePending = 3;
-        private const int StateConnected = 4;
+        private const int StateProxyConnectRequest = 2;
+        private const int StateProxyConnectRequestPending = 3; 
+        private const int StateNeedAuthenticate = 4;
+        private const int StateAuthenticatePending = 5;
+        private const int StateConnected = 6;
     }
 }
