@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -24,6 +24,6 @@ public sealed class TestI : TestIntfDisp_
     public override void deactivate(Ice.Current current)
     {
         current.adapter.deactivate();
-        System.Threading.Thread.Sleep(10 * 1000);
+        System.Threading.Thread.Sleep(100);
     }
 }
