@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,7 +22,7 @@ internal class Acceptor : IceInternal.Acceptor
         _acceptor.listen();
     }
 
-    public bool startAccept(AsyncCallback callback, object state)
+    public bool startAccept(IceInternal.AsyncCallback callback, object state)
     {
         return _acceptor.startAccept(callback, state);
     }
