@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -24,13 +24,11 @@ public class Client
         AllTests.allTests(communicator);
         return 0;
     }
-    
-    public static void Main(string[] args)
+
+    public static int Main(string[] args)
     {
         int status = 0;
         Ice.Communicator communicator = null;
-        
-        Debug.Listeners.Add(new ConsoleTraceListener());
 
         try
         {
@@ -39,10 +37,10 @@ public class Client
         }
         catch(System.Exception ex)
         {
-            System.Console.Error.WriteLine(ex);
+            System.Console.Error.WriteLine(ex.ToString());
             status = 1;
         }
-        
+
         if(communicator != null)
         {
             try
@@ -55,10 +53,7 @@ public class Client
                 status = 1;
             }
         }
-        
-        if(status != 0)
-        {
-            System.Environment.Exit(status);
-        }
+
+        return status;
     }
 }
