@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -15,9 +15,11 @@ public sealed class UnexpectedObjectExceptionTestI : Ice.Blobject
     {
         Ice.Communicator communicator = current.adapter.getCommunicator();
         Ice.OutputStream @out = Ice.Util.createOutputStream(communicator);
+        @out.startEncapsulation(current.encoding, Ice.FormatType.DefaultFormat);
         AlsoEmpty ae = new AlsoEmpty();
         Test.AlsoEmptyHelper.write(@out, ae);
         @out.writePendingObjects();
+        @out.endEncapsulation();
         outParams = @out.finished();
         return true;
     }
