@@ -1,12 +1,12 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
-
+#if !SILVERLIGHT
 using System;
 
 [assembly: CLSCompliant(true)]
@@ -45,3 +45,4 @@ public class Struct // Used to test that null members marshal correctly.
 }
 
 }
+#endif
