@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,16 +12,12 @@ using System.Diagnostics;
 using System.Threading;
 using Test;
 
-public class AllTests
-{
-    private static void test(bool b)
-    {
-        if(!b)
-        {
-            throw new System.Exception();
-        }
-    }
+#if SILVERLIGHT
+using System.Windows.Controls;
+#endif
 
+public class AllTests : TestCommon.TestApp
+{
     private class Callback
     {
         internal Callback()
@@ -327,24 +323,79 @@ public class AllTests
         private Callback callback = new Callback();
     }
 
-    public static TestIntfPrx allTests(Ice.Communicator communicator, bool collocated)
+    private class RelayI : RelayDisp_
     {
-        Console.Out.Write("testing stringToProxy... ");
-        Console.Out.Flush();
+        public override void knownPreservedAsBase(Ice.Current current)
+        {
+            KnownPreservedDerived ex = new KnownPreservedDerived();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            throw ex;
+        }
+
+        public override void knownPreservedAsKnownPreserved(Ice.Current current)
+        {
+            KnownPreservedDerived ex = new KnownPreservedDerived();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            throw ex;
+        }
+
+        public override void unknownPreservedAsBase(Ice.Current current)
+        {
+            Preserved2 ex = new Preserved2();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            ex.p1 = new PreservedClass("bc", "pc");
+            ex.p2 = ex.p1;
+            throw ex;
+        }
+
+        public override void unknownPreservedAsKnownPreserved(Ice.Current current)
+        {
+            Preserved2 ex = new Preserved2();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            ex.p1 = new PreservedClass("bc", "pc");
+            ex.p2 = ex.p1;
+            throw ex;
+        }
+    }
+
+#if SILVERLIGHT
+    public override Ice.InitializationData initData()
+    {
+        Ice.InitializationData initData = new Ice.InitializationData();
+        initData.properties = Ice.Util.createProperties();
+        initData.properties.setProperty("Ice.FactoryAssemblies", "exceptions,version=1.0.0.0");
+        return initData;
+    }
+
+    public override void run(Ice.Communicator communicator)
+#else
+    public static TestIntfPrx allTests(Ice.Communicator communicator, bool collocated)
+#endif
+    {
+        Write("testing stringToProxy... ");
+        Flush();
         String @ref = "Test:default -p 12010 -t 2000";
         Ice.ObjectPrx @base = communicator.stringToProxy(@ref);
         test(@base != null);
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("testing checked cast... ");
-        Console.Out.Flush();
+        Write("testing checked cast... ");
+        Flush();
         TestIntfPrx testPrx = TestIntfPrxHelper.checkedCast(@base);
         test(testPrx != null);
         test(testPrx.Equals(@base));
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("base... ");
-        Console.Out.Flush();
+        Write("base... ");
+        Flush();
         {
             try
             {
@@ -361,19 +412,19 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("base (AMI)... ");
-        Console.Out.Flush();
+        Write("base (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_baseAsBase().whenCompleted(cb.response, cb.exception_baseAsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown derived... ");
-        Console.Out.Flush();
+        Write("slicing of unknown derived... ");
+        Flush();
         {
             try
             {
@@ -390,19 +441,19 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown derived (AMI)... ");
-        Console.Out.Flush();
+        Write("slicing of unknown derived (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_unknownDerivedAsBase().whenCompleted(cb.response, cb.exception_unknownDerivedAsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known derived as base... ");
-        Console.Out.Flush();
+        Write("non-slicing of known derived as base... ");
+        Flush();
         {
             try
             {
@@ -420,19 +471,19 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known derived as base (AMI)... ");
-        Console.Out.Flush();
+        Write("non-slicing of known derived as base (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_knownDerivedAsBase().whenCompleted(cb.response, cb.exception_knownDerivedAsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known derived as derived... ");
-        Console.Out.Flush();
+        Write("non-slicing of known derived as derived... ");
+        Flush();
         {
             try
             {
@@ -450,20 +501,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known derived as derived (AMI)... ");
-        Console.Out.Flush();
+        Write("non-slicing of known derived as derived (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_knownDerivedAsKnownDerived().whenCompleted(
                         cb.response, cb.exception_knownDerivedAsKnownDerived);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown intermediate as base... ");
-        Console.Out.Flush();
+        Write("slicing of unknown intermediate as base... ");
+        Flush();
         {
             try
             {
@@ -480,20 +531,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown intermediate as base (AMI)... ");
-        Console.Out.Flush();
+        Write("slicing of unknown intermediate as base (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_unknownIntermediateAsBase().whenCompleted(
                         cb.response, cb.exception_unknownIntermediateAsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of known intermediate as base... ");
-        Console.Out.Flush();
+        Write("slicing of known intermediate as base... ");
+        Flush();
         {
             try
             {
@@ -511,20 +562,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of known intermediate as base (AMI)... ");
-        Console.Out.Flush();
+        Write("slicing of known intermediate as base (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_knownIntermediateAsBase().whenCompleted(
                         cb.response, cb.exception_knownIntermediateAsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of known most derived as base... ");
-        Console.Out.Flush();
+        Write("slicing of known most derived as base... ");
+        Flush();
         {
             try
             {
@@ -543,20 +594,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of known most derived as base (AMI)... ");
-        Console.Out.Flush();
+        Write("slicing of known most derived as base (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_knownMostDerivedAsBase().whenCompleted(
                         cb.response, cb.exception_knownMostDerivedAsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known intermediate as intermediate... ");
-        Console.Out.Flush();
+        Write("non-slicing of known intermediate as intermediate... ");
+        Flush();
         {
             try
             {
@@ -574,20 +625,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known intermediate as intermediate (AMI)... ");
-        Console.Out.Flush();
+        Write("non-slicing of known intermediate as intermediate (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_knownIntermediateAsKnownIntermediate().whenCompleted(
                         cb.response, cb.exception_knownIntermediateAsKnownIntermediate);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known most derived as intermediate... ");
-        Console.Out.Flush();
+        Write("non-slicing of known most derived as intermediate... ");
+        Flush();
         {
             try
             {
@@ -606,20 +657,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known most derived as intermediate (AMI)... ");
-        Console.Out.Flush();
+        Write("non-slicing of known most derived as intermediate (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_knownMostDerivedAsKnownIntermediate().whenCompleted(
                         cb.response, cb.exception_knownMostDerivedAsKnownIntermediate);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known most derived as most derived... ");
-        Console.Out.Flush();
+        Write("non-slicing of known most derived as most derived... ");
+        Flush();
         {
             try
             {
@@ -638,20 +689,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("non-slicing of known most derived as most derived (AMI)... ");
-        Console.Out.Flush();
+        Write("non-slicing of known most derived as most derived (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_knownMostDerivedAsKnownMostDerived().whenCompleted(
                         cb.response, cb.exception_knownMostDerivedAsKnownMostDerived);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown most derived, known intermediate as base... ");
-        Console.Out.Flush();
+        Write("slicing of unknown most derived, known intermediate as base... ");
+        Flush();
         {
             try
             {
@@ -669,20 +720,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown most derived, known intermediate as base (AMI)... ");
-        Console.Out.Flush();
+        Write("slicing of unknown most derived, known intermediate as base (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_unknownMostDerived1AsBase().whenCompleted(
                         cb.response, cb.exception_unknownMostDerived1AsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown most derived, known intermediate as intermediate... ");
-        Console.Out.Flush();
+        Write("slicing of unknown most derived, known intermediate as intermediate... ");
+        Flush();
         {
             try
             {
@@ -700,20 +751,20 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown most derived, known intermediate as intermediate (AMI)... ");
-        Console.Out.Flush();
+        Write("slicing of unknown most derived, known intermediate as intermediate (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_unknownMostDerived1AsKnownIntermediate().whenCompleted(
                         cb.response, cb.exception_unknownMostDerived1AsKnownIntermediate);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown most derived, unknown intermediate thrown as base... ");
-        Console.Out.Flush();
+        Write("slicing of unknown most derived, unknown intermediate thrown as base... ");
+        Flush();
         {
             try
             {
@@ -730,18 +781,183 @@ public class AllTests
                 test(false);
             }
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
-        Console.Out.Write("slicing of unknown most derived, unknown intermediate thrown as base (AMI)... ");
-        Console.Out.Flush();
+        Write("slicing of unknown most derived, unknown intermediate thrown as base (AMI)... ");
+        Flush();
         {
             AsyncCallback cb = new AsyncCallback();
             testPrx.begin_unknownMostDerived2AsBase().whenCompleted(
-                        cb.response, cb.exception_unknownMostDerived2AsBase);
+                cb.response, cb.exception_unknownMostDerived2AsBase);
             cb.check();
         }
-        Console.Out.WriteLine("ok");
+        WriteLine("ok");
 
+        Write("unknown most derived in compact format... ");
+        Flush();
+        {
+            try
+            {
+                testPrx.unknownMostDerived2AsBaseCompact();
+                test(false);
+            }
+            catch(Base)
+            {
+                //
+                // For the 1.0 encoding, the unknown exception is sliced to Base.
+                //
+                test(testPrx.ice_getEncodingVersion().Equals(Ice.Util.Encoding_1_0));
+            }
+            catch(Ice.UnknownUserException)
+            {
+                //
+                // A MarshalException is raised for the compact format because the
+                // most-derived type is unknown and the exception cannot be sliced.
+                //
+                test(!testPrx.ice_getEncodingVersion().Equals(Ice.Util.Encoding_1_0));
+            }
+            catch(Ice.OperationNotExistException)
+            {
+            }
+            catch(Exception)
+            {
+                test(false);
+            }
+        }
+        WriteLine("ok");
+
+        //
+        // No server side in Silverlight
+        //
+#if !SILVERLIGHT
+        Write("preserved exceptions... ");
+        Flush();
+        {
+            Ice.ObjectAdapter adapter = communicator.createObjectAdapterWithEndpoints("Relay", "default");
+            RelayPrx relay = RelayPrxHelper.uncheckedCast(adapter.addWithUUID(new RelayI()));
+            adapter.activate();
+
+            try
+            {
+                testPrx.relayKnownPreservedAsBase(relay);
+                test(false);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                test(ex.b.Equals("base"));
+                test(ex.kp.Equals("preserved"));
+                test(ex.kpd.Equals("derived"));
+            }
+            catch(Ice.OperationNotExistException)
+            {
+            }
+            catch(Exception)
+            {
+                test(false);
+            }
+
+            try
+            {
+                testPrx.relayKnownPreservedAsKnownPreserved(relay);
+                test(false);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                test(ex.b.Equals("base"));
+                test(ex.kp.Equals("preserved"));
+                test(ex.kpd.Equals("derived"));
+            }
+            /*
+            catch(Exception)
+            {
+                test(false);
+            }
+            */
+            catch(Ice.OperationNotExistException)
+            {
+            }
+            catch(Exception)
+            {
+                test(false);
+            }
+
+            try
+            {
+                testPrx.relayUnknownPreservedAsBase(relay);
+                test(false);
+            }
+            catch(Preserved2 ex)
+            {
+                test(ex.b.Equals("base"));
+                test(ex.kp.Equals("preserved"));
+                test(ex.kpd.Equals("derived"));
+                test(ex.p1.ice_id().Equals(PreservedClass.ice_staticId()));
+                PreservedClass pc = ex.p1 as PreservedClass;
+                test(pc.bc.Equals("bc"));
+                test(pc.pc.Equals("pc"));
+                test(ex.p2 == ex.p1);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                //
+                // For the 1.0 encoding, the unknown exception is sliced to KnownPreserved.
+                //
+                test(testPrx.ice_getEncodingVersion().Equals(Ice.Util.Encoding_1_0));
+                test(ex.b.Equals("base"));
+                test(ex.kp.Equals("preserved"));
+                test(ex.kpd.Equals("derived"));
+            }
+            catch(Ice.OperationNotExistException)
+            {
+            }
+            catch(Exception)
+            {
+                test(false);
+            }
+
+            try
+            {
+                testPrx.relayUnknownPreservedAsKnownPreserved(relay);
+                test(false);
+            }
+            catch(Preserved2 ex)
+            {
+                test(ex.b.Equals("base"));
+                test(ex.kp.Equals("preserved"));
+                test(ex.kpd.Equals("derived"));
+                test(ex.p1.ice_id().Equals(PreservedClass.ice_staticId()));
+                PreservedClass pc = ex.p1 as PreservedClass;
+                test(pc.bc.Equals("bc"));
+                test(pc.pc.Equals("pc"));
+                test(ex.p2 == ex.p1);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                //
+                // For the 1.0 encoding, the unknown exception is sliced to KnownPreserved.
+                //
+                test(testPrx.ice_getEncodingVersion().Equals(Ice.Util.Encoding_1_0));
+                test(ex.b.Equals("base"));
+                test(ex.kp.Equals("preserved"));
+                test(ex.kpd.Equals("derived"));
+            }
+            catch(Ice.OperationNotExistException)
+            {
+            }
+            catch(Exception)
+            {
+                test(false);
+            }
+
+            adapter.destroy();
+        }
+        WriteLine("ok");
+#endif
+
+#if SILVERLIGHT
+        testPrx.shutdown();
+#else
         return testPrx;
+#endif
     }
 }
