@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -17,7 +17,7 @@ public class Client
         try
         {
             Ice.Communicator communicator = Ice.Util.initialize(args);
-            HelloPrx hello = HelloPrxHelper.checkedCast(communicator.stringToProxy("hello:tcp -p 10000"));
+            HelloPrx hello = HelloPrxHelper.checkedCast(communicator.stringToProxy("hello:tcp -h localhost -p 10000"));
             hello.sayHello();
             communicator.destroy();
         }
