@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,6 +22,11 @@ class ObjectFactory implements Ice.ObjectFactory
             return new DerivedPrinterI();
         }
 
+        if(type.equals("::Demo::ClientPrinter"))
+        {
+            return new ClientPrinterI();
+        }
+
         assert(false);
         return null;
     }
