@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -18,6 +18,9 @@ public class Server extends Ice.Application
             return 1;
         }
 
+	Ice.ObjectFactory factory = new ObjectFactory();
+	communicator().addObjectFactory(factory, Demo.Printer.ice_staticId());
+
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("Value");
         Ice.Object object = new InitialI(adapter);
         adapter.add(object, communicator().stringToIdentity("initial"));
