@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -17,7 +17,7 @@ public class Client
         try
         {
             ic = Ice.Util.initialize(args);
-            Ice.ObjectPrx base = ic.stringToProxy("SimplePrinter:default -p 10000");
+            Ice.ObjectPrx base = ic.stringToProxy("SimplePrinter:default -h localhost -p 10000");
             Demo.PrinterPrx printer = Demo.PrinterPrxHelper.checkedCast(base);
             if(printer == null)
             {
