@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -27,8 +27,8 @@ import Freeze.Util;
 public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     implements Map<K, V>, KeyCodec<K>, IteratorModel<K, V>
 {
-    public abstract byte[] encodeValue(V o, Ice.Communicator communicator);
-    public abstract V decodeValue(byte[] b, Ice.Communicator communicator);
+    public abstract byte[] encodeValue(V o, Ice.Communicator communicator, Ice.EncodingVersion encoding);
+    public abstract V decodeValue(byte[] b, Ice.Communicator communicator, Ice.EncodingVersion encoding);
 
     protected
     MapI(Connection connection, String dbName, String key, String value, boolean createDb,
@@ -277,7 +277,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     public void
     fastPut(K key, V value)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(k);
         putImpl(dbKey, value);
     }
@@ -285,7 +285,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     public boolean
     fastRemove(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(k);
         return removeImpl(dbKey);
     }
@@ -455,35 +455,35 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     public java.util.Map.Entry<K, V>
     ceilingEntry(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         return entrySearch(Search.Type.CEILING, k, true);
     }
 
     public java.util.Map.Entry<K, V>
     floorEntry(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         return entrySearch(Search.Type.FLOOR, k, true);
     }
 
     public java.util.Map.Entry<K, V>
     higherEntry(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         return entrySearch(Search.Type.HIGHER, k, true);
     }
 
     public java.util.Map.Entry<K, V>
     lowerEntry(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         return entrySearch(Search.Type.LOWER, k, true);
     }
 
     public K
     ceilingKey(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         java.util.Map.Entry<K, V> e = entrySearch(Search.Type.CEILING, k, false);
         return e != null ? e.getKey() : null;
     }
@@ -491,7 +491,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     public K
     floorKey(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         java.util.Map.Entry<K, V> e = entrySearch(Search.Type.FLOOR, k, false);
         return e != null ? e.getKey() : null;
     }
@@ -499,7 +499,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     public K
     higherKey(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         java.util.Map.Entry<K, V> e = entrySearch(Search.Type.HIGHER, k, false);
         return e != null ? e.getKey() : null;
     }
@@ -507,7 +507,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     public K
     lowerKey(K key)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         java.util.Map.Entry<K, V> e = entrySearch(Search.Type.LOWER, k, false);
         return e != null ? e.getKey() : null;
     }
@@ -757,8 +757,8 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
                 {
                     if(_trace.deadlockWarning)
                     {
-                        _trace.logger.warning("Deadlock in Freeze.MapInternal.MapI.containsValue while " + "iterating over Db \"" +
-                                              _dbName  + "\"; retrying...");
+                        _trace.logger.warning("Deadlock in Freeze.MapInternal.MapI.containsValue while " + 
+                                              "iterating over Db \"" + _dbName  + "\"; retrying...");
                     }
 
                     //
@@ -789,7 +789,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
             throw ex;
         }
 
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
 
         com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(k);
         com.sleepycat.db.DatabaseEntry dbValue = new com.sleepycat.db.DatabaseEntry();
@@ -839,7 +839,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
         @SuppressWarnings("unchecked")
         K key = (K)o;
 
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(k);
         byte[] v = getImpl(dbKey);
         if(v == null)
@@ -848,20 +848,20 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
         }
         else
         {
-            return decodeValue(v, _connection.getCommunicator());
+            return decodeValue(v, _connection.getCommunicator(), _connection.getEncoding());
         }
     }
 
     public V
     put(K key, V value)
     {
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(k);
         byte[] v = getImpl(dbKey);
         V old = null;
         if(v != null)
         {
-            old = decodeValue(v, _connection.getCommunicator());
+            old = decodeValue(v, _connection.getCommunicator(), _connection.getEncoding());
         }
         putImpl(dbKey, value);
         return old;
@@ -873,13 +873,13 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
         @SuppressWarnings("unchecked")
         K key = (K)o;
 
-        byte[] k = encodeKey(key, _connection.getCommunicator());
+        byte[] k = encodeKey(key, _connection.getCommunicator(), _connection.getEncoding());
         com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(k);
         byte[] v = getImpl(dbKey);
 
         if(v != null && removeImpl(dbKey))
         {
-            return decodeValue(v, _connection.getCommunicator());
+            return decodeValue(v, _connection.getCommunicator(), _connection.getEncoding());
         }
         else
         {
@@ -898,13 +898,55 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
         }
 
         com.sleepycat.db.Transaction txn = _connection.dbTxn();
+        if(txn == null)
+        {
+            closeAllIterators();
+        }
 
+        com.sleepycat.db.DatabaseEntry keyEntry = new com.sleepycat.db.DatabaseEntry();
+        keyEntry.setPartial(true);
+
+        com.sleepycat.db.DatabaseEntry valueEntry = new com.sleepycat.db.DatabaseEntry();
+        valueEntry.setPartial(true);
+        
         for(;;)
         {
             try
             {
-                _db.db().truncate(txn, false);
-                break;
+                com.sleepycat.db.Cursor dbc = null;
+                Transaction tx = null;
+                try
+                {
+                    if(txn == null)
+                    {
+                        tx = _connection.beginTransaction();
+                        txn = _connection.dbTxn();
+                    }
+
+                    dbc = _db.db().openCursor(txn, null);
+                    while(dbc.getNext(keyEntry, valueEntry, null) == com.sleepycat.db.OperationStatus.SUCCESS)
+                    {
+                        dbc.delete();
+                    }
+                    break;
+                }
+                finally
+                {
+                    try
+                    {
+                        if(dbc != null)
+                        {
+                            dbc.close();
+                        }
+                    }
+                    finally 
+                    {
+                        if(tx != null)
+                        {
+                            tx.commit();
+                        }
+                    }
+                }
             }
             catch(com.sleepycat.db.DeadlockException e)
             {
@@ -958,7 +1000,8 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
                     V value = entry.getValue();
 
                     byte[] v = getImpl(entry.getDbKey());
-                    return v != null && valEquals(decodeValue(v, _connection.getCommunicator()), value);
+                    return v != null && valEquals(decodeValue(v, _connection.getCommunicator(),
+                                                              _connection.getEncoding()), value);
                 }
 
                 public boolean
@@ -973,7 +1016,8 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
                     V value = entry.getValue();
 
                     byte[] v = getImpl(entry.getDbKey());
-                    if(v != null && valEquals(decodeValue(v, _connection.getCommunicator()), value))
+                    if(v != null && valEquals(decodeValue(v, _connection.getCommunicator(), _connection.getEncoding()),
+                                              value))
                     {
                         return removeImpl(entry.getDbKey());
                     }
@@ -1051,18 +1095,15 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     {
         int count = 0;
 
-        synchronized(_iteratorList)
-        {
-            java.util.Iterator<IteratorI> p = _iteratorList.iterator();
+        java.util.Iterator<IteratorI> p = _iteratorList.iterator();
 
-            while(p.hasNext())
+        while(p.hasNext())
+        {
+            IteratorI i = p.next();
+            if(i != except)
             {
-                IteratorI i = p.next();
-                if(i != except)
-                {
-                    i.close();
-                    ++count;
-                }
+                i.close();
+                ++count;
             }
         }
 
@@ -1072,13 +1113,10 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
     Object
     addIterator(IteratorI i)
     {
-        synchronized(_iteratorList)
-        {
-            _iteratorList.addFirst(i);
-            java.util.Iterator<IteratorI> p = _iteratorList.iterator();
-            p.next();
-            return p;
-        }
+        _iteratorList.addFirst(i);
+        java.util.Iterator<IteratorI> p = _iteratorList.iterator();
+        p.next();
+        return p;
     }
 
     void
@@ -1087,10 +1125,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
         @SuppressWarnings("unchecked")
         java.util.Iterator<IteratorI> i = (java.util.Iterator<IteratorI>)token;
 
-        synchronized(_iteratorList)
-        {
-            i.remove();
-        }
+        i.remove();
     }
 
     //
@@ -1143,7 +1178,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
             com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry();
             com.sleepycat.db.DatabaseEntry dbValue = new com.sleepycat.db.DatabaseEntry();
 
-            byte[] k = encodeKey(fromKey, _connection.getCommunicator());
+            byte[] k = encodeKey(fromKey, _connection.getCommunicator(), _connection.getEncoding());
             dbKey.setData(k);
             dbKey.setReuseBuffer(false);
 
@@ -1203,7 +1238,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
 
         if(fromKey != null)
         {
-            byte[] k = encodeKey(fromKey, _connection.getCommunicator());
+            byte[] k = encodeKey(fromKey, _connection.getCommunicator(), _connection.getEncoding());
             dbKey.setData(k);
             dbKey.setReuseBuffer(false);
 
@@ -1260,7 +1295,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
             throw ex;
         }
 
-        byte[] v = encodeValue(value, _connection.getCommunicator());
+        byte[] v = encodeValue(value, _connection.getCommunicator(), _connection.getEncoding());
         com.sleepycat.db.DatabaseEntry dbValue = new com.sleepycat.db.DatabaseEntry(v);
 
         if(_trace.level >= 2)
@@ -1428,7 +1463,7 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
         K key = null;
         if(fromKey != null || toKey != null)
         {
-            key = decodeKey(dbKey.getData(), _connection.getCommunicator());
+            key = decodeKey(dbKey.getData(), _connection.getCommunicator(), _connection.getEncoding());
             if(!checkRange(key, fromKey, fromInclusive, toKey, toInclusive))
             {
                 return null;
@@ -1483,7 +1518,8 @@ public abstract class MapI<K, V> extends java.util.AbstractMap<K, V>
         compare(byte[] d1, byte[] d2)
         {
             Ice.Communicator communicator = _connection.getCommunicator();
-            return _comparator.compare(decodeKey(d1, communicator), decodeKey(d2, communicator));
+            Ice.EncodingVersion encoding = _connection.getEncoding();
+            return _comparator.compare(decodeKey(d1, communicator, encoding), decodeKey(d2, communicator, encoding));
         }
 
         //
