@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -21,6 +21,8 @@ class ObjectStore implements IceUtil.Store
         _evictor = evictor;
         _indices = indices;
         _communicator = evictor.communicator();
+        _encoding = evictor.encoding();
+        _keepStats = false;
 
         if(facet.equals(""))
         {
@@ -53,11 +55,18 @@ class ObjectStore implements IceUtil.Store
             Catalog catalog = new Catalog(connection, Util.catalogName(), true);
             CatalogData catalogData = catalog.get(evictor.filename());
 
-            if(catalogData != null && catalogData.evictor == false)
+            if(catalogData != null)
             {
-                DatabaseException ex = new DatabaseException();
-                ex.message = _evictor.errorPrefix() + evictor.filename() + " is not an evictor database";
-                throw ex;
+                if(catalogData.evictor)
+                {
+                    _keepStats = catalogData.value.isEmpty();
+                }
+                else
+                {
+                    DatabaseException ex = new DatabaseException();
+                    ex.message = _evictor.errorPrefix() + evictor.filename() + " is not an evictor database";
+                    throw ex;
+                }
             }
 
             com.sleepycat.db.Environment dbEnv = evictor.dbEnv().getEnv();
@@ -121,8 +130,7 @@ class ObjectStore implements IceUtil.Store
 
                 if(catalogData == null)
                 {
-                    catalogData = new CatalogData();
-                    catalogData.evictor = true;
+                    catalogData = new CatalogData(true, "::Ice::Identity", "Object");
                     catalog.put(evictor.filename(), catalogData);
                 }
 
@@ -191,8 +199,7 @@ class ObjectStore implements IceUtil.Store
             }
         }
 
-        byte[] key = marshalKey(ident, _communicator);
-        com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(key);
+        com.sleepycat.db.DatabaseEntry dbKey = marshalKey(ident, _communicator, _encoding);
 
         //
         // Keep 0 length since we're not interested in the data
@@ -243,7 +250,8 @@ class ObjectStore implements IceUtil.Store
     }
 
     void
-    save(byte[] key, byte[] value, byte status, com.sleepycat.db.Transaction tx)
+    save(com.sleepycat.db.DatabaseEntry key, com.sleepycat.db.DatabaseEntry value, byte status,
+         com.sleepycat.db.Transaction tx)
         throws com.sleepycat.db.DatabaseException
     {
         assert tx != null;
@@ -253,16 +261,14 @@ class ObjectStore implements IceUtil.Store
             case BackgroundSaveEvictorI.created:
             case BackgroundSaveEvictorI.modified:
             {
-                com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(key);
-                com.sleepycat.db.DatabaseEntry dbValue = new com.sleepycat.db.DatabaseEntry(value);
                 com.sleepycat.db.OperationStatus err;
                 if(status == BackgroundSaveEvictorI.created)
                 {
-                    err = _db.putNoOverwrite(tx, dbKey, dbValue);
+                    err = _db.putNoOverwrite(tx, key, value);
                 }
                 else
                 {
-                    err = _db.put(tx, dbKey, dbValue);
+                    err = _db.put(tx, key, value);
                 }
                 if(err != com.sleepycat.db.OperationStatus.SUCCESS)
                 {
@@ -272,8 +278,7 @@ class ObjectStore implements IceUtil.Store
             }
             case BackgroundSaveEvictorI.destroyed:
             {
-                com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(key);
-                com.sleepycat.db.OperationStatus err = _db.delete(tx, dbKey);
+                com.sleepycat.db.OperationStatus err = _db.delete(tx, key);
                 if(err != com.sleepycat.db.OperationStatus.SUCCESS)
                 {
                     throw new DatabaseException();
@@ -287,63 +292,80 @@ class ObjectStore implements IceUtil.Store
         }
     }
 
-    static byte[]
-    marshalKey(Ice.Identity v, Ice.Communicator communicator)
+    static com.sleepycat.db.DatabaseEntry
+    marshalKey(Ice.Identity v, Ice.Communicator communicator, Ice.EncodingVersion encoding)
     {
         IceInternal.BasicStream os =
-            new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), false, false);
+            new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), encoding, true, false);
         v.__write(os);
-        IceInternal.Buffer buf = os.prepareWrite();
-        byte[] r = new byte[buf.size()];
-        buf.b.get(r);
-        return r;
+        return new com.sleepycat.db.DatabaseEntry(os.prepareWrite().b);
     }
 
     static Ice.Identity
-    unmarshalKey(byte[] b, Ice.Communicator communicator)
+    unmarshalKey(com.sleepycat.db.DatabaseEntry e, Ice.Communicator communicator, Ice.EncodingVersion encoding)
     {
-        IceInternal.BasicStream is =
-            new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), false, false);
-        is.resize(b.length, true);
-        IceInternal.Buffer buf = is.getBuffer();
-        buf.b.position(0);
-        buf.b.put(b);
-        buf.b.position(0);
+        IceInternal.BasicStream is;
+        if(e.getDataNIO() != null)
+        {
+            is = new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), encoding, e.getDataNIO());
+        }
+        else
+        {
+            is = new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), encoding, e.getData());
+        }
         Ice.Identity key = new Ice.Identity();
         key.__read(is);
         return key;
     }
 
-    static byte[]
-    marshalValue(ObjectRecord v, Ice.Communicator communicator)
+    static com.sleepycat.db.DatabaseEntry
+    marshalValue(ObjectRecord v, Ice.Communicator communicator, Ice.EncodingVersion encoding, boolean keepStats)
     {
         IceInternal.BasicStream os =
-            new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), false, false);
+            new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), encoding, true, false);
         os.startWriteEncaps();
-        v.__write(os);
+
+        if(keepStats)
+        {
+            v.__write(os);
+        }
+        else
+        {
+            os.writeObject(v.servant);
+        }
         os.writePendingObjects();
         os.endWriteEncaps();
-        IceInternal.Buffer buf = os.prepareWrite();
-        byte[] r = new byte[buf.size()];
-        buf.b.get(r);
-        return r;
+        return new com.sleepycat.db.DatabaseEntry(os.prepareWrite().b);
     }
 
     static ObjectRecord
-    unmarshalValue(byte[] b, Ice.Communicator communicator)
+    unmarshalValue(com.sleepycat.db.DatabaseEntry e, Ice.Communicator communicator, Ice.EncodingVersion encoding,
+                   boolean keepStats)
     {
-        IceInternal.BasicStream is =
-            new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), false, false);
+        IceInternal.BasicStream is;
+        if(e.getDataNIO() != null)
+        {
+            is = new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), encoding, e.getDataNIO());
+        }
+        else
+        {
+            is = new IceInternal.BasicStream(IceInternal.Util.getInstance(communicator), encoding, e.getData());
+        }
         is.sliceObjects(false);
-        is.resize(b.length, true);
-        IceInternal.Buffer buf = is.getBuffer();
-        buf.b.position(0);
-        buf.b.put(b);
-        buf.b.position(0);
-        ObjectRecord rec= new ObjectRecord();
+        ObjectRecord rec = new ObjectRecord();
         is.startReadEncaps();
-        rec.__read(is);
-        is.readPendingObjects();
+        if(keepStats)
+        {
+            rec.__read(is);
+            is.readPendingObjects();
+        }
+        else
+        {
+            Ice.ObjectHolder holder = new Ice.ObjectHolder();
+            is.readObject(holder);
+            is.readPendingObjects();
+            rec.servant = holder.value;
+        }
         is.endReadEncaps();
         return rec;
     }
@@ -366,6 +388,18 @@ class ObjectStore implements IceUtil.Store
         return _communicator;
     }
 
+    final Ice.EncodingVersion
+    encoding()
+    {
+        return _encoding;
+    }
+
+    final boolean
+    keepStats()
+    {
+        return _keepStats;
+    }
+
     final EvictorI
     evictor()
     {
@@ -399,9 +433,7 @@ class ObjectStore implements IceUtil.Store
     {
         Ice.Identity ident = (Ice.Identity)identObj;
 
-        byte[] key = marshalKey(ident, _communicator);
-
-        com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(key);
+        com.sleepycat.db.DatabaseEntry dbKey = marshalKey(ident, _communicator, _encoding);
         com.sleepycat.db.DatabaseEntry dbValue = new com.sleepycat.db.DatabaseEntry();
 
         for(;;)
@@ -439,7 +471,7 @@ class ObjectStore implements IceUtil.Store
             }
         }
 
-        ObjectRecord rec = unmarshalValue(dbValue.getData(), _communicator);
+        ObjectRecord rec = unmarshalValue(dbValue, _communicator, _encoding, _keepStats);
         _evictor.initialize(ident, _facet, rec.servant);
 
         Object result = _evictor.createEvictorElement(ident, rec, this);
@@ -463,9 +495,7 @@ class ObjectStore implements IceUtil.Store
             throw new DatabaseException(_evictor.errorPrefix() + "inactive transaction");
         }
 
-        byte[] key = marshalKey(ident, _communicator);
-
-        com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(key);
+        com.sleepycat.db.DatabaseEntry dbKey = marshalKey(ident, _communicator, _encoding);
         com.sleepycat.db.DatabaseEntry dbValue = new com.sleepycat.db.DatabaseEntry();
 
         try
@@ -497,7 +527,7 @@ class ObjectStore implements IceUtil.Store
             throw new DatabaseException(_evictor.errorPrefix() + "Db.get: " + dx.getMessage(), dx);
         }
 
-        ObjectRecord rec = unmarshalValue(dbValue.getData(), _communicator);
+        ObjectRecord rec = unmarshalValue(dbValue, _communicator, _encoding, _keepStats);
         _evictor.initialize(ident, _facet, rec.servant);
         return rec;
     }
@@ -523,9 +553,8 @@ class ObjectStore implements IceUtil.Store
             throw new DatabaseException(msg);
         }
 
-        com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(marshalKey(ident, _communicator));
-        com.sleepycat.db.DatabaseEntry dbValue =
-            new com.sleepycat.db.DatabaseEntry(marshalValue(objectRecord, _communicator));
+        com.sleepycat.db.DatabaseEntry dbKey = marshalKey(ident, _communicator, _encoding);
+        com.sleepycat.db.DatabaseEntry dbValue = marshalValue(objectRecord, _communicator, _encoding, _keepStats);
 
         try
         {
@@ -565,9 +594,8 @@ class ObjectStore implements IceUtil.Store
             }
         }
 
-        com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(marshalKey(ident, _communicator));
-        com.sleepycat.db.DatabaseEntry dbValue =
-            new com.sleepycat.db.DatabaseEntry(marshalValue(objectRecord, _communicator));
+        com.sleepycat.db.DatabaseEntry dbKey = marshalKey(ident, _communicator, _encoding);
+        com.sleepycat.db.DatabaseEntry dbValue = marshalValue(objectRecord, _communicator, _encoding, _keepStats);
 
         if(_sampleServant != null && !objectRecord.servant.ice_id().equals(_sampleServant.ice_id()))
         {
@@ -621,7 +649,7 @@ class ObjectStore implements IceUtil.Store
             }
         }
 
-        com.sleepycat.db.DatabaseEntry dbKey = new com.sleepycat.db.DatabaseEntry(marshalKey(ident, _communicator));
+        com.sleepycat.db.DatabaseEntry dbKey = marshalKey(ident, _communicator, _encoding);
 
         for(;;)
         {
@@ -661,6 +689,8 @@ class ObjectStore implements IceUtil.Store
     private final EvictorI _evictor;
     private final java.util.List<Index> _indices;
     private final Ice.Communicator _communicator;
+    private final Ice.EncodingVersion _encoding;
+    private boolean _keepStats;
 
     private com.sleepycat.db.Database _db;
     private Ice.Object _sampleServant;
