@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -108,7 +108,8 @@ public abstract class Application extends Ice.Application
 
     /**
      * Called to restart the application's Glacier2 session. This
-     * method never returns.
+     * method never returns. The exception produce an application restart
+     * when called from the Application main thread.
      *
      * @throws RestartSessionException This exception is always thrown.
      **/
@@ -124,16 +125,17 @@ public abstract class Application extends Ice.Application
      * <code>createSession</code> always precedes a call to
      * <code>runWithSession</code>. If <code>Ice.LocalException</code>
      * is thrown from this method, the application is terminated.
-
+     *
      * @return The Glacier2 session.
      **/
     abstract public Glacier2.SessionPrx
     createSession();
 
     /**
-     * Called when the base class detects that the session has been destroyed.
-     * A subclass can override this method to take action after the loss of
-     * connectivity with the Glacier2 router.
+     * Called when the session refresh thread detects that the session has been
+     * destroyed. A subclass can override this method to take action after the
+     * loss of connectivity with the Glacier2 router. This method is always
+     * called from the session refresh thread.
      **/
     public void
     sessionDestroyed()
@@ -245,15 +247,15 @@ public abstract class Application extends Ice.Application
             {
                 try
                 {
-                    _router.refreshSession_async(new Glacier2.AMI_Router_refreshSession()
+                    _router.begin_refreshSession(new Glacier2.Callback_Router_refreshSession()
                         {
                             public void
-                            ice_response()
+                            response()
                             {
                             }
 
                             public void
-                            ice_exception(Ice.LocalException ex)
+                            exception(Ice.LocalException ex)
                             {
                                 //
                                 // Here the session has gone. The thread terminates, and we notify the
@@ -264,7 +266,7 @@ public abstract class Application extends Ice.Application
                             }
 
                             public void
-                            ice_exception(Ice.UserException ex)
+                            exception(Ice.UserException ex)
                             {
                                 //
                                 // Here the session has gone. The thread terminates, and we notify the
