@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -10,7 +10,7 @@
 package Ice;
 
 /**
- * Utility base class that makes it easy to to correctly initialize and finalize
+ * Utility base class that makes it easy to correctly initialize and finalize
  * the Ice run time, as well as handle signals. Unless the application specifies
  * a logger, <Application> installs a per-process logger that logs to the standard
  * error output.
@@ -166,8 +166,20 @@ public abstract class Application
             initData = new InitializationData();
         }
         StringSeqHolder argHolder = new StringSeqHolder(args);
-        initData.properties = Util.createProperties(argHolder, initData.properties);
-
+        try
+        {
+            initData.properties = Util.createProperties(argHolder, initData.properties);
+        }
+        catch(LocalException ex)
+        {
+            Util.getProcessLogger().error(IceInternal.Ex.toString(ex));
+            return 1;
+        }
+        catch(java.lang.Exception ex)
+        {
+            Util.getProcessLogger().error("unknown exception: " + IceInternal.Ex.toString(ex));
+            return 1;
+        }
         _appName = initData.properties.getPropertyWithDefault("Ice.ProgramName", _appName);
 
         //
