@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -328,15 +328,6 @@ public final class Util
         }
     }
 
-    /**
-     * @deprecated generateUUID() is deprecated, use java.util.UUID instead.
-     **/
-    public static String
-    generateUUID()
-    {
-    	return java.util.UUID.randomUUID().toString();
-    }
-
     /**
      * Compares the object identities of two proxies.
      *
@@ -439,7 +430,8 @@ public final class Util
     }
 
     /**
-     * Creates an input stream for dynamic invocation and dispatch.
+     * Creates an input stream for dynamic invocation and dispatch. The stream uses
+     * the communicator's default encoding version.
      *
      * @param communicator The communicator for the stream.
      * @param bytes An encoded request or reply.
@@ -448,11 +440,61 @@ public final class Util
     public static InputStream
     createInputStream(Communicator communicator, byte[] bytes)
     {
-        return new InputStreamI(communicator, bytes);
+        return new InputStreamI(communicator, bytes, true);
     }
 
     /**
-     * Creates an output stream for dynamic invocation and dispatch.
+     * Creates an input stream for dynamic invocation and dispatch. The stream uses
+     * the given encoding version.
+     *
+     * @param communicator The communicator for the stream.
+     * @param bytes An encoded request or reply.
+     * @param v The desired encoding version.
+     * @return The input stream.
+     **/
+    public static InputStream
+    createInputStream(Communicator communicator, byte[] bytes, EncodingVersion v)
+    {
+        return new InputStreamI(communicator, bytes, v, true);
+    }
+
+    /**
+     * Wraps encoded data with an input stream for dynamic invocation
+     * and dispatch. The stream uses the communicator's default
+     * encoding version.
+     *
+     * @param communicator The communicator for the stream.
+     * @param bytes An encoded request or reply.
+     * @param copyBytes True if the given bytes should be copied,
+     * false otherwise.
+     * @return The input stream.
+     **/
+    public static InputStream
+    wrapInputStream(Communicator communicator, byte[] bytes)
+    {
+        return new InputStreamI(communicator, bytes, false);
+    }
+
+    /**
+     * Wraps encoded data with an input stream for dynamic invocation
+     * and dispatch. The stream uses the given encoding version.
+     *
+     * @param communicator The communicator for the stream.
+     * @param bytes An encoded request or reply.
+     * @param v The desired encoding version.
+     * @param copyBytes True if the given bytes should be copied,
+     * false otherwise.
+     * @return The input stream.
+     **/
+    public static InputStream
+    wrapInputStream(Communicator communicator, byte[] bytes, EncodingVersion v)
+    {
+        return new InputStreamI(communicator, bytes, v, false);
+    }
+
+    /**
+     * Creates an output stream for dynamic invocation and dispatch. The stream uses
+     * the communicator's default encoding version.
      *
      * @param communicator The communicator for the stream.
      * @return The output stream.
@@ -463,6 +505,20 @@ public final class Util
         return new OutputStreamI(communicator);
     }
 
+    /**
+     * Creates an output stream for dynamic invocation and dispatch. The stream uses
+     * the given encoding version.
+     *
+     * @param communicator The communicator for the stream.
+     * @param v The desired encoding version.
+     * @return The output stream.
+     **/
+    public static OutputStream
+    createOutputStream(Communicator communicator, EncodingVersion v)
+    {
+        return new OutputStreamI(communicator, v);
+    }
+
     /**
      * Returns the process-wide logger.
      *
@@ -509,7 +565,7 @@ public final class Util
     public static String
     stringVersion()
     {
-        return "3.4.2"; // "A.B.C", with A=major, B=minor, C=patch
+        return "3.5.1"; // "A.B.C", with A=major, B=minor, C=patch
     }
 
     /**
@@ -522,9 +578,154 @@ public final class Util
     public static int
     intVersion()
     {
-        return 30402; // AABBCC, with AA=major, BB=minor, CC=patch
+        return 30501; // AABBCC, with AA=major, BB=minor, CC=patch
     }
 
+    /**
+     * Converts a string to a protocol version.
+     *
+     * @param version The string to convert.
+     *
+     * @return The converted protocol version.
+     **/
+    static public Ice.ProtocolVersion
+    stringToProtocolVersion(String version)
+    {
+        return new Ice.ProtocolVersion(stringToMajor(version), stringToMinor(version));
+    }
+
+    /**
+     * Converts a string to an encoding version.
+     *
+     * @param version The string to convert.
+     *
+     * @return The converted object identity.
+     **/
+    static public Ice.EncodingVersion
+    stringToEncodingVersion(String version)
+    {
+        return new Ice.EncodingVersion(stringToMajor(version), stringToMinor(version));
+    }
+
+    /**
+     * Converts a protocol version to a string.
+     *
+     * @param v The protocol version to convert.
+     *
+     * @return The converted string.
+     **/
+    static public String 
+    protocolVersionToString(Ice.ProtocolVersion v)
+    {
+        return majorMinorToString(v.major, v.minor);
+    }
+
+    /**
+     * Converts an encoding version to a string.
+     *
+     * @param v The encoding version to convert.
+     *
+     * @return The converted string.
+     **/
+    static public String 
+    encodingVersionToString(Ice.EncodingVersion v)
+    {
+        return majorMinorToString(v.major, v.minor);
+    }
+
+    /**
+     * Returns the supported Ice protocol version.
+     *
+     * @return The Ice protocol version.
+     **/
+    static public Ice.ProtocolVersion
+    currentProtocol()
+    {
+        return (Ice.ProtocolVersion)IceInternal.Protocol.currentProtocol.clone();
+    }
+    
+    /**
+     * Returns the supported Ice encoding version.
+     *
+     * @return The Ice encoding version.
+     **/
+    static public Ice.EncodingVersion
+    currentEncoding() 
+    {
+        return (Ice.EncodingVersion)IceInternal.Protocol.currentEncoding.clone();
+    }
+
+    static private byte 
+    stringToMajor(String str)
+    {
+        int pos = str.indexOf('.');
+        if(pos == -1)
+        {
+            throw new Ice.VersionParseException("malformed version value `" + str + "'");
+        }
+            
+        String majStr = str.substring(0, pos);
+        int majVersion;
+        try
+        {
+            majVersion = Integer.parseInt(majStr);
+        }
+        catch(NumberFormatException ex)
+        {
+            throw new Ice.VersionParseException("invalid version value `" + str + "'");
+        }
+        
+        if(majVersion < 1 || majVersion > 255)
+        {
+            throw new Ice.VersionParseException("range error in version `" + str + "'");
+        }
+
+        return (byte)majVersion;
+    }
+
+    static private byte
+    stringToMinor(String str)
+    {
+        int pos = str.indexOf('.');
+        if(pos == -1)
+        {
+            throw new Ice.VersionParseException("malformed version value `" + str + "'");
+        }
+            
+        String minStr = str.substring(pos + 1, str.length());
+        int minVersion;
+        try
+        {
+            minVersion = Integer.parseInt(minStr);
+        }
+        catch(NumberFormatException ex)
+        {
+            throw new Ice.VersionParseException("invalid version value `" + str + "'");
+        }
+        
+        if(minVersion < 0 || minVersion > 255)
+        {
+            throw new Ice.VersionParseException("range error in version `" + str + "'");
+        }
+
+        return (byte)minVersion;
+    }
+
+    static private String 
+    majorMinorToString(byte major, byte minor)
+    {
+        StringBuilder str = new StringBuilder();
+        str.append(major < 0 ? (int)major + 255 : (int)major);
+        str.append(".");
+        str.append(minor < 0 ? (int)minor + 255 : (int)minor);
+        return str.toString();
+    }
+
+    public final static Ice.ProtocolVersion Protocol_1_0 = new Ice.ProtocolVersion((byte)1, (byte)0);
+
+    public final static Ice.EncodingVersion Encoding_1_0 = new Ice.EncodingVersion((byte)1, (byte)0);
+    public final static Ice.EncodingVersion Encoding_1_1 = new Ice.EncodingVersion((byte)1, (byte)1);
+
     private static String _localAddress = null;
     private static java.lang.Object _processLoggerMutex = new java.lang.Object();
     private static Logger _processLogger = null;
