@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -18,20 +18,20 @@ final class _AMD_Object_ice_invoke extends IceInternal.IncomingAsync implements
     }
 
     public void
-    ice_response(boolean ok, byte[] outParams)
+    ice_response(boolean ok, byte[] outEncaps)
     {
         if(__validateResponse(ok))
         {
             try
             {
-                __os().writeBlob(outParams);
+                __writeParamEncaps(outEncaps, ok);
             }
             catch(Ice.LocalException ex)
             {
                 __exception(ex);
                 return;
             }
-            __response(ok);
+            __response();
         }
     }
 }
