@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -33,12 +33,13 @@ import com.jgoodies.forms.layout.CellConstraints;
 import IceGrid.*;
 import IceGridGUI.*;
 
+@SuppressWarnings("unchecked")
 class AdapterEditor extends CommunicatorChildEditor
 {
     AdapterEditor()
     {
-        _objects = new ArrayMapField(this, true, "Identity", "Type", "Property");
-        _allocatables = new ArrayMapField(this, true, "Identity", "Type", "Property");
+        _objects = new ArrayMapField(this, true, "Identity", "Type", "Property", "Proxy Options");
+        _allocatables = new ArrayMapField(this, true, "Identity", "Type", "Property", "Proxy Options");
 
         //
         // Create buttons
@@ -151,6 +152,9 @@ class AdapterEditor extends CommunicatorChildEditor
             + " ssl -h venus.foo.com (accepts SSL connections instead of plain TCP)"
             + "</html>");
 
+        _proxyOptions.getDocument().addDocumentListener(_updateListener);
+        _proxyOptions.setToolTipText("<html>The proxy options used for proxies created by the object adapter.</html>");
+
         _description.getDocument().addDocumentListener(_updateListener);
         _description.setToolTipText("An optional description for this object adapter");
 
@@ -238,6 +242,10 @@ class AdapterEditor extends CommunicatorChildEditor
         builder.append(_publishedEndpoints, 3);
         builder.nextLine();
 
+        builder.append("Proxy Options" );
+        builder.append(_proxyOptions, 3);
+        builder.nextLine();
+
         builder.append("", _registerProcess);
         builder.nextLine();
         builder.append("", _serverLifetime);
@@ -287,6 +295,7 @@ class AdapterEditor extends CommunicatorChildEditor
         {
             adapter.removeProperty(_oldName + ".Endpoints");
             adapter.removeProperty(_oldName + ".PublishedEndpoints");
+            adapter.removeProperty(_oldName + ".ProxyOptions");
             _oldName = name;
         }
 
@@ -302,6 +311,11 @@ class AdapterEditor extends CommunicatorChildEditor
             adapter.setProperty(name + ".PublishedEndpoints", published.toString().trim());
         }
 
+        if(!_proxyOptions.getText().trim().isEmpty())
+        {
+            adapter.setProperty(name + ".ProxyOptions", _proxyOptions.getText().trim());
+        }
+
         //
         // Set all objects and allocatables properties
         //
@@ -516,6 +530,10 @@ class AdapterEditor extends CommunicatorChildEditor
         _publishedEndpoints.setEnabled(isEditable);
         _publishedEndpoints.setEditable(isEditable);
 
+        _proxyOptions.setEnabled(true);
+        _proxyOptions.setEditable(true);
+        _proxyOptions.setText(Utils.substitute(adapter.getProperty(oaPrefix + "ProxyOptions"), resolver));
+
         //
         // Objects
         //
@@ -549,7 +567,7 @@ class AdapterEditor extends CommunicatorChildEditor
         for(ObjectDescriptor p : objects)
         {
             String k = Ice.Util.identityToString(p.id);
-            result.put(k, new String[]{p.type, getAdapter().lookupPropertyValue(k)});
+            result.put(k, new String[]{p.type, getAdapter().lookupPropertyValue(k),p.proxyOptions});
         }
         return result;
     }
@@ -564,7 +582,7 @@ class AdapterEditor extends CommunicatorChildEditor
             {
                 Ice.Identity id = Ice.Util.stringToIdentity(p.getKey());
                 String[] val = p.getValue();
-                result.add(new ObjectDescriptor(id, val[0]));
+                result.add(new ObjectDescriptor(id, val[0], val[2]));
             }
             catch(Ice.IdentityParseException ex)
             {
@@ -611,6 +629,7 @@ class AdapterEditor extends CommunicatorChildEditor
 
     private JTextField _endpoints = new JTextField(20);
     private JComboBox _publishedEndpoints = new JComboBox(new Object[]{PUBLISH_ACTUAL});
+    private JTextField _proxyOptions = new JTextField(20);
 
     private JTextField _currentStatus = new JTextField(20);
     private JTextField _currentEndpoints = new JTextField(20);
