@@ -34,7 +34,7 @@ public class BareBonesBrowserLaunch {
                 Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
             else { //assume Unix or Linux
                 String[] browsers = {
-                    "firefox", "opera", "konqueror", "mozilla", "netscape" };
+                    "google-chrome", "firefox", "opera", "konqueror", "mozilla", "netscape" };
                 String browser = null;
                 for (int count = 0; count < browsers.length && browser == null; count++)
                     if (Runtime.getRuntime().exec(
