@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,7 +22,7 @@ public interface Tab
     boolean close();
 
     void save();
-    void saveToRegistry();
+    void saveToRegistry(boolean restart);
     void saveToFile();
     void discardUpdates();
 }
