@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,9 +12,13 @@ package IceInternal;
 public class ConnectionReaper
 {
     synchronized public void
-    add(Ice.ConnectionI connection)
+    add(Ice.ConnectionI connection, Ice.Instrumentation.Observer observer)
     {
         _connections.add(connection);
+        if(observer != null)
+        {
+            observer.detach();
+        }
     }
 
     synchronized public java.util.List<Ice.ConnectionI>
