@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -51,12 +51,24 @@ public final class DefaultsAndOverrides
             overrideConnectTimeoutValue = -1;
         }
 
+        value = properties.getProperty("Ice.Override.CloseTimeout");
+        if(value.length() > 0)
+        {
+            overrideCloseTimeout = true;
+            overrideCloseTimeoutValue = properties.getPropertyAsInt("Ice.Override.CloseTimeout");
+        }
+        else
+        {
+            overrideCloseTimeout = false;
+            overrideCloseTimeoutValue = -1;
+        }
+
         value = properties.getProperty("Ice.Override.Compress");
         if(value.length() > 0)
         {
             overrideCompress = true;
             boolean b = properties.getPropertyAsInt("Ice.Override.Compress") > 0;
-            if(!BasicStream.compressible() && b)
+            if(b && !BasicStream.compressible())
             {
                 System.err.println("warning: bzip2 support not available, Ice.Override.Compress ignored");
                 b = false;
@@ -65,7 +77,7 @@ public final class DefaultsAndOverrides
         }
         else
         {
-            overrideCompress = !BasicStream.compressible();
+            overrideCompress = false;
             overrideCompressValue = false;
         }
 
@@ -96,13 +108,21 @@ public final class DefaultsAndOverrides
         else
         {
             Ice.EndpointSelectionTypeParseException ex = new Ice.EndpointSelectionTypeParseException();
-            ex.str = value;
+            ex.str = "illegal value `" + value + "'; expected `Random' or `Ordered'";
             throw ex;
         }
 
         defaultLocatorCacheTimeout = properties.getPropertyAsIntWithDefault("Ice.Default.LocatorCacheTimeout", -1);
 
         defaultPreferSecure = properties.getPropertyAsIntWithDefault("Ice.Default.PreferSecure", 0) > 0;
+
+        value = properties.getPropertyWithDefault("Ice.Default.EncodingVersion", 
+                                                  Ice.Util.encodingVersionToString(Protocol.currentEncoding));
+        defaultEncoding = Ice.Util.stringToEncodingVersion(value);
+        Protocol.checkSupportedEncoding(defaultEncoding);        
+
+        boolean slicedFormat = properties.getPropertyAsIntWithDefault("Ice.Default.SlicedFormat", 0) > 0;
+        defaultFormat = slicedFormat ? Ice.FormatType.SlicedFormat : Ice.FormatType.CompactFormat;
     }
 
     final public String defaultHost;
@@ -111,11 +131,15 @@ public final class DefaultsAndOverrides
     final public Ice.EndpointSelectionType defaultEndpointSelection;
     final public int defaultLocatorCacheTimeout;
     final public boolean defaultPreferSecure;
+    final public Ice.EncodingVersion defaultEncoding;
+    final public Ice.FormatType defaultFormat;
 
     final public boolean overrideTimeout;
     final public int overrideTimeoutValue;
     final public boolean overrideConnectTimeout;
     final public int overrideConnectTimeoutValue;
+    final public boolean overrideCloseTimeout;
+    final public int overrideCloseTimeoutValue;
     final public boolean overrideCompress;
     final public boolean overrideCompressValue;
     final public boolean overrideSecure;
