@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -11,6 +11,28 @@ package IceInternal;
 
 public final class Instance
 {
+    private class ObserverUpdaterI implements Ice.Instrumentation.ObserverUpdater
+    {
+        ObserverUpdaterI(Instance instance)
+        {
+            _instance = instance;
+        }
+
+        @Override public void
+        updateConnectionObservers()
+        {
+            _instance.updateConnectionObservers();
+        }
+
+        @Override public void
+        updateThreadObservers()
+        {
+            _instance.updateThreadObservers();
+        }
+
+        final private Instance _instance;
+    }
+
     public Ice.InitializationData
     initializationData()
     {
@@ -135,17 +157,24 @@ public final class Instance
         return _objectAdapterFactory;
     }
 
-    public synchronized int
+    public int
     protocolSupport()
     {
-        if(_state == StateDestroyed)
-        {
-            throw new Ice.CommunicatorDestroyedException();
-        }
-
         return _protocolSupport;
     }
 
+    public boolean
+    preferIPv6()
+    {
+        return _preferIPv6;
+    }
+
+    public NetworkProxy
+    networkProxy()
+    {
+        return _networkProxy;
+    }
+
     public synchronized ThreadPool
     clientThreadPool()
     {
@@ -159,14 +188,14 @@ public final class Instance
     }
 
     public synchronized ThreadPool
-    serverThreadPool()
+    serverThreadPool(boolean create)
     {
         if(_state == StateDestroyed)
         {
             throw new Ice.CommunicatorDestroyedException();
         }
 
-        if(_serverThreadPool == null) // Lazy initialization.
+        if(_serverThreadPool == null && create) // Lazy initialization.
         {
             int timeout = _initData.properties.getPropertyAsInt("Ice.ServerIdleTime");
             _serverThreadPool = new ThreadPool(this, "Ice.ThreadPool.Server", timeout);
@@ -463,10 +492,10 @@ public final class Instance
         }
 
         Ice.Object result = null;
+
         if(_adminAdapter == null || (!_adminFacetFilter.isEmpty() && !_adminFacetFilter.contains(facet)))
         {
             result = _adminFacets.remove(facet);
-
             if(result == null)
             {
                 throw new Ice.NotRegisteredException("facet", facet);
@@ -476,9 +505,39 @@ public final class Instance
         {
             result = _adminAdapter.removeFacet(_adminIdentity, facet);
         }
+
         return result;
     }
 
+    public synchronized Ice.Object
+    findAdminFacet(String facet)
+    {
+        if(_state == StateDestroyed)
+        {
+            throw new Ice.CommunicatorDestroyedException();
+        }
+
+        Ice.Object result = null;
+
+        if(_adminAdapter == null || (!_adminFacetFilter.isEmpty() && !_adminFacetFilter.contains(facet)))
+        {
+            result = _adminFacets.get(facet);
+        }
+        else
+        {
+            result = _adminAdapter.findFacet(_adminIdentity, facet);
+        }
+
+        return result;
+    }
+
+    public Ice.Instrumentation.CommunicatorObserver
+    getObserver()
+    {
+        return _observer; // Immutable
+    }
+
+
     public synchronized void
     setDefaultLocator(Ice.LocatorPrx locator)
     {
@@ -513,7 +572,7 @@ public final class Instance
     public void
     setThreadHook(Ice.ThreadNotification threadHook)
     {
-        // 
+        //
         // No locking, as it can only be called during plug-in loading
         //
         _initData.threadHook = threadHook;
@@ -525,6 +584,37 @@ public final class Instance
         return Util.findClass(className, _initData.classLoader);
     }
 
+    public synchronized String
+    getClassForType(String type)
+    {
+        return _typeToClassMap.get(type);
+    }
+
+    public synchronized void
+    addClassForType(String type, String className)
+    {
+        if(_typeToClassMap.containsKey(type))
+        {
+            assert(_typeToClassMap.get(type).equals(className));
+        }
+        else
+        {
+            _typeToClassMap.put(type, className);
+        }
+    }
+
+    public String[]
+    getPackages()
+    {
+        return _packages;
+    }
+
+    public boolean
+    useApplicationClassLoader()
+    {
+        return _useApplicationClassLoader;
+    }
+
     //
     // Only for use by Ice.CommunicatorI
     //
@@ -600,7 +690,8 @@ public final class Instance
             if(_initData.logger == null)
             {
                 String logfile = _initData.properties.getProperty("Ice.LogFile");
-                if(_initData.properties.getPropertyAsInt("Ice.UseSyslog") > 0)
+                if(_initData.properties.getPropertyAsInt("Ice.UseSyslog") > 0 &&
+                   !System.getProperty("os.name").startsWith("Windows"))
                 {
                     if(logfile.length() != 0)
                     {
@@ -619,7 +710,9 @@ public final class Instance
                 }
             }
 
-            validatePackages();
+            _packages = validatePackages();
+
+            _useApplicationClassLoader = _initData.properties.getPropertyAsInt("Ice.UseApplicationClassLoader") > 0;
 
             _traceLevels = new TraceLevels(_initData.properties);
 
@@ -660,8 +753,17 @@ public final class Instance
 
             _proxyFactory = new ProxyFactory(this);
 
+            final String proxyHost = _initData.properties.getProperty("Ice.SOCKSProxyHost");
+            int defaultIPv6 = 1; // IPv6 enabled by default.
+            if(proxyHost.length() > 0)
+            {
+                final int proxyPort = _initData.properties.getPropertyAsIntWithDefault("Ice.SOCKSProxyPort", 1080);
+                _networkProxy = new SOCKSNetworkProxy(proxyHost, proxyPort);
+                defaultIPv6 = 0; // IPv6 is not supported with SOCKS
+            }
+
             boolean ipv4 = _initData.properties.getPropertyAsIntWithDefault("Ice.IPv4", 1) > 0;
-            boolean ipv6 = _initData.properties.getPropertyAsIntWithDefault("Ice.IPv6", 0) > 0;
+            boolean ipv6 = _initData.properties.getPropertyAsIntWithDefault("Ice.IPv6", defaultIPv6) > 0;
             if(!ipv4 && !ipv6)
             {
                 throw new Ice.InitializationException("Both IPV4 and IPv6 support cannot be disabled.");
@@ -678,15 +780,22 @@ public final class Instance
             {
                 _protocolSupport = Network.EnableIPv6;
             }
+            _preferIPv6 = _initData.properties.getPropertyAsInt("Ice.PreferIPv6Address") > 0;
+
+            if(ipv6 && _networkProxy instanceof SOCKSNetworkProxy)
+            {
+                throw new Ice.InitializationException("IPv6 is not supported with SOCKS4 proxies");
+            }
+
             _endpointFactoryManager = new EndpointFactoryManager(this);
             EndpointFactory tcpEndpointFactory = new TcpEndpointFactory(this);
             _endpointFactoryManager.add(tcpEndpointFactory);
             EndpointFactory udpEndpointFactory = new UdpEndpointFactory(this);
             _endpointFactoryManager.add(udpEndpointFactory);
 
-            _pluginManager = new Ice.PluginManagerI(communicator);
+            _pluginManager = new Ice.PluginManagerI(communicator, this);
 
-            _outgoingConnectionFactory = new OutgoingConnectionFactory(this);
+            _outgoingConnectionFactory = new OutgoingConnectionFactory(communicator, this);
 
             _servantFactoryManager = new ObjectFactoryManager();
 
@@ -703,9 +812,32 @@ public final class Instance
                 _adminFacetFilter.addAll(java.util.Arrays.asList(facetFilter));
             }
 
-            _adminFacets.put("Properties", new PropertiesAdminI(_initData.properties));
             _adminFacets.put("Process", new ProcessI(communicator));
 
+            MetricsAdminI admin = new MetricsAdminI(_initData.properties, _initData.logger);
+            _adminFacets.put("Metrics", admin);
+
+            PropertiesAdminI props = new PropertiesAdminI("Properties", _initData.properties, _initData.logger);
+            _adminFacets.put("Properties", props);
+
+            //
+            // Setup the communicator observer only if the user didn't already set an
+            // Ice observer resolver and if the admininistrative endpoints are set.
+            //
+            if((_adminFacetFilter.isEmpty() || _adminFacetFilter.contains("Metrics")) &&
+               _initData.properties.getProperty("Ice.Admin.Endpoints").length() > 0)
+            {
+                _observer = new CommunicatorObserverI(admin, _initData.observer);
+
+                //
+                // Make sure the admin plugin receives property updates.
+                //
+                props.addUpdateCallback(admin);
+            }
+            else 
+            {
+                _observer = _initData.observer;
+            }
         }
         catch(Ice.LocalException ex)
         {
@@ -718,24 +850,32 @@ public final class Instance
     finalize()
         throws Throwable
     {
-        IceUtilInternal.Assert.FinalizerAssert(_state == StateDestroyed);
-        IceUtilInternal.Assert.FinalizerAssert(_referenceFactory == null);
-        IceUtilInternal.Assert.FinalizerAssert(_proxyFactory == null);
-        IceUtilInternal.Assert.FinalizerAssert(_outgoingConnectionFactory == null);
-        IceUtilInternal.Assert.FinalizerAssert(_connectionMonitor == null);
-        IceUtilInternal.Assert.FinalizerAssert(_servantFactoryManager == null);
-        IceUtilInternal.Assert.FinalizerAssert(_objectAdapterFactory == null);
-        IceUtilInternal.Assert.FinalizerAssert(_clientThreadPool == null);
-        IceUtilInternal.Assert.FinalizerAssert(_serverThreadPool == null);
-        IceUtilInternal.Assert.FinalizerAssert(_endpointHostResolver == null);
-        IceUtilInternal.Assert.FinalizerAssert(_timer == null);
-        IceUtilInternal.Assert.FinalizerAssert(_routerManager == null);
-        IceUtilInternal.Assert.FinalizerAssert(_locatorManager == null);
-        IceUtilInternal.Assert.FinalizerAssert(_endpointFactoryManager == null);
-        IceUtilInternal.Assert.FinalizerAssert(_pluginManager == null);
-        IceUtilInternal.Assert.FinalizerAssert(_retryQueue == null);
-
-        super.finalize();
+        try
+        {
+            IceUtilInternal.Assert.FinalizerAssert(_state == StateDestroyed);
+            IceUtilInternal.Assert.FinalizerAssert(_referenceFactory == null);
+            IceUtilInternal.Assert.FinalizerAssert(_proxyFactory == null);
+            IceUtilInternal.Assert.FinalizerAssert(_outgoingConnectionFactory == null);
+            IceUtilInternal.Assert.FinalizerAssert(_connectionMonitor == null);
+            IceUtilInternal.Assert.FinalizerAssert(_servantFactoryManager == null);
+            IceUtilInternal.Assert.FinalizerAssert(_objectAdapterFactory == null);
+            IceUtilInternal.Assert.FinalizerAssert(_clientThreadPool == null);
+            IceUtilInternal.Assert.FinalizerAssert(_serverThreadPool == null);
+            IceUtilInternal.Assert.FinalizerAssert(_endpointHostResolver == null);
+            IceUtilInternal.Assert.FinalizerAssert(_timer == null);
+            IceUtilInternal.Assert.FinalizerAssert(_routerManager == null);
+            IceUtilInternal.Assert.FinalizerAssert(_locatorManager == null);
+            IceUtilInternal.Assert.FinalizerAssert(_endpointFactoryManager == null);
+            IceUtilInternal.Assert.FinalizerAssert(_pluginManager == null);
+            IceUtilInternal.Assert.FinalizerAssert(_retryQueue == null);
+        }
+        catch(java.lang.Exception ex)
+        {
+        }
+        finally
+        {
+            super.finalize();
+        }
     }
 
     public void
@@ -748,6 +888,14 @@ public final class Instance
         Ice.PluginManagerI pluginManagerImpl = (Ice.PluginManagerI)_pluginManager;
         pluginManagerImpl.loadPlugins(args);
 
+        //
+        // Set observer updater
+        //
+        if(_observer != null)
+        {
+            _observer.setObserverUpdater(new ObserverUpdaterI(this));
+        }
+
         //
         // Create threads.
         //
@@ -931,11 +1079,8 @@ public final class Instance
                 _servantFactoryManager = null;
             }
 
-            if(_referenceFactory != null)
-            {
-                _referenceFactory.destroy();
-                _referenceFactory = null;
-            }
+            //_referenceFactory.destroy(); // No destroy function defined.
+            _referenceFactory = null;
 
             // _proxyFactory.destroy(); // No destroy function defined.
             _proxyFactory = null;
@@ -967,6 +1112,8 @@ public final class Instance
             _adminAdapter = null;
             _adminFacets.clear();
 
+            _typeToClassMap.clear();
+
             _state = StateDestroyed;
         }
 
@@ -994,8 +1141,8 @@ public final class Instance
                 StringBuffer message = new StringBuffer("The following properties were set but never read:");
                 for(String p : unusedProperties)
                 {
-		    message.append("\n    ");
-		    message.append(p);
+                    message.append("\n    ");
+                    message.append(p);
                 }
                 _initData.logger.warning(message.toString());
             }
@@ -1003,10 +1150,51 @@ public final class Instance
     }
 
     private void
+    updateConnectionObservers()
+    {
+        try
+        {
+            assert(_outgoingConnectionFactory != null);
+            _outgoingConnectionFactory.updateConnectionObservers();
+            assert(_objectAdapterFactory != null);
+            _objectAdapterFactory.updateConnectionObservers();
+        }
+        catch(Ice.CommunicatorDestroyedException ex)
+        {
+        }
+    }
+
+    private void
+    updateThreadObservers()
+    {
+        try
+        {
+            if(_clientThreadPool != null)
+            {
+                _clientThreadPool.updateObservers();
+            }
+            if(_serverThreadPool != null)
+            {
+                _serverThreadPool.updateObservers();
+            }
+            assert(_objectAdapterFactory != null);
+            _objectAdapterFactory.updateThreadObservers();
+            if(_endpointHostResolver != null)
+            {
+                _endpointHostResolver.updateObserver();
+            }
+        }
+        catch(Ice.CommunicatorDestroyedException ex)
+        {
+        }
+    }
+
+    private String[]
     validatePackages()
     {
         final String prefix = "Ice.Package.";
         java.util.Map<String, String> map = _initData.properties.getPropertiesForPrefix(prefix);
+        java.util.List<String> packages = new java.util.ArrayList<String>();
         for(java.util.Map.Entry<String, String> p : map.entrySet())
         {
             String key = p.getKey();
@@ -1029,7 +1217,18 @@ public final class Instance
             {
                 _initData.logger.warning("unable to validate package: " + key + "=" + pkg);
             }
+            else
+            {
+                packages.add(pkg);
+            }
         }
+
+        String pkg = _initData.properties.getProperty("Ice.Default.Package");
+        if(pkg.length() > 0)
+        {
+            packages.add(pkg);
+        }
+        return packages.toArray(new String[packages.size()]);
     }
 
     private static final int StateActive = 0;
@@ -1045,6 +1244,7 @@ public final class Instance
     private final int _clientACM; // Immutable, not reset by destroy().
     private final int _serverACM; // Immutable, not reset by destroy().
     private final Ice.ImplicitContextI _implicitContext;
+    private final Ice.Instrumentation.CommunicatorObserver _observer;
     private RouterManager _routerManager;
     private LocatorManager _locatorManager;
     private ReferenceFactory _referenceFactory;
@@ -1054,6 +1254,8 @@ public final class Instance
     private ObjectFactoryManager _servantFactoryManager;
     private ObjectAdapterFactory _objectAdapterFactory;
     private int _protocolSupport;
+    private boolean _preferIPv6;
+    private NetworkProxy _networkProxy;
     private ThreadPool _clientThreadPool;
     private ThreadPool _serverThreadPool;
     private EndpointHostResolver _endpointHostResolver;
@@ -1067,5 +1269,9 @@ public final class Instance
     private java.util.Set<String> _adminFacetFilter = new java.util.HashSet<String>();
     private Ice.Identity _adminIdentity;
 
+    private java.util.Map<String, String> _typeToClassMap = new java.util.HashMap<String, String>();
+    final private String[] _packages;
+    final private boolean _useApplicationClassLoader;
+
     private static boolean _oneOffDone = false;
 }
