@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -27,6 +27,21 @@ public interface ProtocolPluginFacade
     //
     int getProtocolSupport();
 
+    //
+    // Get the protocol support.
+    //
+    boolean getPreferIPv6();
+
+    //
+    // Get the network proxy.
+    //
+    NetworkProxy getNetworkProxy();
+
+    //
+    // Get the default encoding to be used in endpoints.
+    //
+    Ice.EncodingVersion getDefaultEncoding();
+
     //
     // Get the default hostname to be used in endpoints.
     //
