@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,11 +14,11 @@ final class TcpEndpointI extends EndpointI
     public
     TcpEndpointI(Instance instance, String ho, int po, int ti, String conId, boolean co)
     {
+        super(conId);
         _instance = instance;
         _host = ho;
         _port = po;
         _timeout = ti;
-        _connectionId = conId;
         _compress = co;
         calcHashValue();
     }
@@ -26,6 +26,7 @@ final class TcpEndpointI extends EndpointI
     public
     TcpEndpointI(Instance instance, String str, boolean oaEndpoint)
     {
+        super("");
         _instance = instance;
         _host = null;
         _port = 0;
@@ -168,6 +169,7 @@ final class TcpEndpointI extends EndpointI
     public
     TcpEndpointI(BasicStream s)
     {
+        super("");
         _instance = s.instance();
         s.startReadEncaps();
         _host = s.readString();
@@ -270,6 +272,15 @@ final class TcpEndpointI extends EndpointI
         return Ice.TCPEndpointType.value;
     }
 
+    //
+    // Return the protocol name
+    //
+    public String
+    protocol()
+    {
+        return "tcp";
+    }
+
     //
     // Return the timeout for the endpoint in milliseconds. 0 means
     // non-blocking, -1 means no timeout.
@@ -379,15 +390,15 @@ final class TcpEndpointI extends EndpointI
     // is available.
     //
     public java.util.List<Connector>
-    connectors()
+    connectors(Ice.EndpointSelectionType selType)
     {
-        return connectors(Network.getAddresses(_host, _port, _instance.protocolSupport()));
+        return _instance.endpointHostResolver().resolve(_host, _port, selType, this);
     }
 
     public void
-    connectors_async(EndpointI_connectors callback)
+    connectors_async(Ice.EndpointSelectionType selType, EndpointI_connectors callback)
     {
-        _instance.endpointHostResolver().resolve(_host, _port, this, callback);
+        _instance.endpointHostResolver().resolve(_host, _port, selType, this, callback);
     }
 
     //
@@ -401,8 +412,7 @@ final class TcpEndpointI extends EndpointI
     acceptor(EndpointIHolder endpoint, String adapterName)
     {
         TcpAcceptor p = new TcpAcceptor(_instance, _host, _port);
-        endpoint.value =
-            new TcpEndpointI(_instance, _host, p.effectivePort(), _timeout, _connectionId, _compress);
+        endpoint.value = new TcpEndpointI(_instance, _host, p.effectivePort(), _timeout, _connectionId, _compress);
         return p;
     }
 
@@ -435,18 +445,25 @@ final class TcpEndpointI extends EndpointI
     public boolean
     equivalent(EndpointI endpoint)
     {
-        TcpEndpointI tcpEndpointI = null;
-        try
-        {
-            tcpEndpointI = (TcpEndpointI)endpoint;
-        }
-        catch(ClassCastException ex)
+        if(!(endpoint instanceof TcpEndpointI))
         {
             return false;
         }
+        TcpEndpointI tcpEndpointI = (TcpEndpointI)endpoint;
         return tcpEndpointI._host.equals(_host) && tcpEndpointI._port == _port;
     }
 
+    public java.util.List<Connector>
+    connectors(java.util.List<java.net.InetSocketAddress> addresses, NetworkProxy proxy)
+    {
+        java.util.List<Connector> connectors = new java.util.ArrayList<Connector>();
+        for(java.net.InetSocketAddress p : addresses)
+        {
+            connectors.add(new TcpConnector(_instance, p, proxy, _timeout, _connectionId));
+        }
+        return connectors;
+    }
+
     public int
     hashCode()
     {
@@ -456,38 +473,27 @@ final class TcpEndpointI extends EndpointI
     //
     // Compare endpoints for sorting purposes
     //
-    public boolean
-    equals(java.lang.Object obj)
-    {
-        try
-        {
-            return compareTo((EndpointI)obj) == 0;
-        }
-        catch(ClassCastException ee)
-        {
-            assert(false);
-            return false;
-        }
-    }
-
     public int
     compareTo(EndpointI obj) // From java.lang.Comparable
     {
-        TcpEndpointI p = null;
-
-        try
-        {
-            p = (TcpEndpointI)obj;
-        }
-        catch(ClassCastException ex)
+        if(!(obj instanceof TcpEndpointI))
         {
             return type() < obj.type() ? -1 : 1;
         }
 
+        TcpEndpointI p = (TcpEndpointI)obj;
         if(this == p)
         {
             return 0;
         }
+        else
+        {
+            int r = super.compareTo(p);
+            if(r != 0)
+            {
+                return r;
+            }
+        }
 
         if(_port < p._port)
         {
@@ -507,11 +513,6 @@ final class TcpEndpointI extends EndpointI
             return 1;
         }
 
-        if(!_connectionId.equals(p._connectionId))
-        {
-            return _connectionId.compareTo(p._connectionId);
-        }
-
         if(!_compress && p._compress)
         {
             return -1;
@@ -524,32 +525,23 @@ final class TcpEndpointI extends EndpointI
         return _host.compareTo(p._host);
     }
 
-    public java.util.List<Connector>
-    connectors(java.util.List<java.net.InetSocketAddress> addresses)
-    {
-        java.util.List<Connector> connectors = new java.util.ArrayList<Connector>();
-        for(java.net.InetSocketAddress p : addresses)
-        {
-            connectors.add(new TcpConnector(_instance, p, _timeout, _connectionId));
-        }
-        return connectors;
-    }
-
     private void
     calcHashValue()
     {
-        _hashCode = _host.hashCode();
-        _hashCode = 5 * _hashCode + _port;
-        _hashCode = 5 * _hashCode + _timeout;
-        _hashCode = 5 * _hashCode + _connectionId.hashCode();
-        _hashCode = 5 * _hashCode + (_compress ? 1 : 0);
+        int h = 5381;
+        h = IceInternal.HashUtil.hashAdd(h, Ice.TCPEndpointType.value);
+        h = IceInternal.HashUtil.hashAdd(h, _host);
+        h = IceInternal.HashUtil.hashAdd(h, _port);
+        h = IceInternal.HashUtil.hashAdd(h, _timeout);
+        h = IceInternal.HashUtil.hashAdd(h, _connectionId);
+        h = IceInternal.HashUtil.hashAdd(h, _compress);
+        _hashCode = h;
     }
 
     private Instance _instance;
     private String _host;
     private int _port;
     private int _timeout;
-    private String _connectionId = "";
     private boolean _compress;
     private int _hashCode;
 }
