@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -11,12 +11,13 @@ package IceInternal;
 
 public final class ThreadPoolCurrent
 {
-    ThreadPoolCurrent(Instance instance, ThreadPool threadPool)
+    ThreadPoolCurrent(Instance instance, ThreadPool threadPool, ThreadPool.EventHandlerThread thread)
     {
         operation = SocketOperation.None;
-        stream = new BasicStream(instance);
+        stream = new BasicStream(instance, Protocol.currentProtocolEncoding);
 
         _threadPool = threadPool;
+        _thread = thread;
         _ioCompleted = false;
         _leader = false;
     }
@@ -31,6 +32,7 @@ public final class ThreadPoolCurrent
     }
     
     final ThreadPool _threadPool;
+    final ThreadPool.EventHandlerThread _thread;
     EventHandler _handler;
     boolean _ioCompleted;
     boolean _leader;
