@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -11,7 +11,7 @@ package IceInternal;
 
 public interface UserExceptionFactory
 {
-    void createAndThrow()
+    void createAndThrow(String typeId)
         throws Ice.UserException;
 
     void destroy();
