@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -23,7 +23,7 @@ final class TransceiverI implements IceInternal.Transceiver
     }
 
     public int
-    initialize()
+    initialize(IceInternal.Buffer readBuffer, IceInternal.Buffer writeBuffer)
     {
         try
         {
@@ -32,32 +32,90 @@ final class TransceiverI implements IceInternal.Transceiver
                 _state = StateConnectPending;
                 return IceInternal.SocketOperation.Connect;
             }
-            else if(_state <= StateConnectPending)
+            else if(_state == StateConnectPending)
             {
                 IceInternal.Network.doFinishConnect(_fd);
-                _state = StateConnected;
-                _desc = IceInternal.Network.fdToString(_fd);
-            }
-            assert(_state == StateConnected);
+                _desc = IceInternal.Network.fdToString(_fd, _proxy, _addr);
 
-            int status = handshakeNonBlocking();
-            if(status != IceInternal.SocketOperation.None)
+                if(_proxy != null)
+                {
+                    //
+                    // Prepare the read & write buffers in advance.
+                    //
+                    _proxy.beginWriteConnectRequest(_addr, writeBuffer);
+                    _proxy.beginReadConnectRequestResponse(readBuffer);
+
+                    //
+                    // Write the proxy connection message.
+                    //
+                    if(writeRaw(writeBuffer))
+                    {
+                        //
+                        // Write completed without blocking.
+                        //
+                        _proxy.endWriteConnectRequest(writeBuffer);
+
+                        //
+                        // Try to read the response.
+                        //
+                        if(readRaw(readBuffer))
+                        {
+                            //
+                            // Read completed without blocking - fall through.
+                            //
+                            _proxy.endReadConnectRequestResponse(readBuffer);
+                        }
+                        else
+                        {
+                            //
+                            // Return SocketOperationRead to indicate we need to complete the read.
+                            //
+                            _state = StateProxyConnectRequestPending; // Wait for proxy response
+                            return IceInternal.SocketOperation.Read;
+                        }
+                    }
+                    else
+                    {
+                        //
+                        // Return SocketOperationWrite to indicate we need to complete the write.
+                        //
+                        _state = StateProxyConnectRequest; // Send proxy connect request
+                        return IceInternal.SocketOperation.Write;
+                    }
+                }
+
+                _state = StateConnected;
+            }
+            else if(_state == StateProxyConnectRequest)
             {
-                return status;
+                //
+                // Write completed.
+                //
+                _proxy.endWriteConnectRequest(writeBuffer);
+                _state = StateProxyConnectRequestPending; // Wait for proxy response
+                return IceInternal.SocketOperation.Read;
+            }
+            else if(_state == StateProxyConnectRequestPending)
+            {
+                //
+                // Read completed.
+                //
+                _proxy.endReadConnectRequestResponse(readBuffer);
+                _state = StateConnected;
+            }
+
+            if(_state == StateConnected)
+            {
+                return handshakeNonBlocking();
             }
         }
         catch(Ice.LocalException ex)
         {
             if(_instance.networkTraceLevel() >= 2)
             {
-                java.net.Socket fd = (java.net.Socket)_fd.socket();
                 StringBuilder s = new StringBuilder(128);
                 s.append("failed to establish ssl connection\n");
-                s.append("local address = ");
-                s.append(IceInternal.Network.addrToString(fd.getLocalAddress(), fd.getLocalPort()));
-                s.append("\nremote address = ");
-                assert(_connectAddr != null);
-                s.append(IceInternal.Network.addrToString(_connectAddr));
+                s.append(IceInternal.Network.fdToString(_fd, _proxy, _addr));
                 _logger.trace(_instance.networkTraceCategory(), s.toString());
             }
             throw ex;
@@ -145,6 +203,14 @@ final class TransceiverI implements IceInternal.Transceiver
     public boolean
     write(IceInternal.Buffer buf)
     {
+        if(_state == StateProxyConnectRequest)
+        {
+            //
+            // We need to write the proxy message, but we have to use TCP and not SSL.
+            //
+            return writeRaw(buf);
+        }
+
         //
         // If the handshake isn't completed yet, we shouldn't be writing.
         //
@@ -153,6 +219,16 @@ final class TransceiverI implements IceInternal.Transceiver
             throw new Ice.ConnectionLostException();
         }
 
+        //
+        // We don't want write to be called on android main thread as this will cause
+        // NetworkOnMainThreadException to be thrown. If that is the android main thread
+        // we return false and this method will be later called from the thread pool.
+        //
+        if(IceInternal.Util.isAndroidMainThread(Thread.currentThread()))
+        {
+            return false;
+        }
+
         int status = writeNonBlocking(buf.b);
         if(status != IceInternal.SocketOperation.None)
         {
@@ -162,9 +238,20 @@ final class TransceiverI implements IceInternal.Transceiver
         return true;
     }
 
+    @SuppressWarnings("deprecation")
     public boolean
     read(IceInternal.Buffer buf, Ice.BooleanHolder moreData)
     {
+        moreData.value = false;
+
+        if(_state == StateProxyConnectRequestPending)
+        {
+            //
+            // We need to read the proxy reply, but we have to use TCP and not SSL.
+            //
+            return readRaw(buf);
+        }
+
         //
         // If the handshake isn't completed yet, we shouldn't be reading (read can be
         // called by the thread pool when the connection is registered/unregistered
@@ -223,7 +310,6 @@ final class TransceiverI implements IceInternal.Transceiver
                     if(status != IceInternal.SocketOperation.None)
                     {
                         assert(status == IceInternal.SocketOperation.Read);
-                        moreData.value = false;
                         return false;
                     }
                     continue;
@@ -292,21 +378,35 @@ final class TransceiverI implements IceInternal.Transceiver
         }
     }
 
-    //
-    // Only for use by ConnectorI, AcceptorI.
-    //
     TransceiverI(Instance instance, javax.net.ssl.SSLEngine engine, java.nio.channels.SocketChannel fd,
-                 String host, boolean connected, boolean incoming, String adapterName,
-                 java.net.InetSocketAddress connectAddr)
+                 IceInternal.NetworkProxy proxy, String host, java.net.InetSocketAddress addr)
+    {
+        init(instance, engine, fd);
+        _proxy = proxy;
+        _host = host;
+        _incoming = false;
+        _addr = addr;
+        _state = StateNeedConnect;
+        _desc = IceInternal.Network.fdToString(_fd, _proxy, _addr);
+    }
+
+    TransceiverI(Instance instance, javax.net.ssl.SSLEngine engine, java.nio.channels.SocketChannel fd,
+                 String adapterName)
+    {
+        init(instance, engine, fd);
+        _host = "";
+        _adapterName = adapterName;
+        _incoming = true;
+        _state = StateConnected;
+        _desc = IceInternal.Network.fdToString(_fd);
+    }
+
+    @SuppressWarnings("deprecation")
+    private void init(Instance instance, javax.net.ssl.SSLEngine engine, java.nio.channels.SocketChannel fd)
     {
         _instance = instance;
         _engine = engine;
         _fd = fd;
-        _host = host;
-        _incoming = incoming;
-        _adapterName = adapterName;
-        _connectAddr = connectAddr;
-        _state = connected ? StateConnected : StateNeedConnect;
         _logger = instance.communicator().getLogger();
         try
         {
@@ -316,7 +416,6 @@ final class TransceiverI implements IceInternal.Transceiver
         {
             // Ignore.
         }
-        _desc = IceInternal.Network.fdToString(_fd);
         _maxPacketSize = 0;
         if(System.getProperty("os.name").startsWith("Windows"))
         {
@@ -341,9 +440,17 @@ final class TransceiverI implements IceInternal.Transceiver
     finalize()
         throws Throwable
     {
-        IceUtilInternal.Assert.FinalizerAssert(_fd == null);
-
-        super.finalize();
+        try
+        {
+            IceUtilInternal.Assert.FinalizerAssert(_fd == null);
+        }
+        catch(java.lang.Exception ex)
+        {
+        }
+        finally
+        {
+            super.finalize();
+        }
     }
 
     private NativeConnectionInfo
@@ -352,56 +459,48 @@ final class TransceiverI implements IceInternal.Transceiver
         //
         // This can only be called on an open transceiver.
         //
-        assert(_fd != null);
-
         NativeConnectionInfo info = new NativeConnectionInfo();
-        java.net.Socket socket = _fd.socket();
-        if(socket.getLocalAddress() != null)
-        {
-            info.localAddress = socket.getLocalAddress().getHostAddress();
-            info.localPort = socket.getLocalPort();
-        }
-        else
+        if(_fd != null)
         {
+            java.net.Socket socket = _fd.socket();
             //
             // On some platforms (e.g., early Android releases), sockets don't
             // correctly return address information.
             //
-            info.localAddress = "";
-            info.localPort = -1;
-        }
-
-        if(socket.getInetAddress() != null)
-        {
-            info.remoteAddress = socket.getInetAddress().getHostAddress();
-            info.remotePort = socket.getPort();
-        }
-        else
-        {
-            info.remoteAddress = "";
-            info.remotePort = -1;
-        }
-        SSLSession session = _engine.getSession();
-        info.cipher = session.getCipherSuite();
-        try
-        {
-            java.util.ArrayList<String> certs = new java.util.ArrayList<String>();
-            info.nativeCerts = session.getPeerCertificates();
-            for(java.security.cert.Certificate c : info.nativeCerts)
+            if(socket.getLocalAddress() != null)
             {
-                StringBuffer s = new StringBuffer("-----BEGIN CERTIFICATE-----\n");
-                s.append(IceUtilInternal.Base64.encode(c.getEncoded()));
-                s.append("\n-----END CERTIFICATE-----");
-                certs.add(s.toString());
+                info.localAddress = socket.getLocalAddress().getHostAddress();
+                info.localPort = socket.getLocalPort();
+            }
+            
+            if(socket.getInetAddress() != null)
+            {
+                info.remoteAddress = socket.getInetAddress().getHostAddress();
+                info.remotePort = socket.getPort();
+            }
+
+            SSLSession session = _engine.getSession();
+            info.cipher = session.getCipherSuite();
+            try
+            {
+                java.util.ArrayList<String> certs = new java.util.ArrayList<String>();
+                info.nativeCerts = session.getPeerCertificates();
+                for(java.security.cert.Certificate c : info.nativeCerts)
+                {
+                    StringBuffer s = new StringBuffer("-----BEGIN CERTIFICATE-----\n");
+                    s.append(IceUtilInternal.Base64.encode(c.getEncoded()));
+                    s.append("\n-----END CERTIFICATE-----");
+                    certs.add(s.toString());
+                }
+                info.certs = certs.toArray(new String[0]);
+            }
+            catch(java.security.cert.CertificateEncodingException ex)
+            {
+            }
+            catch(javax.net.ssl.SSLPeerUnverifiedException ex)
+            {
+                // No peer certificates.
             }
-            info.certs = certs.toArray(new String[0]);
-        }
-        catch(java.security.cert.CertificateEncodingException ex)
-        {
-        }
-        catch(javax.net.ssl.SSLPeerUnverifiedException ex)
-        {
-            // No peer certificates.
         }
         info.adapterName = _adapterName;
         info.incoming = _incoming;
@@ -569,6 +668,7 @@ final class TransceiverI implements IceInternal.Transceiver
         }
     }
 
+    @SuppressWarnings("deprecation")
     private int
     writeNonBlocking(ByteBuffer buf)
     {
@@ -794,15 +894,126 @@ final class TransceiverI implements IceInternal.Transceiver
         _appInput.compact();
     }
 
+    @SuppressWarnings("deprecation")
+    private boolean
+    writeRaw(IceInternal.Buffer buf)
+    {
+        //
+        // We don't want write to be called on android main thread as this will cause
+        // NetworkOnMainThreadException to be thrown. If that is the android main thread
+        // we return false and this method will be later called from the thread pool.
+        //
+        if(IceInternal.Util.isAndroidMainThread(Thread.currentThread()))
+        {
+            return false;
+        }
+
+        final int size = buf.b.limit();
+        int packetSize = size - buf.b.position();
+
+        while(buf.b.hasRemaining())
+        {
+            try
+            {
+                assert(_fd != null);
+                int ret = _fd.write(buf.b);
+
+                if(ret == -1)
+                {
+                    throw new Ice.ConnectionLostException();
+                }
+                else if(ret == 0)
+                {
+                    return false;
+                }
+
+                if(_instance.networkTraceLevel() >= 3)
+                {
+                    String s = "sent " + ret + " of " + packetSize + " bytes via tcp\n" + toString();
+                    _logger.trace(_instance.networkTraceCategory(), s);
+                }
+
+                if(_stats != null)
+                {
+                    _stats.bytesSent(type(), ret);
+                }
+            }
+            catch(java.io.InterruptedIOException ex)
+            {
+                continue;
+            }
+            catch(java.io.IOException ex)
+            {
+                throw new Ice.SocketException(ex);
+            }
+        }
+        return true;
+    }
+
+    @SuppressWarnings("deprecation")
+    private boolean
+    readRaw(IceInternal.Buffer buf)
+    {
+        int packetSize = buf.b.remaining();
+
+        while(buf.b.hasRemaining())
+        {
+            try
+            {
+                assert(_fd != null);
+                int ret = _fd.read(buf.b);
+
+                if(ret == -1)
+                {
+                    throw new Ice.ConnectionLostException();
+                }
+
+                if(ret == 0)
+                {
+                    return false;
+                }
+
+                if(ret > 0)
+                {
+                    if(_instance.networkTraceLevel() >= 3)
+                    {
+                        String s = "received " + ret + " of " + packetSize + " bytes via tcp\n" + toString();
+                        _logger.trace(_instance.networkTraceCategory(), s);
+                    }
+
+                    if(_stats != null)
+                    {
+                        _stats.bytesReceived(type(), ret);
+                    }
+                }
+
+                packetSize = buf.b.remaining();
+            }
+            catch(java.io.InterruptedIOException ex)
+            {
+                continue;
+            }
+            catch(java.io.IOException ex)
+            {
+                throw new Ice.ConnectionLostException(ex);
+            }
+        }
+
+        return true;
+    }
+
     private Instance _instance;
     private java.nio.channels.SocketChannel _fd;
     private javax.net.ssl.SSLEngine _engine;
+    private IceInternal.NetworkProxy _proxy;
     private String _host;
     private boolean _incoming;
     private String _adapterName;
-    private java.net.InetSocketAddress _connectAddr;
+    private java.net.InetSocketAddress _addr;
     private int _state;
     private Ice.Logger _logger;
+
+    @SuppressWarnings("deprecation")
     private Ice.Stats _stats;
     private String _desc;
     private int _maxPacketSize;
@@ -813,6 +1024,8 @@ final class TransceiverI implements IceInternal.Transceiver
 
     private static final int StateNeedConnect = 0;
     private static final int StateConnectPending = 1;
-    private static final int StateConnected = 2;
-    private static final int StateHandshakeComplete = 3;
+    private static final int StateProxyConnectRequest = 2;
+    private static final int StateProxyConnectRequestPending = 3;
+    private static final int StateConnected = 4;
+    private static final int StateHandshakeComplete = 5;
 }
