@@ -1,12 +1,14 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+package test.Ice.background;
+
 class Acceptor implements IceInternal.Acceptor
 {
     public java.nio.channels.ServerSocketChannel
@@ -30,7 +32,7 @@ class Acceptor implements IceInternal.Acceptor
     public IceInternal.Transceiver
     accept()
     {
-        return new Transceiver(_acceptor.accept());
+        return new Transceiver(_configuration, _acceptor.accept());
     }
 
     public String
@@ -39,17 +41,12 @@ class Acceptor implements IceInternal.Acceptor
         return _acceptor.toString();
     }
 
-    Acceptor(IceInternal.Acceptor acceptor)
+    Acceptor(Configuration configuration, IceInternal.Acceptor acceptor)
     {
+        _configuration = configuration;
         _acceptor = acceptor;
     }
 
-    protected synchronized void
-    finalize()
-        throws Throwable
-    {
-        super.finalize();
-    }
-
     final private IceInternal.Acceptor _acceptor;
+    private Configuration _configuration;
 }
