@@ -1,12 +1,14 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
+package test.Ice.background;
+
 final class Transceiver implements IceInternal.Transceiver
 {
     public java.nio.channels.SelectableChannel
@@ -15,24 +17,24 @@ final class Transceiver implements IceInternal.Transceiver
         return _transceiver.fd();
     }
 
-    public IceInternal.SocketStatus
-    initialize()
+    public int
+    initialize(IceInternal.Buffer readBuffer, IceInternal.Buffer writeBuffer)
     {
-        IceInternal.SocketStatus status = _configuration.initializeSocketStatus();
-        if(status == IceInternal.SocketStatus.NeedConnect || status == IceInternal.SocketStatus.NeedWrite)
+        int status = _configuration.initializeSocketStatus();
+        if(status == IceInternal.SocketOperation.Connect || status == IceInternal.SocketOperation.Write)
         {
             if(!_initialized)
             {
-                status = _transceiver.initialize();
-                if(status != IceInternal.SocketStatus.Finished)
+                status = _transceiver.initialize(readBuffer, writeBuffer);
+                if(status != IceInternal.SocketOperation.None)
                 {
                     return status;
                 }
                 _initialized = true;
             }
-            return IceInternal.SocketStatus.NeedWrite;
+            return IceInternal.SocketOperation.Write;
         }
-        else if(status == IceInternal.SocketStatus.NeedRead)
+        else if(status == IceInternal.SocketOperation.Read)
         {
             return status;
         }
@@ -40,14 +42,14 @@ final class Transceiver implements IceInternal.Transceiver
         _configuration.checkInitializeException();
         if(!_initialized)
         {
-            status = _transceiver.initialize();
-            if(status != IceInternal.SocketStatus.Finished)
+            status = _transceiver.initialize(readBuffer, writeBuffer);
+            if(status != IceInternal.SocketOperation.None)
             {
                 return status;
             }
             _initialized = true;
         }
-        return IceInternal.SocketStatus.Finished;
+        return IceInternal.SocketOperation.None;
     }
 
     public void
@@ -59,14 +61,9 @@ final class Transceiver implements IceInternal.Transceiver
     public boolean
     write(IceInternal.Buffer buf)
     {
-        if(!_initialized)
-        {
-            throw new Ice.SocketException();
-        }
-
         if(!_configuration.writeReady())
         {
-                return false;
+            return false;
         }
         _configuration.checkWriteException();
         return _transceiver.write(buf);
@@ -75,11 +72,6 @@ final class Transceiver implements IceInternal.Transceiver
     public boolean
     read(IceInternal.Buffer buf, Ice.BooleanHolder moreData)
     {
-        if(!_initialized)
-        {
-            throw new Ice.SocketException();
-        }
-
         if(!moreData.value)
         {
             if(!_configuration.readReady())
@@ -103,6 +95,12 @@ final class Transceiver implements IceInternal.Transceiver
         return _transceiver.toString();
     }
 
+    public Ice.ConnectionInfo
+    getInfo()
+    {
+        return _transceiver.getInfo();
+    }
+
     public void
     checkSendSize(IceInternal.Buffer buf, int messageSizeMax)
     {
@@ -112,17 +110,10 @@ final class Transceiver implements IceInternal.Transceiver
     //
     // Only for use by Connector, Acceptor
     //
-    Transceiver(IceInternal.Transceiver transceiver)
+    Transceiver(Configuration configuration, IceInternal.Transceiver transceiver)
     {
         _transceiver = transceiver;
-        _configuration = Configuration.getInstance();
-    }
-
-    protected synchronized void
-    finalize()
-        throws Throwable
-    {
-        super.finalize();
+        _configuration = configuration;
     }
 
     final private IceInternal.Transceiver _transceiver;
