@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -102,7 +102,7 @@ public class AllTests
         String ref = "communicator:default -p 12010";
         RemoteCommunicatorPrx com = RemoteCommunicatorPrxHelper.uncheckedCast(communicator.stringToProxy(ref));
 
-		out.print("testing binding with single endpoint... ");
+                out.print("testing binding with single endpoint... ");
         out.flush();
         {
             RemoteObjectAdapterPrx adapter = com.createObjectAdapter("Adapter", "default");
@@ -843,6 +843,135 @@ public class AllTests
             out.println("ok");
         }
 
+        {
+            out.print("testing ipv4 & ipv6 connections... ");
+            out.flush();
+
+            Ice.Properties ipv4 = Ice.Util.createProperties();
+            ipv4.setProperty("Ice.IPv4", "1");
+            ipv4.setProperty("Ice.IPv6", "0");
+            ipv4.setProperty("Adapter.Endpoints", "tcp -h localhost");
+
+            Ice.Properties ipv6 = Ice.Util.createProperties();
+            ipv6.setProperty("Ice.IPv4", "0");
+            ipv6.setProperty("Ice.IPv6", "1");
+            ipv6.setProperty("Adapter.Endpoints", "tcp -h localhost");
+
+            Ice.Properties bothPreferIPv4 = Ice.Util.createProperties();
+            bothPreferIPv4.setProperty("Ice.IPv4", "1");
+            bothPreferIPv4.setProperty("Ice.IPv6", "1");
+            bothPreferIPv4.setProperty("Ice.PreferIPv6Address", "0");
+            bothPreferIPv4.setProperty("Adapter.Endpoints", "tcp -h localhost");
+
+            Ice.Properties bothPreferIPv6 = Ice.Util.createProperties();
+            bothPreferIPv6.setProperty("Ice.IPv4", "1");
+            bothPreferIPv6.setProperty("Ice.IPv6", "1");
+            bothPreferIPv6.setProperty("Ice.PreferIPv6Address", "1");
+            bothPreferIPv6.setProperty("Adapter.Endpoints", "tcp -h localhost");
+
+            java.util.List<Ice.Properties> clientProps = new java.util.ArrayList<Ice.Properties>();
+            clientProps.add(ipv4);
+            clientProps.add(ipv6);
+            clientProps.add(bothPreferIPv4);
+            clientProps.add(bothPreferIPv6);
+
+            Ice.Properties anyipv4 = ipv4._clone();
+            anyipv4.setProperty("Adapter.Endpoints", "tcp -p 12012");
+            anyipv4.setProperty("Adapter.PublishedEndpoints", "tcp -h 127.0.0.1 -p 12012");
+
+            Ice.Properties anyipv6 = ipv6._clone();
+            anyipv6.setProperty("Adapter.Endpoints", "tcp -p 12012");
+            anyipv6.setProperty("Adapter.PublishedEndpoints", "tcp -h \".1\" -p 12012");
+
+            Ice.Properties anyboth = Ice.Util.createProperties();
+            anyboth.setProperty("Ice.IPv4", "1");
+            anyboth.setProperty("Ice.IPv6", "1");
+            anyboth.setProperty("Adapter.Endpoints", "tcp -p 12012");
+            anyboth.setProperty("Adapter.PublishedEndpoints", "tcp -h \"::1\" -p 12012:tcp -h 127.0.0.1 -p 12012");
+
+            Ice.Properties localipv4 = ipv4._clone();
+            localipv4.setProperty("Adapter.Endpoints", "tcp -h 127.0.0.1");
+
+            Ice.Properties localipv6 = ipv6._clone();
+            localipv6.setProperty("Adapter.Endpoints", "tcp -h \"::1\"");
+
+            java.util.List<Ice.Properties> serverProps = new java.util.ArrayList<Ice.Properties>(clientProps);
+            serverProps.add(anyipv4);
+            serverProps.add(anyipv6);
+            serverProps.add(anyboth);
+            serverProps.add(localipv4);
+            serverProps.add(localipv6);
+
+            boolean ipv6NotSupported = false;
+            for(Ice.Properties p : serverProps)
+            {
+                Ice.InitializationData serverInitData = new Ice.InitializationData();
+                serverInitData.properties = p;
+                Ice.Communicator serverCommunicator = Ice.Util.initialize(serverInitData);
+                Ice.ObjectAdapter oa;
+                try
+                {
+                    oa = serverCommunicator.createObjectAdapter("Adapter");
+                    oa.activate();
+                }
+                catch(Ice.DNSException ex)
+                {
+                    serverCommunicator.destroy();
+                    continue; // IP version not supported.
+                }
+                catch(Ice.SocketException ex)
+                {
+                    if(p == ipv6)
+                    {
+                        ipv6NotSupported = true;
+                    }
+                    serverCommunicator.destroy();
+                    continue; // IP version not supported.
+                }
+
+                String strPrx = oa.createProxy(serverCommunicator.stringToIdentity("dummy")).toString();
+                for(Ice.Properties q : clientProps)
+                {
+                    Ice.InitializationData clientInitData = new Ice.InitializationData();
+                    clientInitData.properties = q;
+                    Ice.Communicator clientCommunicator = Ice.Util.initialize(clientInitData);
+                    Ice.ObjectPrx prx = clientCommunicator.stringToProxy(strPrx);
+                    try
+                    {
+                        prx.ice_ping();
+                        test(false);
+                    }
+                    catch(Ice.ObjectNotExistException ex)
+                    {
+                        // Expected, no object registered.
+                    }
+                    catch(Ice.DNSException ex)
+                    {
+                        // Expected if no IPv4 or IPv6 address is
+                        // associated to localhost or if trying to connect
+                        // to an any endpoint with the wrong IP version,
+                        // e.g.: resolving an IPv4 address when only IPv6
+                        // is enabled fails with a DNS exception.
+                    }
+                    catch(Ice.SocketException ex)
+                    {
+                        test((p == ipv4 && q == ipv6) || (p == ipv6 && q == ipv4) ||
+                             (p == bothPreferIPv4 && q == ipv6) || (p == bothPreferIPv6 && q == ipv4) ||
+                             (p == bothPreferIPv6 && q == ipv6 && ipv6NotSupported) ||
+                             (p == anyipv4 && q == ipv6) || (p == anyipv6 && q == ipv4) ||
+                             (p == localipv4 && q == ipv6) || (p == localipv6 && q == ipv4) ||
+                             (p == ipv6 && q == bothPreferIPv4) || (p == bothPreferIPv6 && q == ipv6) ||
+                             (p == ipv6 && q == bothPreferIPv4) || (p == ipv6 && q == bothPreferIPv6) || 
+                             (p == bothPreferIPv6 && q == ipv6));
+                    }
+                    clientCommunicator.destroy();
+                }
+                serverCommunicator.destroy();
+            }
+            
+            out.println("ok");
+        }
+
         com.shutdown();
     }
 }
