@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -21,6 +21,7 @@ import test.Ice.exceptions.Test.AMI_Thrower_throwCasA;
 import test.Ice.exceptions.Test.AMI_Thrower_throwCasB;
 import test.Ice.exceptions.Test.AMI_Thrower_throwCasC;
 import test.Ice.exceptions.Test.AMI_Thrower_throwLocalException;
+import test.Ice.exceptions.Test.AMI_Thrower_throwLocalExceptionIdempotent;
 import test.Ice.exceptions.Test.AMI_Thrower_throwNonIceException;
 import test.Ice.exceptions.Test.AMI_Thrower_throwUndeclaredA;
 import test.Ice.exceptions.Test.AMI_Thrower_throwUndeclaredB;
@@ -42,6 +43,7 @@ import test.Ice.exceptions.Test.Callback_Thrower_throwCasA;
 import test.Ice.exceptions.Test.Callback_Thrower_throwCasB;
 import test.Ice.exceptions.Test.Callback_Thrower_throwCasC;
 import test.Ice.exceptions.Test.Callback_Thrower_throwLocalException;
+import test.Ice.exceptions.Test.Callback_Thrower_throwLocalExceptionIdempotent;
 import test.Ice.exceptions.Test.Callback_Thrower_throwNonIceException;
 import test.Ice.exceptions.Test.Callback_Thrower_throwUndeclaredA;
 import test.Ice.exceptions.Test.Callback_Thrower_throwUndeclaredB;
@@ -1245,6 +1247,9 @@ public class AllTests
             catch(Ice.UnknownLocalException ex)
             {
             }
+            catch(Ice.OperationNotExistException ex)
+            {
+            }
             catch(Throwable ex)
             {
                 ex.printStackTrace();
@@ -1734,6 +1739,57 @@ public class AllTests
             out.println("ok");
         }
 
+        out.print("testing memory limit marshal exception...");
+        out.flush();
+        {
+            try
+            {
+                thrower.throwMemoryLimitException(null);
+                test(collocated);
+            }
+            catch(Ice.UnknownLocalException ex)
+            {
+            }
+            catch(Throwable ex)
+            {
+                ex.printStackTrace();
+                test(false);
+            }
+
+            try
+            {
+                thrower.throwMemoryLimitException(new byte[20 * 1024]); // 20KB
+                test(collocated);
+            }
+            catch(Ice.MemoryLimitException ex)
+            {
+            }
+            catch(Throwable ex)
+            {
+                ex.printStackTrace();
+                test(false);
+            }
+            
+            if(!collocated)
+            {
+                try
+                {
+                    thrower.end_throwMemoryLimitException(
+                        thrower.begin_throwMemoryLimitException(new byte[20 * 1024])); // 20KB
+                    test(false);
+                }
+                catch(Ice.MemoryLimitException ex)
+                {
+                }
+                catch(Throwable ex)
+                {
+                    ex.printStackTrace();
+                    test(false);
+                }
+            }
+        }
+        out.println("ok");
+
         out.print("catching object not exist exception... ");
         out.flush();
 
@@ -1820,6 +1876,23 @@ public class AllTests
             test(false);
         }
 
+        try
+        {
+            thrower.throwLocalExceptionIdempotent();
+            test(false);
+        }
+        catch(Ice.UnknownLocalException ex)
+        {
+        }
+        catch(Ice.OperationNotExistException ex)
+        {
+        }
+        catch(Throwable ex)
+        {
+            ex.printStackTrace();
+            test(false);
+        }
+
         out.println("ok");
 
         out.print("catching unknown non-Ice exception... ");
@@ -2179,6 +2252,25 @@ public class AllTests
                 cb.check();
             }
 
+            {
+                final Callback_Thrower_throwLocalExceptionI cb = new Callback_Thrower_throwLocalExceptionI();
+                thrower.begin_throwLocalExceptionIdempotent(new Callback_Thrower_throwLocalExceptionIdempotent()
+                    {
+                        @Override
+                        public void response()
+                        {
+                            cb.response();
+                        }
+
+                        @Override
+                        public void exception(Ice.LocalException exc)
+                        {
+                            cb.exception(exc);
+                        }
+                    });
+                cb.check();
+            }
+
             out.println("ok");
 
             out.print("catching unknown non-Ice exception with new AMI mapping... ");
