@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -9,25 +9,7 @@
 
 package test.Ice.slicing.exceptions;
 
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_baseAsBase;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_knownDerivedAsBase;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_knownDerivedAsKnownDerived;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_knownIntermediateAsBase;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_knownIntermediateAsKnownIntermediate;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_knownMostDerivedAsBase;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_knownMostDerivedAsKnownIntermediate;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_knownMostDerivedAsKnownMostDerived;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_unknownDerivedAsBase;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_unknownIntermediateAsBase;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_unknownMostDerived1AsBase;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_unknownMostDerived1AsKnownIntermediate;
-import test.Ice.slicing.exceptions.Test.Callback_TestIntf_unknownMostDerived2AsBase;
-import test.Ice.slicing.exceptions.Test.Base;
-import test.Ice.slicing.exceptions.Test.KnownDerived;
-import test.Ice.slicing.exceptions.Test.KnownIntermediate;
-import test.Ice.slicing.exceptions.Test.KnownMostDerived;
-import test.Ice.slicing.exceptions.Test.TestIntfPrx;
-import test.Ice.slicing.exceptions.Test.TestIntfPrxHelper;
+import test.Ice.slicing.exceptions.client.Test.*;
 
 public class AllTests
 {
@@ -641,10 +623,57 @@ public class AllTests
         private Callback callback = new Callback();
     }
 
+    private static class RelayI extends _RelayDisp
+    {
+        public void knownPreservedAsBase(Ice.Current current)
+            throws Base
+        {
+            KnownPreservedDerived ex = new KnownPreservedDerived();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            throw ex;
+        }
+
+        public void knownPreservedAsKnownPreserved(Ice.Current current)
+            throws KnownPreserved
+        {
+            KnownPreservedDerived ex = new KnownPreservedDerived();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            throw ex;
+        }
+
+        public void unknownPreservedAsBase(Ice.Current current)
+            throws Base
+        {
+            Preserved2 ex = new Preserved2();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            ex.p1 = new PreservedClass("bc", "pc");
+            ex.p2 = ex.p1;
+            throw ex;
+        }
+
+        public void unknownPreservedAsKnownPreserved(Ice.Current current)
+            throws KnownPreserved
+        {
+            Preserved2 ex = new Preserved2();
+            ex.b = "base";
+            ex.kp = "preserved";
+            ex.kpd = "derived";
+            ex.p1 = new PreservedClass("bc", "pc");
+            ex.p2 = ex.p1;
+            throw ex;
+        }
+    }
+
     public static TestIntfPrx
     allTests(Ice.Communicator communicator, boolean collocated, java.io.PrintWriter out)
     {
-		out.print("testing stringToProxy... ");
+                out.print("testing stringToProxy... ");
         out.flush();
         String ref = "Test:default -p 12010 -t 10000";
         Ice.ObjectPrx base = communicator.stringToProxy(ref);
@@ -1051,6 +1080,156 @@ public class AllTests
         }
         out.println("ok");
 
+        out.print("unknown most derived in compact format... ");
+        out.flush();
+        {
+            try
+            {
+                test.unknownMostDerived2AsBaseCompact();
+                test(false);
+            }
+            catch(Base ex)
+            {
+                //
+                // For the 1.0 encoding, the unknown exception is sliced to Base.
+                //
+                test(test.ice_getEncodingVersion().equals(Ice.Util.Encoding_1_0));
+            }
+            catch(Ice.UnknownUserException ex)
+            {
+                //
+                // An UnknownUserException is raised for the compact format because the
+                // most-derived type is unknown and the exception cannot be sliced.
+                //
+                test(!test.ice_getEncodingVersion().equals(Ice.Util.Encoding_1_0));
+            }
+            catch(Ice.OperationNotExistException ex)
+            {
+            }
+            catch(Exception ex)
+            {
+                test(false);
+            }
+        }
+        out.println("ok");
+
+        out.print("preserved exceptions... ");
+        out.flush();
+        {
+            Ice.ObjectAdapter adapter = communicator.createObjectAdapterWithEndpoints("Relay", "default");
+            RelayPrx relay = RelayPrxHelper.uncheckedCast(adapter.addWithUUID(new RelayI()));
+            adapter.activate();
+
+            try
+            {
+                test.relayKnownPreservedAsBase(relay);
+                test(false);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                test(ex.b.equals("base"));
+                test(ex.kp.equals("preserved"));
+                test(ex.kpd.equals("derived"));
+            }
+            catch(Ice.OperationNotExistException ex)
+            {
+            }
+            catch(Exception ex)
+            {
+                test(false);
+            }
+
+            try
+            {
+                test.relayKnownPreservedAsKnownPreserved(relay);
+                test(false);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                test(ex.b.equals("base"));
+                test(ex.kp.equals("preserved"));
+                test(ex.kpd.equals("derived"));
+            }
+            catch(Ice.OperationNotExistException ex)
+            {
+            }
+            catch(Exception ex)
+            {
+                test(false);
+            }
+
+            try
+            {
+                test.relayUnknownPreservedAsBase(relay);
+                test(false);
+            }
+            catch(Preserved2 ex)
+            {
+                test(ex.b.equals("base"));
+                test(ex.kp.equals("preserved"));
+                test(ex.kpd.equals("derived"));
+                test(ex.p1.ice_id().equals(PreservedClass.ice_staticId()));
+                PreservedClass pc = (PreservedClass)ex.p1;
+                test(pc.bc.equals("bc"));
+                test(pc.pc.equals("pc"));
+                test(ex.p2 == ex.p1);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                //
+                // For the 1.0 encoding, the unknown exception is sliced to KnownPreserved.
+                //
+                test(test.ice_getEncodingVersion().equals(Ice.Util.Encoding_1_0));
+                test(ex.b.equals("base"));
+                test(ex.kp.equals("preserved"));
+                test(ex.kpd.equals("derived"));
+            }
+            catch(Ice.OperationNotExistException ex)
+            {
+            }
+            catch(Exception ex)
+            {
+                test(false);
+            }
+
+            try
+            {
+                test.relayUnknownPreservedAsKnownPreserved(relay);
+                test(false);
+            }
+            catch(Preserved2 ex)
+            {
+                test(ex.b.equals("base"));
+                test(ex.kp.equals("preserved"));
+                test(ex.kpd.equals("derived"));
+                test(ex.p1.ice_id().equals(PreservedClass.ice_staticId()));
+                PreservedClass pc = (PreservedClass)ex.p1;
+                test(pc.bc.equals("bc"));
+                test(pc.pc.equals("pc"));
+                test(ex.p2 == ex.p1);
+            }
+            catch(KnownPreservedDerived ex)
+            {
+                //
+                // For the 1.0 encoding, the unknown exception is sliced to KnownPreserved.
+                //
+                test(test.ice_getEncodingVersion().equals(Ice.Util.Encoding_1_0));
+                test(ex.b.equals("base"));
+                test(ex.kp.equals("preserved"));
+                test(ex.kpd.equals("derived"));
+            }
+            catch(Ice.OperationNotExistException ex)
+            {
+            }
+            catch(Exception ex)
+            {
+                test(false);
+            }
+
+            adapter.destroy();
+        }
+        out.println("ok");
+
         return test;
     }
 }
