@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,32 +19,7 @@ import Ice.ShortSeqHelper;
 import Ice.IntSeqHelper;
 import Ice.LongSeqHelper;
 import Ice.StringSeqHelper;
-import test.Ice.stream.Test.BoolSSHelper;
-import test.Ice.stream.Test.ByteSSHelper;
-import test.Ice.stream.Test.DoubleSSHelper;
-import test.Ice.stream.Test.FloatSSHelper;
-import test.Ice.stream.Test.ShortSSHelper;
-import test.Ice.stream.Test.IntSSHelper;
-import test.Ice.stream.Test.LongSSHelper;
-import test.Ice.stream.Test.StringSSHelper;
-import test.Ice.stream.Test.MyClass;
-import test.Ice.stream.Test.MyClassPrxHelper;
-import test.Ice.stream.Test.MyClassSHelper;
-import test.Ice.stream.Test.MyClassSSHelper;
-import test.Ice.stream.Test.MyEnum;
-import test.Ice.stream.Test.MyEnumSHelper;
-import test.Ice.stream.Test.MyEnumSSHelper;
-import test.Ice.stream.Test.MyInterface;
-import test.Ice.stream.Test.MyInterfaceHelper;
-import test.Ice.stream.Test.MyInterfaceHolder;
-import test.Ice.stream.Test.SmallStruct;
-import test.Ice.stream.Test.MyException;
-import test.Ice.stream.Test.ByteBoolDHelper;
-import test.Ice.stream.Test.ShortIntDHelper;
-import test.Ice.stream.Test.LongFloatDHelper;
-import test.Ice.stream.Test.StringStringDHelper;
-import test.Ice.stream.Test.StringMyClassDHelper;
-import test.Ice.stream.Test._MyInterfaceDisp;
+import test.Ice.stream.Test.*;
 
 import java.io.PrintWriter;
 
@@ -80,10 +55,10 @@ public class Client extends test.Util.Application
     private static class TestObjectReader extends Ice.ObjectReader
     {
         public void
-        read(Ice.InputStream in, boolean rid)
+        read(Ice.InputStream in)
         {
             obj = new MyClass();
-            obj.__read(in, rid);
+            obj.__read(in);
             called = true;
         }
 
@@ -173,7 +148,7 @@ public class Client extends test.Util.Application
     public int
     run(String[] args)
     {
-    	Ice.Communicator comm = communicator();
+        Ice.Communicator comm = communicator();
         MyClassFactoryWrapper factoryWrapper = new MyClassFactoryWrapper();
         comm.addObjectFactory(factoryWrapper, MyClass.ice_staticId());
         comm.addObjectFactory(new MyInterfaceFactory(), _MyInterfaceDisp.ice_staticId());
@@ -204,6 +179,12 @@ public class Client extends test.Util.Application
             test(in.readBool());
             in.endEncapsulation();
             in.destroy();
+
+            in = Ice.Util.wrapInputStream(comm, data);
+            in.startEncapsulation();
+            test(in.readBool());
+            in.endEncapsulation();
+            in.destroy();
         }
 
         {
@@ -338,6 +319,60 @@ public class Client extends test.Util.Application
             in.destroy();
         }
 
+        {
+            out = Ice.Util.createOutputStream(comm);
+            OptionalClass o = new OptionalClass();
+            o.bo = true;
+            o.by = (byte)5;
+            o.setSh((short)4);
+            o.setI(3);
+            out.writeObject(o);
+            out.writePendingObjects();
+            byte[] data = out.finished();
+            in = Ice.Util.createInputStream(comm, data);
+            TestReadObjectCallback cb = new TestReadObjectCallback();
+            in.readObject(cb);
+            in.readPendingObjects();
+            OptionalClass o2 = (OptionalClass)cb.obj;
+            test(o2.bo == o.bo);
+            test(o2.by == o.by);
+            if(comm.getProperties().getProperty("Ice.Default.EncodingVersion").equals("1.0"))
+            {
+                test(!o2.hasSh());
+                test(!o2.hasI());
+            }
+            else
+            {
+                test(o2.getSh() == o.getSh());
+                test(o2.getI() == o.getI());
+            }
+            out.destroy();
+            in.destroy();
+        }
+
+        {
+            out = Ice.Util.createOutputStream(comm, Ice.Util.Encoding_1_0);
+            OptionalClass o = new OptionalClass();
+            o.bo = true;
+            o.by = (byte)5;
+            o.setSh((short)4);
+            o.setI(3);
+            out.writeObject(o);
+            out.writePendingObjects();
+            byte[] data = out.finished();
+            in = Ice.Util.createInputStream(comm, data, Ice.Util.Encoding_1_0);
+            TestReadObjectCallback cb = new TestReadObjectCallback();
+            in.readObject(cb);
+            in.readPendingObjects();
+            OptionalClass o2 = (OptionalClass)cb.obj;
+            test(o2.bo == o.bo);
+            test(o2.by == o.by);
+            test(!o2.hasSh());
+            test(!o2.hasI());
+            out.destroy();
+            in.destroy();
+        }
+
         {
             final boolean[] arr =
             {
