@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -16,14 +16,12 @@ if len(head) > 0:
     path = [os.path.join(head, p) for p in path]
 path = [os.path.abspath(p) for p in path if os.path.exists(os.path.join(p, "scripts", "TestUtil.py")) ]
 if len(path) == 0:
-    raise "can't find toplevel directory!"
-sys.path.append(os.path.join(path[0]))
-from scripts import *
+    raise RuntimeError("can't find toplevel directory!")
+sys.path.append(os.path.join(path[0], "scripts"))
+import TestUtil
 
-print "starting test...",
-clientProc = TestUtil.startClient("test.Ice.stream.Client",startReader=False)
-print "ok"
-clientProc.startReader()
-
-clientProc.waitTestSuccess()
+print("Running test with default encoding...")
+TestUtil.simpleTest("test.Ice.stream.Client")
 
+print("Running test with 1.0 encoding...")
+TestUtil.simpleTest("test.Ice.stream.Client", "--Ice.Default.EncodingVersion=1.0")
