@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,13 +13,7 @@ public class Server extends test.Util.Application
 {
     public int run(String[] args)
     {
-        //
-        // We don't want connection warnings because of the timeout test.
-        //
-        communicator().getProperties().setProperty("Ice.Warn.Connections", "0");
         communicator().getProperties().setProperty("TestAdapter.Endpoints", "default -p 12010 -t 10000:udp");
-        communicator().getProperties().setProperty("Ice.ThreadPool.Server.ThreadPriority", "10");
-
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("TestAdapter");
         adapter.add(new PriorityI(), communicator().stringToIdentity("test"));
         adapter.activate();
@@ -28,6 +22,14 @@ public class Server extends test.Util.Application
         return 0;
     }
 
+    protected Ice.InitializationData getInitData(Ice.StringSeqHolder argsH)
+    {
+        Ice.InitializationData initData = new Ice.InitializationData();
+        initData.properties = Ice.Util.createProperties(argsH);
+        initData.properties.setProperty("Ice.ThreadPool.Server.ThreadPriority", "10");
+        return initData;
+    }
+
     public static void main(String[] args)
     {
         Server c = new Server();
