@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -46,22 +46,29 @@ public class Server extends test.Util.Application
     protected Ice.InitializationData getInitData(Ice.StringSeqHolder argsH)
     {
         Ice.InitializationData initData = new Ice.InitializationData();
-	initData.properties = Ice.Util.createProperties(argsH);
+        initData.properties = Ice.Util.createProperties(argsH);
         initData.properties.setProperty("Ice.Package.Test", "test.Ice.udp");
         initData.properties.setProperty("Ice.Warn.Connections", "0");
         initData.properties.setProperty("Ice.UDP.RcvSize", "16384");
         initData.properties.setProperty("Ice.UDP.SndSize", "16384");
 
-        String host;
+        String endpoint;
         if(initData.properties.getProperty("Ice.IPv6").equals("1"))
         {
-            host = "\"ff01::1:1\"";
+            if(System.getProperty("os.name").contains("OS X"))
+            {
+                endpoint = "udp -h \"ff02::1:1\" -p 12020 --interface \"lo0\"";
+            }
+            else
+            {
+                endpoint = "udp -h \"ff01::1:1\" -p 12020";
+            }
         }
         else
         {
-            host = "239.255.1.1";
+            endpoint = "udp -h 239.255.1.1 -p 12020";
         }
-        initData.properties.setProperty("McastTestAdapter.Endpoints", "udp -h " + host + " -p 12020");
+        initData.properties.setProperty("McastTestAdapter.Endpoints", endpoint);
         return initData;
     }
 
