@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -10,15 +10,16 @@
 
 import os, sys, re, getopt
 
-for toplevel in [".", "..", "../..", "../../..", "../../../.."]:
-    toplevel = os.path.normpath(toplevel)
-    if os.path.exists(os.path.join(toplevel, "scripts", "TestUtil.py")):
-        break
-else:
-    raise "can't find toplevel directory!"
+path = [ ".", "..", "../..", "../../..", "../../../.." ]
+head = os.path.dirname(sys.argv[0])
+if len(head) > 0:
+    path = [os.path.join(head, p) for p in path]
+path = [os.path.abspath(p) for p in path if os.path.exists(os.path.join(p, "scripts", "TestUtil.py")) ]
+if len(path) == 0:
+    raise RuntimeError("can't find toplevel directory!")
 
-sys.path.append(os.path.join(toplevel))
-from scripts import *
+sys.path.append(os.path.join(path[0], "scripts"))
+import TestUtil
 
 #
 # List of all basic tests.
@@ -36,10 +37,13 @@ tests = [
     ("Ice/slicing/exceptions", ["core"]),
     ("Ice/slicing/objects", ["core"]),
     ("Ice/defaultValue", ["core"]),
+    ("Ice/optional", ["core"]),
+    ("Ice/enums", ["core"]),
     ("Ice/ini", ["once"]),
     ("Ice/scope", ["once"]),
     ("Slice/keyword", ["once"]),
-    ("Slice/structure", ["once"])
+    ("Slice/structure", ["once"]),
+    ("Slice/macros", ["once"]),
     ]
 
 if __name__ == "__main__":
