@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -26,6 +26,18 @@ ZEND_FUNCTION(IcePHP_Operation_call);
 namespace IcePHP
 {
 
+class ParamInfo : public IceUtil::Shared
+{
+public:
+
+    TypeInfoPtr type;
+    bool optional;
+    int tag;
+    int pos;
+};
+typedef IceUtil::Handle<ParamInfo> ParamInfoPtr;
+typedef list<ParamInfoPtr> ParamInfoList;
+
 //
 // Receives an out parameter or return value.
 //
@@ -38,6 +50,8 @@ public:
 
     virtual void unmarshaled(zval*, zval*, void* TSRMLS_DC);
 
+    void unset(TSRMLS_D);
+
     zval* zv;
 };
 typedef IceUtil::Handle<ResultCallback> ResultCallbackPtr;
@@ -50,7 +64,8 @@ class OperationI : public Operation
 {
 public:
 
-    OperationI(const char*, Ice::OperationMode, Ice::OperationMode, zval*, zval*, zval*, zval* TSRMLS_DC);
+    OperationI(const char*, Ice::OperationMode, Ice::OperationMode, Ice::FormatType, zval*, zval*, zval*, zval*
+               TSRMLS_DC);
     ~OperationI();
 
     virtual zend_function* function();
@@ -58,9 +73,12 @@ public:
     string name; // On-the-wire name.
     Ice::OperationMode mode;
     Ice::OperationMode sendMode;
-    TypeInfoList inParams;
-    TypeInfoList outParams;
-    TypeInfoPtr returnType;
+    Ice::FormatType format;
+    ParamInfoList inParams;
+    ParamInfoList optionalInParams;
+    ParamInfoList outParams;
+    ParamInfoList optionalOutParams;
+    ParamInfoPtr returnType;
     ExceptionInfoList exceptions;
     bool sendsClasses;
     bool returnsClasses;
@@ -70,8 +88,9 @@ private:
 
     zend_internal_function* _zendFunction;
 
-    static void convertParams(zval*, TypeInfoList&, bool& TSRMLS_DC);
-    static void getArgInfo(zend_arg_info&, const TypeInfoPtr&, bool);
+    static void convertParams(zval*, ParamInfoList&, bool& TSRMLS_DC);
+    static ParamInfoPtr convertParam(zval*, int TSRMLS_DC);
+    static void getArgInfo(zend_arg_info&, const ParamInfoPtr&, bool);
 };
 typedef IceUtil::Handle<OperationI> OperationIPtr;
 
@@ -110,7 +129,7 @@ protected:
 
     OperationIPtr _op;
 
-    bool prepareRequest(int, zval**, Ice::ByteSeq& TSRMLS_DC);
+    bool prepareRequest(int, zval**, Ice::OutputStreamPtr&, pair<const Ice::Byte*, const Ice::Byte*>& TSRMLS_DC);
     void unmarshalResults(int, zval**, zval*, const pair<const Ice::Byte*, const Ice::Byte*>& TSRMLS_DC);
     zval* unmarshalException(const pair<const Ice::Byte*, const Ice::Byte*>& TSRMLS_DC);
     bool validateException(const ExceptionInfoPtr& TSRMLS_DC) const;
@@ -129,6 +148,35 @@ public:
     virtual void invoke(INTERNAL_FUNCTION_PARAMETERS);
 };
 
+class UserExceptionReaderFactoryI : public Ice::UserExceptionReaderFactory
+{
+public:
+
+    UserExceptionReaderFactoryI(const CommunicatorInfoPtr& communicator TSRMLS_DC) :
+        _communicator(communicator)
+    {
+#ifdef ZTS
+        this->TSRMLS_C = TSRMLS_C;
+#endif
+    }
+
+    virtual void createAndThrow(const string& id) const
+    {
+        ExceptionInfoPtr info = getExceptionInfo(id TSRMLS_CC);
+        if(info)
+        {
+            throw ExceptionReader(_communicator, info TSRMLS_CC);
+        }
+    }
+
+private:
+
+    const CommunicatorInfoPtr _communicator;
+#if ZTS
+    TSRMLS_D;
+#endif
+};
+
 }
 
 //
@@ -157,16 +205,24 @@ IcePHP::ResultCallback::unmarshaled(zval* val, zval*, void* TSRMLS_DC)
     Z_ADDREF_P(zv);
 }
 
+void
+IcePHP::ResultCallback::unset(TSRMLS_D)
+{
+    MAKE_STD_ZVAL(zv);
+    assignUnset(zv TSRMLS_CC);
+}
+
 //
 // OperationI implementation.
 //
-IcePHP::OperationI::OperationI(const char* n, Ice::OperationMode m, Ice::OperationMode sm, zval* in, zval* out,
-                               zval* ret, zval* ex TSRMLS_DC) :
-    name(n), mode(m), sendMode(sm), sendsClasses(false), returnsClasses(false), _zendFunction(0)
+IcePHP::OperationI::OperationI(const char* n, Ice::OperationMode m, Ice::OperationMode sm, Ice::FormatType f, zval* in,
+                               zval* out, zval* ret, zval* ex TSRMLS_DC) :
+    name(n), mode(m), sendMode(sm), format(f), _zendFunction(0)
 {
     //
     // inParams
     //
+    sendsClasses = false;
     if(in)
     {
         convertParams(in, inParams, sendsClasses TSRMLS_CC);
@@ -175,25 +231,61 @@ IcePHP::OperationI::OperationI(const char* n, Ice::OperationMode m, Ice::Operati
     //
     // outParams
     //
+    returnsClasses = false;
     if(out)
     {
         convertParams(out, outParams, returnsClasses TSRMLS_CC);
     }
 
-    numParams = static_cast<int>(inParams.size() + outParams.size());
-
     //
     // returnType
     //
     if(ret)
     {
-        returnType = Wrapper<TypeInfoPtr>::value(ret TSRMLS_CC);
+        returnType = convertParam(ret, 0 TSRMLS_CC);
         if(!returnsClasses)
         {
-            returnsClasses = returnType->usesClasses();
+            returnsClasses = returnType->type->usesClasses();
         }
     }
 
+    numParams = static_cast<int>(inParams.size() + outParams.size());
+
+    class SortFn
+    {
+    public:
+        static bool compare(const ParamInfoPtr& lhs, const ParamInfoPtr& rhs)
+        {
+            return lhs->tag < rhs->tag;
+        }
+
+        static bool isRequired(const ParamInfoPtr& i)
+        {
+            return !i->optional;
+        }
+    };
+
+    //
+    // The inParams list represents the parameters in the order of declaration.
+    // We also need a sorted list of optional parameters.
+    //
+    ParamInfoList l = inParams;
+    copy(l.begin(), remove_if(l.begin(), l.end(), SortFn::isRequired), back_inserter(optionalInParams));
+    optionalInParams.sort(SortFn::compare);
+
+    //
+    // The outParams list represents the parameters in the order of declaration.
+    // We also need a sorted list of optional parameters. If the return value is
+    // optional, we must include it in this list.
+    //
+    l = outParams;
+    copy(l.begin(), remove_if(l.begin(), l.end(), SortFn::isRequired), back_inserter(optionalOutParams));
+    if(returnType && returnType->optional)
+    {
+        optionalOutParams.push_back(returnType);
+    }
+    optionalOutParams.sort(SortFn::compare);
+
     //
     // exceptions
     //
@@ -218,7 +310,7 @@ IcePHP::OperationI::~OperationI()
     if(_zendFunction)
     {
         delete []_zendFunction->arg_info;
-        efree(_zendFunction->function_name);
+        efree(const_cast<char*>(_zendFunction->function_name));
         efree(_zendFunction);
     }
 }
@@ -234,16 +326,20 @@ IcePHP::OperationI::function()
         zend_arg_info* argInfo = new zend_arg_info[numParams];
 
         int i = 0;
-        TypeInfoList::const_iterator p;
+        ParamInfoList::const_iterator p;
         for(p = inParams.begin(); p != inParams.end(); ++p, ++i)
         {
             getArgInfo(argInfo[i], *p, false);
+#if PHP_VERSION_ID < 50400
             argInfo[i].required_num_args = static_cast<zend_uint>(numParams);
+#endif
         }
         for(p = outParams.begin(); p != outParams.end(); ++p, ++i)
         {
             getArgInfo(argInfo[i], *p, true);
+#if PHP_VERSION_ID < 50400
             argInfo[i].required_num_args = static_cast<zend_uint>(numParams);
+#endif
         }
 
         string fixed = fixIdent(name);
@@ -255,9 +351,11 @@ IcePHP::OperationI::function()
         _zendFunction->prototype = 0;
         _zendFunction->num_args = static_cast<zend_uint>(numParams);
         _zendFunction->arg_info = argInfo;
-        _zendFunction->pass_rest_by_reference = 0;
         _zendFunction->required_num_args = _zendFunction->num_args;
+#if PHP_VERSION_ID < 50400
+        _zendFunction->pass_rest_by_reference = 0;
         _zendFunction->return_reference = 0;
+#endif
         _zendFunction->handler = ZEND_FN(IcePHP_Operation_call);
     }
 
@@ -265,43 +363,79 @@ IcePHP::OperationI::function()
 }
 
 void
-IcePHP::OperationI::convertParams(zval* p, TypeInfoList& params, bool& usesClasses TSRMLS_DC)
+IcePHP::OperationI::convertParams(zval* p, ParamInfoList& params, bool& usesClasses TSRMLS_DC)
 {
-    usesClasses = false;
-
     assert(Z_TYPE_P(p) == IS_ARRAY);
     HashTable* arr = Z_ARRVAL_P(p);
     HashPosition pos;
     zend_hash_internal_pointer_reset_ex(arr, &pos);
     void* data;
+    int i = 0;
     while(zend_hash_get_current_data_ex(arr, &data, &pos) != FAILURE)
     {
         zval** val = reinterpret_cast<zval**>(data);
-        TypeInfoPtr type = Wrapper<TypeInfoPtr>::value(*val TSRMLS_CC);
-        params.push_back(type);
-        if(!usesClasses)
+        ParamInfoPtr param = convertParam(*val, i TSRMLS_CC);
+        params.push_back(param);
+        if(!param->optional && !usesClasses)
         {
-            usesClasses = type->usesClasses();
+            usesClasses = param->type->usesClasses();
         }
         zend_hash_move_forward_ex(arr, &pos);
+        ++i;
     }
 }
 
+ParamInfoPtr
+IcePHP::OperationI::convertParam(zval* p, int pos TSRMLS_DC)
+{
+    assert(Z_TYPE_P(p) == IS_ARRAY);
+    HashTable* arr = Z_ARRVAL_P(p);
+    assert(zend_hash_num_elements(arr) == 3);
+
+    ParamInfoPtr param = new ParamInfo;
+    zval** m;
+
+    zend_hash_index_find(arr, 0, reinterpret_cast<void**>(&m));
+    param->type = Wrapper<TypeInfoPtr>::value(*m TSRMLS_CC);
+    zend_hash_index_find(arr, 1, reinterpret_cast<void**>(&m));
+    assert(Z_TYPE_PP(m) == IS_BOOL);
+    param->optional = Z_BVAL_PP(m) ? true : false;
+    zend_hash_index_find(arr, 2, reinterpret_cast<void**>(&m));
+    assert(Z_TYPE_PP(m) == IS_LONG);
+    param->tag = Z_LVAL_PP(m);
+    param->pos = pos;
+
+    return param;
+}
+
 void
-IcePHP::OperationI::getArgInfo(zend_arg_info& arg, const TypeInfoPtr& info, bool out)
+IcePHP::OperationI::getArgInfo(zend_arg_info& arg, const ParamInfoPtr& info, bool out)
 {
     arg.name = 0;
     arg.class_name = 0;
     arg.allow_null = 1;
-    if(SequenceInfoPtr::dynamicCast(info) || DictionaryInfoPtr::dynamicCast(info))
+
+    if(!info->optional)
     {
-        arg.array_type_hint = 1;
+        const bool isArray = SequenceInfoPtr::dynamicCast(info->type) || DictionaryInfoPtr::dynamicCast(info->type);
+
+#if PHP_VERSION_ID < 50400
+        arg.array_type_hint = isArray ? 1 : 0;
+        arg.return_reference = 0;
+#else
+        arg.type_hint = isArray ? IS_ARRAY : 0;
+#endif
     }
     else
     {
+#if PHP_VERSION_ID < 50400
         arg.array_type_hint = 0;
+        arg.return_reference = 0;
+#else
+        arg.type_hint = 0;
+#endif
     }
-    arg.return_reference = 0;
+
     arg.pass_by_reference = out ? 1 : 0;
 }
 
@@ -326,7 +460,8 @@ IcePHP::TypedInvocation::TypedInvocation(const Ice::ObjectPrx& prx, const Commun
 }
 
 bool
-IcePHP::TypedInvocation::prepareRequest(int argc, zval** args, Ice::ByteSeq& bytes TSRMLS_DC)
+IcePHP::TypedInvocation::prepareRequest(int argc, zval** args, Ice::OutputStreamPtr& os, 
+                                        pair<const Ice::Byte*, const Ice::Byte*>& params TSRMLS_DC)
 {
     //
     // Verify that the expected number of arguments are supplied. The context argument is optional.
@@ -353,19 +488,51 @@ IcePHP::TypedInvocation::prepareRequest(int argc, zval** args, Ice::ByteSeq& byt
             //
             // Marshal the in parameters.
             //
-            Ice::OutputStreamPtr os = Ice::createOutputStream(_communicator->getCommunicator());
+            os = Ice::createOutputStream(_communicator->getCommunicator());
+            os->startEncapsulation(_prx->ice_getEncodingVersion(), _op->format);
 
             ObjectMap objectMap;
-            int i = 0;
-            for(TypeInfoList::iterator p = _op->inParams.begin(); p != _op->inParams.end(); ++p, ++i)
+            ParamInfoList::iterator p;
+
+            //
+            // Validate the supplied arguments.
+            //
+            for(p = _op->inParams.begin(); p != _op->inParams.end(); ++p)
             {
-                zval* arg = args[i];
-                if(!(*p)->validate(arg TSRMLS_CC))
+                ParamInfoPtr info = *p;
+                zval* arg = args[info->pos];
+                if((!info->optional || !isUnset(arg TSRMLS_CC)) && !info->type->validate(arg TSRMLS_CC))
                 {
-                    invalidArgument("invalid value for argument %d in operation `%s'" TSRMLS_CC, i, _op->name.c_str());
+                    invalidArgument("invalid value for argument %d in operation `%s'" TSRMLS_CC, info->pos + 1,
+                                    _op->name.c_str());
                     return false;
                 }
-                (*p)->marshal(arg, os, &objectMap TSRMLS_CC);
+            }
+
+            //
+            // Marshal the required parameters.
+            //
+            for(p = _op->inParams.begin(); p != _op->inParams.end(); ++p)
+            {
+                ParamInfoPtr info = *p;
+                if(!info->optional)
+                {
+                    zval* arg = args[info->pos];
+                    info->type->marshal(arg, os, &objectMap, false TSRMLS_CC);
+                }
+            }
+
+            //
+            // Marshal the optional parameters.
+            //
+            for(p = _op->optionalInParams.begin(); p != _op->optionalInParams.end(); ++p)
+            {
+                ParamInfoPtr info = *p;
+                zval* arg = args[info->pos];
+                if(!isUnset(arg TSRMLS_CC) && os->writeOptional(info->tag, info->type->optionalFormat()))
+                {
+                    info->type->marshal(arg, os, &objectMap, true TSRMLS_CC);
+                }
             }
 
             if(_op->sendsClasses)
@@ -373,7 +540,8 @@ IcePHP::TypedInvocation::prepareRequest(int argc, zval** args, Ice::ByteSeq& byt
                 os->writePendingObjects();
             }
 
-            os->finished(bytes);
+            os->endEncapsulation();
+            params = os->finished();
         }
         catch(const AbortMarshaling&)
         {
@@ -393,7 +561,19 @@ void
 IcePHP::TypedInvocation::unmarshalResults(int argc, zval** args, zval* ret,
                                           const pair<const Ice::Byte*, const Ice::Byte*>& bytes TSRMLS_DC)
 {
-    Ice::InputStreamPtr is = Ice::createInputStream(_communicator->getCommunicator(), bytes);
+    Ice::InputStreamPtr is = Ice::wrapInputStream(_communicator->getCommunicator(), bytes);
+
+    //
+    // Store a pointer to a local SlicedDataUtil object as the stream's closure.
+    // This is necessary to support object unmarshaling (see ObjectReader).
+    //
+    SlicedDataUtil util;
+    assert(!is->closure());
+    is->closure(&util);
+
+    is->startEncapsulation();
+
+    ParamInfoList::iterator p;
 
     //
     // These callbacks collect references to the unmarshaled values. We copy them into
@@ -402,23 +582,56 @@ IcePHP::TypedInvocation::unmarshalResults(int argc, zval** args, zval* ret,
     ResultCallbackList outParamCallbacks;
     ResultCallbackPtr retCallback;
 
+    outParamCallbacks.resize(_op->outParams.size());
+
     //
-    // Unmarshal the out parameters.
+    // Unmarshal the required out parameters.
     //
-    for(TypeInfoList::iterator p = _op->outParams.begin(); p != _op->outParams.end(); ++p)
+    for(p = _op->outParams.begin(); p != _op->outParams.end(); ++p)
     {
-        ResultCallbackPtr cb = new ResultCallback;
-        outParamCallbacks.push_back(cb);
-        (*p)->unmarshal(is, cb, _communicator, 0, 0 TSRMLS_CC);
+        ParamInfoPtr info = *p;
+        if(!info->optional)
+        {
+            ResultCallbackPtr cb = new ResultCallback;
+            outParamCallbacks[info->pos] = cb;
+            info->type->unmarshal(is, cb, _communicator, 0, 0, false TSRMLS_CC);
+        }
     }
 
     //
-    // Unmarshal the return value.
+    // Unmarshal the required return value, if any.
     //
-    if(_op->returnType)
+    if(_op->returnType && !_op->returnType->optional)
     {
         retCallback = new ResultCallback;
-        _op->returnType->unmarshal(is, retCallback, _communicator, 0, 0 TSRMLS_CC);
+        _op->returnType->type->unmarshal(is, retCallback, _communicator, 0, 0, false TSRMLS_CC);
+    }
+
+    //
+    // Unmarshal the optional results. This includes an optional return value.
+    //
+    for(p = _op->optionalOutParams.begin(); p != _op->optionalOutParams.end(); ++p)
+    {
+        ParamInfoPtr info = *p;
+
+        ResultCallbackPtr cb = new ResultCallback;
+        if(_op->returnType && info->tag == _op->returnType->tag)
+        {
+            retCallback = cb;
+        }
+        else
+        {
+            outParamCallbacks[info->pos] = cb;
+        }
+
+        if(is->readOptional(info->tag, info->type->optionalFormat()))
+        {
+            info->type->unmarshal(is, cb, _communicator, 0, 0, true TSRMLS_CC);
+        }
+        else
+        {
+            cb->unset(TSRMLS_C);
+        }
     }
 
     if(_op->returnsClasses)
@@ -426,6 +639,10 @@ IcePHP::TypedInvocation::unmarshalResults(int argc, zval** args, zval* ret,
         is->readPendingObjects();
     }
 
+    is->endEncapsulation();
+
+    util.update(TSRMLS_C);
+
     int i = static_cast<int>(_op->inParams.size());
     for(ResultCallbackList::iterator q = outParamCallbacks.begin(); q != outParamCallbacks.end(); ++q, ++i)
     {
@@ -451,71 +668,48 @@ IcePHP::TypedInvocation::unmarshalResults(int argc, zval** args, zval* ret,
 zval*
 IcePHP::TypedInvocation::unmarshalException(const pair<const Ice::Byte*, const Ice::Byte*>& bytes TSRMLS_DC)
 {
-    int traceSlicing = -1;
+    Ice::InputStreamPtr is = Ice::wrapInputStream(_communicator->getCommunicator(), bytes);
 
-    Ice::InputStreamPtr is = Ice::createInputStream(_communicator->getCommunicator(), bytes);
+    //
+    // Store a pointer to a local SlicedDataUtil object as the stream's closure.
+    // This is necessary to support object unmarshaling (see ObjectReader).
+    //
+    SlicedDataUtil util;
+    assert(!is->closure());
+    is->closure(&util);
 
-    bool usesClasses;
-    is->read(usesClasses);
+    is->startEncapsulation();
 
-    string id;
-    is->read(id);
-    const string origId = id;
-
-    while(!id.empty())
+    try
     {
-        ExceptionInfoPtr info = getExceptionInfo(id TSRMLS_CC);
-        if(info)
-        {
-            zval* ex = info->unmarshal(is, _communicator TSRMLS_CC);
-            if(ex)
-            {
-                if(info->usesClasses)
-                {
-                    is->readPendingObjects();
-                }
+        Ice::UserExceptionReaderFactoryPtr factory = new UserExceptionReaderFactoryI(_communicator TSRMLS_CC);
+        is->throwException(factory);
+    }
+    catch(const ExceptionReader& r)
+    {
+        is->endEncapsulation();
 
-                if(validateException(info TSRMLS_CC))
-                {
-                    return ex;
-                }
-                else
-                {
-                    zval_ptr_dtor(&ex);
-                    Ice::UnknownUserException uue(__FILE__, __LINE__,
-                                                  "operation raised undeclared exception `" + id + "'");
-                    return convertException(uue TSRMLS_CC);
-                }
+        zval* ex = r.getException();
+        ExceptionInfoPtr info = r.getInfo();
+
+        if(validateException(info TSRMLS_CC))
+        {
+            util.update(TSRMLS_C);
+
+            Ice::SlicedDataPtr slicedData = r.getSlicedData();
+            if(slicedData)
+            {
+                SlicedDataUtil::setMember(ex, slicedData TSRMLS_CC);
             }
+
+            return ex;
         }
         else
         {
-            if(traceSlicing == -1)
-            {
-                traceSlicing =
-                    _communicator->getCommunicator()->getProperties()->getPropertyAsInt("Ice.Trace.Slicing") > 0;
-            }
-
-            if(traceSlicing > 0)
-            {
-                _communicator->getCommunicator()->getLogger()->trace("Slicing", "unknown exception type `" + id + "'");
-            }
-
-            is->skipSlice(); // Slice off what we don't understand.
-
-            try
-            {
-                is->read(id); // Read type id for next slice.
-            }
-            catch(Ice::UnmarshalOutOfBoundsException& ex)
-            {
-                //
-                // When readString raises this exception it means we've seen the last slice,
-                // so we set the reason member to a more helpful message.
-                //
-                ex.reason = "unknown exception type `" + origId + "'";
-                return convertException(ex TSRMLS_CC);
-            }
+            zval_ptr_dtor(&ex);
+            Ice::UnknownUserException uue(__FILE__, __LINE__,
+                                          "operation raised undeclared exception `" + info->id + "'");
+            return convertException(uue TSRMLS_CC);
         }
     }
 
@@ -525,7 +719,7 @@ IcePHP::TypedInvocation::unmarshalException(const pair<const Ice::Byte*, const I
     // have a factory for. This means that sender and receiver disagree
     // about the Slice definitions they use.
     //
-    Ice::UnknownUserException uue(__FILE__, __LINE__, "unknown exception type `" + origId + "'");
+    Ice::UnknownUserException uue(__FILE__, __LINE__, "unknown exception");
     return convertException(uue TSRMLS_CC);
 }
 
@@ -576,9 +770,10 @@ IcePHP::SyncTypedInvocation::invoke(INTERNAL_FUNCTION_PARAMETERS)
         runtimeError("unable to get arguments" TSRMLS_CC);
         return;
     }
-
-    Ice::ByteSeq params;
-    if(!prepareRequest(ZEND_NUM_ARGS(), *args, params TSRMLS_CC))
+    
+    Ice::OutputStreamPtr os;
+    pair<const Ice::Byte*, const Ice::Byte*> params;
+    if(!prepareRequest(ZEND_NUM_ARGS(), *args, os, params TSRMLS_CC))
     {
         return;
     }
@@ -668,13 +863,14 @@ ZEND_FUNCTION(IcePHP_defineOperation)
     int nameLen;
     long mode;
     long sendMode;
+    long format;
     zval* inParams;
     zval* outParams;
     zval* returnType;
     zval* exceptions;
 
-    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, const_cast<char*>("oslla!a!o!a!"), &cls, &name, &nameLen, &mode,
-                             &sendMode, &inParams, &outParams, &returnType, &exceptions) == FAILURE)
+    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, const_cast<char*>("osllla!a!a!a!"), &cls, &name, &nameLen,
+                             &mode, &sendMode, &format, &inParams, &outParams, &returnType, &exceptions) == FAILURE)
     {
         return;
     }
@@ -684,8 +880,8 @@ ZEND_FUNCTION(IcePHP_defineOperation)
     assert(c);
 
     OperationIPtr op = new OperationI(name, static_cast<Ice::OperationMode>(mode),
-                                      static_cast<Ice::OperationMode>(sendMode), inParams, outParams, returnType,
-                                      exceptions TSRMLS_CC);
+                                      static_cast<Ice::OperationMode>(sendMode), static_cast<Ice::FormatType>(format),
+                                      inParams, outParams, returnType, exceptions TSRMLS_CC);
 
     c->addOperation(name, op);
 }
