@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -15,20 +15,20 @@ import Demo
 
 class CallbackI(Demo.Callback):
     def initiateCallback(self, proxy, current=None):
-        print "initiating callback to: " + current.adapter.getCommunicator().proxyToString(proxy)
+        print("initiating callback to: " + current.adapter.getCommunicator().proxyToString(proxy))
         try:
             proxy.callback()
         except:
             traceback.print_exc()
 
     def shutdown(self, current=None):
-        print "shutting down..."
+        print("shutting down...")
         current.adapter.getCommunicator().shutdown()
 
 class Server(Ice.Application):
     def run(self, args):
         if len(args) > 1:
-            print self.appName() + ": too many arguments"
+            print(self.appName() + ": too many arguments")
             return 1
 
         adapter = self.communicator().createObjectAdapter("Callback.Server")
