@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -15,14 +15,14 @@ import Demo
 
 class CallbackSenderI(Demo.CallbackSender):
     def initiateCallback(self, proxy, current=None):
-        print "initiating callback"
+        print("initiating callback")
         try:
             proxy.callback()
         except:
             traceback.print_exc()
 
     def shutdown(self, current=None):
-        print "Shutting down..."
+        print("Shutting down...")
         try:
             current.adapter.getCommunicator().shutdown()
         except:
@@ -31,7 +31,7 @@ class CallbackSenderI(Demo.CallbackSender):
 class Server(Ice.Application):
     def run(self, args):
         if len(args) > 1:
-            print self.appName() + ": too many arguments"
+            print(self.appName() + ": too many arguments")
             return 1
 
         adapter = self.communicator().createObjectAdapter("Callback.Server")
