@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -11,13 +11,14 @@
 import sys, traceback, time, Ice
 
 Ice.loadSlice('Hello.ice')
+Ice.updateModules()
 import Demo
 
 class HelloI(Demo.Hello):
     def sayHello(self, delay, current=None):
         if delay != 0:
             time.sleep(delay / 1000.0)
-        print "Hello World!"
+        print("Hello World!")
 
     def shutdown(self, current=None):
         current.adapter.getCommunicator().shutdown()
@@ -25,7 +26,7 @@ class HelloI(Demo.Hello):
 class Server(Ice.Application):
     def run(self, args):
         if len(args) > 1:
-            print self.appName() + ": too many arguments"
+            print(self.appName() + ": too many arguments")
             return 1
 
         adapter = self.communicator().createObjectAdapter("Hello")
