@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -16,19 +16,19 @@ import Demo
 class Client(Ice.Application):
     def run(self, args):
         if len(args) > 1:
-            print self.appName() + ": too many arguments"
+            print(self.appName() + ": too many arguments")
             return 1
 
         ping = Demo.PingPrx.checkedCast(self.communicator().propertyToProxy('Ping.Proxy'))
         if not ping:
-            print "invalid proxy"
+            print("invalid proxy")
             return 1
 
         # Initial ping to setup the connection.
         ping.ice_ping();
 
         repetitions = 100000
-        print "pinging server " + str(repetitions) + " times (this may take a while)"
+        print("pinging server " + str(repetitions) + " times (this may take a while)")
 
         tsec = time.time()
 
@@ -40,8 +40,8 @@ class Client(Ice.Application):
         tsec = time.time() - tsec
         tmsec = tsec * 1000.0
 
-        print "time for %d pings: %.3fms" % (repetitions, tmsec)
-        print "time per ping: %.3fms" % (tmsec / repetitions)
+        print("time for %d pings: %.3fms" % (repetitions, tmsec))
+        print("time per ping: %.3fms" % (tmsec / repetitions))
 
         return 0
 
