@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -17,7 +17,7 @@ status = 0
 ice = None
 try:
     ic = Ice.initialize(sys.argv)
-    base = ic.stringToProxy("SimplePrinter:default -p 10000")
+    base = ic.stringToProxy("SimplePrinter:default -h localhost -p 10000")
     printer = Demo.PrinterPrx.checkedCast(base)
     if not printer:
         raise RuntimeError("Invalid proxy")
