@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -15,13 +15,13 @@ import Demo
 
 class PrinterI(Demo.Printer):
     def printString(self, s, current=None):
-        print s
+        print(s)
 
 status = 0
 ice = None
 try:
     ic = Ice.initialize(sys.argv)
-    adapter = ic.createObjectAdapterWithEndpoints("SimplePrinterAdapter", "default -p 10000")
+    adapter = ic.createObjectAdapterWithEndpoints("SimplePrinterAdapter", "default -h localhost -p 10000")
     object = PrinterI()
     adapter.add(object, ic.stringToIdentity("SimplePrinter"))
     adapter.activate()
