@@ -1,25 +1,31 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2009 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
 #
 # **********************************************************************
 
-import os, sys, traceback
+import os, sys, traceback, array
 
 import Ice
-Ice.loadSlice('TestAMD.ice')
+slice_dir = Ice.getSliceDir()
+if not slice_dir:
+    print(sys.argv[0] + ': Slice directory not found.')
+    sys.exit(1)
+
+Ice.loadSlice('"-I' + slice_dir + '" TestAMD.ice')
 import Test
 
-class ThrowerI(Test.Thrower):
-    def __init__(self, adapter):
-        self._adapter = adapter
+def test(b):
+    if not b:
+        raise RuntimeError('test assertion failed')
 
+class ThrowerI(Test.Thrower):
     def shutdown_async(self, cb, current=None):
-        self._adapter.getCommunicator().shutdown()
+        current.adapter.getCommunicator().shutdown()
         cb.ice_response()
 
     def supportsUndeclaredExceptions_async(self, cb, current=None):
@@ -107,24 +113,47 @@ class ThrowerI(Test.Thrower):
         cb.ice_exception(Ice.TimeoutException())
 
     def throwNonIceException_async(self, cb, current=None):
+        # Python-specific: make sure the argument is validated.
+        try:
+            cb.ice_exception('foo')
+            test(False)
+        except TypeError:
+            pass
+
         cb.ice_exception(RuntimeError("12345"))
 
     def throwAssertException_async(self, cb, current=None):
         raise RuntimeError("operation `throwAssertException' not supported")
 
+    def throwMemoryLimitException_async(self, cb, seq, current=None):
+        cb.ice_response(bytearray(20 * 1024))
+
+    def throwLocalExceptionIdempotent_async(self, cb, current=None):
+        cb.ice_exception(Ice.TimeoutException())
+
+    def throwAfterResponse_async(self, cb, current=None):
+        cb.ice_response()
+        raise RuntimeError("12345")
+
+    def throwAfterException_async(self, cb, current=None):
+        cb.ice_exception(Test.A())
+        raise RuntimeError("12345")
+
 def run(args, communicator):
-    properties = communicator.getProperties()
-    properties.setProperty("Ice.Warn.Dispatch", "0")
-    properties.setProperty("TestAdapter.Endpoints", "default -p 12010 -t 10000:udp")
     adapter = communicator.createObjectAdapter("TestAdapter")
-    object = ThrowerI(adapter)
+    object = ThrowerI()
     adapter.add(object, communicator.stringToIdentity("thrower"))
     adapter.activate()
     communicator.waitForShutdown()
     return True
 
 try:
-    communicator = Ice.initialize(sys.argv)
+    initData = Ice.InitializationData()
+    initData.properties = Ice.createProperties(sys.argv)
+    initData.properties.setProperty("Ice.Warn.Dispatch", "0")
+    initData.properties.setProperty("TestAdapter.Endpoints", "default -p 12010:udp")
+    initData.properties.setProperty("Ice.MessageSizeMax", "10")
+    communicator = Ice.initialize(sys.argv, initData)
     status = run(sys.argv, communicator)
 except:
     traceback.print_exc()
