@@ -1,6 +1,6 @@
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -151,6 +151,12 @@ class InitialI(Test.Initial):
     def setI(self, i, current=None):
         pass
 
+    def opBaseSeq(self, inSeq, current=None):
+        return (inSeq, inSeq)
+
+    def getCompact(self, current=None):
+        return Test.CompactExt()
+
 class UnexpectedObjectExceptionTestI(Test.UnexpectedObjectExceptionTest):
     def op(self, current=None):
         return Test.AlsoEmpty()
