@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -27,7 +27,7 @@ extern "C"
 {
 
 void
-Init_IceRuby()
+ICE_DECLSPEC_EXPORT Init_IceRuby()
 {
     iceModule = rb_define_module("Ice");
     initCommunicator(iceModule);
