@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -9,10 +9,11 @@
 # **********************************************************************
 
 require 'pathname'
-
 require 'Ice'
-Ice::loadSlice('Test.ice')
-require 'AllTests'
+
+slice_dir = Ice.getSliceDir
+Ice::loadSlice("'-I" + slice_dir + "' Test.ice")
+require './AllTests'
 
 def run(args, communicator)
     thrower = allTests(communicator)
@@ -21,7 +22,10 @@ def run(args, communicator)
 end
 
 begin
-    communicator = Ice.initialize(ARGV)
+    initData = Ice::InitializationData.new
+    initData.properties = Ice.createProperties(ARGV)
+    initData.properties.setProperty("Ice.MessageSizeMax", "10")
+    communicator = Ice.initialize(ARGV, initData)
     status = run(ARGV, communicator)
 rescue => ex
     puts $!
