@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -11,7 +11,7 @@
 require 'pathname'
 require 'Ice'
 Ice::loadSlice('Test.ice')
-require 'AllTests'
+require './AllTests'
 
 def run(args, communicator)
     g = allTests(communicator)
