@@ -1,14 +1,14 @@
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
 #
 # **********************************************************************
 
-require 'Twoways'
-require 'BatchOneways'
+require './Twoways'
+require './BatchOneways'
 
 def allTests(communicator)
     ref = "test:default -p 12010"
