@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2011 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2013 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -28,7 +28,7 @@ nodeOptions = r' --Ice.Warn.Connections=0' + \
               r' --IceGrid.Node.Trace.Adapter=0' + \
               r' --IceGrid.Node.Trace.Server=0' + \
               r' --IceGrid.Node.ThreadPool.SizeWarn=0' + \
-              r' --IceGrid.Node.PrintServersReady=node' + \
+              r' --IceGrid.Node.PrintServersReady=node1' + \
               r' --Ice.NullHandleAbort' + \
               r' --Ice.ThreadPool.Server.Size=0' + \
               r' --Ice.ServerIdleTime=0'
@@ -70,11 +70,7 @@ def getDefaultLocatorProperty():
 
 def startIceGridRegistry(testdir, dynamicRegistration = False):
 
-    iceGrid = ""
-    if TestUtil.isBCC2010() or TestUtil.isVC6():
-        iceGrid = os.path.join(TestUtil.getServiceDir(), "icegridregistry")
-    else:
-        iceGrid = os.path.join(TestUtil.getCppBinDir(), "icegridregistry")
+    iceGrid = TestUtil.getIceGridRegistry()
 
     command = ' --nowarn ' + registryOptions
     if dynamicRegistration:
@@ -95,7 +91,8 @@ def startIceGridRegistry(testdir, dynamicRegistration = False):
         else:
             cleanDbDir(dataDir)
 
-        print "starting icegrid " + name + "...",
+        sys.stdout.write("starting icegrid " + name + "... ")
+        sys.stdout.flush()
         cmd = command + ' ' + TestUtil.getQtSqlOptions('IceGrid') + \
               r' --Ice.ProgramName=' + name + \
               r' --IceGrid.Registry.Client.Endpoints="default -p ' + str(iceGridPort + i) + '" ' + \
@@ -106,9 +103,9 @@ def startIceGridRegistry(testdir, dynamicRegistration = False):
 
         driverConfig = TestUtil.DriverConfig("server")
         driverConfig.lang = "cpp"
-        proc = TestUtil.startServer(iceGrid, cmd, driverConfig, count = 4)
+        proc = TestUtil.startServer(iceGrid, cmd, driverConfig, count = 5)
         procs.append(proc)
-        print "ok"
+        print("ok")
 
         i = i + 1
     return procs
@@ -117,14 +114,16 @@ def shutdownIceGridRegistry(procs):
 
     i = nreplicas
     while i > 0:
-        print "shutting down icegrid replica-" + str(i) + "...",
+        sys.stdout.write("shutting down icegrid replica-" + str(i) + "... ")
+        sys.stdout.flush()
         iceGridAdmin("registry shutdown replica-" + str(i))
-        print "ok"
+        print("ok")
         i = i - 1
 
-    print "shutting down icegrid registry...",
+    sys.stdout.write("shutting down icegrid registry... ")
+    sys.stdout.flush()
     iceGridAdmin("registry shutdown")
-    print "ok"
+    print("ok")
 
     for p in procs:
         p.waitTestSuccess()
@@ -136,6 +135,7 @@ def iceGridNodePropertiesOverride():
     #
     overrideOptions = ''
     for opt in shlex.split(TestUtil.getCommandLineProperties("", TestUtil.DriverConfig("server"))):
+       opt = opt.strip().replace("--", "")
        index = opt.find("=")
        if index == -1:
           overrideOptions += ("%s=1 ") % opt
@@ -145,18 +145,17 @@ def iceGridNodePropertiesOverride():
           if(value.find(' ') == -1):
              overrideOptions += ("%s=%s ") % (key, value)
           else:
-             overrideOptions += ("%s=\\\"%s\\\" ") % (key, value)
+             #
+             # NOTE: We need 2 backslash before the quote to run the
+             # C# test/IceGrid/simple test with SSL.
+             #
+             overrideOptions += ("%s=\\\"%s\\\" ") % (key, value.replace('"', '\\\\\\"'))
 
     return overrideOptions
 
 def startIceGridNode(testdir):
 
-    iceGrid = ""
-    if TestUtil.isBCC2010() or TestUtil.isVC6():
-        iceGrid = os.path.join(TestUtil.getServiceDir(), "icegridnode")
-    else:
-        iceGrid = os.path.join(TestUtil.getCppBinDir(), "icegridnode")
-
+    iceGrid = TestUtil.getIceGridNode()
     dataDir = os.path.join(testdir, "db", "node")
     if not os.path.exists(dataDir):
         os.mkdir(dataDir)
@@ -166,7 +165,8 @@ def startIceGridNode(testdir):
     overrideOptions = '" ' + iceGridNodePropertiesOverride()
     overrideOptions += ' Ice.ServerIdleTime=0 Ice.PrintProcessId=0 Ice.PrintAdapterReady=0"'
 
-    print "starting icegrid node...",
+    sys.stdout.write("starting icegrid node... ")
+    sys.stdout.flush()
     command = r' --nowarn ' + nodeOptions + getDefaultLocatorProperty() + \
               r' --IceGrid.Node.Data="' + dataDir + '"' \
               r' --IceGrid.Node.Name=localnode' + \
@@ -174,9 +174,9 @@ def startIceGridNode(testdir):
 
     driverConfig = TestUtil.DriverConfig("server")
     driverConfig.lang = "cpp"
-    proc = TestUtil.startServer(iceGrid, command, driverConfig, adapter='node')
+    proc = TestUtil.startServer(iceGrid, command, driverConfig, adapter='node1')
         
-    print "ok"
+    print("ok")
 
     return proc
 
@@ -202,7 +202,7 @@ def iceGridAdmin(cmd, ignoreFailure = False):
         TestUtil.appVerifierAfterTestEnd([TestUtil.getIceGridAdmin()])
 
     if not ignoreFailure and status:
-        print proc.buf
+        print(proc.buf)
         sys.exit(1)
     return proc.buf
     
@@ -218,7 +218,7 @@ def iceGridTest(application, additionalOptions = "", applicationOptions = ""):
     testdir = os.getcwd()
     if not TestUtil.isWin32() and os.getuid() == 0:
         print
-        print "*** can't run test as root ***"
+        print("*** can't run test as root ***")
         print
         return
 
@@ -240,26 +240,29 @@ def iceGridTest(application, additionalOptions = "", applicationOptions = ""):
     iceGridNodeProc = startIceGridNode(testdir)
     
     if application != "":
-        print "adding application...",
+        sys.stdout.write("adding application... ")
+        sys.stdout.flush()
         iceGridAdmin("application add -n '" + os.path.join(testdir, application) + "' " + \
                      "test.dir='" + testdir + "' ice.bindir='" + TestUtil.getCppBinDir() + "' " + applicationOptions)
-        print "ok"
+        print("ok")
 
-    print "starting client...",
+    sys.stdout.write("starting client... ")
+    sys.stdout.flush()
     clientProc = TestUtil.startClient(client, clientOptions, TestUtil.DriverConfig("client"), startReader = False)
-    print "ok"
+    print("ok")
     clientProc.startReader()
-
     clientProc.waitTestSuccess()
 
     if application != "":
-        print "remove application...",
+        sys.stdout.write("remove application... ")
+        sys.stdout.flush()
         iceGridAdmin("application remove Test")
-        print "ok"
+        print("ok")
 
-    print "shutting down icegrid node...",
+    sys.stdout.write("shutting down icegrid node... ")
+    sys.stdout.flush()
     iceGridAdmin("node shutdown localnode")
-    print "ok"
+    print("ok")
     shutdownIceGridRegistry(registryProcs)
     iceGridNodeProc.waitTestSuccess()
 
@@ -288,13 +291,15 @@ def iceGridClientServerTest(additionalClientOptions, additionalServerOptions):
     
     registryProcs = startIceGridRegistry(testdir, True)
 
-    print "starting server...",
+    sys.stdout.write("starting server... ")
+    sys.stdout.flush()
     serverProc= TestUtil.startServer(server, serverOptions, TestUtil.DriverConfig("server"))
-    print "ok"
+    print("ok")
 
-    print "starting client...",
+    sys.stdout.write("starting client... ")
+    sys.stdout.flush()
     clientProc = TestUtil.startClient(client, clientOptions, TestUtil.DriverConfig("client"))
-    print "ok"
+    print("ok")
 
     clientProc.waitTestSuccess()
     serverProc.waitTestSuccess()
